/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qar.h"

extern "C" QplugReturnStatus QarPOWER(QplugArgList *);

QplugReturnStatus QarPOWER  ( QplugArgList *call_list )

{
QplugReturnStatus	status;
int qar_status;


QdObject_var in1_ptr;
QdObject_var in2_ptr;
QdObject_var out_ptr;

char * attr_ptr;
char * string_buf;
char * out_name;

status = QPLUG_SUCCESS;


//	Unpack the input data objects

 in1_ptr = (*call_list)[0];

 in2_ptr = (*call_list)[1];


// Get name of output object

 char * slot_ptr = GetSlotText(0);
 if ( slot_ptr[0] != '\0' ) {
    out_name = QplugNewSTR(slot_ptr);
 }
 else{
    out_name = QplugNewSTR("RaisedToPower");
    SetSlotText(0,out_name);
 }

// Test validity of inputs...

qar_status =  QarTestConformal(in1_ptr, in2_ptr);
  if(qar_status == QAR_ERR)     
  {
    QplugAppendTextDisplay ("Arithmetic>> input sequences not joined properly\n");
     return QPLUG_FAILURE; 
  } 

// get base SI for input data


  if( QarHasUnits(in2_ptr) ){
    // issue warning
    QplugAppendTextDisplay ("WARNING>> Exponent has units.\a \n");  
  }


// do arithmetic...

out_ptr = pow(in1_ptr, in2_ptr);

if (out_ptr.is_nil()) {
    QplugAppendTextDisplay (
        "Arithmetic>> This Operation is not yet supported between these input types\n");
    return QPLUG_FAILURE; 
}

 // Add timeline: Either timeline is safe if it exists
 
 QdTimeSeq_var tt = get_timetags(in1_ptr);
 if (tt.is_nil()) tt = get_timetags(in2_ptr);
 if (!tt.is_nil()) set_timetags(out_ptr, tt);


 // Set QSAS essential attributes
  
  QdRScalar_var B_ptr = QdRScalar_var::narrow(in2_ptr);

 if(!B_ptr.is_nil()){   // can only do these if power is unique
  
   // SI_conversion
   // the value for constant power is a global
   
   double constB = (double)(*B_ptr);
   if( (attr_ptr = QarSIConv_power(in1_ptr, constB) ) != NULL){
     QuSetTxtAttr("SI_conversion", attr_ptr, out_ptr);
     delete[] attr_ptr;
   }
  
   // UNITS
   // the value for constant power is a global
 
   if( (attr_ptr = QarUNITS_power(in1_ptr, constB) ) != NULL){
     QuSetTxtAttr("UNITS", attr_ptr, out_ptr);
     delete[] attr_ptr;
   }

 }
 
 // FIELDNAM

 // ignore fieldnames as concatenating these rapidly gets too long

   QuSetTxtAttr("FIELDNAM", out_name, out_ptr);
   free (out_name);
 
 // Frame
 
   QuSetTxtAttr("Frame", "scalar>na", out_ptr);
 
 
  //	..add  the output series to the list

  call_list->push_back(out_ptr);

  QplugAppendTextDisplay ("Power: Done\n");
  
return status;

}

