#include "qOperators.h"

QdObject_var QdDiv( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{
	QdObject_var        qdResult;
	if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
	{
		// The object will be empty - this should never happen
		// The qdResultt object is NULL at this point
		return ( qdResult );
	}

	QplugReturnStatus	status      = QPLUG_SUCCESS;;
	QdObject_var        qdInv;
	const char*               attrPtr     =  NULL;
	char*               stringBuf   =  NULL;

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}

	// do arithmetic...
	try
	{
		qdResult = qdOperand1 / qdOperand2;
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		QcalcAppendTextDisplay( "This Operation Divide is not yet supported between these input types");
		rqstrErrorMsg = QString( "This Operation Divide is not yet supported between these input types");
		return ( qdResult );
	}

	if (qdResult.is_nil())
	{
	    QcalcAppendTextDisplay( "This Operation Divide is not yet supported between these input types" );
		rqstrErrorMsg = QString( "This Operation Divide is not yet supported between these input types");
		return ( qdResult );
	}

	// Add timeline: Either timeline is safe if it exists
	QdTimeSeq_var qdtt = get_timetags(qdOperand1);
	if ( qdtt.is_nil() )
	{
		qdtt = get_timetags( qdOperand2 );
	}

	if ( !qdtt.is_nil() )
	{
		set_timetags( qdResult, qdtt);
	}

	// Set QSAS essential attriutes
	// find inverses first and set its attributes, then do for result

	// SI_conversion
	attrPtr = QarSIConv_inverse( qdOperand2 );
	qdInv   = qdOperand1->create(); // this is empty as we only need metadata

	if( attrPtr != NULL )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr, qdInv );
		delete[] attrPtr;
	}

	// UNITS
	attrPtr = QuGetAttrText( "UNITS", qdOperand2 );
	if( strcmp( attrPtr, "" ) != 0 )
	{
		stringBuf = ( char * ) malloc( strlen( attrPtr ) + 6 );
		strcpy( stringBuf, "("     );
		strcat( stringBuf, attrPtr );
		strcat( stringBuf, ")^-1"  );
		QuSetTxtAttr( "UNITS", stringBuf, qdInv );
		free ( stringBuf );
	}

	// use inverse for attributes for returned object
	// SI_conversion
	if ( ( attrPtr = QarSIConv_product( qdOperand1, qdInv ) ) != NULL )
	{
		QuSetTxtAttr("SI_conversion", attrPtr, qdResult);
		delete[] attrPtr;
	}

	// Frame (must come from first object for divide)
	attrPtr = QuGetFrameAttr( qdOperand1 );
	QuSetFrame( qdResult, attrPtr );

	// UNITS
	if ( ( attrPtr = QarUNITS_product( qdOperand1, qdInv ) ) != NULL )
	{
		QuSetTxtAttr( "UNITS", attrPtr, qdResult );
		delete[] attrPtr;
	}

	// DEPEND_i
    if (qdOperand1->xref_exists(DEPEND_1)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_1);
        qdResult->set_xref( DEPEND_1, d1_xref  );
    }
   
    if (qdOperand1->xref_exists(DEPEND_2)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_2);
        qdResult->set_xref( DEPEND_2, d1_xref  );
    }

    if (qdOperand1->xref_exists(DEPEND_3)) 
    {
        QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_3);
        qdResult->set_xref( DEPEND_3, d1_xref  );
    }
   
	return ( qdResult );

}





