#include "qOperators.h"
    
void setProductDepends(QdObject_var resultObj, QdObject_var obj1, QdObject_var obj2);
         
QdObject_var QdMult( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{

    QdObject_var  qdResult;

    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
	// The qdResult object is NULL at this point
        return ( qdResult );
    }

 	// Test validity of inputs...
	int nQarStatus =  QarTestConformal( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Sequence size or join error: Notify csc_support" );
		return qdResult;
	}
	
	// Test Frames for vectors
	
	const char*  attrPtr;
	char*  stringBuf;
     const char * flag1 = QuGetOrder(qdOperand1);
	const char * flag2 = QuGetOrder(qdOperand2);
	bool isV2 = false;
	bool isR1 = false;
	bool isR2 = false;

	if(strcmp(QuToUpper(flag2), "VECTOR") == 0) isV2 = true;
	if(strcmp(QuToUpper(flag1), "ROT") == 0) isR1 = true;
	if(strcmp(QuToUpper(flag2), "ROT") == 0) isR2 = true;
	
	if(isR1 && isV2){
	   const char * fr1 = QuGetFrameAttr(qdOperand1);
	   const char * fr2 = QuGetFrameAttr(qdOperand2);
	   if(strcmp(fr1, fr2) != 0){
		QcalcAppendTextDisplay( "Frames for Rotation Matrix and Vector differ" );
		return qdResult;
	   }
	}
	if(isR1 && isR2){
	   const char * fr1 = QuGetFrameAttr(qdOperand1);
	   const char * fr2 = QuGetAttrText("ToFrame", qdOperand2);
	   if(strcmp(fr1, fr2) != 0){
		QcalcAppendTextDisplay( "Frames for Rotation Matrices differ..." );
		QcalcAppendTextDisplay( "R1.Frame != R2.ToFrame" );
		return qdResult;
	   }
	   
	}
	
	// do arithmetic...
	try
	{
		qdResult = qdOperand1 * qdOperand2;
	}
	catch( Exception &e )
	{
		QcalcAppendTextDisplay ( e.msg().c_str() );
		QcalcAppendTextDisplay( "The operation Multiplication is not yet supported between these input types");
		rqstrErrorMsg = QString( "The operation Multiplication is not yet supported between these input types");
		return ( qdResult );
	}

	if ( qdResult.is_nil() )
	{
		QcalcAppendTextDisplay( "Arithmetic>> This Operation is not yet supported between these input types" );
		rqstrErrorMsg = QString( "Arithmetic>> This Operation is not yet supported between these input types" );
		return ( qdResult );
	}

	// Add timeline: Either timeline is safe if it exists
	QdTimeSeq_var qdtt = get_timetags( qdOperand1 );
	if ( qdtt.is_nil() )
	{
		qdtt = get_timetags( qdOperand2 );
	}

	if ( !qdtt.is_nil() )
	{
		set_timetags( qdResult, qdtt );
	}

	// Set QSAS essential attriutes
	// SI_conversion
	if ( ( attrPtr = QarSIConv_product( qdOperand1, qdOperand2 ) ) != NULL )
	{
		QuSetTxtAttr( "SI_conversion", attrPtr, qdResult );
	}

	// Frame
	attrPtr = QarFrame_product( qdOperand1, qdOperand2 );
	QuSetFrame(qdResult, attrPtr );
	
	if(isR1 && isR2){
	  // new ToFrame from R1 
	  // Note Frame is from ToFrame of R2 and is handled by QarFrame_producet
	   const char * toFr = QuGetAttrText("ToFrame", qdOperand1);
	   QuSetTxtAttr( "ToFrame", toFr, qdResult );
	}
	

	// UNITS
	if ( ( attrPtr = QarUNITS_product( qdOperand1, qdOperand2 ) ) != NULL)
	{
		QuSetTxtAttr("UNITS", attrPtr, qdResult);
	}

	// DEPEND_i
	
	// special case of one operand is a scalar
	QdRScalarSeq_var sObj2 = QdRScalarSeq_var::narrow(qdOperand2);
	QdRScalarSeq_var sObj1 = QdRScalarSeq_var::narrow(qdOperand1);
	QdRScalar_var Obj2 = QdRScalar_var::narrow(qdOperand2);
	QdRScalar_var Obj1 = QdRScalar_var::narrow(qdOperand1);
	if ( !sObj2.is_nil() || !Obj2.is_nil() )
	{
		// 2nd obj is scalar use depends from first
   	 	if (qdOperand1->xref_exists(DEPEND_1)) 
    		{
        		QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_1);
        		qdResult->set_xref( DEPEND_1, d1_xref  );
    		}
   
    		if (qdOperand1->xref_exists(DEPEND_2)) 
    		{
        		QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_2);
        		qdResult->set_xref( DEPEND_2, d1_xref  );
    		}

    		if (qdOperand1->xref_exists(DEPEND_3)) 
    		{
        		QdObject_var d1_xref = qdOperand1->get_xref(DEPEND_3);
        		qdResult->set_xref( DEPEND_3, d1_xref  );
    		}
	}
	else if ( !sObj1.is_nil() ||  !Obj1.is_nil() )
	{
		// 1st obj is scalar use depends from second
   	 	if (qdOperand2->xref_exists(DEPEND_1)) 
    		{
        		QdObject_var d1_xref = qdOperand2->get_xref(DEPEND_1);
        		qdResult->set_xref( DEPEND_1, d1_xref  );
    		}
   
    		if (qdOperand2->xref_exists(DEPEND_2)) 
    		{
        		QdObject_var d1_xref = qdOperand2->get_xref(DEPEND_2);
        		qdResult->set_xref( DEPEND_2, d1_xref  );
    		}

    		if (qdOperand2->xref_exists(DEPEND_3)) 
    		{
        		QdObject_var d1_xref = qdOperand2->get_xref(DEPEND_3);
        		qdResult->set_xref( DEPEND_3, d1_xref  );
    		}
	}
	else{
 	   // handle matrix
	   QdRMatrixSeq_var Mobj2s = QdRMatrixSeq_var::narrow(qdOperand2);
	   QdRMatrixSeq_var Mobj1s = QdRMatrixSeq_var::narrow(qdOperand1);
	   QdRMatrix_var    Mobj2 = QdRMatrix_var::narrow(qdOperand2);
	   QdRMatrix_var    Mobj1 = QdRMatrix_var::narrow(qdOperand1);
	   if( (!Mobj2s.is_nil() || !Mobj2.is_nil()) && (!Mobj1s.is_nil() || !Mobj1.is_nil())){
	     // both objects are arrays or array sequences
		 setProductDepends(qdResult, qdOperand1, qdOperand2);
	   }
	}
	
    return ( qdResult );
}

// *************************************************************************

void setProductDepends(QdObject_var resultObj, QdObject_var obj1, QdObject_var obj2){
   int dims1 = QuArrayDims(obj1);
   int dims2 = QuArrayDims(obj2);

   int thisDim=1;
   while( thisDim < dims1 ){ // this ignores last dim = first dim of obj2
     QString depName = QString("DEPEND_%1").arg(thisDim);
	 if (obj1->xref_exists(depName.latin1())) 
     {
        	QdObject_var d_xref = obj1->get_xref(depName.latin1());
#if QT_VERSION < 0x040000
                resultObj->set_xref( depName, d_xref  );
#else
                resultObj->set_xref( depName.toStdString(), d_xref  );
#endif
	 }
     thisDim++;
   }
   
   int i=2;
   while( i <= dims2 ){ // this ignores first dim of obj2
     QString depNamei = QString("DEPEND_%1").arg(i);
     QString depName = QString("DEPEND_%1").arg(thisDim);
	 if (obj2->xref_exists(depNamei.latin1())) 
     {
        	QdObject_var d_xref = obj2->get_xref(depNamei.latin1());
#if QT_VERSION < 0x040000
                resultObj->set_xref( depName, d_xref 
                                   ); // shuffles dependencies along as needed
#else
                resultObj->set_xref( depName.toStdString(), d_xref);
#endif
	 }
     thisDim++;
	 i++;
   }
}


