


#include "qOperators.h"


#define QPAL_DEBUG  0   // print out debug info as label is constructed

char* QpAxisLabelMakeStem ( QdObject_var data_obj )
{
	char*   stem_ptr            = NULL;
	static char stem_buf[ 300 ] = "\0";

	//  For the stem, use LABLAXIS attribute if it exists, else try for
	//  LABL_PTR (may exist for vector data objects) and perform string
	//  comparision on its entries to synthesise a stem from common parts.
	//  If neither LABLAXIS nor LABL_PTR_1 exists, try for FIELDNAM attribute,
	//  and if that is absent finally fall back to data object name itself.


	//  first use LABLAXIS attribute if it exists

	stem_ptr = QuGetAttrText( "LABLAXIS", data_obj );

	if ( strcmp( stem_ptr, "" ) != 0 )
	{
		if ( QPAL_DEBUG )
			printf ("QpAxisLabelMakeStem: using LABLAXIS = /%s/\n", stem_ptr);

		strcpy (stem_buf, stem_ptr);

	}
	else
	{
		stem_ptr = QuGetAttrText("FIELDNAM", data_obj);

		// fall back to using FIELDNAM attribute if it exists

		if ( QPAL_DEBUG )
			printf("QpAxisLabelMakeStem: using FIELDNAM = /%s/\n", stem_ptr);

		strcpy ( stem_buf, stem_ptr );

	}

	return stem_buf;
}


QdObject_var QdComponent( QString& rqstrErrorMsg, QdObject_var qdOperand1, QdObject_var qdOperand2 )
{
	QdObject_var   qdResult;
	QdRScalar_var  componentdo;
	int            icomponent;
	char*          string_ptr;
	char           fieldnam[ 150 ];                  // attributes
	char           siconversion[ 150 ];
	char           units[ 150 ];
	char           lablaxis[ 150 ];
	char           stem[ 150 ];
	double         hs = 0. , hsp = 0.;
	int            ndata_in = 0;                                  /* number of data points  */
													               /* array new'ed below  */

	if ( qdOperand1.is_nil() )
    {
        QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand1 is NULL" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand1 is NULL" );
	    return ( qdResult );
    }

	if ( qdOperand2.is_nil() )
    {
     	QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand2 is NULL" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand2 is NULL" );
        return ( qdResult );
    }


	if ( !QuIsThreeVector( qdOperand1 ) )
	{
		QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand1 not a Vector Time Series!" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand1 not a Vector Time Series!" );
		return ( qdResult );
	}

	ndata_in    = qdOperand1->sequence_size();
	componentdo = QdRScalar_var::narrow( qdOperand2 );
	if ( componentdo.is_nil() )
	{
		QcalcAppendTextDisplay ( "VectorTSDecomp>> ERROR - Operand2 not an integer flag!" );
		rqstrErrorMsg = QString( "VectorTSDecomp>> ERROR - Operand2 not an integer flag!" );
		return ( qdResult );
	}

	icomponent = ( int )( *componentdo );

	strcpy( stem, QpAxisLabelMakeStem( qdOperand1 ) );

	string_ptr = QuGetAttrText( "FIELDNAM", qdOperand1 );
	strcpy( fieldnam, string_ptr );


	switch( icomponent )
	{
		case( 0 ):            // x component
			string_ptr = QuGetAttrText( "UNITS", qdOperand1 );
			strcpy( units, string_ptr );

			string_ptr = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( strcmp( string_ptr, "" ) != 0 )
			{
				strcpy( siconversion, string_ptr );
			} 
			else
			{                                           // set SI_conversion
				strcpy( siconversion, "1>(unknown)" );     //  empty if fail
			}

			strcpy( lablaxis, stem );
			strcat( lablaxis, "_x" );
			strcat( fieldnam, "_x" );
			qdResult = QuGetVectorComponent( qdOperand1, "x" );
			break;

		case( 1 ):            // y component
			string_ptr = QuGetAttrText( "UNITS", qdOperand1 );
			strcpy( units,string_ptr );

			string_ptr = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( strcmp( string_ptr, "" ) != 0 )
			{
				strcpy( siconversion, string_ptr );
			}
			else
			{                                              // set SI_conversion
				strcpy( siconversion, "1>(unknown)" );     //  empty if fail
			}      

			strcpy( lablaxis, stem );
			strcat( lablaxis, "_y" );
			strcat( fieldnam, "_y" );
			qdResult = QuGetVectorComponent( qdOperand1, "y" );
			break;

		case( 2 ):            // z component
			string_ptr = QuGetAttrText( "UNITS", qdOperand1 );
			strcpy( units,string_ptr );

			string_ptr = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( strcmp( string_ptr, "" ) != 0 )
			{
				strcpy( siconversion, string_ptr );
			} 
			else
			{                                              // set SI_conversion
				strcpy( siconversion, "1>(unknown)" );     //  empty if fail
			}

			strcpy( lablaxis, stem );
			strcat( lablaxis, "_z" );
			strcat( fieldnam, "_z" );
			qdResult = QuGetVectorComponent( qdOperand1, "z" );
			break;

		case( 3 ):            // r component
			string_ptr = QuGetAttrText( "UNITS", qdOperand1 );
			strcpy( units,string_ptr );

			string_ptr = QuGetAttrText( "SI_conversion", qdOperand1 );
			if ( strcmp( string_ptr, "" ) != 0 )
			{
				strcpy(siconversion, string_ptr);
			}
			else
			{                                           // set SI_conversion
				strcpy( siconversion,"1>(unknown)" );     //  empty if fail
			}      

			strcpy( lablaxis, "|" );
			strcat( lablaxis, stem );
			strcat( lablaxis, "|" );
			strcat( fieldnam, "_magnitude" );
			qdResult = QuGetVectorComponent( qdOperand1, "r(mag)" );
			break;

		case( 4 ):            // Lat(deg) component
			strcpy( units, "deg" );
			strcpy( siconversion,"1>degree" );
			strcpy( lablaxis, "Lat" );
			strcat( lablaxis, "_" );
			strcat( lablaxis, stem );
			strcat( fieldnam, "_Lat(deg)" );
			qdResult = QuGetVectorComponent( qdOperand1, "Lat(deg)" );
			break;

		case( 5 ):            // theta(deg) component
			strcpy( units,"deg" );
			strcpy( siconversion,"1>degree" );
			strcpy( lablaxis,"theta" );
			strcat( lablaxis,"_" );
			strcat( lablaxis,stem );
			strcat( fieldnam,"_theta(deg)" );
			qdResult = QuGetVectorComponent( qdOperand1, "theta(deg)" );
			break;

		case( 6 ):            // phi(deg) component
			strcpy( units,         "deg" );
			strcpy( siconversion,  "1>degree" );
			strcpy( lablaxis,      "phi" );
			strcat( lablaxis,      "_" );
			strcat( lablaxis,      stem );
			strcat( fieldnam,      "_phi(deg)" );
			qdResult = QuGetVectorComponent( qdOperand1, "phi(deg)" );
			break;

		case( 7 ):            // Lat(rad) component
			strcpy( units,        "rad" );
			strcpy( siconversion, "1>rad");
			strcpy( lablaxis,     "Lat");
			strcat( lablaxis,     "_");
			strcat( lablaxis,     stem);
			strcat( fieldnam,     "_Lat(rad)");
			qdResult = QuGetVectorComponent( qdOperand1, "Lat(rad)" );
			break;

		case( 8 ):            // theta(rad) component
			strcpy( units,        "rad" );
			strcpy( siconversion, "1>rad" );
			strcpy( lablaxis,     "theta" );
			strcat( lablaxis,     "_" );
			strcat( lablaxis,     stem );
			strcat( fieldnam,     "_theta(rad)" );
			qdResult = QuGetVectorComponent( qdOperand1, "theta(rad)" );
			break;

		case( 9 ):            // phi(rad) component
			strcpy( units,        "rad" );
			strcpy( siconversion, "1>rad" );
			strcpy( lablaxis,     "phi" );
			strcat( lablaxis,     "_" );
			strcat( lablaxis,     stem );
			strcat( fieldnam,     "_phi(deg)" );
			qdResult = QuGetVectorComponent( qdOperand1, "phi(rad)" );
			break;

		default:                     // Shouldn't get here!
			QcalcAppendTextDisplay( "VectorTSDecompAVG>> Unknown component requested; Failure" );
			rqstrErrorMsg = QString( "VectorTSDecompAVG>> Unknown component requested; Failure" );
			return ( qdResult );
			break;
	}                                 // end switch(icomp)


	// Create the output Time Series and fill it

	QdTimeSeq_var tt     = get_timetags( qdOperand1 );
	QdTimeSeq_var tt_out = ( QdTimeSeq_var ) Factory::clone( tt );

	set_timetags( qdResult, tt_out );

	// load all the attributes, and the half spacing/sample period:

	QuSetFrame(qdResult, "scalar", "",  "" );
	QuSetTxtAttr( "SI_conversion", siconversion, qdResult );
	QuSetTxtAttr( "UNITS",         units,        qdResult );
	QuSetTxtAttr( "LABLAXIS",      lablaxis,     qdResult) ;


	QcalcAppendTextDisplay ( "VectorTSDecomp>> data object created" );
	rqstrErrorMsg = QString( "VectorTSDecomp>> data object created" );

	return ( qdResult );

}


