
#include "qcalc.h"

using namespace QSAS;


// Class QcalcTab
/*!
    \class QcalcTab
    \brief  This class defines the widget that is added to the calculator tab view when a new operation is created (added to
	the current list view).

	This class specialises the QT QWidget class\n
*/


QcalcTab::QcalcTab()
: QWidget()
{
  this->opWasJoin = false; // initialise flag
}

QcalcTab::~QcalcTab()
{

}


void QcalcTab::createCalcTabWidgets(QcalcListVirtualItem * VirtItem)
{
   if  ( QuSlotDebug() ){
		printf("QcalcTab::createCalcTabWidgets %ld of %ld \n", this, VirtItem);	
		QDateTime  qdatetime ; 
		qdatetime = QDateTime::currentDateTime();
		printf(" ---- START tabs -----  %s\n", qdatetime.toString( "dd.MM.yyyy-hh:mm:ss.zzz" ).latin1());

   }
   
   if ( calcListview == NULL )  return;
   
   this->virtualItem = VirtItem;
   this->WLnameUnset = true;
   this->nSlots = 0;
   
  // Fill in the CALC tab
  QVBoxLayout *pageTab2layout = new QVBoxLayout( this );
  pageTab2layout->setSpacing( 2 );

  // Selected object details
#if QT_VERSION < 0x040000
  details_groupbox = new QGroupBox( 1, Qt::Vertical, this, "details_groupbox");

  details_groupbox->setTitle( "New Operation" );

  details_widget = new QWidget( details_groupbox, "details_widget" );

  details_layout = new QGridLayout( details_widget, 2, 2);
  details_layout->setSpacing( 2 );
  details_layout->setMargin(0);


  // Aliase of object being selected
  objAlias_label = new QLabel( "Object Alias ", details_widget, "objAlias_label");
  details_layout->addWidget( objAlias_label, 0, 0, Qt::AlignRight );

  objAliase_field = new QguWLinput( details_widget,  "objAliase_field" );
  objAliase_field->setMinimumWidth( 270 );
  objAliase_field->setMaxLength( 200 );
  objAliase_field->acceptText = true;  
  objAliase_field->setText("");  
  objAliase_field->setEnabled( true);
  connect(objAliase_field, SIGNAL(validated_TEXT(const QString&)), this, SLOT(reNameAliasTab(const QString&)));  
  connect(objAliase_field->lineBox, SIGNAL(lostFocus()), this, SLOT(confirmAliasSlot()) );
    
  objAliase_field->resize( objAliase_field->sizeHint() );
  details_layout->addWidget( objAliase_field, 0, 1, Qt::AlignLeft );
  
 // Check BOX
  saveWL_check = new QCheckBox( "Save to WL as", details_widget, "saveWL_check");
  saveWL_check->setEnabled( TRUE );
  saveWL_check->resize( saveWL_check->sizeHint() );
  details_layout->addWidget( saveWL_check, 1, 0, Qt::AlignRight  );
  connect( saveWL_check, SIGNAL( clicked() ), this, SLOT( toggleWLName() ) );

 // Name of the working list Edit box
  _nameWL_field = new QguWLinput( details_widget, "nameWL_field" );
  _nameWL_field->setMinimumWidth( 270 );
  _nameWL_field->setMaxLength( 200 );
  _nameWL_field->acceptText = TRUE;
  _nameWL_field->resize( _nameWL_field->sizeHint() );
  QToolTip::add(  _nameWL_field, "Name of the object as saved within the working list" );
  details_layout->addWidget( _nameWL_field, 1, 1, Qt::AlignLeft  );
  connect( _nameWL_field, SIGNAL( validated_TEXT(const QString&)), this, SLOT(setNameForSave(const QString&)));
  
  saveWL_check->setChecked( FALSE );
  _nameWL_field->setEnabled( FALSE );
 
  pageTab2layout->addWidget( details_groupbox );
#else
  details_groupbox = new QGroupBox(this);

  details_groupbox->setTitle( "New Operation" );

//   details_widget = new QWidget( details_groupbox, "details_widget" );

  details_layout = new QGridLayout;
  details_layout->setSpacing( 2 );
  details_layout->setMargin(0);


  // Aliase of object being selected
  objAlias_label = new QLabel( "Object Alias ");
  details_layout->addWidget( objAlias_label, 0, 0, Qt::AlignRight );

  objAliase_field = new QguWLinput( details_groupbox,  "objAliase_field" );
  objAliase_field->setMinimumWidth( 270 );
  objAliase_field->setMaxLength( 200 );
  objAliase_field->acceptText = true;  
  objAliase_field->setText("");  
  objAliase_field->setEnabled( true);
  connect(objAliase_field, SIGNAL(validated_TEXT(const QString&)), this, SLOT(reNameAliasTab(const QString&)));  
  connect(objAliase_field->lineBox, SIGNAL(lostFocus()), this, SLOT(confirmAliasSlot()) );
    
  objAliase_field->resize( objAliase_field->sizeHint() );
  details_layout->addWidget( objAliase_field, 0, 1, Qt::AlignLeft );
  
 // Check BOX
  saveWL_check = new QCheckBox( "Save to WL as");
  saveWL_check->setEnabled( TRUE );
  saveWL_check->resize( saveWL_check->sizeHint() );
  details_layout->addWidget( saveWL_check, 1, 0, Qt::AlignRight  );
  connect( saveWL_check, SIGNAL( clicked() ), this, SLOT( toggleWLName() ) );

 // Name of the working list Edit box
  _nameWL_field = new QguWLinput( details_groupbox, "nameWL_field" );
  _nameWL_field->setMinimumWidth( 270 );
  _nameWL_field->setMaxLength( 200 );
  _nameWL_field->acceptText = TRUE;
  _nameWL_field->resize( _nameWL_field->sizeHint() );
  QToolTip::add(  _nameWL_field, "Name of the object as saved within the working list" );
  details_layout->addWidget( _nameWL_field, 1, 1, Qt::AlignLeft  );
  connect( _nameWL_field, SIGNAL( validated_TEXT(const QString&)), this, SLOT(setNameForSave(const QString&)));
  
  saveWL_check->setChecked( FALSE );
  _nameWL_field->setEnabled( FALSE );
 
  details_groupbox->setLayout(details_layout);
  pageTab2layout->addWidget( details_groupbox );
#endif


  // Operation group box
#if QT_VERSION < 0x040000
  operation_groupbox = new QGroupBox( 1, Qt::Horizontal, this, "operation_groupbox");
  operation_groupbox->setTitle( " Operation " );

  // Operator
  operation_widget = new QWidget( operation_groupbox, "operation_widget" );
  
  operation_layout = new QGridLayout( operation_widget, 1, 2);
#else
  operation_groupbox = new QGroupBox(this);
  operation_groupbox->setTitle( " Operation " );
  
  operation_layout = new QGridLayout;
#endif
  operation_layout->setSpacing( 2 );
  operation_layout->setColStretch( 1, 10 );
  operation_layout->setMargin(2);

#if QT_VERSION < 0x040000
  operator_label = new QLabel( "Operator   ", operation_widget, "operator_label");
#else
  operator_label = new QLabel( "Operator   ");
#endif
  operation_layout->addWidget( operator_label, 0, 0, Qt::AlignRight );


  // Pop menu arithmetic
  arithmetic = new Q_POPUP_MENU( this );
  arithmetic->insertItem( strOpAdd,    1 );
  arithmetic->insertItem( strOpSub,    2 );
  arithmetic->insertItem( strOpMult,   3 );
  arithmetic->insertItem( strOpDiv,    4 );
  arithmetic->insertItem( strOpMod,    5 );
  arithmetic->insertItem( strOpPower,  6 );
  arithmetic->insertItem( strOpChSign, 7 );
  connect( arithmetic, SIGNAL( activated( int ) ),   this,   SLOT( SetOperatorArithemtic( int ) ) );
  connect( arithmetic, SIGNAL( highlighted( int ) ), this,   SLOT( DisplayOpArithemticDesc( int ) ) );

  // Pop menu Trig
  trig = new Q_POPUP_MENU( this );
  trig->insertItem( strOpCos,     1 );
  trig->insertItem( strOpSin,     2 );
  trig->insertItem( strOpTan,     3 );
  trig->insertItem( strOpCot,     4 );
  trig->insertItem( strOpACos,    5 );
  trig->insertItem( strOpASin,    6 );
  trig->insertItem( strOpATan,    7 );
  trig->insertItem( strOpATan2,   8 );
  trig->insertItem( strOpACot,    9 );
  trig->insertItem( strOpCosh,    10 );
  trig->insertItem( strOpSinh,    11 );
  trig->insertItem( strOptanh,    12 );
  trig->insertItem( strOpCoth,    13 );
  trig->insertItem( strOpACosh,   14 );
  trig->insertItem( strOpASinh,   15 );
  trig->insertItem( strOpAtanh,   16 );
  trig->insertItem( strOpACoth,   17 );
  connect( trig, SIGNAL( activated( int ) ),   this,   SLOT( SetOperatorTrig( int ) ) );
  connect( trig, SIGNAL( highlighted( int ) ), this,   SLOT( DisplayOpTrigDesc( int ) ) );


  // Pop menu Vector Ops
  vectorOps = new Q_POPUP_MENU( this );
  vectorOps->insertItem( strOpVecProd,           1 );
  vectorOps->insertItem( strOpDotProd,           2 );

  connect( vectorOps, SIGNAL( activated( int ) ),   this,   SLOT( SetOperatorVectorOps( int ) ) );
  connect( vectorOps, SIGNAL( highlighted( int ) ), this,   SLOT( DisplayOpVectorOpsDesc( int ) ) );

  // Pop menu Filter
  filter = new Q_POPUP_MENU( this );
  filter->insertItem( strOpRemainder,      1 );
  filter->insertItem( strOPAbsValue,       2 );
  filter->insertItem( strOpLog,            3 );
  filter->insertItem( strOpLog10,          4 );
  filter->insertItem( strOpInvers,         5 );
  filter->insertItem( strOpSqrt,           6 );
  filter->insertItem( strOpExp,            7 );
  connect( filter, SIGNAL( activated( int ) ),      this,   SLOT( SetOperatorFilter( int ) ) );
  connect( filter, SIGNAL( highlighted( int ) ),   this,   SLOT( DisplayOpFilterDesc( int ) ) );


    // Pop menu Filter
  modify = new Q_POPUP_MENU( this );

  modify->insertItem( strOpUnits,          1 );
  modify->insertItem( strOpFills,          2 );
  connect( modify, SIGNAL( activated( int ) ),      this,   SLOT( SetOperatorModify( int ) ) );
  connect( modify, SIGNAL( highlighted( int ) ),   this,   SLOT( DisplayOpModifyDesc( int ) ) );

#if QT_VERSION < 0x040000
  tb4 = new QPushButton( "            ", operation_widget, "popup" );
#else
  tb4 = new QPushButton( "            ");
#endif
  menu = new Q_POPUP_MENU( tb4 );
  menu->insertItem(strOpJoin,   JOIN_MENU_ITEM_ID  );
  connect( menu, SIGNAL( activated( int ) ),    this,   SLOT( SetOperatorJoin( int ) ) );
  menu->insertItem("Arith f (A, B)...", arithmetic, ARITHMETIC_MENU_ITEM_ID );
  menu->insertItem("Arith f (A)...",  filter,     FILTER_MENU_ITEM_ID );
  menu->insertItem("Trig...",       trig,       TRIG_MENU_ITEM_ID       );
  menu->insertItem("Vector Ops...", vectorOps,  VECTOR_OPS_MENU_ITEM_ID );

  menu->insertItem("Modify...",     modify,     MODIFY_MENU_ITEM_ID     );
  menu->setItemEnabled( JOIN_MENU_ITEM_ID,        TRUE  );
  menu->setItemEnabled( ARITHMETIC_MENU_ITEM_ID,  TRUE );
  menu->setItemEnabled( TRIG_MENU_ITEM_ID,        TRUE );
  menu->setItemEnabled( VECTOR_OPS_MENU_ITEM_ID,  TRUE );
  menu->setItemEnabled( MODIFY_MENU_ITEM_ID,  TRUE );
  menu->setItemEnabled( FILTER_MENU_ITEM_ID,      TRUE );
  
  tb4->setPopup(menu);
  tb4->resize( tb4->sizeHint() );
  operation_layout->addWidget( tb4, 0, 1, Qt::AlignLeft );

  
    // Operands Stuff
#if QT_VERSION < 0x040000
  operand_widget = new QWidget( operation_groupbox, "operation_widget" );
  
  operand_layout = new QGridLayout( operand_widget, MAX_N_OPERANDS, 1);
  operand_layout->setSpacing( 2 );
  operand_layout->setMargin(0);

// operation_layout->addWidget(operand_widget);
  
  // create all slots, but hide them
  for(int nOps=0; nOps<MAX_N_OPERANDS; nOps++)
  {
	QString title = getOpName(nOps);
	QcalcInput *OpInput_i = new QcalcInput( operand_widget, title, title, this ); // its name is also title to be unique
	OpInput_i->slot->setText("");
	operandStack.push_back(OpInput_i);
	operand_layout->addWidget( OpInput_i, nOps, 0);  
	// connect the underlying QlineEdit signal as the QguWLinput signal is only emitted if box has focus. 
	// Needed for pulldown updates for components etc

	connect(calcListview, SIGNAL(listItemDeleted()), OpInput_i->slot, SLOT(reValidate()) );
	connect(OpInput_i->slot, SIGNAL(validated_WL_OBJ(const QString &)), calcListview, SLOT(validateWLitems()) );
	connect(OpInput_i->slot->lineBox, SIGNAL(hadDropEvent()), this, SLOT(setThisSelected()) );
	connect(OpInput_i->slot, SIGNAL(componentSelected(const char *, const char * )), calcListview, SLOT(propagateSelection(const char *, const char * )) );
	connect(OpInput_i->slot, SIGNAL(dropDetected(const char * )), calcListview, SLOT(propagateDrop(const char * )) );
   }
#else
   operand_layout = new QGridLayout;
   operand_layout->setSpacing( 2 );
   operand_layout->setMargin(0);

// operation_layout->addWidget(operand_widget);
  
  // create all slots, but hide them
   for(int nOps=0; nOps<MAX_N_OPERANDS; nOps++)
   {
	   QString title = getOpName(nOps);
	   QcalcInput *OpInput_i = new QcalcInput( operation_groupbox, title, title, this ); // its name is also title to be unique
	   OpInput_i->slot->setText("");
	   operandStack.push_back(OpInput_i);
	   operand_layout->addWidget( OpInput_i, nOps, 0);  
	// connect the underlying QlineEdit signal as the QguWLinput signal is only emitted if box has focus. 
	// Needed for pulldown updates for components etc

	   connect(calcListview, SIGNAL(listItemDeleted()), OpInput_i->slot, SLOT(reValidate()) );
	   connect(OpInput_i->slot, SIGNAL(validated_WL_OBJ(const QString &)), calcListview, SLOT(validateWLitems()) );
	   connect(OpInput_i->slot->lineBox, SIGNAL(hadDropEvent()), this, SLOT(setThisSelected()) );
	   connect(OpInput_i->slot, SIGNAL(componentSelected(const char *, const char * )), calcListview, SLOT(propagateSelection(const char *, const char * )) );
	   connect(OpInput_i->slot, SIGNAL(dropDetected(const char * )), calcListview, SLOT(propagateDrop(const char * )) );
   }
   operation_layout->addLayout(operand_layout, 1, 0, 1, 2);
#endif
   
   
     // Extras
#if QT_VERSION < 0x040000
   extras_widget = new QWidget( operation_groupbox, "operation_widget" );
  
   extras_layout = new QGridLayout( extras_widget, 1, 1);
   extras_layout->setSpacing( 2 );
   extras_layout->setColStretch( 1, 10 );
   
  AngleUnits = new QcalcInput(extras_widget, "AngleUnits", "Angle in  ", this);
  extras_layout->addWidget( AngleUnits, 0, 0 );
  AngleUnits->setAnglePulldown();

  QToolTip::add(  AngleUnits, "Angle Units for Trig Function if not in SI_conversion" );

 // AngleUnits->hide();
#else
  extras_layout = new QGridLayout;
  extras_layout->setSpacing( 2 );
  extras_layout->setColStretch( 1, 10 );
   
  AngleUnits = new QcalcInput(operation_groupbox, "AngleUnits", "Angle in  ", this);
  extras_layout->addWidget( AngleUnits, 0, 0 );
  AngleUnits->setAnglePulldown();

  QToolTip::add(  AngleUnits, "Angle Units for Trig Function if not in SI_conversion" );
  operation_layout->addLayout(extras_layout, 2, 0, 1, 2);
 // AngleUnits->hide();
  
  operation_groupbox->setLayout(operation_layout);
#endif

  pageTab2layout->addWidget( operation_groupbox );

  
    // Join Setting group box
#if QT_VERSION < 0x040000
  join_groupbox = new QGroupBox( 1, Qt::Vertical, this, "join_groupbox" );
  join_groupbox->setTitle("Join Settings");

  // Widget Join Setting
  qwJoinSetting = new QWidget( join_groupbox, "page_join_widget" );

  // ( 1 ) Create the GRID
  join_layout = new QGridLayout( qwJoinSetting, 5, 2 );
  join_layout->setSpacing( 2 );
  join_layout->setColStretch( 1,10 );

  // ( 2 ) Algoritm label
  algorithm_label = new QLabel( "Join Algorithm ", qwJoinSetting, "algorithm_label" );
  join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
  
  // Combo box
  _algorithm = new QComboBox( false, qwJoinSetting, "algorithm" );
  _algorithm->clear();
  _algorithm->insertItem( LINEAR, -1 );
  _algorithm->insertItem( BOXCAR, -1 );
  join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
  connect( _algorithm, SIGNAL( activated( int ) ), this,  SLOT( ValidateAlgorithm( int ) ) );


  // Average width
  width_label = new QLabel( "Ave width (sec) ", qwJoinSetting, "width_label" );
  join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );

  _width_field = new QguWLinput( qwJoinSetting, "width_field" );
  _width_field->setMinimumWidth( 150 );
  _width_field->setMaxLength( 250 );
  _width_field->acceptNumber = TRUE;
  _width_field->setEnabled(false);
   connect( _width_field,  SIGNAL( textChanged(const QString &) ), this, SLOT( setWidthField() ) );
  QToolTip::add(  _width_field, "width in seconds of the boxcar" );
  join_layout->addWidget( _width_field, 1, 1 );

  // Blank labels
  blank_label = new QLabel( " ", qwJoinSetting, "blank_label");
  join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );

  blank2_label = new QLabel(" ", qwJoinSetting, "blank2_label");
  join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );

  // Gap hadling label
  gap_label = new QLabel("Gap Handling ", qwJoinSetting, "gap_label");
  join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );

   // Gap handling combo box
  _gap = new QComboBox( false, qwJoinSetting, "gap" );
  _gap->clear();
  _gap->insertItem( LINEAR_INTERP_GAP,     -1 );
  _gap->insertItem( ZERO_FILL_GAP,         -1 );
  _gap->insertItem( FILL_GAP,              -1 );
  _gap->insertItem( NEAREST_NEIGHBOUR_GAP, -1 );
  _gap->insertItem( REMOVE_GAP,            -1 );

  connect( _gap, SIGNAL( activated(int) ), this, SLOT( setGap(int) ) );
  join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );

  // Gap threshold label
  gapSize_label = new QLabel( "Gap threshold (sec) ", qwJoinSetting, "gapSize_label");
  join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight ); 

  // Gap field input box
  _gap_field = new QguWLinput( qwJoinSetting, "gap_field" );
  _gap_field->setMinimumWidth( 150 );
  _gap_field->setMaxLength( 100 );
  _gap_field->acceptNumber = TRUE;
  _gap_field->setEnabled( true);
   connect( _gap_field, SIGNAL(textChanged(const QString &)), this, SLOT(setGapField()) );
   QToolTip::add(  _gap_field, "minimum time interval in seconds to be treated as a gap if empty" );
   join_layout->addWidget( _gap_field, 4, 1 );

  pageTab2layout->addWidget( join_groupbox );
#else
  join_groupbox = new QGroupBox(this);
  join_groupbox->setTitle("Join Settings");

  // Widget Join Setting
//   qwJoinSetting = new QWidget( join_groupbox, "page_join_widget" );

  // ( 1 ) Create the GRID
  join_layout = new QGridLayout;
  join_layout->setSpacing( 2 );
  join_layout->setColStretch( 1,10 );

  // ( 2 ) Algoritm label
  algorithm_label = new QLabel( "Join Algorithm ");
  join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
  
  // Combo box
  _algorithm = new QComboBox( false);
  _algorithm->clear();
  _algorithm->insertItem( LINEAR, -1 );
  _algorithm->insertItem( BOXCAR, -1 );
  join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
  connect( _algorithm, SIGNAL( activated( int ) ), this,  SLOT( ValidateAlgorithm( int ) ) );


  // Average width
  width_label = new QLabel( "Ave width (sec) ");
  join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );

  _width_field = new QguWLinput( join_groupbox, "width_field" );
  _width_field->setMinimumWidth( 150 );
  _width_field->setMaxLength( 250 );
  _width_field->acceptNumber = TRUE;
  _width_field->setEnabled(false);
  connect( _width_field,  SIGNAL( textChanged(const QString &) ), this, SLOT( setWidthField() ) );
  QToolTip::add(  _width_field, "width in seconds of the boxcar" );
  join_layout->addWidget( _width_field, 1, 1 );

//   // Blank labels
//   blank_label = new QLabel( " ");
//   join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );
// 
//   blank2_label = new QLabel(" ");
//   join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );

  // Gap hadling label
  gap_label = new QLabel("Gap Handling ");
  join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );

   // Gap handling combo box
  _gap = new QComboBox( false);
  _gap->clear();
  _gap->insertItem( LINEAR_INTERP_GAP,     -1 );
  _gap->insertItem( ZERO_FILL_GAP,         -1 );
  _gap->insertItem( FILL_GAP,              -1 );
  _gap->insertItem( NEAREST_NEIGHBOUR_GAP, -1 );
  _gap->insertItem( REMOVE_GAP,            -1 );

  connect( _gap, SIGNAL( activated(int) ), this, SLOT( setGap(int) ) );
  join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );

  // Gap threshold label
  gapSize_label = new QLabel( "Gap threshold (sec) ");
  join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight ); 

  // Gap field input box
  _gap_field = new QguWLinput( join_groupbox, "gap_field" );
  _gap_field->setMinimumWidth( 150 );
  _gap_field->setMaxLength( 100 );
  _gap_field->acceptNumber = TRUE;
  _gap_field->setEnabled( true);
  connect( _gap_field, SIGNAL(textChanged(const QString &)), this, SLOT(setGapField()) );
  QToolTip::add(  _gap_field, "minimum time interval in seconds to be treated as a gap if empty" );
  join_layout->addWidget( _gap_field, 4, 1 );
  
  join_groupbox->setLayout(join_layout);
  pageTab2layout->addWidget( join_groupbox );
#endif


}

void QcalcTab::SetTabFeedback( ) 
{
 
	if  ( QuSlotDebug() )
		printf("QcalcTab::SetTabFeedback %ld of %ld \n", this, virtualItem);
		
	virtualItem->SetItemVirtualName(false);

	// set save to WL name here AFTER the name has been set in text(Obj_Name_)
	// don't update once the user has entered their own choice
	if(this->WLnameUnset || this->_nameWL_field->text() == QString("") )
	{
		this->WLnameUnset = true; // reset if field was blank
   	  	QString joined_name = QString(virtualItem->text(Obj_Name_).latin1());
		if(virtualItem->m_qstrOperator == strOpJoin)
		{
			// for WL item add _J and remove path
			int compPosn = joined_name.find( " [ " );
			if (compPosn != -1) {
			   joined_name.replace(compPosn+1, 1, "(");
			   compPosn = joined_name.find( "]" );
			   if (compPosn != -1) joined_name.replace(compPosn, 1, ")");
			}
			else{
				joined_name        += "_J";
			}
			// remove path
     		int posn            = joined_name.find( "}" );
     		joined_name.remove( 0, posn+1 );
		}
		
		// sanitise / from divide op
		int posn = joined_name.find(" } ");
		while (posn >=0){
			joined_name.replace(posn, 3, " | ");
			posn = joined_name.find(" } ");
		}
		
		this->_nameWL_field->setText(joined_name);
		virtualItem->m_NameForSave = QString(joined_name.latin1()); // used only to allow name to be held when toggle is off
	}
	 confirmAliasSlot(); // this will silently set alias in slot to item->text(Alias_)
	 
	 // set Angle slot on/off according to availability of SI_conversion
	 if(virtualItem->m_qstrOperator == strOpCos ||
 	  virtualItem->m_qstrOperator == strOpSin  ||
	  virtualItem->m_qstrOperator == strOpTan  ||
	  virtualItem->m_qstrOperator == strOpCot  ){
	  
		bool SIc_unset = true;
		QdObject_var dobj = this->operandStack[0]->slot->getSlotObject();
		char * SIc = QuGetAttrText( "SI_conversion", dobj ); // safe against nil dobj
		if(strstr(SIc, "rad") != 0 || strstr(SIc, "deg") != 0){
			// SI conversion should be used for units
			SIc_unset = false;
		}
		if( SIc_unset ) this->AngleUnits->show(); // not hyperbolic trig
		else this->AngleUnits->hide(); // hyperbolic trig or has deg/rad set in SI_conversion
	 }
}

void QcalcTab::SetTab( ) 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::SetTab %ld of %ld \n", this, virtualItem);
		
	SetTabFeedback();	
	
	updateFlags();

	bool isSelfLInked = false;
	QcalcListView * qView = (QcalcListView *)virtualItem->listView();
	if(qView && this->isVisible()){
		// test for self link
		for(int i=0; i<this->nSlots; i++){
		   if( this->objAliase_field->text() == this->operandStack[i]->slot->textObject() ){
			  QPalette palette = QPalette(qApp->palette());
			  QColorGroup active_g = QColorGroup(palette.active());
			  active_g.setColor(QColorGroup::Base,active_g.link());
			  active_g.setColor(QColorGroup::Text,active_g.dark());
			  palette.setActive(active_g);
			  this->operandStack[i]->slot->setPalette(palette);

			  qView->owner->messageHandler( "Self Link!", true );
			  isSelfLInked = true;
		   }
		}
		if(!isSelfLInked) {
			qView->owner->messageHandler(" ", false);
		}
	}

}


void QcalcTab::updateFlags() 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::updateFlags %ld  %ld\n", this, virtualItem);
	
	if(virtualItem->setArrayFlags()){ 

		// if flags changed, recursively update virtual items that depend on it
		QcalcListView * qView = (QcalcListView *)virtualItem->listView();
		if(qView){
			Q_LIST_VIEW_ITEM_ITERATOR it( qView );
			while ( it.current() ) {
				QcalcListVirtualItem *item = dynamic_cast <QcalcListVirtualItem *> (it.current());
				++it;
				if (item == virtualItem || item->m_eObjType == WORKING_LIST_OBJECT) continue; // skip this as already done
				for (int i=0; i<item->m_pcalcTab->nSlots; i++){
					bool updateThis = false;
					if( item->m_pcalcTab->operandStack[i]->slot->textObject() == virtualItem->text( Alias_) ){
				   		updateThis = true;
						item->m_pcalcTab->operandStack[i]->slot->reValidate();
					}
					if(updateThis) item->m_pcalcTab->updateFlags();
				}
			}
		}
	}
}

bool QcalcTab::AddValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Add/Sub" );
		return false;
    }

    // Get base SI for input data
    int nQarStatus = QarAreUnitsSame( qdOperand1, qdOperand2 );
    if( nQarStatus == QAR_ERR )
    {
          QcalcAppendTextDisplay( "This Operation requires inputs in same units (from SI_conversion)" );
     	  return true; // allow Add and Subtract to try to convert on the fly
    }
    else if ( nQarStatus == QAR_WARN )
    {
		QcalcAppendTextDisplay( "WARNING: units (SI_conversion) unknown for at least one input - Proceed with caution!" );
    }

    //  Test frames...
    nQarStatus = QarAreFramesSame( qdOperand1, qdOperand2 );
    if( nQarStatus == QAR_ERR ) {
		QcalcAppendTextDisplay( "Frames differ for inputs" );
		return false;
	}
    
    return true;
}

bool QcalcTab::DivideValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to divide" );
		return false;
    }

	// trap divide by vector or array

	if ( !QuIsScalar( qdOperand2 ) )
	{
		QcalcAppendTextDisplay( "Only divide by scalars or scalar sequences is allowed" );
		return false;
	}
	
	return true;
}

bool QcalcTab::VectorProdValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Vector Prod" );
		return false;
    }
	
	// test special for multiply ...
	int nQarStatus =  QarTestConformalForVecProds( qdOperand1, qdOperand2 );
	if ( nQarStatus == QAR_ERR )
	{
		QcalcAppendTextDisplay( "Inner and Outer product only valid between vectors or tensors" );
		return false;
	}
	
	return true;
}

bool QcalcTab::MultiplyValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Multiply" );
		return false;
    }
	
	return true;
}

bool QcalcTab::FilterValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	
    if ( qdOperand1.is_nil()  )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object to Filter" );
		return false;
    }
	
	// test special for exp 
	if (QarHasUnits( qdOperand1) )
	{
		QcalcAppendTextDisplay( "WARNING!  Operation Exp() or Log() -> Subject has units" );
	}
	
	return true;
}

bool QcalcTab::PowerValid( ) 
{
	QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
	QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
	
    if ( ( qdOperand1.is_nil() ) || ( qdOperand2.is_nil() ) )
    {
        // The object will be empty - this should never happen
		// The qdResult object is NULL at this point
		// Display error message within the window status
		QcalcAppendTextDisplay( "NULL input object" );
		return false;
    }

	// get base SI for input data
	if( QarHasUnits( qdOperand2 ) )
	{
		// issue warning
		QcalcAppendTextDisplay( "WARNING! Operation Power - Exponent has units.");
	}

	return true;
}



bool QcalcTab::isOpAllowed( ) 
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::isOpAllowed %ld of %ld \n", this, virtualItem);
		
	QcalcListView * qView = (QcalcListView *)virtualItem->listView();
		
	bool isVector0 = false;
	bool isVector1 = false;
		
		QcalcListVirtualItem * qItem = ( QcalcListVirtualItem* )
		                               qView->findItem( this->operandStack[0]->slot->textObject(), Alias_ );
		if(qItem) isVector0 = qItem->getIsVector();
		
		if(isVector0){
			// allow for component selection in slot
			if(this->operandStack[0]->slot->getComponent() != WL_OBJ) isVector0 = false;
		}
		
		if (nSlots > 1){
		 	qItem = ( QcalcListVirtualItem* )qView->findItem( this->operandStack[1]->slot->textObject(), Alias_ );
		     if(qItem) isVector1 = qItem->getIsVector();
			
			if(isVector1) {
				// allow for component selection in slot
				if(this->operandStack[1]->slot->getComponent() != WL_OBJ) isVector1 = false;
			}

		}

		// JOIN
		if(virtualItem->m_qstrOperator == strOpJoin){
			return true; // always allowed
		}
		
		// ADD or SUBTRACT
		else if(virtualItem->m_qstrOperator == strOpAdd || virtualItem->m_qstrOperator == strOpSub){
			if( ( isVector0 && !isVector1 ) || ( !isVector0 && isVector1 ) ) {
				QcalcAppendTextDisplay( "Cannot do operation: objects of different dimensionality, e.g. scalar + vector" );
				return false; 
			}
			if( !(this->AddValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Add or Subtract" );
				return false;
			}
		}
		
		// MULTIPLY scalars and matrices
		else if(virtualItem->m_qstrOperator == strOpMult){
			if( isVector0 &&  isVector1)  {
				QcalcAppendTextDisplay( "Use vector operations for multiplying vectors" );
				return false; 
			}
			if( !(this->MultiplyValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Multiply" );
				return false;
			}
		}
		
		// DIVIDE
		else if(virtualItem->m_qstrOperator == strOpDiv){
			// only numerator may be vector
			if( isVector1 )  {
				QcalcAppendTextDisplay( "cannot divide by vector or array" );
				return false; 
			}
			if( !(this->DivideValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Divide" );
				return false;
			}
		}
		
		// MOD
		else if(virtualItem->m_qstrOperator == strOpMod){
			// always scalar but allowed for vector
		}
		
		// POWER
		else if(virtualItem->m_qstrOperator == strOpPower){
			// always scalar but allowed for vector
			// only base may be vector
			if( isVector1 )  {
				QcalcAppendTextDisplay( "cannot raise to power of vector or array" );
				return false; 
			}
			
			if( !(this->PowerValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Power" );
				return false;
			}
		}
		
		// CHANGE SIGN
		else if(virtualItem->m_qstrOperator == strOpChSign){
		}
		
		// TRIG
		else if(virtualItem->m_qstrOperator == strOpCos){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpSin){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpTan){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCot){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCosh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpSinh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOptanh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpCoth){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACos){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpASin){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpATan){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACot){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACosh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpASinh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpAtanh){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		else if(virtualItem->m_qstrOperator == strOpACoth){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// CROSS PRODUCT
		else if(virtualItem->m_qstrOperator == strOpVecProd){
			if( !isVector0 || !isVector1 ) {
				QcalcAppendTextDisplay( "Operation is only valid between vectors or tensors" );
				return false; 
			}
			if( !(this->VectorProdValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Cross Product" );
				return false;
			}
		}
		
		// DOT PRODUCT
		else if(virtualItem->m_qstrOperator == strOpDotProd){
			if( !isVector0 || !isVector1 ) {
				QcalcAppendTextDisplay( "Operation is only valid between vectors or tensors" );
				return false; 
			}
			if( !(this->VectorProdValid()) ) {
				QcalcAppendTextDisplay( "Mismatch in Xrefs for inputs to Inner Product" );
				return false;
			}
		}
		
		// REMAINDER
		else if(virtualItem->m_qstrOperator == strOpRemainder){
			if( isVector0 ){
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// ABS
		else if(virtualItem->m_qstrOperator == strOPAbsValue){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// LOG e
		else if(virtualItem->m_qstrOperator == strOpLog){
			if( isVector0 ){
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Log e" );
				return false;
			}
		}
		
		// LOG 10
		else if(virtualItem->m_qstrOperator == strOpLog10){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Log 10" );
				return false;
			}
		}
		
		// INVERSE
		else if(virtualItem->m_qstrOperator == strOpInvers){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// SQRT
		else if(virtualItem->m_qstrOperator == strOpSqrt){
			if( isVector0 )  {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
		}
		
		// EXPONENT
		else if(virtualItem->m_qstrOperator == strOpExp){
			if( isVector0 ) {
				QcalcAppendTextDisplay( "Operation is not valid for vector objects" );
				return false; 
			}
			if( !(this->FilterValid()) ) {
				QcalcAppendTextDisplay( "Input not compatible with Exp" );
				return false;
			}
		}
		
		// CONVERT UNITS
		else if(virtualItem->m_qstrOperator == strOpUnits){
			QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
			if( !QarHasUnits( qdOperand1 ) ){
				QcalcAppendTextDisplay( "Input needs SI_conversion for this filter " );
				return false;
			}
			QdObject_var qdOperand2 = this->operandStack[1]->slot->getSlotObject();
			if( !QarHasUnits( qdOperand2 ) ){
				QdString_var strObj = QdString_var::narrow(qdOperand2);
				if(strObj.is_nil() || strstr(strObj->c_str(), ">") == NULL ) {
					QcalcAppendTextDisplay( "SI_conversion slot must be text or object with SI_conversion" );
					return false;
				}
			}
		}
		
		// CONVERT FILLS
		else if(virtualItem->m_qstrOperator == strOpFills){
			QdObject_var qdOperand1 = this->operandStack[0]->slot->getSlotObject();
			QdObject_var fill_xref = qdOperand1->get_xref(FILL_VALUE);
			if( fill_xref.is_nil()){
				QcalcAppendTextDisplay( "Input needs FILLVAL for this filter " );
				return false;
			}
		}
		
	return true;
}


void QcalcTab::reNameAliasTab(const QString& NewVal)
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::reNameAliasTab %ld of %s \n", this, NewVal.latin1());
		
	if ( NewVal.isEmpty()  ) return;
 
	if (calcListview == NULL) return;

	// stop revalidation from non-user input
	if ( !(this->objAliase_field->lineBox->hasFocus()) ) return;

	if ( virtualItem->AliasAlreadyUsed( NewVal ) )
	{
		QString qstrMsg = QString("Alias ") + QString(NewVal.latin1()) + QString(" already used. Please re-enter a valid alias");
		calcListview->owner->messageHandler( qstrMsg, true );
		return;
	}
	
	int cursorPosn = this->objAliase_field->lineBox->cursorPosition();
	calcListview->owner->messageHandler( " ", false );
	virtualItem->reNameAlias( NewVal );
	this->objAliase_field->lineBox->setFocus();
	this->objAliase_field->setCursorPosition(cursorPosn);
}

void QcalcTab::ValidateAlgorithm( int nIndex )
{
	if ( _algorithm )
	{
		if (_width_field  )
		{
			if ( nIndex == 1  )
			{
				_width_field->setEnabled( TRUE );
			}
			else
			{
				_width_field->setEnabled( FALSE );
			}
		}
		
		if( !(_algorithm->isVisible()) ) return; // ensure only active tab can update selection 
				
		// now instruct any seleceted item to also update
		for( int i=0; i < calcListview->localList.size(); i++)
		{
			QcalcListVirtualItem *item = (QcalcListVirtualItem *) calcListview->localList[i];
			if (item == virtualItem) continue; // this item
			if( item->getEntryType() != ListTypeEntry) continue;
			// set algorithm for selected item
			item->m_pcalcTab->_algorithm->setCurrentItem(_algorithm->currentItem()); // may emit validate signal which is trapped above
			if (item->m_pcalcTab->_width_field  )
			{
				if ( nIndex == 1  )
				{
					item->m_pcalcTab->_width_field->setEnabled( TRUE );
				}
				else
				{
					item->m_pcalcTab->_width_field->setEnabled( FALSE );
				}
			}
		}
	}
	
}


void QcalcTab::setGap(int nIndex)
{

	if ( QuSlotDebug() )
		printf( "QcalcTab::setGap %ld\n", this );

	if ( _gap )
	{
		if( !(_gap->isVisible()) ) return; // ensure only active tab can validate - avoid incorrect reset and inf loop

		// now instruct any seleceted item to also update
		for( int i=0; i < calcListview->localList.size(); i++)
		{
		   QcalcListVirtualItem *item = (QcalcListVirtualItem *) calcListview->localList[i];
		   if (item == virtualItem) continue; // this item
		   if( item->getEntryType() != ListTypeEntry) continue;
		   // set gap for selected item gap;
		   item->m_pcalcTab->_gap->setCurrentItem(_gap->currentItem()); // this may emit validate signal which is trapped above
		}
	}

}


void QcalcTab::setWidthField()
{

	if ( QuSlotDebug() )
		printf( "QcalcTab::setWidthField %ld\n", this );

	if (  this->_width_field->edited() )
	{
		// now instruct any seleceted item to also update
		for( int i=0; i < calcListview->localList.size(); i++)
		{
		   QcalcListVirtualItem *item = (QcalcListVirtualItem *) calcListview->localList[i];
		   if (item == virtualItem) continue; // this item
		   if( item->getEntryType() != ListTypeEntry) continue;
		   // set gap for selected item gap;
		   item->m_pcalcTab->_width_field->setText(this->_width_field->text()); // this may emit validate signal which is trapped above
		}

	}

}


void QcalcTab::setGapField()
{

	if  ( QuSlotDebug() )
		printf("QcalcTab::setGapField %ld\n", this);

	if(  this->_gap_field->edited() )
	{
		// now instruct any seleceted item to also update
		for( int i=0; i < calcListview->localList.size(); i++)
		{
		   QcalcListVirtualItem *item = (QcalcListVirtualItem *) calcListview->localList[i];
		   if (item == virtualItem) continue; // this item
		   if( item->getEntryType() != ListTypeEntry) continue;
		   // set gap for selected item gap;
		   item->m_pcalcTab->_gap_field->setText(this->_gap_field->text()); // this may emit validate signal which is trapped above
		}

	}

}

void QcalcTab::confirmAliasSlot()
{

	if  ( QuSlotDebug() )
		printf("QcalcTab::confirmAliasSlot %ld\n", this);

	if(  this->objAliase_field->text() != this->virtualItem->text(Alias_) )
	{
		// if user goes elsewhere before valid alias is entered, reset to existing alias
		this->objAliase_field->setText(this->virtualItem->text(Alias_));
	}

}

void QcalcTab::SetOperand( const QString & opStr )
{
	if ( QuSlotDebug() )
		printf("QcalcTab:: SetOperand %ld [%s]\n", this, opStr.latin1());
		
	QcalcListVirtualItem*   qItem = this->virtualItem;
	if ( qItem == NULL ) return;
	
	if(qItem->m_qstrOperator.isNull() ) return;

	SetTab();
     return;
}


void QcalcTab::SetLink( const QString & objStr )
{
	if ( QuSlotDebug() )
		printf("QcalcTab::SetLink %ld %s\n", this, objStr.latin1());
	
	QcalcListVirtualItem*   qItem = dynamic_cast <QcalcListVirtualItem *> (this->virtualItem);
	if (qItem == NULL  ) return;

	if( qItem->m_qstrOperator == strOpJoin )
	{
		qItem->WLcontext = QString(objStr.latin1());
		// rename item on list
		qItem->setText(Obj_Name_, objStr);
	}
		
	SetTab();

     return;
}


void QcalcTab::toggleWLName()
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::toggleWLName %ld\n", this);

	QcalcListVirtualItem*   qItem = dynamic_cast <QcalcListVirtualItem *> (this->virtualItem);
	if ( qItem == NULL ) return;

	if ( this->_nameWL_field == NULL || this->saveWL_check == NULL)	return;
	
	if ( this->saveWL_check->isChecked() )
	{
		this->_nameWL_field->setEnabled( TRUE );
		this->_nameWL_field->setText(qItem->m_NameForSave);
	}
	else
	{
		this->_nameWL_field->setEnabled( FALSE );
		this->_nameWL_field->setText("");
	}
	
	// propogate the same choice to other selected items
	for(int i=0; i<calcListview->localList.size(); i++)
	{
		QcalcListVirtualItem *listItem = dynamic_cast <QcalcListVirtualItem *>(calcListview->localList[i]);
		if (listItem == qItem) continue; // already done

		// set toggle for other selected item
		if(this->saveWL_check->isChecked()){
			listItem->m_pcalcTab->saveWL_check->setChecked(TRUE);
			listItem->m_pcalcTab->_nameWL_field->setEnabled( TRUE );
			listItem->m_pcalcTab->_nameWL_field->setText(listItem->m_NameForSave);
		}
		else
		{
			listItem->m_pcalcTab->saveWL_check->setChecked(FALSE);
			listItem->m_pcalcTab->_nameWL_field->setEnabled( FALSE );
			listItem->m_pcalcTab->_nameWL_field->setText("");
		}
	}
}    
  
void QcalcTab::setNameForSave(const QString& newNameForSave)
{
	if  ( QuSlotDebug() )
		printf("QcalcTab::setNameForSave %ld\n", this);

	QcalcListVirtualItem*   qItem = dynamic_cast <QcalcListVirtualItem *> (this->virtualItem);
	if ( qItem == NULL ) return;
	
	if (this->_nameWL_field->lineBox->hasFocus()) 
	{
		qItem->m_NameForSave = QString(newNameForSave.latin1());
	
		this->WLnameUnset = false;  // stops auto-set overwriting user choice
	}
}


void QcalcTab::SetOperator( const QString& strOperator )
{
	if ( QuSlotDebug() )
		printf("QcalcTab::SetOperator %ld %s \n", this, strOperator.latin1());
	
	if( strOperator.isEmpty() ) return;
	
	QcalcListVirtualItem* qItem = dynamic_cast <QcalcListVirtualItem*> (this->virtualItem);
	if (qItem == NULL ) return;
	qItem->m_qstrOperator = QString(strOperator);
	
	SetSlotsForOperator(strOperator);

	QString joined_name;

	if ( this->tb4 && !(strOperator.isNull()) )
	{
		QdOperator qdOpDetails; // this is initialised with defaults
		this->tb4->setText( strOperator );

	     if(strOperator != strOpJoin)  
		{
			this->opWasJoin = false; // flag used to empty slot A if operation changes FROM join (as WL item is lost)
			calcListview->owner->m_mapOps.FindOperator( strOperator, qdOpDetails ) ;
		
			this->nSlots = qdOpDetails.GetNumOperands();
			if ( qdOpDetails.GetTrigFlag() ) //  atan2(y,x) does NOT have trig flag set
			{
				bool SIc_set = false;
				QdObject_var dobj = this->operandStack[0]->slot->getSlotObject();
				char * SIc = QuGetAttrText( "SI_conversion", dobj ); // safe against nil dobj

				if(strstr(SIc, "rad") != 0 || strstr(SIc, "deg") != 0){
					// SI conversion should be used for units
					SIc_set = true;
				}

				if(           strOperator == strOpCosh
				           || strOperator == strOpSinh
				           || strOperator == strOptanh
				           || strOperator == strOpCoth
				           || strOperator == strOpACosh
				           || strOperator == strOpASinh
				           || strOperator == strOpAtanh
				           || strOperator == strOpACoth) this->AngleUnits->hide(); // hyperbolic trig
				else if ( (  strOperator == strOpCos
				           || strOperator == strOpSin
				           || strOperator == strOpTan
				           || strOperator == strOpCot)
					    && SIc_set ) this->AngleUnits->hide(); // trig with SI_conversion
				else {
					this->AngleUnits->show(); // inverse trig or trig with no SI_conversion
					this->AngleUnits->pulldown->setCurrentItem( 0 );
				}
			}
			else
			{
				this->AngleUnits->hide(); // not trig menu
			}
		}
		else{
			this->AngleUnits->hide();		
		}
		
		for(int i = this->nSlots; i < MAX_N_OPERANDS; i++) 
		{
			this->operandStack[i]->hide();
			this->operandStack[i]->slot->active = false;
		}
		
		for(int i = 0; i < this->nSlots; i++)
		{
			this->operandStack[i]->show();
			this->operandStack[i]->slot->active = true;
			this->operandStack[i]->slot->reValidate();
		}

		// Set the operator name within the selected list view item
		qItem->SetItemVirtualName( true );

		// set save to WL name here AFTER the name has been set in text(Obj_Name_)
		// don't update once the user has entered their own choice
		if(this->WLnameUnset || this->_nameWL_field->text() == QString("") )
		{
   	  		this->WLnameUnset = true; // reset if field was blank
			joined_name = QString(qItem->text(Obj_Name_).latin1());
			if(strOperator == strOpJoin)
			{
				// for WL item add _J and remove path
				int compPosn = joined_name.find( " [ " );
				if (compPosn != -1) {
					joined_name.replace(compPosn+1, 1, "(");
					compPosn = joined_name.find( "]" );
					if (compPosn != -1) joined_name.replace(compPosn, 1, ")");
				}
			
				joined_name        += "_J";
				
     			int posn            = joined_name.find( "}" );
     			if(posn != -1) joined_name.remove( 0, posn+1 );
			}

			// sanitise / from divide op. Will be } from text() sanitisation
			int posn = joined_name.find(" } ");
			while (posn >=0){
				joined_name.replace(posn, 3, " | ");
				posn = joined_name.find(" } ");
			}

			this->_nameWL_field->setText(joined_name);
			qItem->m_NameForSave = QString(joined_name.latin1()); // used only to allow name to be held when toggle is off
		}
	}
}

void QcalcTab::SetSlotsForOperator(const QString& strOperator )
{	
	this->operandStack[1]->inputLabel->setText("Operand B    "); // gets reset by Filter units
	this->operandStack[1]->slot->acceptText = false;
	if( this->opWasJoin ) {
		this->operandStack[0]->slot->setText("");
	}
	
	// all operations are virtual except join
	this->operandStack[0]->slot->acceptAlias = true;  
	this->operandStack[0]->slot->acceptObjectPlus = false;
	QToolTip::remove(  this->operandStack[0] );
	QToolTip::remove(  this->operandStack[1] );
	if(strOperator != strOpUnits){
		QToolTip::add(  this->operandStack[0], " Drop object from WL or calculator list, or type in value " );
		QToolTip::add(  this->operandStack[1], " Drop object from WL or calculator list, or type in value " );
	}
			
	// Arithmetic
	if(     strOperator == strOpMod
		|| strOperator == strOpChSign ){
	  this->operandStack[0]->slot->acceptAliasToScalar = false;
	  this->operandStack[0]->slot->acceptAliasToVector = false;
	}
	else if(strOperator == strOpDiv 
	     || strOperator == strOpPower){
	  this->operandStack[0]->slot->acceptAliasToScalar = false;
	  this->operandStack[1]->slot->acceptAliasToScalar = true;
	  this->operandStack[0]->slot->acceptAliasToVector = false;
	  this->operandStack[1]->slot->acceptAliasToVector = false;
	}
	else if(strOperator == strOpAdd 
	     || strOperator == strOpSub
		|| strOperator == strOpMult){
	  // note multiply will operate on matrices, so set no extra constraints
	  this->operandStack[0]->slot->acceptAliasToScalar = false;
	  this->operandStack[1]->slot->acceptAliasToScalar = false;
	  this->operandStack[0]->slot->acceptAliasToVector = false;
	  this->operandStack[1]->slot->acceptAliasToVector = false;
	}
	// Trig
	else if(strOperator == strOpCos 
	     || strOperator == strOpSin
		|| strOperator == strOpTan
		|| strOperator == strOpCot
		|| strOperator == strOpCosh
		|| strOperator == strOpSinh
		|| strOperator == strOptanh
		|| strOperator == strOpCoth
		|| strOperator == strOpACos
		|| strOperator == strOpASin
		|| strOperator == strOpATan
		|| strOperator == strOpACot
		|| strOperator == strOpACosh
		|| strOperator == strOpASinh
		|| strOperator == strOpAtanh
		|| strOperator == strOpACoth){
			this->operandStack[0]->slot->acceptAliasToScalar = true;
			this->operandStack[0]->slot->acceptAliasToVector = false;
	}
	else if(strOperator == strOpATan2){ 
			this->operandStack[1]->slot->acceptAliasToScalar = true;
			this->operandStack[1]->slot->acceptAliasToVector = false;
	}
	// Vector Ops
	else if(strOperator == strOpVecProd
		|| strOperator == strOpDotProd){ 
	  		this->operandStack[0]->slot->acceptAliasToScalar = false;
	  		this->operandStack[1]->slot->acceptAliasToScalar = false;
	  		this->operandStack[0]->slot->acceptAliasToVector = true;
	  		this->operandStack[1]->slot->acceptAliasToVector = true;
	}
     // Filters
	else if(strOperator == strOpRemainder){
	  	this->operandStack[0]->slot->acceptAliasToScalar = true;
	  	this->operandStack[1]->slot->acceptAliasToScalar = true;
	  	this->operandStack[0]->slot->acceptAliasToVector = false;
	  	this->operandStack[1]->slot->acceptAliasToVector = false;
	}
	else if(strOperator == strOpInvers){
	  	// will accept square matrices
	  	this->operandStack[0]->slot->acceptAliasToScalar = false;
	  	this->operandStack[0]->slot->acceptAliasToVector = false;
	}
	else if(strOperator == strOpUnits){
	  	// will accept anything
	  	this->operandStack[0]->slot->acceptAliasToScalar = false;
	  	this->operandStack[0]->slot->acceptAliasToVector = false;
	  	this->operandStack[1]->slot->acceptText = true;
	  	this->operandStack[1]->inputLabel->setText("SI_conv/Object  ");
		QToolTip::add(  this->operandStack[0], " Drop object from WL or calculator list, or type in value " );
		QToolTip::add(  this->operandStack[1], 
				        " Accepts SI_conversion text (e.g. 1.0e3>m), or object with SI_conversion xref ");
	}
	else if(strOperator == strOpFills){
	  // will accept anything
	  this->operandStack[0]->slot->acceptAliasToScalar = false;
	  this->operandStack[0]->slot->acceptAliasToVector = false;
	  this->operandStack[1]->slot->acceptAliasToScalar = true;
	  this->operandStack[1]->inputLabel->setText("Replace fill w/  ");
	}
	else if(strOperator == strOPAbsValue
		|| strOperator == strOpLog
		|| strOperator == strOpLog10
		|| strOperator == strOpSqrt
		|| strOperator == strOpExp){
	  	this->operandStack[0]->slot->acceptAliasToScalar = true;
	  	this->operandStack[0]->slot->acceptAliasToVector = false;
	}
	// Join only
	else if(strOperator == strOpJoin){
		this->operandStack[1]->inputLabel->setText("Operand B   "); // gets reset by Filter units
	
		this->opWasJoin = true;
		this->nSlots = 1;
			
		this->operandStack[0]->slot->acceptAlias = false;
		this->operandStack[0]->slot->acceptObjectPlus = true;
		this->operandStack[0]->slot->acceptAliasToScalar = false;
		this->operandStack[0]->slot->acceptAliasToVector = false;
		QToolTip::remove(  this->operandStack[0] );
		QToolTip::add(  this->operandStack[0], "Drop object from WL" );
	
		if ( this->menu )
		{
			QToolTip::remove( this->menu );
			QToolTip::add( this->menu, QString("returns object or component joined") );
		}
	}
	else{
		cout << "UNKNOWN OPERATION, NOTIFY CSC SUPPORT [" << strOperator.latin1() << "]" << endl;
	}

}

void QcalcTab::SetOperatorArithemtic( int nOperator )
{	
	QString strOperator = this->arithmetic->text( nOperator );
	SetOperator( strOperator );
}

void QcalcTab::SetOperatorTrig( int nOperator )
{
     QString strOperator = this->trig->text( nOperator );
	SetOperator( strOperator);
}


void QcalcTab::SetOperatorVectorOps(  int nOperator )
{	  
     QString strOperator = this->vectorOps->text( nOperator );
	SetOperator( strOperator);
}

void QcalcTab::SetOperatorFilter( int nOperator )
{
     QString strOperator = this->filter->text( nOperator );
	SetOperator( strOperator);
}

void QcalcTab::SetOperatorModify( int nOperator )
{
     QString strOperator = this->modify->text( nOperator );
	SetOperator( strOperator);
}


void QcalcTab::SetOperatorJoin( int nOperator )
{
	if(nOperator != JOIN_MENU_ITEM_ID) return; // not the Join item, so is a sub-menu
     SetOperator( strOpJoin  );
}


void QcalcTab::DisplayOperatorDesc( int nOperator, Q_POPUP_MENU* qMenu )
{

	QcalcListVirtualItem* qItem = dynamic_cast <QcalcListVirtualItem*> (this->virtualItem);
	if (qItem == NULL ) return;
	
	if ( qMenu )
	{
		QString strOperator = qMenu->text( nOperator );
		if ( !strOperator.isNull() )
		{
			QdOperator qdOpDetails;
			if ( calcListview->owner )
			{
				if ( calcListview->owner->m_mapOps.FindOperator( strOperator, qdOpDetails ) )
				{
					QString qstrDesc = QString( qdOpDetails.GetDescription().c_str() );
					QToolTip::remove( qMenu );
					QToolTip::add( qMenu, qstrDesc );
				}
			}
		}
	}
}


void QcalcTab::DisplayOpArithemticDesc( int nOperator )
{
	DisplayOperatorDesc( nOperator, this->arithmetic );
}

void QcalcTab::DisplayOpTrigDesc(       int nOperator )
{
	DisplayOperatorDesc( nOperator, this->trig );
}

void QcalcTab::DisplayOpVectorOpsDesc(  int nOperator )
{
	DisplayOperatorDesc( nOperator, this->vectorOps );
}

void QcalcTab::DisplayOpFilterDesc(     int nOperator )
{
	DisplayOperatorDesc( nOperator, this->filter );
}

void QcalcTab::DisplayOpModifyDesc(     int nOperator )
{
	DisplayOperatorDesc( nOperator, this->modify );
}

void QcalcTab::setThisSelected(  ){
  
  if(calcListview && virtualItem){
    calcListview->clearSelectedList();
    calcListview->setSelectedList(virtualItem);

  }
}







