#include "qcalc.h"
#include "qdutil.h"


using namespace QSAS;


bool QcalcWindow::joinSlotObjects(QcalcTab *qTab, vector <QdObject_var >   &returnList )
{
    QdObject_var   qdfillval;
    returnList.clear();
 
    QdObjectSeq_var  qdToJoinList = new QdObjectSeq();
    vector <int> ToJoinIndx;
    QdObjectSeq_var qdJoinedList= new QdObjectSeq();
    vector <QdObject_var>   noActionList;
    vector <int> noActionIndx;

    if ( qdToJoinList.is_nil() ||  qdJoinedList.is_nil() ) return true;

    vector<KVDataBase_var> vOptionList;

    for ( int nIndx = 0; nIndx < qTab->nSlots; nIndx++ )
    {
		QcalcListVirtualItem*  qOperandItem = NULL;
    		if(qTab->virtualItem->m_qstrOperator == strOpJoin) 
		{
			if( !(qTab->saveWL_check->isChecked()) ) return false; // join only and not saved so don't join
			// use join options of this tab
			qOperandItem = qTab->virtualItem;
		}
		else
		{
			// get item slot refers to for join options
			QString Op_i = qTab->operandStack[nIndx]->slot->textObject();
			qOperandItem = dynamic_cast< QcalcListVirtualItem* >( this->_join_listview->findItem(Op_i, Alias_) );
		}
		QdObject_var  qdObj = qTab->operandStack[nIndx]->slot->getSlotObject();
        	if ( qdObj.is_nil()  ) return true; // can't get object at all
		
		// if object is already joined do nothing (e.g. if this object used as target tt or joined earlier)
		if( qOperandItem &&
		    qOperandItem->m_pcalcTab->_algorithm->currentText() != QString("boxcar average") && 
		    QuIsJoined(qdObj, (QdObject_var)_join_listview->qdtargetTT) ) {
		    
				// pass back  as already joined	
				QString qstrNote = QString("Already joined: ") + qTab->operandStack[nIndx]->slot->text() ;
				QcalcAppendTextDisplay( qstrNote.latin1() );
				noActionList.push_back( qdObj );
				int OpID = nIndx;
				noActionIndx.push_back(OpID);
		}	
		// if object has timetags add to join list
		else if ( qdObj->xref_exists(TIME_TAGS) )
		{
			if( qOperandItem == NULL ) return true; // data object but no item REALLY shouldn't happen
			
			if ( qOperandItem->m_eObjType == WORKING_LIST_OBJECT )
			{
				if ( qdObj->xref_exists( "FILLVAL" ) )
				{
					 qdfillval  = qdObj->get_xref( "FILLVAL" );
				}
			}

			if ( qdfillval.is_nil() )
			{
				qdfillval = new QdRScalar( ( double ) 0.1e-30 );
			}

			KVDataBase_var kvJoinOptions = new KVDataBase();
			if ( kvJoinOptions.is_nil() ) return true;
				
				kvJoinOptions->set( FILL_VALUE, qdfillval );
		
			bool bOK;
			// gap tolerance in seconds
			QdTime_var gap_ptr = new QdTime( Seconds( (qOperandItem->m_pcalcTab->_gap_field->text()).toDouble( &bOK ) ) );
			if (bOK) kvJoinOptions->set( GAP_TOLERANCE, gap_ptr );

	 		// boxcar width in seconds
			QdTime_var boxcar_ptr = new QdTime( Seconds( (qOperandItem->m_pcalcTab->_width_field->text()).toDouble( &bOK ) ) );
			if (bOK) kvJoinOptions->set( BOXCAR_WIDTH, boxcar_ptr );

			// interpolation method
			QdString_var interp_ptr = new QdString( (qOperandItem->m_pcalcTab->_algorithm->currentText()).latin1() );
			kvJoinOptions->set( INTERP_METHOD, interp_ptr );

				// gap handling option
			QdString_var gap_opt_ptr = new QdString ( (qOperandItem->m_pcalcTab->_gap->currentText()).latin1() );
			kvJoinOptions->set( GAP_METHOD, gap_opt_ptr );

			QString qName = ( ( qOperandItem->m_eObjType == WORKING_LIST_OBJECT )  ? qOperandItem->getWLcontext() : qOperandItem->text(Alias_) );

			QdString_var qdNewObjName = new QdString( qName.latin1() );
			kvJoinOptions->set( NEW_NAME, qdNewObjName );

			qdToJoinList->push_back( qdObj );
			vOptionList.push_back( kvJoinOptions );
			int OpID = nIndx;
			ToJoinIndx.push_back(OpID);
		}
		else
		{
		 	// pass back unjoined, no tt either constant or text or data series			
			noActionList.push_back( qdObj );
			int OpID = nIndx;
			noActionIndx.push_back(OpID);
		}
	}

	if( qdToJoinList->size() > 0  )
	{	
		try
		{
			QuMultiJoin( qdToJoinList, _join_listview->qdtargetTT, qdJoinedList, vOptionList );
		}
		catch( Exception &e )
		{
			messageHandler( QString( e.msg().c_str( ) ), true );
			return true;
		}
	}

	// assemble return stack and add constants etc back
	int J_id=0;
	int nJ_id=0;
	for ( int nIndx = 0; nIndx < qTab->nSlots; nIndx++ )
	{
		if( ToJoinIndx.size() > J_id && ToJoinIndx[J_id] == nIndx)
		{
			// this was put on joined list
			returnList.push_back(qdJoinedList[J_id]);
			J_id++;
		}
		else if ( noActionIndx.size() > nJ_id && noActionIndx[nJ_id] == nIndx)  
		{
			// this was put on no ACtion list
			returnList.push_back(noActionList[nJ_id]);
			nJ_id++;
		}
		else return true;  // failed to resolve slot
	}
	
	return false;  // no errors found, returnList contains as many values as there were slots
}


KVDataBase_var QcalcWindow::GetJoinOptionObject( QcalcListVirtualItem* item,
                                                 QdObject_var          obj  )
{

	KVDataBase_var Join_options = new KVDataBase();
	if ( Join_options.is_nil() ) return ( Join_options );

	// fill value
	QdRScalar_var    fillval_ptr;
	bool OK = false;
	if ( obj->xref_exists( "FILLVAL" ) )
	{
		QdObject_var fill_xref = obj->get_xref( "FILLVAL" );
		fillval_ptr            = QdRScalar_var::narrow( fill_xref );
	}
	if ( fillval_ptr.is_nil() )
	{
		fillval_ptr = new QdRScalar( ( double ) 0.1e-30 );
	}
	Join_options->set( FILL_VALUE, fillval_ptr );

	// gap tolerance in seconds
	QdTime_var gap_ptr = new QdTime( Seconds( item->m_pcalcTab->_gap_field->text().toDouble( &OK ) ) );
	Join_options->set( GAP_TOLERANCE, gap_ptr );

	// boxcar width in seconds
	QdTime_var boxcar_ptr = new QdTime( Seconds( item->m_pcalcTab->_width_field->text().toDouble( &OK ) ) );
	Join_options->set( BOXCAR_WIDTH, boxcar_ptr );

	// interpolation method
	QdString_var interp_ptr = new QdString( item->m_pcalcTab->_algorithm->currentText().latin1() );
	Join_options->set( INTERP_METHOD, interp_ptr );

	// gap handling option
	QdString_var gap_opt_ptr = new QdString( item->m_pcalcTab->_gap->currentText().latin1() );
	Join_options->set( GAP_METHOD, gap_opt_ptr );

	QdString_var new_obj_name = new QdString( item->text(Alias_).latin1() );
	Join_options->set( NEW_NAME, new_obj_name );

	return ( Join_options );
}


bool QcalcWindow::joinObjects(QdObjectSeq_var         JoinedList,
                             	vector<KVDataBase_var>  &OptionList,
						bool                    bSkipVirtual )
{

	if  ( QuSlotDebug() )
		printf( "QcalcWindow::joinObjects %ld\n", this );

	QdObjectSeq_var  ToJoinList = new QdObjectSeq();
	QdRScalar_var    fillval_ptr;
	QcalcListVirtualItem*   next = NULL ;

	// For each item on Join List, set choices

	next = ( QcalcListVirtualItem * ) _join_listview->firstChild();
	while ( next )
	{
		bool OK;
		QcalcListVirtualItem*  qItem = dynamic_cast< QcalcListVirtualItem* >( next );
		if(!qItem) continue;
		
		QdObject_var obj;
		if( qItem->m_eObjType == VIRTUAL_OBJECT && !bSkipVirtual)
		{
			obj = qItem->m_qdObjResult;
		}
		else if(qItem->m_eObjType == WORKING_LIST_OBJECT)
		{
			obj = qItem->m_pcalcTab->operandStack[0]->slot->getSlotObject();
		}
		
		QdTimeSeq_var qdtt = get_timetags( obj ); // safe against null obj
		
		if ( qdtt.is_nil() )
		{
			next = ( QcalcListVirtualItem* ) next->QguGetNext( );
			continue;
		}

		if ( disjointSeries( obj ) )
		{
			QString ctxt = next->getWLcontext();
			( void )QMessageBox::warning( this, "Join", "disjoint series - "+ctxt );
			qApp->processEvents();
		}

		KVDataBase_var Join_options = GetJoinOptionObject( next, obj );
		if ( Join_options.is_nil() ) return ( false );

		ToJoinList->push_back( obj );
		OptionList.push_back( Join_options );

		next = ( QcalcListVirtualItem* ) next->QguGetNext( );
	}

	if ( ToJoinList->size() == 0 )  return ( true ); // no joining was needed

	// Do actual join
	try
	{
		messageHandler( "Performing multijoin...", false );
		QuMultiJoin( ToJoinList, _join_listview->qdtargetTT, JoinedList, OptionList );
	}
	catch( Exception &e )
	{
		messageHandler( QString(e.msg().c_str() ), true );
		QcalcAppendTextDisplay( e.msg().c_str() );
		return ( false );
	}

	return ( true );

}


bool QcalcWindow::disjointSeries( QdObject_var obj )
{
// checks whether obj is disjoint from target Time Series.
//
//
	if ( obj.is_nil() ) return true;

	// fetch time interval associated with this object
	QdTimeInterval_var obj_ivl = get_timeinterval( obj );
	if ( obj_ivl.is_nil() ) return false;
	
	QdTimeInterval_var join_ivl = get_timeinterval( _join_listview->qdtargetTT );
	if ( join_ivl.is_nil() ) return false;
	
	if ( obj_ivl->int_intersect( *join_ivl ).empty() )
	{
		// are disjoint
		return true;
	}
	else return false;

}




QdTimeInterval_var QcalcWindow::QcalcGetInterval( )
{

	QdTimeInterval_var ivl;

	if ( _TimeIvlSelect->currentItem() == 0)
	{
		// interval from data
		ivl = QcalcGetIntervalFromData();
	}
	else
	{
		// interval from object (utility call get_timeinterval is safe against nil object
	        ivl = get_timeinterval( _IvlSource->getSlotObject() );
	}

	if ( !ivl.is_nil() )
	{
		if(QuSlotDebug()){
			cout << "Time interval set:-" << endl;
			ivl->start().display();
			ivl->end().display();
		}
	}
	
	return ivl;
}

QdTimeInterval_var QcalcWindow::QcalcGetIntervalFromData()
{
	QdTimeInterval_var ivl;
	QdTimeInterval_var ivlUnion;
	QcalcListVirtualItem *    next;

	next = ( QcalcListVirtualItem * ) _join_listview->firstChild();

	while ( next )
	{
		if ( next->getEntryType() == ListTypeEntry && next->m_eObjType == WORKING_LIST_OBJECT)
		{
			QdObject_var ivl_obj = QuGetObjByName( next->getWLcontext() );
			if ( !ivl_obj.is_nil() )
			{
				ivl = QdTimeInterval_var::narrow( ivl_obj );
				if ( ivl.is_nil() )
				{
					QdTimeSeq_var ts = get_timetags( ivl_obj );
					if ( !ts.is_nil() )
					{
						ivl = new QdTimeInterval( ts->get_interval_bounds() );
					}
				}
			}

			if ( ivlUnion.is_nil() )
			{
				ivlUnion = ivl;
			}
			else if ( !ivl.is_nil() )
			{
				QdTimeInterval_var ivlNew = new QdTimeInterval( ivlUnion->int_union( *ivl ) );
				ivlUnion = ivlNew;
			}
		}

		next = ( QcalcListVirtualItem * ) next->QguGetNext();
	}

	return ivlUnion;
}


QdTimeSeq_var QcalcWindow::getTimeTags( )
{

	QdTimeSeq_var      tags;
	QdTimeInterval_var ivl;
	double             Spacing;
	double             Tinterval;
	
	if  ( QuSlotDebug() )
	 	printf( "QcalcWindow::getTimeTags %ld\n", this );
		
	// set interval
	ivl = QcalcGetInterval( );
	if (ivl.is_nil() ) return tags;   // will be .is_nil()
	Tinterval = Seconds( ivl->duration_abs() );

	// set tags

	if ( _TimeTagSelect->currentItem() == 0 )
	{
 		// create regular tags
		bool OK;
		Spacing = _TagSource->text().toDouble(&OK);
		if ( !OK || Spacing < 1.e-20 )
		{
			return tags;
		}

		int Ntags = (int) ( Tinterval / Spacing  +  1.0 );

		tags = new QdTimeSeq();
		tags->assign_regular( Ntags, ivl->start(), Seconds( Spacing ) );

	}
	else
	{
		// tags from object, utility get_timetags() is safe against nil object
		QdTimeSeq_var  ts = get_timetags( _TagSource->getSlotObject());
		if ( ts.is_nil() )
		{
			messageHandler("Time tag source did not give valid tags", true);
			return tags;    // will be .is_nil()
		}

		// truncate to interval if necessary

		if (   ( ivl->start()  != ts->get_interval_bounds().start() )
			|| ( ivl->end()   != ts->get_interval_bounds().end()    )    )
		{
			SequenceMask msk( ts->get_mask( QdTimeSeq::InRange( ivl->start(),ivl->end() ) ) );
			tags = new QdTimeSeq();
			ts->get_subsequence( *tags, msk );
		}
		else
		{
			tags = new QdTimeSeq(*ts);
		}
	}

	return tags;
}

void QcalcWindow::messageHandler( QString txt, bool bError )
{
	if ( QuSlotDebug() )
		printf( "QcalcWindow::messageHandler %ld\n", this );

	QPalette    palette  = QPalette(qApp->palette());
	QColorGroup g        = QColorGroup(palette.active());
	if ( bError ){
		g.setColor( QColorGroup::Text,        g.dark());
		g.setColor( QColorGroup::Background,  g.link() );
	}
	else
	{
		g.setColor( QColorGroup::Text,      g.text() );
		g.setColor( QColorGroup::Background, g.background()   );
	}
	palette.setActive( g );
	palette.setInactive( g );
	this->msgBar->setPalette( palette );

	this->status( txt, 50000 );

}


QString getOpName(int OpNum){
 
  QString nextName = QString("Operand ") + getOpChar(OpNum) + QString("  ");
 
  return nextName;
  
}

QString getOpChar(int OpNum){
  // operand number starts at 0
  int charNum = OpNum + 65;
  
  if(charNum < 92) {
    return QString(QChar(charNum));
  }
  else
  {
    QString num;
    num.setNum(charNum);
    return num;
  }
  
}

