//Dwt_set_plot.cc

#include "Dwt_set_plot.h"

QplugReturnStatus ReadFilter(const char* filter_name, QdRScalarSeq_var gfilt)		 
{
   QString filter_bank_path;
   char* file_name;
   if ( !(filter_bank_path = QSAS_profiles->current()->FilterBankPath).isEmpty() ){
      file_name = (char *)malloc(strlen(filter_bank_path.latin1())+12);
      strcpy(file_name, filter_bank_path);
   } else {
       QplugAppendTextDisplay("Error: Cannot read FILTER_BANK.");
       QplugAppendTextDisplay("Check path in current profile");
       return QPLUG_FAILURE;
   }
   strcat(file_name, "/Bank.flt");
   //cout<<"file_name="<<file_name<<endl;
   int len = gfilt->sequence_size();
   
   ifstream  fin(file_name, ios::in);
   if(!fin){
       QplugAppendTextDisplay("Error: Cannot open filter file.");
       QplugAppendTextDisplay("Check path in current profile.");
       return QPLUG_FAILURE;
   }
   char s[100], ch[20], readch;
   double gvalue;
   int rlen;
   
   fin.getline(s, 100, '\n');
   fin.getline(s, 100, '\n');
   fin.getline(s, 100, '\n');
   fin >> ch >> rlen;
   try{
     while((strcmp(ch, filter_name) !=0) || rlen != len){
	for(int i=0; i<rlen; i++)
		fin >> gvalue;
	if((readch=fin.get()) == EOF) {
	   QplugAppendTextDisplay("Error: No match found for the selected filter/length combination\n");
           return QPLUG_FAILURE;
	}	
        fin >> ch >> rlen;
     }
     for(int l=0; l<len; l++){
         fin >> gvalue;
	 gfilt[l]= gvalue;
     }
     fin.close();
     delete file_name;

  } catch(Exception &e){
      QplugAppendTextDisplay("Error: Fails to read the scaling filter file.\n");  
      return QPLUG_FAILURE;
  }
  return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus  Run_Dwt(QdRScalarSeq_var ts_in, int jsize,  
			QdRScalarSeq_var hf, QdRScalarSeq_var gf, 
			QdRScalarSeq_var wavlcoef)			
{
  long ts_size = ts_in->sequence_size();
  int j = 1;
  long Mj = (long)floor(ts_size/2.0);
  try{
    QdRScalarSeq_var 	vin, vout, wout;
    vin = new QdRScalarSeq(*ts_in); //_var::narrow(ts_in); 
    while(j <= jsize ){  
      int Nj = (int)Mj;
      vout = new QdRScalarSeq(Nj);
      wout = new QdRScalarSeq(Nj);
      for(int t=0; t<Nj; t++){
	int tt = 2*t +1;
	vout[t]= gf[0]*vin[tt]; 
	wout[t]= hf[0]*vin[tt]; 
	for(int l=1; l<hf->sequence_size(); l++){
		--tt;
		if(tt < 0) tt = 2*Nj - 1;
		vout[t] = vout[t] + gf[l]*vin[tt];
		wout[t] = wout[t] + hf[l]*vin[tt];
	}
    }
    //now insert at the front. the first inserted become the last bit
    reverse(wout->begin(),wout->end());
    wavlcoef->insert(wavlcoef->begin(),wout->begin(),wout->end());
    vin = vout;
    Mj = (long)(Mj/2.0);
    ++j;
  }
  reverse(vout->begin(),vout->end());
  wavlcoef->insert(wavlcoef->begin(),vout->begin(),vout->end());  
  reverse(wavlcoef->begin(), wavlcoef->end());
  } catch(Exception &e){
	e.print_msg();
	return QPLUG_FAILURE;
   }
    return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus  Run_MoDwt(QdRScalarSeq_var ts_in, int jsize,  
			QdRScalarSeq_var hf, QdRScalarSeq_var gf, 
			QdRScalarSeq_var wavlcoef)			
{
  long ts_size = ts_in->sequence_size();
  int j = 1, k = 1, tt;
  try{
    QdRScalarSeq_var 	vin, vout, wout;
    vin = QdRScalarSeq_var::narrow(ts_in); 
    while(j <= jsize ){  
      vout = new QdRScalarSeq(ts_size);
      wout = new QdRScalarSeq(ts_size);
      for(int t=0; t<ts_size; t++){
	tt = t;
	vout[t]= gf[0]*vin[tt]; 
	wout[t]= hf[0]*vin[tt]; 
	for(int l=1; l<hf->sequence_size(); l++){
		tt -= k; 
		if(tt < 0) tt = tt + ts_size;
		vout[t] = vout[t] + gf[l]*vin[tt];
		wout[t] = wout[t] + hf[l]*vin[tt];		
	}

      }
   //insert at the front.the first row inserted becomes the last row
   //WARNING: wavlcoef MUSTN'T be given any initial size
    reverse(wout->begin(),wout->end());
    wavlcoef->insert(wavlcoef->begin(),wout->begin(),wout->end());
    vin = vout;
    ++j;
    k = 2*k;
  }
  reverse(vout->begin(),vout->end());
  wavlcoef->insert(wavlcoef->begin(),vout->begin(),vout->end()); 
  reverse(wavlcoef->begin(), wavlcoef->end());
  } catch(Exception &e){
	e.print_msg();
	return QPLUG_FAILURE;
   }
    return QPLUG_SUCCESS;
}
/*-------------------------------------------------------------*/

QplugReturnStatus SPectro_set(QdRScalarSeq_var wav_mat, QdTimeSeq_var ttag,
		int level, PGSpectro *spectrop, QdRScalarSeq_var gfilter, 
		Yes_or_No dwt_yes_no,
		char *x_lab, char *y_lab, char *w_lab)
{
   int nx;
   if(dwt_yes_no == YES)  nx = wav_mat->sequence_size();
   else  nx = (int) (wav_mat->sequence_size()/(level+1.0));
   int nz = ttag->sequence_size(); //size of data without padding
   //cout<<"nx= "<<nx <<" nz= "<< nz <<endl;
   int filtlen = gfilter->sequence_size();
 
   QdRMatrix_var z = new QdRMatrix(Dim(level,nx), 0.0);//z[level][nx]; 
      
  try {
     spectrop->z = new double[level*nx];//for DWT or MODWT
     spectrop->nx = nx;                  
     spectrop->ny = level;

     spectrop->domain.xndcmax = 0.8;
     spectrop->domain.yndcmin   = 0.1;     
     spectrop->domain.yndcmax   = 0.4;

     spectrop->xlabel = x_lab;        
     spectrop->ylabel = y_lab; 
     spectrop->wedge.label = w_lab; //Wedge label;
     spectrop->ylinlog = PGSPECTRO_LOG;
     
     spectrop->skip_missing = YES;
     spectrop->zmissing = 0.0;
     spectrop->zlinlog = PGSPECTRO_LOG; //displays Log(|w_{j,t}|^2)

     float delta_t = (float)Seconds(ttag[1]-ttag[0]); //sampling interval
     if(delta_t <= 0.0 ){
        QplugAppendTextDisplay("Error: Sampling interval <= zero.\n");
        return QPLUG_FAILURE;
     }
     
     spectrop->ybinlow = new double[level];
     spectrop->ybinhigh = new double[level];
     //float lamda_j = pow(2.0,j+1)*delta_t; tau_j = pow(2.0,j)*delta_t;
     for (int j=0; j<level; j++){	
        spectrop->ybinlow[j] = pow(2.0,j)*delta_t; 
        spectrop->ybinhigh[j] = 2.0*spectrop->ybinlow[j];	   
     }            
     spectrop->domain.yworldmin = spectrop->ybinlow[0];
     spectrop->domain.yworldmax = spectrop->ybinhigh[level-1];
 

     QdRScalar_var eg = new QdRScalar(0.0); 
     if(dwt_yes_no == YES){
        //gfilt_var MUST be of unit energy =sum_{l} (g_l)^2
        for(int l=0; l<filtlen; l++) *eg += l*gfilter[l]*gfilter[l];
     }
     else {
	for(int l=0; l<filtlen; l++){
	       gfilter[l] = sqrt(2.0)*gfilter[l];
               *eg +=  l*gfilter[l]*gfilter[l];
        }
    }
	
    QplugReturnStatus ret_stat;
    //now we update the spectrop
    ret_stat = SPectro_fill(nz, filtlen, eg, wav_mat, spectrop, dwt_yes_no, z);

  } catch(Exception &e){
	e.print_msg();
	return QPLUG_FAILURE;
    }
    return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus SPectro_fill(int nz,  int Llen, QdRScalar_var eg, 
		QdRScalarSeq_var wseq, PGSpectro *spectrop,
		Yes_or_No dwt_yes_no, QdRMatrix_var z)
{
 //cout<<"start SPectro_fill."<<endl;
 int level = (z->dimdata())[0]; //spectrop->ny;
 int nx = (z->dimdata())[1];
 QdRScalarSeq_var tmpseq = new QdRScalarSeq();
 QdRScalarSeq_var wavmax =  new QdRScalarSeq(level);
 QdRScalarSeq_var wavmin =  new QdRScalarSeq(level);
 QdRScalarSeq_var tmpwavj;
 int ii, jump, nj = nx/2, nwj, nstart = 0, t=0; 
 
 if(dwt_yes_no == YES){
    for (int k=0; k<level; k++){
       int j = k+1;
       int Lj = (int)((pow(2.0, j) - 1)*(Llen - 1)+1);
       int nujH = (int)( pow(2.0, j-1)*(Llen - 1) - ceil(*eg));
       int wearlyt= 0;
       int ttj = (int)(pow(2.0,j)*(wearlyt+1) - 1 - nujH);
       while(ttj <0){
         ++wearlyt;
         ttj = (int)(pow(2.0,j)*(wearlyt+1) - 1 - nujH);
       } 
       //compute the associated early time in X_t
       int xearlyt = (int) (pow(2.0,j)*(wearlyt+1)-1-nujH);
       //cout<<"j="<<j<<" nj= "<<nj<<" wearlyt= "<<wearlyt;
       int r = (int) (pow(2.0,j-1));
       jump = (int)(pow(2.0,j) - 1.0);   
       //cout<<" xearlyt= "<<xearlyt <<" r= "<< r <<" Lj= "<<Lj <<endl;	

       //pick up the right j-level wav. coef.
       tmpwavj = new QdRScalarSeq(nj);
       wseq->get_subsequence(*tmpwavj ,nstart, nj);
       //divide by the number of time we repeat the energy
       *tmpwavj = *tmpwavj/pow(2.0,j); //works fine
       //shift the wav. coef. to ligne up time to X_t
       if(wearlyt > 0) Circular_shift(tmpwavj, -wearlyt);
	
       //--------next fill the matrix z	  
	//tmp = new QdRScalarSeq(*tmpwavj);    	    	    
	if(xearlyt == 0){
	   //fill to the right and shift left by r
	   ii = 0;	    
	   for(t=0; t<nx; t+=2*r) {
	       for(int l=0; l<2*r; l++){
		   (*z)[k][t+l]= tmpwavj[ii];
		}
	    	++ii; 	
	    }      
	    *tmpseq = (*z)[k].matrix_to_seq();
	    Circular_shift(tmpseq, -r);//shift to left
	    (*z)[k] = *tmpseq;
	} else if(xearlyt >0 & xearlyt == r){	      
	    ii = -1;
	    for(t=xearlyt; t<nx-2*r; t += 2*r){
	       ++ii;
	       //fill by r before t (t-r,..,t-1) and by r-1 after t
	       for(int l=0; l<r+1; l++) (*z)[k][t-l]= tmpwavj[ii];
	       if(r>1){
	         for(int l=1; l<r; l++)(*z)[k][t+l]= tmpwavj[ii];
	       }
	     }
	     ++ii; 
	     for(int l=0; l<r+1; l++) (*z)[k][t-l]= tmpwavj[ii];
	     if(r>1) {
		  for(int l=1; l<r; l++) (*z)[k][t+l]= tmpwavj[ii];   
	     }   		   		
	} else if(xearlyt > r){	   
	     ii = -1;
	     for(t=xearlyt; t<nx-2*r; t+=2*r){
	        //fill by r before t (t-r,..,t-1) and by r-1 after t
	         ++ii;
		 for(int l=0; l<r+1; l++) (*z)[k][t-l]= tmpwavj[ii];
		 if(r>1) {
		       for(int l=1; l<r; l++)(*z)[k][t+l]= tmpwavj[ii];
		 }
	      }
	      ++ii;
	      for(int l=0; l<r+1; l++) (*z)[k][t-l]= tmpwavj[ii];
	      while(nx-t-1>0 & t<nx){
		    ++t;
		   (*z)[k][t]= tmpwavj[ii];
	      }
	      //fill between t=0 and t=xearlyt-r
	      for(int l=0; l<xearlyt-r; l++) (*z)[k][l]= tmpwavj[ii];
		
	} else if(xearlyt >0 & xearlyt<r) {
	      //fill always to the left
	       for(t=0; t<xearlyt+1; t++) (*z)[k][t]= tmpwavj[0];
	       ii = 1;	       
               for(t= xearlyt+1; t<nx-jump; t+=jump+1){ 
	           (*z)[k][t]= tmpwavj[ii];
                   for(int i=t+1; i< t+1+jump; i++) (*z)[k][i]= tmpwavj[ii];	
                   ++ii ;  
 	        }	       
		int tend = nx - (jump - xearlyt);
		if(tend >0){
		   for(t=tend; t<nx; t++) (*z)[k][t]=tmpwavj[0];
	        }
	      //now shift to the right by r-1
	      *tmpseq = (*z)[k].matrix_to_seq();
	      if(r>1) Circular_shift(tmpseq, r-1);//shift to right
	      (*z)[k] = *tmpseq;
	}
	nstart += nj;
	nj = nj/2;
	//*tmpseq = (*z)[k].matrix_to_seq();
	wavmax[k] = tmpwavj->max_value();
	wavmin[k] = tmpwavj->min_value();
//cout<<"wmax="<<wavmax[k]<<" wmin="<<wavmin[k]<<endl;
//cout<<"wmax="<<SafeLog10(wavmax[k])<<" wmin="<<SafeLog10(wavmin[k])<<endl;	      
     }//end of for loop over k	   		
     //spectrop->zscalemin = (float)wavmin->min_value();            
     spectrop->zscalemax = (float)(wavmax->max_value());  
     spectrop->zscalemin = (float)(spectrop->zscalemax/pow(2.0,1.5*level));   	 	    	    
  } else { //if dwt_yes_no = NO
	wseq->get_subsequence(*tmpseq ,0, (level*nx));//off scal. coef
	//spectrop->zscalemin = (float)tmpseq->min_value();            
	spectrop->zscalemax = (float)tmpseq->max_value();
	spectrop->zscalemin = (float)(spectrop->zscalemax/pow(2.0,1.5*level));
	//gfilt_var must be be of unit enery
	for(int k=0; k<level; k++){ 
	       int j = k+1;
               int Lj = (int)((pow(2.0, j) - 1)*(Llen - 1)+1);
               int nujH = (int)( pow(2.0, j-1)*(Llen - 1) - ceil(*eg));
               int w_earlyt= 0;
               int ttj = (int)(w_earlyt - nujH);
	       //cout<<"nujH= "<<nujH<<endl;
               while(ttj <0){
                 ++w_earlyt;
                 ttj = (int)(w_earlyt - nujH);
               } 
	       int x_earlyt = (int)(w_earlyt - nujH);
	       tmpwavj = new QdRScalarSeq(nx);
	       wseq->get_subsequence(*tmpwavj, k*nx, nx);
	       if(w_earlyt > 0) Circular_shift(tmpwavj, -w_earlyt);
	       (*z)[k] = *tmpwavj;
       }
  } 
  //now fill in the vector z. spectro->z[k]=z[i][j]; //z[nx][ny] 
  try{
     if(dwt_yes_no == YES && isPowerOf2(nz) == NO){
         spectrop->nx = nz;   
         spectrop->z = new double[level*nz];
         QdRMatrix_var zz = new QdRMatrix(Dim(level,nz), 0.0);//z[level][nx];
         tmpseq = new QdRScalarSeq(nx);
         tmpwavj = new QdRScalarSeq(nz);
         for(int j=0; j<level; j++) {        
	    *tmpseq = (*z)[j].matrix_to_seq();
            tmpseq->get_subsequence(*tmpwavj, 0, nz);
	    (*zz)[j] = *tmpwavj;
         }
         spectrop->zscalemax = (double)(max(*zz));
         spectrop->zscalemin = (double)(spectrop->zscalemax/pow(2.0,1.5*level));	 
         for(int i=0; i<nz; i++){
            for(int j=0; j<level; j++){
                int k=level*i+j;
	        spectrop->z[k] = (*zz)[j][i];
            }
         }
    } else {
          for(int i=0; i<nx; i++){
             for(int j=0; j<level; j++){
                int k=level*i+j;
	        spectrop->z[k] = (*z)[j][i];
	        //cout<<"z["<<j<<"]["<<i<<"]="<<(*z)[j][i]<<endl;
             }
          }
    }     
   //cout<<"end SPectro_fill."<<endl;
 } catch(Exception &e){
	e.print_msg();
	return QPLUG_FAILURE;
 }
 return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
// QplugReturnStatus SPectro_dev(PGSpectro *spectrop, QdRScalar pgdev, 
// 			const char *filename, int& iddev) 
//   {   
//    PGSpectro_ret return_val;
// 
//      try{
//          return_val = PGSpectro_Plot(spectrop);      
//      } catch(Exception &e){
// 	e.print_msg();
// 	return QPLUG_FAILURE;
//     }
//     return QPLUG_SUCCESS;
// }

/*-------------------------------------------------------------*/

// QplugReturnStatus XYtrace_set(QdRScalarSeq_var yin, PGXYtrace *trace,
// 			char *x_lab, char *y_lab, 
// 			char *title, char *sbtitle, char *tlegend)
// 			//QdTimeSeq_var xtags);
// {
//    int n = yin->sequence_size();
//    trace->y = new double[n]; 
//    for(int i=0; i<n; i++)  trace->y[i] = yin[i];
// 
//    try{ 
//      PGXYtrace_set_defaults(trace); 
//      
//      trace->npts = n;
//      trace->timex = YES;
//      trace->title = title;
//      trace->subtitle = sbtitle;
//      trace->legend_text = tlegend;
//      trace->xlabel = x_lab;
//      trace->ylabel = y_lab;
//    
//      trace->plot_domain.yworldmax = yin->max_value();
//      trace->plot_domain.yworldmin = yin->min_value();
//      trace->plot_domain.xndcmax = 0.8;
//      trace->plot_domain.yndcmin = 0.6;
//      trace->plot_domain.yndcmax = 0.85;
//      //legend position
//      trace->legend_domain.xndcmin = 0.6;
//      trace->legend_domain.xndcmax = 0.8;
//      trace->legend_domain.yndcmin = 0.86;
//      trace->legend_domain.yndcmax = 0.95;
//    
//      // trace.YoNlegendbox = NO;
//      trace->xlinlog = PGSPECTRO_LIN;  // PGSPECTRO_LOG;
//      trace->ylinlog = PGSPECTRO_LIN;  //PGSPECTRO_LOG;
//      trace->colour = QPGGREEN;
//      trace->tick_length = 1.0;    
//      trace->box_thickness = 1;
//      trace->line_width = 1;
//      trace->symbol_size = 1;
//      trace->label_char_size = PGSPECTRO_DEFAULT_CHAR_SIZE;
//      trace->legend_text_char_size = 0.8;
//      trace->text_line_width = 1;
//      //trace->title_disp = 1.5;
//      trace->Ynumlab_left = YES; //NO;
//      trace->YoNlegend = NO; 
//      trace->YoNlegendbox = NO; 
//    } catch(Exception &e){
// 	e.print_msg();
// 	return QPLUG_FAILURE;
//    }
//    return QPLUG_SUCCESS;
// }

/*-------------------------------------------------------------*/

// QplugReturnStatus  Time_set(QdTimeSeq_var ttag, PGSpectro *spgp, 
// 			PGXYtrace *tracep, Yes_or_No dwt_yes)
// {
//   int n = ttag->sequence_size();
//   
//   try{
//      PGSpectro_set_defaults(spgp);
//      PGSpectro_domain_set_defaults( &(spgp->domain) );
//      
//     QdTime_var t_start = new QdTime(ttag[0]);
//     QdTime_var t_end = new QdTime(ttag[n-1]);
//     QdTime_var t_midnite = new QdTime(*t_start);
//     *t_midnite -= Seconds(t_start->second_of_day());
//     //cout<<"*t_midnite= "<< *t_midnite<<endl;
//    
// 
//     string tstart_str, tend_str;
//     char  tstartchar[20], tendchar[20];     
//     tstart_str = (t_start->iso_srep()).substr(0,10);
//     tend_str = (t_end->iso_srep()).substr(0,10); 
//     strcpy(tstartchar, tstart_str.c_str());//local non-const C-copy
//     strcpy(tendchar, tend_str.c_str());
//   
//     tracep->datestart = new char[20];
//     tracep->dateend = new char[20];
//     spgp->datestart = new char[20];
//     spgp->dateend = new char[20];
//     for(int i=0; i<20; i++){
//       tracep->datestart[i] = tstartchar[i];
//       tracep->dateend[i] = tendchar[i];
//       spgp->datestart[i] = tstartchar[i];
//       spgp->dateend[i] = tendchar[i];
//     }
//    
//     float halfspacing = (float)Seconds(ttag[1]-ttag[0])/2.0;
//   
//     spgp->xnums = YES;
//     spgp->timex = YES; 
//     tracep->x = new double[n];
//     spgp->xbinlow = new double[n];
//     spgp->xbinhigh = new double[n];
//  
//     double tmp ; //= (float)Seconds(*t_midnite);
//     QdTimeSeq_var newttag = new QdTimeSeq(*ttag);
//   
//     if(dwt_yes == YES){
//       for(int i=0; i<n; i++){
//         tmp = (double)Seconds(newttag[i] - *t_midnite );
// 	tracep->x[i] = tmp ;//set the x-axis unit to seconds
//         spgp->xbinlow[i] = tmp ; //- halfspacing;
//         spgp->xbinhigh[i] = tmp + 2.0*halfspacing;     
//       }
//     } else {
//       for(int i=0; i<n; i++){
//         tmp = (double)Seconds(newttag[i] - *t_midnite );
//         tracep->x[i] = tmp ;//set the x-axis unit to seconds
//         spgp->xbinlow[i] = tmp - halfspacing;
//         spgp->xbinhigh[i] = tmp + halfspacing;     
//       }
//     }
// 
//     tracep->plot_domain.xworldmax = tracep->x[n-1];
//     tracep->plot_domain.xworldmin = tracep->x[0];
//     spgp->domain.xworldmax = spgp->xbinhigh[n-1];
//     spgp->domain.xworldmin = spgp->xbinlow[0]; 	
//   
//   } catch(Exception &e){
// 	e.print_msg();
// 	return QPLUG_FAILURE;
//    }
//    return QPLUG_SUCCESS;       
// }

/*-------------------------------------------------------------*/
void Circular_shift(QdRScalarSeq_var seq, int shft){
        //alter the input by shifting it by the amount shift
	QdRScalarSeq_var seq_copy = new QdRScalarSeq(*seq);
	if(shft >0 && shft <= seq->sequence_size()){	 
	//shift to the right side    
           QdRScalarSeq::iterator iter_pos = seq_copy->end();
	   iter_pos -= shft;
           vector<double> tmp(seq_copy->begin(), iter_pos);
	   tmp.insert(tmp.begin(), iter_pos,seq_copy->end());
	   *seq = tmp;
	   tmp.clear();
	}
	else if(shft <0 &&  -shft <= seq->sequence_size()){
	//shift to the left side  	   
	   QdRScalarSeq::iterator iter_pos = seq_copy->begin();
           iter_pos += -shft;
	   vector<double> tmp(iter_pos, seq_copy->end());
	   tmp.insert(tmp.end(), seq_copy->begin(), iter_pos);
	   *seq = tmp;
	   tmp.clear();
	}	
 }
 
 /*-------------------------------------------------------------*/
 Yes_or_No isPowerOf2(long m){
       float f = log((float)m)/log(2.0);
      if((f - floor(f)) != 0.0) return NO;
      else return YES;
 }
 
/*-------------------------------------------------------------*/
char* str_dup(string& s, int n){
	char* buf = new char[n+1];
	for(int i=0; i<n; i++)
		buf[i] = s[i];
	buf[n] = '\0';
	return buf;
 }
 	
/*-------------------------------------------------------------*/
void clean_all(PGSpectro *spectrop/*, PGXYtrace *tracep*/){
//       delete [] tracep->x, delete [] tracep->y;
//       delete [] tracep->datestart, delete [] tracep->dateend;
      delete [] spectrop->z;
      delete [] spectrop->ybinlow;
      delete [] spectrop->ybinhigh;
}      
	 

