/*****************************************************************
* PROJECT:  Cluster UK CDHF
* COMPONENT:  Plug In library
* MODULE:  InvTrigoFunct
* LANGUAGE:  C++
* FUNCTION: QSAS InvTrigoFunct  Plug In
* PURPOSE: .
* 	
* ARGUMENTS:  
*    IN: ANY DATA OBJECT
*        integer: function identifier
*	 integer: deg/rad
*    OUT: 
*	ANY DATA OBJECT 
*
* RETURN: QplugReturnStatus
*
* History: version 1.0 Created Jul. 2002. 
* 	   version 1.1 July 2002.
* A. Serroukh, 
******************************************************************/

#include<stdio.h>
#include<math.h>
#include<string.h>
#include<stdlib.h>
#include<fstream>

#include "qar.h"
#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus InvTrigo(QplugArgList *);

using namespace QSAS;

char* str_dup(string& , int );

/*****************************************************************/
QplugReturnStatus InvTrigo(QplugArgList *itrig_list){

   QdObject_var  any_obj_in = (* itrig_list)[0];
   QdObject_var  obj_fct_in = (* itrig_list)[1]; 
   QdObject_var  deg_rad_out= (* itrig_list)[2];
   
   const double PI = 3.14159265358979323846;
          
   char rad_yes[] = "rad", deg_yes[] = "degree"; 
   const int   rad_id = 1, deg_id = 0, default_id = 2;
   int angle_unit = 1; 
   char* win_msg = new char[100];
    
   QdRScalar_var unit = new QdRScalar(1.0);  
   QdRScalar_var RadToDeg = new QdRScalar(180.0/PI);
   QdRScalar_var one = new QdRScalar(1.0);
   QdRScalar_var half = new QdRScalar(0.5);
   //QdObject_var RadToDegObj = new QdRScalar(180.0/PI);
   //QdObject_var onej = new QdRScalar(1.0);
   //QdObject_var halfj = new QdRScalar(0.5);
   QdObject_var  outObj; 
   QdObject_var tmpObj, anyObj = any_obj_in;
   
   //-----unpack and test the narrow operations ------------   
   if( any_obj_in.is_nil() ){
     QplugAppendTextDisplay("Error>> input neither const nor sequence.\n");
     return QPLUG_FAILURE;
   } 
               
   QdRScalar_var obj_fct = QdRScalar_var::narrow(obj_fct_in);
   //int deg_rad_id = (int) *QdScalar_var::narrow(fct_in);
   if(obj_fct.is_nil()){
      QplugAppendTextDisplay("Error>> undefined function\n");
      return QPLUG_FAILURE;
   }       
   int obj_fct_id = (int)(* obj_fct);
   if(obj_fct_id < 4 ) angle_unit = 0; //dealing with no angle unit


   QdRScalar_var deg_or_rad = QdRScalar_var::narrow(deg_rad_out);
   if(deg_or_rad.is_nil()){
      QplugAppendTextDisplay("Error>> Deg/Rad unspecified.\n");
      return QPLUG_FAILURE;
   }        
   int deg_or_rad_id = (int)(*deg_or_rad);  
     
   //-----read and check what's in the SI_conversion ----------  
   static char* no_unit = "1 > ()";
   QdString_var SItxtvar = 
   		new QdString(QuGetAttrText("SI_conversion",any_obj_in));
   char* SItxt = str_dup(*SItxtvar, SItxtvar->length());
   char *SIconvr, *rbuf, *tmpbuf;
   if(strcmp(SItxt,"") == 0){
      SIconvr = new char[strlen(no_unit)+1];
      strcpy(SIconvr,no_unit);
      sprintf(win_msg, "Data SI_conversion: %s \n", SIconvr);
   } else {
      SIconvr = new char[strlen(SItxt)+1];
      strcpy(SIconvr,SItxt);
      sprintf(win_msg, "Data SI_conversion: %s \n", SIconvr);
   }   
   QplugAppendTextDisplay(win_msg);

   if( (tmpbuf = strtok(SIconvr, ">")) != NULL) {
      unit = new QdRScalar(atof(tmpbuf));
      //cout<<" unit="<<*unit<<endl;
   }
   if(*unit != 1.0) anyObj = ((QdObject_var)unit)*any_obj_in;
   
   //pick up the RHS of the SI_conversion and put it into rbuf 
   if( (tmpbuf = strtok(NULL, " ")) != NULL){
       rbuf = new char[strlen(tmpbuf)+1]; 
       strcpy(rbuf,tmpbuf);
   } else {
    rbuf = new char[strlen(SItxt)+1];
    strcpy(rbuf,SItxt); 
   }
   //cout<<"SIconvr="<<SIconvr<<" rbuf="<<rbuf<<" buf="<<*buf<<endl; 
   QdString_var newSI = new QdString("1.0 >(");
   QdString_var funct = new QdString("");
   // char* func =  new char[strlen(SIconvr)+11];
   // strcat(func, "1 > ");   cout<<"func= "<<func<<endl;       
   
   //----now test rbuf againt user input ---------------- 
   if(deg_or_rad_id == 0 || deg_or_rad_id == 1){
      if(obj_fct_id > 3 ){
        QplugAppendTextDisplay(
	"Error>> output in degree/rad for non-hyperbolic functions only.\n");
	return QPLUG_FAILURE;	  
      }
   }
   if(deg_or_rad_id == 2 && obj_fct_id < 4 ){
      QplugAppendTextDisplay(
	"Error>> must specify degree or rad for non-hyperbolic functions.\n");
	return QPLUG_FAILURE;	  
   }

 try{ 
   switch(obj_fct_id){
      case 0: 
       tmpObj = abs(anyObj);
       if(tmpObj > (QdObject_var)one) {
	  QplugAppendTextDisplay("Error>> Cannot compute acos of |data|>1.\n");
	  return QPLUG_FAILURE; 
        }       
        outObj = QuCleanObject(acos(anyObj));
	*funct += "acos("; 	
	break;
      case 1:  
       tmpObj = abs(anyObj);
       if(tmpObj > (QdObject_var)one) {
	  QplugAppendTextDisplay("Error>> Cannot compute asin of |data|>1.\n");
	  return QPLUG_FAILURE; 
        }             
         outObj = QuCleanObject(asin(anyObj));
	*funct += "sin(";	
	break;
      case 2:  
	*funct += "atan(";
	outObj = QuCleanObject(atan(anyObj));
	break;
      case 3:   
	*funct += "acot(";
	outObj = QuCleanObject(atan((QdObject_var)one/anyObj));	      
	break;
      case 4:
       if(anyObj < (QdObject_var)one){
	  QplugAppendTextDisplay("Error>> Cannot compute acosh of data<1.\n");
	  return QPLUG_FAILURE; 
        }             
        outObj = QuCleanObject(log(anyObj + sqrt(anyObj*anyObj - (QdObject_var)one))); 
	*funct += "acosh(";
        break;
      case 5: 
        outObj = QuCleanObject(log(anyObj + sqrt(anyObj*anyObj + (QdObject_var)one)));	
	*funct += "asinh(";	       
	break;	
      case 6:  
       tmpObj = abs(anyObj); 
       if(tmpObj >= (QdObject_var)one){     
	  QplugAppendTextDisplay("Error>> Cannot compute atanh of |data|>= 1.\n");
	  return QPLUG_FAILURE; 
        }                   
	 outObj = QuCleanObject((QdObject_var)half*log(((QdObject_var)one + anyObj)/((QdObject_var)one - anyObj)));
	 *funct += "atanh(";
        break;
      case 7:           
	 outObj = QuCleanObject((QdObject_var)half*log(((QdObject_var)one + anyObj)/(anyObj - (QdObject_var)one)));
	 *funct += "acoth(";
	break;				
      default:         
        QplugAppendTextDisplay("Error>> Please choose from acos,asin,...\n");
        return QPLUG_FAILURE;
     } 
   } catch(Exception &e){
	e.print_msg();
	QplugAppendTextDisplay("Error>> Nil Exception...\n");
	return QPLUG_FAILURE;
   }    
   *newSI += *funct;
   *newSI += rbuf;
   *newSI += "))";
   if( outObj.is_nil() || is_empty(outObj) ){
       QplugAppendTextDisplay(
       "Error>> Unvalid input, nil or empty object\n");
       return QPLUG_FAILURE;
   } 
   if(deg_or_rad_id == 0)//then transform to output to degree
      outObj = ((QdObject_var)RadToDeg)*outObj;
     
     char* newSI_s_rep = str_dup(*newSI, newSI->length());
     QuSetTxtAttr("SI_conversion", newSI_s_rep, outObj);

     char* unit_txt= QuGetAttrText("UNITS",any_obj_in);
     QdString_var unit_s = new QdString(*funct);
     *unit_s += unit_txt;
     *unit_s += ")";
     char* unit_s_rep = str_dup(*unit_s, unit_s->length());
     QuSetTxtAttr("UNITS", unit_s_rep, outObj);
     
     char* labl_txt = QuGetAttrText("LABLAXIS",any_obj_in);
     QdString_var labl_s = new QdString(*funct);
     *labl_s += labl_txt;
     *labl_s += ")";
     char* labl_s_rep = str_dup(*labl_s, labl_s->length());
     QuSetTxtAttr("LABLAXIS", labl_s_rep, outObj);
     QuSetTxtAttr(FIELDNAM, QuGetAttrText("FIELDNAM",any_obj_in), outObj);
     char* fillval = QuGetAttrText("FILLVAL",any_obj_in);
     QuSetTxtAttr("FILLVAL", fillval, outObj);     
     QuSetTxtAttr(SCALEMAX, QuGetAttrText("SCALEMAX",any_obj_in), outObj);
     QuSetTxtAttr(SCALEMIN, QuGetAttrText("SCALEMIN",any_obj_in), outObj);
     QuSetTxtAttr(SCALETYP, QuGetAttrText("SCALETYP",any_obj_in), outObj);
              
     //add time if necessary
     QdTimeSeq_var tt = get_timetags(any_obj_in );
     if (!tt.is_nil()) set_timetags(outObj, tt);
          
     itrig_list->push_back(outObj);  
         		
   return QPLUG_SUCCESS;

}
/************************************************************/
char* str_dup(string& s, int n){
	char* buf = new char[n+1];
	for(int i=0; i<n; i++)
		buf[i] = s[i];
	buf[n] = '\0';
	return buf;
 }
 /***********************************************************/


