/*****************************************************************
* PROJECT:  Cluster UK CDHF
* COMPONENT:  Plug In library
* MODULE:  TrigoFunct
* LANGUAGE:  C++
* FUNCTION: QSAS TrigoFunct  Plug In
* PURPOSE: .
* 	
* ARGUMENTS:  
*    IN: ANY DATA OBJECT
*        integer: function identifier
*	 integer: deg/rad
*    OUT: 
*	ANY DATA OBJECT 
*
* RETURN: QplugReturnStatus
*
* History: version 1.0 Created Jul. 2002. 
* 	   version 1.1 July 2002.
* A. Serroukh, 
******************************************************************/

#include<cstdio>
#include<cmath>
#include<string>
#include<cstdlib>
#include<fstream>

#include "qar.h"
#include "qplug_if.h"
#include "qdutil.h"
#include "Qdos.h"
#include "Xrefs.h"

extern "C" QplugReturnStatus TrigoFunction(QplugArgList *);

using namespace QSAS;

char* str_dup(string& , int );

/*****************************************************************/
QplugReturnStatus TrigoFunction(QplugArgList *trig_list){

   QdObject_var  any_obj_in = (* trig_list)[0];
   QdObject_var  obj_fct_in = (* trig_list)[1]; 
   QdObject_var  deg_rad_in= (* trig_list)[2];
     
   const double PI = 3.14159265358979323846;
          
   char *rad_yes = "rad", *deg_yes = "degree"; 
   const int   rad_id = 1, deg_id = 0, default_id = 2; 
   char* win_msg = new char[100];
    
   QdRScalar_var unit = new QdRScalar(1.0);  
   //QdObject_var unitObj = new QdRScalar(1.0);
   QdObject_var DegToRadObj = new QdRScalar(PI/180.0);
   QdObject_var  outObj; 
   QdObject_var tmpObj = any_obj_in , anyObj = any_obj_in;
   
   //-----unpack and test the narrow operations ------------   
   if( any_obj_in.is_nil() ){
     QplugAppendTextDisplay("Error>> input neither const nor sequence.\n");
     return QPLUG_FAILURE;
   } 
               
   QdRScalar_var obj_fct = QdRScalar_var::narrow(obj_fct_in);
   //int deg_rad_id = (int) *QdScalar_var::narrow(fct_in);
   if(obj_fct.is_nil()){
      QplugAppendTextDisplay("Error>> undefined function\n");
      return QPLUG_FAILURE;
   }       
   int obj_fct_id = (int)(* obj_fct);

   QdRScalar_var deg_rad1 = QdRScalar_var::narrow(deg_rad_in);
   if(deg_rad1.is_nil()){
      QplugAppendTextDisplay("Error>> Deg/Rad unspecified.\n");
      return QPLUG_FAILURE;
   }        
   int deg_rad_id_in = (int)(*deg_rad1);   
   
   //-----check and read what's in the SI_conversion ----------  
   static char* no_unit = "1 > ()";
   QdString_var SItxtvar = 
   		new QdString(QuGetAttrText("SI_conversion",any_obj_in));
   char* SItxt = str_dup(*SItxtvar, SItxtvar->length());
   char *SIconvr, *rbuf, *tmpbuf;
   if(strcmp(SItxt,"") == 0){
      SIconvr = new char[strlen(no_unit)+1];
      strcpy(SIconvr,no_unit);
      sprintf(win_msg, "Data SI_conversion: %s \n", SIconvr);
   } else {
      SIconvr = new char[strlen(SItxt)+1];
      strcpy(SIconvr,SItxt);
      sprintf(win_msg, "Data SI_conversion: %s \n", SIconvr);
   }   
   QplugAppendTextDisplay(win_msg);
   //cout<<"SItxt= "<<SItxt<<" SIconvr= "<<SIconvr<<endl;
   
   if( (tmpbuf = strtok(SIconvr, ">")) != NULL) {
      unit = new QdRScalar(atof(tmpbuf));
      //cout<<" unit="<<*unit<<endl;
   } 
   //pick up the RHS of the SI_conversion and put it into rbuf 
   if( (tmpbuf = strtok(NULL, " ")) != NULL){
       rbuf = new char[strlen(tmpbuf)+1]; 
       strcpy(rbuf,tmpbuf);
   } else {
    rbuf = new char[strlen(SItxt)+1];
    strcpy(rbuf,SItxt); 
   }
   //cout<<" rbuf="<<rbuf<<endl;
   //----now test rbuf againt user input ----------------   
   
  //if deg_rad_id_in is degree or rad than ignore SI_conversion
  switch(deg_rad_id_in){
    case deg_id:           
      //if(deg_rad_id_in == deg_id){        
       if(strcmp(rbuf,deg_yes) != 0){ 
         QplugAppendTextDisplay(
	  "WARNING>> Ignoring data SI_conversion.\n");    
       } else {//if there's a match, then get the right unit
         if(*unit != 1.0) tmpObj  = ((QdObject_var)unit)*any_obj_in;
       }
       anyObj = DegToRadObj*tmpObj;//transform to rad  
       break;
   case rad_id:    
       //} else if(deg_rad_id_in == rad_id)
       if(strcmp(rbuf,rad_yes) != 0){           
          QplugAppendTextDisplay(
	  "WARNING>> Ignoring data SI_conversion.\n");
       } else {
         if(*unit != 1.0) anyObj = ((QdObject_var)unit)*any_obj_in;
       }
       break;
   case default_id:
       if(strcmp(rbuf,rad_yes) == 0 ){
	   if(*unit != 1.0) anyObj  = ((QdObject_var)unit)*any_obj_in;  
       }
       else if(strcmp(rbuf,deg_yes) == 0){
	   if(*unit != 1.0) tmpObj  = ((QdObject_var)unit)*any_obj_in;
	   anyObj = DegToRadObj*tmpObj;//transform to rad  
       }
       else if( obj_fct_id > 3){//if hyperbolic function
          if(*unit != 1.0) anyObj  = ((QdObject_var)unit)*any_obj_in;
       } else {	
         QplugAppendTextDisplay(
         "Error>> Input is neither in rad nor in degree. Please specify\n");
         return QPLUG_FAILURE;	  
       }  
       break;    
   default:
       QplugAppendTextDisplay(
	 "WARNING>> Input is neither in rad nor in degree. Please specify\n");
	  return QPLUG_FAILURE;
       break;
  } 
   //cout<<"SIconvr="<<SIconvr<<" rbuf="<<rbuf<<endl; 
   QdString_var newSI = new QdString("1.0 >(");
   QdString_var funct = new QdString("");
   // char* func =  new char[strlen(SIconvr)+11];
   // strcat(func, "1 > ");   cout<<"func= "<<func<<endl;               	 
 try{ 
   switch(obj_fct_id){
      case 0:           
        outObj = cos(anyObj);
	*funct += "cos("; 	
	break;
      case 1:  
         outObj = sin(anyObj);
	*funct += "sin(";	
	break;
      case 2:  
	*funct += "tan(";
	outObj = QuCleanObject(sin(anyObj)/cos(anyObj));
	break;
      case 3:   
	*funct += "cot(";
	outObj = QuCleanObject(cos(anyObj)/sin(anyObj));	      
	break;
      case 4:  
        outObj = cosh(anyObj); 
	*funct += "cosh(";
        break;
      case 5: 
        outObj = sinh(anyObj);	
	*funct += "sinh(";	       
	break;	
      case 6:  
	 *funct += "tanh(";
	 outObj = QuCleanObject(sinh(anyObj)/cosh(anyObj));
        break;
      case 7:           
	 *funct += "coth(";
	 outObj = QuCleanObject(cosh(anyObj)/sinh(anyObj));
	break;				
      default:         
        QplugAppendTextDisplay("Error>> Please choose from sin, cos,...\n");
        return QPLUG_FAILURE;
     } 
   } catch(Exception &e){
	e.print_msg();
	QplugAppendTextDisplay("Error>> Nil Exception...\n");
	return QPLUG_FAILURE;
   }    
   *newSI += *funct;
   *newSI += rbuf;
   *newSI += "))";
    if( outObj.is_nil() || is_empty(outObj) ){
       QplugAppendTextDisplay(
       "Error>> unvalid input to the trigonometry function.\n");
       QplugAppendTextDisplay("Error>> outTrigo is nil or empty object. \n");
       return QPLUG_FAILURE;
     } 


     char* newSI_s_rep = str_dup(*newSI, newSI->length());
     QuSetTxtAttr("SI_conversion", newSI_s_rep, outObj);

     char* unit_txt= QuGetAttrText("UNITS",any_obj_in);
     QdString_var unit_s = new QdString(*funct);
     *unit_s += unit_txt;
     *unit_s += ")";
     char* unit_s_rep = str_dup(*unit_s, unit_s->length());
     QuSetTxtAttr("UNITS", unit_s_rep, outObj);
     
     char* labl_txt = QuGetAttrText("LABLAXIS",any_obj_in);
     QdString_var labl_s = new QdString(*funct);
     *labl_s += labl_txt;
     *labl_s += ")";
     char* labl_s_rep = str_dup(*labl_s, labl_s->length());
     QuSetTxtAttr("LABLAXIS", labl_s_rep, outObj);
     QuSetTxtAttr(FIELDNAM, QuGetAttrText("FIELDNAM",any_obj_in), outObj);
     char* fillval = QuGetAttrText("FILLVAL",any_obj_in);
     QuSetTxtAttr("FILLVAL", fillval, outObj);     
     QuSetTxtAttr(SCALEMAX, QuGetAttrText("SCALEMAX",any_obj_in), outObj);
     QuSetTxtAttr(SCALEMIN, QuGetAttrText("SCALEMIN",any_obj_in), outObj);
     QuSetTxtAttr(SCALETYP, QuGetAttrText("SCALETYP",any_obj_in), outObj);
     
     
     
     //add time if necessary
     QdTimeSeq_var tt = get_timetags(any_obj_in);
     if (!tt.is_nil()) set_timetags(outObj, tt);
          
     trig_list->push_back(outObj);  
      
   delete[] win_msg;
   delete[] SIconvr;
   delete[] rbuf;
   		
   return QPLUG_SUCCESS;

}
/************************************************************/
char* str_dup(string& s, int n){
	char* buf = new char[n+1];
	for(int i=0; i<n; i++)
		buf[i] = s[i];
	buf[n] = '\0';
	return buf;
 }
 /***********************************************************/
