#include "QbEditors.h"

#include "qdutil.h"
#include "TimeUtils.h"

#if QT_VERSION < 0x040000
#include "QbTimeWidgets.h"
#else
#include "QbTimeWidgets_qt4.h"
#endif

#if QT_VERSION < 0x040000
QbEditor::QbEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QVGroupBox(parent,name)
{
  this->setPalette(qApp->palette());
 QFontMetrics qbu_font_mt(qApp->font());
 int char_width = qbu_font_mt.width("n");

  valueBox = new QHBox(this, "valueBox");

  valueLabel = new QLabel("  Value ", valueBox, "valueLabel");
  valueField = new QguWLinput(valueBox,"value");
  valueField->setMinimumWidth(char_width*23);
  valueField->setEnabled(true);
  
 // set in specific instances valueField->acceptNumber = true;
  
  timeBox = new QHBox(this, "timeBox");
  QLabel * timeLabel = new QLabel("at Time ", timeBox, "timeLabel");

#if QT_VERSION >= 0x040000 && QT_VERSION < 0x040300
  timeField = new QbDateTimeL(timeBox);
#else
  timeField = new QbDateTimeWidget(timeBox);
#endif
  //timeField->setMinimumWidth(char_width*23);

  timeField->setEnabled(true);
  
  maxRec = 0;
  record = 0;
}
#else
QbEditor::QbEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
	: QGroupBox(parent,name)
{
	this->setPalette(qApp->palette());
	QFontMetrics qbu_font_mt(qApp->font());
	int char_width = qbu_font_mt.width("n");

	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	valueBox = new QWidget(this);
	QHBoxLayout * valueLayout=new QHBoxLayout;
	valueLayout->setSpacing(0);
#if QT_VERSION < 0x040300
	valueLayout->setMargin(0);
#else
	valueLayout->setContentsMargins(0,0,0,0);
#endif

	valueLabel = new QLabel("  Value ");
	valueLayout->addWidget(valueLabel);
	
	valueField = new QguWLinput(valueBox,"value");
	valueField->setMinimumWidth(char_width*23);
	valueField->setEnabled(true);
	valueLayout->addWidget(valueField);
	
	valueBox->setLayout(valueLayout);
// 	valueBox->setMaximumHeight(30);
	mainLayout->addWidget(valueBox);
  
 // set in specific instances valueField->acceptNumber = true;
  
	timeBox = new QWidget(this);
	QHBoxLayout * timeLayout=new QHBoxLayout;
	timeLayout->setSpacing(0);
#if QT_VERSION < 0x040300
	timeLayout->setMargin(0);
#else
	timeLayout->setContentsMargins(0,0,0,0);
#endif

	QLabel * timeLabel = new QLabel("at Time ");
	timeLayout->addWidget(timeLabel, Qt::AlignCenter);

#if QT_VERSION >= 0x040000 && QT_VERSION < 0x040300
	timeField = new QbDateTimeLineEdit(timeBox);
#else
	timeField = new QbDateTimeWidget(timeBox);
#endif
	//timeField->setMinimumWidth(char_width*23);
	timeField->setEnabled(true);
	timeLayout->addWidget(timeField);
	
	timeBox->setLayout(timeLayout);
// 	timeBox->setMaximumHeight(30);
	mainLayout->addWidget(timeBox);
  
// 	mainLayout->addStretch(1);
	
	this->setLayout(mainLayout);
	
	maxRec = 0;
	record = 0;
}
#endif

QbEditor::~QbEditor(){
}

//defined in QbTimeIntervalEditors.cc for dependencies reasons
// void QbEditor::updateForRec(){
// 
//    if(record > maxRec) return;
// 
//    if (editorType == "QbScalarSeqEditor") ((QbScalarSeqEditor *)this)->updateForRec();
//    else if (editorType == "QbMatrixSeqEditor") ((QbMatrixSeqEditor *)this)->updateForRec();
//    else if (editorType == "QbTextSeqEditor") ((QbTextSeqEditor *)this)->updateForRec();
//    else if (editorType == "QbTtagEditor") ((QbTtagEditor *)this)->updateForRec();
//    else if (editorType == "QbTimeIntervalSeqEditor") ((QbTimeIntervalSeqEditor *)this)->updateForRec();
// 
// }

void QbEditor::setData(QdObject_var dobj){
   // generic failsafe method

   if( dobj.is_nil() ) return;
}

void QbEditor::gotInput(){
   emit saveNow();
}

void QbEditor::showValueField(bool OKtoShow){
   
   if ( OKtoShow ){
     valueBox->show();
   }
   else{
     valueBox->hide();
   }
   
   return;
}

void QbEditor::showTimeField(bool OKtoShow){
   
   if ( OKtoShow ){
     timeBox->show();
   }
   else{
     timeBox->hide();
   }
   
   return;
}



//-----------------------------------------------------------------------
QbScalarEditor::QbScalarEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Constant Scalar");
  
  valueField->acceptNumber = true;
  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  setData(dobj);
  editorType = "QbScalarEditor";

}


void QbScalarEditor::setData(QdObject_var dobj){
   QString txt;
   if( dobj.is_nil() ) return;
   QdRScalar_var Snarrow = QdRScalar_var::narrow(dobj);
   
   if( !Snarrow.is_nil() ){
     maxRec = 0;
     record = 0;
     S_var = Factory::clone(Snarrow);
     double value = (double) *S_var;
     txt.setNum(value);
     valueField->setText(txt);
     S_var->copy_xrefs_from(dobj);
          
	 QdTimeSeq_var tt_orig = get_timetags(S_var);
     
     if ( tt_orig.is_nil() ){
       showTimeField(false);
     }
     else{
       showTimeField(true);
       ttags = Factory::clone(tt_orig); // ensure we don't edit tags shared with another object
       ttags->copy_xrefs_from(tt_orig);
	   if( ! ttags.is_nil() ) S_var->change_xref(TIME_TAGS, ttags);
       if ( ttags->sequence_size() == 1 ) 
       {
	       timeField->setTime(ttags[0]);
       }

       else
          QuMsgBar("Wrong number of Time tags", true);
     }
   }
   else
     QuMsgBar("Data object not recognised", true);

#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
       
}


void QbScalarEditor::valueChanged(){

  if ( !valueField->edited() ) return;
  
  bool OK;
  double val;
  
  if(QuSlotDebug())
  {
	printf("QbScalarEditor::valueChanged %s\n", valueField->text().latin1());
	fflush(stdout);
  }
 
  val = valueField->text().toDouble(&OK);
  if (OK){
     *S_var = val;
     edited_var = S_var;
     emit editorHasInput();
  }
}

void QbScalarEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
 
  QDOS::Time tag;
  tag=timeField->time();
      if(!ttags.is_nil() && ttags->sequence_size() > 0) {
        ttags[0] = tag;
        emit editorHasTimeInput();
      }

}

//-----------------------------------------------------------------------

QbScalarSeqEditor::QbScalarSeqEditor(QdObject_var dobj, QWidget *parent, const char *name) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Scalar Sequence");
  
  valueField->acceptNumber = true;
  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );

  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  setData(dobj);
  editorType = "QbScalarSeqEditor";

}

void QbScalarSeqEditor::updateForRec(){
   QString txt;
          
     double value = (double) Sseq_var[record];
     txt.setNum(value);
     valueField->setText(txt);
          
     if ( ttags.is_nil() ){
       showTimeField(false);
     }
     else{ 
       showTimeField(true);
       if ( ttags->sequence_size() > record ) 
       {
	       timeField->setTime(ttags[record]);
       }
     }     
}


void QbScalarSeqEditor::setData(QdObject_var dobj){
   QString txt;

   if( dobj.is_nil() ) return;
   QdRScalarSeq_var Sseqnarrow = QdRScalarSeq_var::narrow(dobj);
   
   if( !Sseqnarrow.is_nil() ){
     maxRec = Sseqnarrow->sequence_size() - 1;
     record = 0;
     Sseq_var = Factory::clone(Sseqnarrow);
     double value = (double) Sseq_var[record];
     txt.setNum(value);
     valueField->setText(txt);
     Sseq_var->copy_xrefs_from(dobj);
          
     QdTimeSeq_var tt_orig = get_timetags(Sseq_var);
          
     if ( tt_orig.is_nil() ){
       showTimeField(false);
     }
     else{
       showTimeField(true);
	   ttags = Factory::clone(tt_orig); // ensure we don't edit tags shared with another object
       ttags->copy_xrefs_from(tt_orig);
	   if( ! ttags.is_nil() ) Sseq_var->change_xref(TIME_TAGS, ttags);
       if ( ttags->sequence_size() == dobj->sequence_size() )
       {
      	 timeField->setTime(ttags[record]);
       }

       else
          QuMsgBar("Wrong number of Time tags", true);
     }
   }
   else
     QuMsgBar("Data object not recognised", true);
   
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
   
}

void QbScalarSeqEditor::valueChanged(){

  if ( !valueField->edited() ) return;

  bool OK;
  double val;
  
  if(QuSlotDebug())
  {
	printf("QbScalarSeqEditor::valueChanged %s\n", valueField->text().latin1());
	fflush(stdout);
  }

  val = valueField->text().toDouble(&OK);
  if (OK){
     Sseq_var[record] = val;
     edited_var = Sseq_var;
     emit editorHasInput();
  }
}

void QbScalarSeqEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
  
  QDOS::Time tag;

  tag=timeField->time();
      if(!ttags.is_nil() && ttags->sequence_size() > 0) {
        ttags[record] = tag;
        emit editorHasTimeInput();
      }


}

//-----------------------------------------------------------------------

QbTtagEditor::QbTtagEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Time Tags");
  
  valueField->acceptTime = true;
  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  setData(dobj);
  editorType = "QbTtagEditor";

}

void QbTtagEditor::updateForRec(){
   QString txt;
     
     valueField->setText(Ttag_var[record].iso().c_str());
               
}


void QbTtagEditor::setData(QdObject_var dobj){

   if( dobj.is_nil() ) return;
   QdTimeSeq_var TTnarrow = QdTimeSeq_var::narrow(dobj);
   
   if( !TTnarrow.is_nil() ){
     maxRec = TTnarrow->sequence_size() - 1;
     record = 0;
     Ttag_var = Factory::clone(TTnarrow);
     valueField->setText(Ttag_var[record].iso().c_str());
     Ttag_var->copy_xrefs_from(dobj);
     
     showTimeField(false);
   }
   else
     QuMsgBar("Data object not recognised", true);
   
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
}

void QbTtagEditor::valueChanged(){

  if ( !valueField->edited() ) return;

  QDOS::Time tag;

   //if( valid_iso_time_str( valueField->text().latin1(), tag ) ){
      if(!Ttag_var.is_nil() && Ttag_var->sequence_size() > 0) {
        Ttag_var[record] = tag;
        edited_var = Ttag_var;
        emit editorHasInput();
      }
   //}
   
}

//-----------------------------------------------------------------------

QbTimeObjEditor::QbTimeObjEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Time Object");
  
  valueField->acceptTime = true;
  valueField->acceptTimeObject = true;
  valueField->setMinimumWidth(250);
  valueField->adjustSize();
  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  connect(valueField, SIGNAL(TimeChosen()), this, SLOT(setValue())  );
  setData(dobj);
  editorType = "QbTimeObjEditor";

}

void QbTimeObjEditor::setData(QdObject_var dobj){

   if( dobj.is_nil() ) return;
   QdTime_var Tnarrow = QdTime_var::narrow(dobj);
   
   if( !Tnarrow.is_nil() ){
     T_var = Factory::clone(Tnarrow);
     valueField->setText(T_var->iso().c_str());
     
     showTimeField(false);
   }
   else
     QuMsgBar("Data object not recognised", true);
   
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
}

void QbTimeObjEditor::valueChanged(){

  if ( !valueField->edited() ) return;

  QDOS::Time tag;

   //if( valid_iso_time_str( valueField->text().latin1(), tag ) ){
        T_var = new QdTime (tag);
        edited_var = T_var;
        emit editorHasInput();
   //}
   
}

void QbTimeObjEditor::setValue(){

   QdTime_var tobj = QdTime_var::narrow(valueField->getSlotObject());
   if( !tobj.is_nil() ){

        T_var = Factory::clone(tobj);
        valueField->setText(T_var->iso().c_str());
        edited_var = T_var;
        emit editorHasInput();
   }
   
}

//-----------------------------------------------------------------------

QbMatrixEditor::QbMatrixEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Constant Matrix");
  
  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  editorType = "QbMatrixEditor";
  
  QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
  QdSMatrix_var SMnarrow = QdSMatrix_var::narrow(dobj);
   
  if(!Mnarrow.is_nil())
  {
    setTitle("Constant Numeric Matrix");
    valueField->acceptNumber = true;
    connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  ); 
  }
  else if(!SMnarrow.is_nil())
  {  
    setTitle("Constant String Matrix");
    valueField->acceptText = true;
    connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  }
  
  setData(dobj);
}


void QbMatrixEditor::setData(QdObject_var dobj){
   QString txt;

   if( dobj.is_nil() ) return;
   
   QdTimeSeq_var tt_orig = get_timetags(dobj);
     
   if ( tt_orig.is_nil() ){
     showTimeField(false);
   }
   else{
     showTimeField(true);
     ttags = Factory::clone(tt_orig);
	 ttags->copy_xrefs_from(tt_orig);
     if ( ttags->sequence_size() == 1 )
     {
	     timeField->setTime(ttags[0]);
     }

     else
        QuMsgBar("Wrong number of Time tags", true);
   }

   QdRMatrix_var Mnarrow = QdRMatrix_var::narrow(dobj);
   QdSMatrix_var SMnarrow = QdSMatrix_var::narrow(dobj);
  
   if( !Mnarrow.is_nil() ){
     dimSize = (Mnarrow->dims()).size();
     index.clear();
     for (int i=0; i<dimSize; i++){
       int maxN = (Mnarrow->dims())[i];
       int nextInd = 0;
       index.push_back(nextInd);
       argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
       connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int, int)) );
#if QT_VERSION >=0x040000
       layout()->addWidget(argS);
#endif
     }

     M_var = Factory::clone(Mnarrow);
     double value = (double) M_var->matrix_indexelt(index);
     txt.setNum(value);
     valueField->setText(txt);
     M_var->copy_xrefs_from(dobj); 
	 if( ! ttags.is_nil() ) M_var->change_xref(TIME_TAGS, ttags);

   }
   else if( !SMnarrow.is_nil() ){
     dimSize = (SMnarrow->dims()).size();
     index.clear();
     for (int i=0; i<dimSize; i++){
       int maxN = (SMnarrow->dims())[i];
       int nextInd = 0;
       index.push_back(nextInd);
       argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
       connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int, int)) );
#if QT_VERSION >=0x040000
       layout()->addWidget(argS);
	((QHBoxLayout*)layout())->addStretch(1);
#endif
     }

     SM_var = Factory::clone(SMnarrow);
     string txtValue = SM_var->matrix_indexelt(index);
     valueField->setText(txtValue.c_str());
     SM_var->copy_xrefs_from(dobj);
	 if( ! ttags.is_nil() ) SM_var->change_xref(TIME_TAGS, ttags);
   } 
   else
   {
     QuMsgBar("Data object not recognised", true);
     return;
   }
   
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
  
}


void QbMatrixEditor::valueChanged(){

  if ( !valueField->edited() ) return;

  if(!M_var.is_nil())
  {
    bool OK;
    double val;
   
   if(QuSlotDebug())
   {
	printf("QbMatrixEditor::valueChanged %s\n", valueField->text().latin1());
	fflush(stdout);
	}
 
    val = valueField->text().toDouble(&OK);
    if (OK){
       M_var->matrix_indexelt(index) = val;
       edited_var = M_var;
       emit editorHasInput();
    }
  }
  else if(!SM_var.is_nil())
  {
    string val=valueField->text().latin1();
    SM_var->matrix_indexelt(index) = val;
    edited_var = SM_var;
    emit editorHasInput();
  }  
}

void QbMatrixEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
 
 QDOS::Time tag;
  tag=timeField->time();
      if(!ttags.is_nil() && ttags->sequence_size() > 0) {
        ttags[0] = tag;
        emit editorHasTimeInput();
      }
}

void QbMatrixEditor::changeElement(int id, int idx){
   QString txt;
   
   index[id] = idx;
   
   if( !M_var.is_nil() ){

     double value = (double) M_var->matrix_indexelt(index);  
     txt.setNum(value);
     valueField->setText(txt);
   }
   else if( !SM_var.is_nil() ){

     string textValue = SM_var->matrix_indexelt(index);  
     valueField->setText(textValue.c_str());
   }
   else
   {
     QuMsgBar("Data object not recognised", true);
     return;
   }
   
   if ( ttags.is_nil() ){
     showTimeField(false);
   }
   else{
     showTimeField(true);
     if ( ttags->sequence_size() == 1 )
     {
	     timeField->setTime( ttags[0] );
     }
     else
        QuMsgBar("Wrong number of Time tags", true);
   }   
}


//-----------------------------------------------------------------------

QbMatrixSeqEditor::QbMatrixSeqEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{

  setTitle("Sequence of Arrays");
  QdRMatrixSeq_var Mseqnarrow = QdRMatrixSeq_var::narrow(dobj);
  QdSMatrixSeq_var SMseqnarrow = QdSMatrixSeq_var::narrow(dobj);
  
  if(!Mseqnarrow.is_nil())
  {
    valueField->acceptNumber = true;
    connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  }
  else if(!SMseqnarrow.is_nil())
  {
    valueField->acceptText=true;
    connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  }
    
  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  editorType = "QbMatrixSeqEditor";
  
  setData(dobj);

}

void QbMatrixSeqEditor::updateForRec(){

 if(QuSlotDebug())
 {
	printf("QbMatrixSeqEditor::updateForRec %ld \n", this);
	fflush(stdout);
 }

   QString txt;
   if(!Mseq_var.is_nil())
   {
     double value = (double) Mseq_var[record].matrix_indexelt(index);  
     txt.setNum(value);
     valueField->setText(txt);
   }
   else if(!SMseq_var.is_nil())
   {
     string textValue=SMseq_var[record].matrix_indexelt(index);  
     valueField->setText(textValue.c_str());
   }
        
   if ( ttags.is_nil() ){
     showTimeField(false);
   }
   else{
     showTimeField(true);
     if ( ttags->sequence_size() > record )
     {
	     timeField->setTime( ttags[record] );
     }
   }     
}


void QbMatrixSeqEditor::setData(QdObject_var dobj){
   QString txt;

   if( dobj.is_nil() ) return;
   
    QdTimeSeq_var tt_orig = get_timetags(dobj);
     
   if ( tt_orig.is_nil() ){
     showTimeField(false);
   }
   else{
     showTimeField(true);
     ttags = Factory::clone(tt_orig);
	 ttags->copy_xrefs_from(tt_orig);

     if ( ttags->sequence_size() == dobj->sequence_size() )
     {
	     timeField->setTime( tt_orig[record] );
     }

     else
        QuMsgBar("Wrong number of Time tags", true);
   }


   QdRMatrixSeq_var Mseqnarrow = QdRMatrixSeq_var::narrow(dobj);
   QdSMatrixSeq_var SMseqnarrow = QdSMatrixSeq_var::narrow(dobj);
        
   if( !Mseqnarrow.is_nil() ){
    
     dimSize = (Mseqnarrow[0].dims()).size();
     index.clear();
    for (int i=0; i<dimSize; i++){
       int maxN = (Mseqnarrow[0].dims())[i];
       int nextInd = 0;
       index.push_back(nextInd);
       argSlider *argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
       connect(argS, SIGNAL(QbEchanged(int, int)), this, SLOT(changeElement(int,int)) );
#if QT_VERSION >=0x040000
       layout()->addWidget(argS);
#endif
     }    
     
     maxRec = Mseqnarrow->sequence_size() - 1;
     record = 0;
     Mseq_var = Factory::clone(Mseqnarrow);
     double value = (double) Mseq_var[record].matrix_indexelt(index);  
     txt.setNum(value);
     valueField->setText(txt);
     Mseq_var->copy_xrefs_from(dobj);
	 if( ! ttags.is_nil() ) Mseq_var->change_xref(TIME_TAGS, ttags);
   }
    else if( !SMseqnarrow.is_nil() ){
  
     dimSize = (SMseqnarrow[0].dims()).size();
     index.clear();
     for (int i=0; i<dimSize; i++){
       int maxN = (SMseqnarrow[0].dims())[i];
       int nextInd = 0;
       index.push_back(nextInd);
       argSlider * argS = new argSlider(tr("index %1").arg(i+1), i, maxN, this);
       connect(argS, SIGNAL(QbEchanged(int,int)), this, SLOT(changeElement(int,int)) );
#if QT_VERSION >=0x040000
       layout()->addWidget(argS);
#endif
     }
    

     maxRec = SMseqnarrow->sequence_size() - 1;
     record = 0;
     SMseq_var = Factory::clone(SMseqnarrow);
     string textValue = SMseq_var[record].matrix_indexelt(index);  
     valueField->setText(textValue.c_str());
     SMseq_var->copy_xrefs_from(dobj);
	 if( ! ttags.is_nil() ) SMseq_var->change_xref(TIME_TAGS, ttags);
   }
   else
   {
     QuMsgBar("Data object not recognised", true);
     return;
   }
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif
}

void QbMatrixSeqEditor::changeElement(int id, int idx){
   QString txt;
      
   index[id] = idx;
   
   if( !Mseq_var.is_nil() ){
          
     double value = (double) Mseq_var[record].matrix_indexelt(index);  
     txt.setNum(value);
     valueField->setText(txt);
   }
   else if( !SMseq_var.is_nil() ){

     string textValue = SMseq_var[record].matrix_indexelt(index);  
     valueField->setText(textValue.c_str());
   }       
   else
   {
     QuMsgBar("Data object not recognised", true);
     return;
   }
   
   if ( ttags.is_nil() ){
     showTimeField(false);
   }
   else{
     showTimeField(true);
     if ( ttags->sequence_size() == Mseq_var->sequence_size() )
     {
	     timeField->setTime( ttags[record] );
     }
     else
        QuMsgBar("Wrong number of Time tags", true);
   }
}

void QbMatrixSeqEditor::valueChanged(){
  
  if ( !valueField->edited() ) return;
  
   if(QuSlotDebug())
   {
	printf("QbMatrixSeqEditor::valueChanged %s\n", valueField->text().latin1());
	fflush(stdout);
	}
  
  if(!Mseq_var.is_nil())
  {
    bool OK;
    double val;
    val = valueField->text().toDouble(&OK);
    if (OK){
       Mseq_var[record].matrix_indexelt(index) = val;  
       edited_var = Mseq_var;
       emit editorHasInput();
    }
  }
  else if(!SMseq_var.is_nil())
  {
     string textValue;
     textValue = valueField->text().latin1();
     SMseq_var[record].matrix_indexelt(index) = textValue;  
     edited_var = SMseq_var;
     emit editorHasInput();
  }
}

void QbMatrixSeqEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
 
  QDOS::Time tag;
  tag=timeField->time();
      if(!ttags.is_nil() && ttags->sequence_size() > 0) {
        ttags[record] = tag;
        // object has TIME_TAGS tt_orig unless tt get changed
        emit editorHasTimeInput();
      }

}


// text editor
//----------------

QbTextEditor::QbTextEditor(QdObject_var dobj, QWidget *parent, const char *name)
  : QbEditor(dobj, parent,name)
{
  setTitle("Text");
  valueLabel->hide();
  valueField->setMinimumWidth(250);
  
  valueField->acceptText = true;

  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );

  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  editorType = "QbTextEditor";
  setData(dobj);

}


void QbTextEditor::setData(QdObject_var dobj){

   if( dobj.is_nil() ) return;
   QdString_var Strnarrow = QdString_var::narrow(dobj);
   
   if( !Strnarrow.is_nil() ){
     maxRec = 0;
     record = 0;
     Str_var = Factory::clone(Strnarrow);
     valueField->setText(QString(Str_var->c_str()));
     
     int txt_len = valueField->text().length() * 7 + 20;     
     if(txt_len < 450) valueField->setMinimumWidth(txt_len);
     else valueField->setMinimumWidth(450);
     valueField->setCursorPosition(0);
     
     Str_var->copy_xrefs_from(dobj);
          
     QdTimeSeq_var tt_orig = get_timetags(dobj);
     
     if ( tt_orig.is_nil() ){
       showTimeField(false);
     }
     else{
       showTimeField(true);
       ttags = Factory::clone(tt_orig);
	   ttags->copy_xrefs_from(tt_orig);
	   if( ! ttags.is_nil() ) Str_var->change_xref(TIME_TAGS, ttags);
       if ( ttags->sequence_size() == 1 )
       {
	       timeField->setTime( ttags[0] );
       }

       else
          QuMsgBar("Wrong number of Time tags", true);
     }
   }
   else
     QuMsgBar("Data object not recognised", true);
        
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif

}

void QbTextEditor::valueChanged(){

  if ( !valueField->edited() ) return;
  
  bool OK;
  QdString_var new_str;
  
  new_str = new QdString(valueField->text().latin1());
  new_str->copy_xrefs_from(Str_var);
  Str_var = new_str;
  edited_var = Str_var;
  emit editorHasInput();
  
}

void QbTextEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
 
  QDOS::Time tag;
  tag=timeField->time();
      if(!ttags.is_nil() && ttags->sequence_size() > 0) {
        ttags[0] = tag;
		emit editorHasTimeInput();
      }

}

// text sequence editor
//----------------

QbTextSeqEditor::QbTextSeqEditor(QdObject_var dobj, QWidget *parent, const char *name ) 
  : QbEditor(dobj, parent,name)
{
  setTitle("Text Sequence");
  valueLabel->hide();
  valueField->setMinimumWidth(300);
  valueField->acceptText = true;

  connect(valueField, SIGNAL(validated_OK()), this, SLOT(valueChanged())  );
  connect(timeField,  SIGNAL(valueChanged()), this, SLOT(timeValueChanged()) );

  editorType = "QbTextSeqEditor";
  setData(dobj);

}


void QbTextSeqEditor::setData(QdObject_var dobj){

   if( dobj.is_nil() ) return;
   QdStringSeq_var Strseqnarrow = QdStringSeq_var::narrow(dobj);
   
   if( !Strseqnarrow.is_nil() ){
     maxRec = Strseqnarrow->sequence_size() - 1;
     record = 0;
     Strseq_var = Factory::clone(Strseqnarrow);
     valueField->setText(QString(Strseq_var[record].c_str()));

     int txt_len = valueField->text().length() * 7 + 20;
     
     if(txt_len < 450) valueField->setMinimumWidth(txt_len);
     else valueField->setMinimumWidth(450);
     valueField->setCursorPosition(0);
     
     Strseq_var->copy_xrefs_from(dobj);
          
     QdTimeSeq_var tt_orig = get_timetags(dobj);
     
     if ( tt_orig.is_nil() ){
       showTimeField(false);
     }
     else{
       showTimeField(true);
       ttags = Factory::clone(tt_orig);
	   ttags->copy_xrefs_from(tt_orig);
	   if( ! ttags.is_nil() ) Strseq_var->change_xref(TIME_TAGS, ttags);

       if ( ttags->sequence_size() == dobj->sequence_size() )
       {
	       timeField->setTime( ttags[record] );
       }

       else
          QuMsgBar("Wrong number of Time tags", true);
     }
   }
   else
     QuMsgBar("Data object not recognised", true);
        
#if QT_VERSION >=0x040000
   ((QHBoxLayout*)layout())->addStretch(1);
#endif

}

void QbTextSeqEditor::updateForRec(){

   QString txt;


     txt = QString(Strseq_var[record].c_str());  
     valueField->setText(txt);

     int txt_len = valueField->text().length() * 7 + 20;     
     if(txt_len < 450) valueField->setMinimumWidth(txt_len);
     else valueField->setMinimumWidth(450);
     valueField->setCursorPosition(0);
                    
     if ( ttags.is_nil() ){
       showTimeField(false);
     }
     else{
       showTimeField(true);
       if ( ttags->sequence_size() > record )
       {
	       timeField->setTime( ttags[record] );
       }
     }     
}


void QbTextSeqEditor::valueChanged(){

  if ( !valueField->edited() ) return;
  
  bool OK;
  QdString_var new_str;
  
  new_str = new QdString(valueField->text().latin1());
  Strseq_var[record] = *new_str;
  edited_var = Strseq_var;
  emit editorHasInput();
  
}

void QbTextSeqEditor::timeValueChanged(){

 if( !timeField->edited() ) return;
 
  QDOS::Time tag;
  tag=timeField->time();
      if(!ttags.is_nil() &&  ttags->sequence_size() > record) {
        ttags[record] = tag;
		emit editorHasTimeInput();
      }


}




// class argSlider

 argSlider::argSlider( QString title, int indx, int maxN, QWidget *parent, char * name)
   : QWidget(parent,name)
{

   maxNum = maxN-1;
   num = 0;
   id = indx;
   
   QGridLayout * layout=new QGridLayout(this);
   layout->setSpacing(0);
   layout->setMargin(0);
   
   QToolButton * leftSlide = new QToolButton( Qt::LeftArrow, this, "leftSlide");
   leftSlide->setFixedSize(18, 20);
   layout->addWidget(leftSlide, 0,0);
   
   numField = new QguWLinput(this, "numField");
   numField->acceptInteger = true;
   numField->setEnabled(true);
   numField->setText("1");
   numField->setFixedSize(40, 20);
   numField->setAlignment(Qt::AlignHCenter);
   layout->addWidget(numField, 0,1);
   
   QToolButton * rightSlide = new QToolButton( Qt::RightArrow, this, "leftSlide");
   rightSlide->setFixedSize(18, 20);
   layout->addWidget(rightSlide, 0,2);
   
   QLabel * Spacer=new QLabel(this);
   Spacer->setFixedSize(18, 20);
   layout->addWidget(Spacer, 0,3);
   
   QLabel *sliderLabel = new QLabel(tr("in range [1,%1]").arg(maxNum+1), this);
   layout->addWidget(sliderLabel, 0, 4);
   
   layout->setColStretch(4, 1);
   
   connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
   connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
   connect( numField,   SIGNAL(validated_OK()), this, SLOT(adjustRec()) );
   
}

argSlider::~argSlider(){
  delete numField;
}

void argSlider::incRec(){
  if(QuSlotDebug())
  {
	printf("argSlider::incRec %ld\n", this);
	fflush(stdout);
  }
   
    num += 1;
    numField->setText(tr("%1").arg(num+1));
    adjustRec();  // done explicitly as textChanged() is not emitted if slot does not have focus
}

void argSlider::decRec(){
  if(QuSlotDebug())
  {
	printf("argSlider::decRec %ld\n", this);
	fflush(stdout);
  }
    
    num -= 1;
    numField->setText(tr("%1").arg(num+1));
    adjustRec(); // done explicitly as textChanged() is not emitted if slot does not have focus
}

void argSlider::adjustRec(){
  if(QuSlotDebug())
  {
	printf("argSlider::adjustRec %ld\n", this);
	fflush(stdout);
  }

  bool OK;
  num = numField->text().toInt(&OK);
  num -=1; // as feedback goes 1-N while code goes 0-N-1
  if(OK){
    if(num < 0) num = 0;
    if(num > maxNum) num = maxNum;
    numField->setText(tr("%1").arg(num+1));
    emit QbEchanged(id, num);
  }
  else
    numField->setText("1");
    emit QbEchanged(id, num);
}
