//  qbu.cc
//
// Data Browser Widget
//
// July 2002: Tony Allen
//

#include <stdlib.h>
#include "qt_version.h"
#include <qobject.h>

#include "qt_version.h"

#if QT_VERSION < 0x040000
#include <qvbox.h>
#include "qgu.h"
#include "qbu.h"
#else
#include "qgu_qt4.h"
#include "qbu_qt4.h"
#endif

#include "qdutil.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;
extern void QdaLaunchWindow(QString,QWidget*);

/// \var  extern vector<QguListItem *> dragList
/// \brief Global variable that contains the list of items being dragged
///
extern vector<QguListItem *> dragList;

/// \var  extern QguListView * dragFromList
/// \brief Global variable that defines the list view from which the items are being dragged
///
extern QguListView * dragFromList;

#if QT_VERSION < 0x040000
DataBrowseWindow::DataBrowseWindow(QdObject_var dobj, QString WLcontext,
                                   QWidget *parent, const char *name ) 
  : QMainWindow(parent,name)
{
  QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);

  //this->setPalette(qsasWindow->palette());
  this->setPalette(qApp->palette());

  QWidget::setWFlags(Qt::WDestructiveClose);
  
   msgBar = statusBar();
   
   // turn off save buttons until after objects set
   disallowSaves = true;
   
   browserVB = new QVGroupBox( this, "TopWidget" );
   setCentralWidget( browserVB );
   
   name_panel=new QHGroupBox(browserVB, "name");
   
   QLabel *name_label = new QLabel("Name", name_panel);
   name_field = new QguWLinput(name_panel, "name_field");
   name_field->acceptText = true;
   name_field->setEnabled(true);
   
   recSlider = new QHGroupBox(browserVB, "recSlider");
   recSlider->setTitle("Record Number");
   QToolButton * leftEnd = new QToolButton( recSlider, "leftEnd" );
   leftEnd->setText("|");
   leftEnd->setFixedSize(20,21);
   QToolButton * leftSlide = new QToolButton( Qt::LeftArrow, recSlider, "");
   leftSlide->setFixedSize(20,21);

   recBar = new QSlider(Qt::Horizontal, recSlider, "recBar");
  
   QToolButton * rightSlide = new QToolButton( Qt::RightArrow, recSlider, "");
   rightSlide->setFixedSize(20,21);
   QToolButton * rightEnd = new QToolButton( recSlider, "rightEnd" );
   rightEnd->setText("|");
   rightEnd->setFixedSize(20,21);
   connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
   connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
   connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
   connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
   connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(adjustRec(int)) );
   
   recField = new QguWLinput(recSlider,"recField");
   recField->acceptInteger = true;
   recField->setEnabled(true);
   recField->setFixedWidth(60);
   recField->setAlignment(Qt::AlignHCenter);

   setObject(dobj, WLcontext);
   connect( recField, SIGNAL(validated_OK()), this, SLOT(inputRec()) );

   editMenu = new QPopupMenu( this );
   menuBar()->insertItem( "  Edit ", editMenu );
   editMenu->insertItem("New Metadata", browser, SLOT( newXref() ), Qt::CTRL+Qt::Key_M );
   editMenu->insertItem("Attach Metadata", browser, SLOT( attachXref() ), Qt::CTRL+Qt::Key_A );
   editMenu->insertItem("Add time tags", browser, SLOT( addTimeTags() ));
   editMenu->insertSeparator();
   editMenu->insertItem("Delete Metadata", browser, SLOT( deleteXref() ), Qt::CTRL+Qt::Key_X );
   
   
   menuBar()->insertSeparator();
   QPopupMenu * help = new QPopupMenu( this );
   menuBar()->insertItem( "  Help ", help );
   help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
   help->insertSeparator();
   help->insertItem("...Data Import", this, SLOT( launchManBrowser() ));
   help->insertSeparator();
   help->insertItem("Module Version", this, SLOT( launchVersion() ));
 

  buttons_layout=new QHGroupBox(browserVB, "buttons");
    
  saveAsBtn = new QPushButton( "Save as...", buttons_layout, "saveAsBtn" );
  saveAsBtn->setMinimumSize(saveAsBtn->sizeHint());
  connect( saveAsBtn, SIGNAL(clicked()), this, SLOT(saveAs()) );
  saveAsBtn->setEnabled(false);
  saveAsBtn->setFocusPolicy(STRONG_FOCUS);

  saveEditsBtn = new QPushButton( "Save Edits", buttons_layout, "saveEditsBtn" );
  saveEditsBtn->setMinimumSize(saveEditsBtn->sizeHint());
  connect( saveEditsBtn, SIGNAL(clicked()), this, SLOT(saveEditsAndClose()) );
  saveEditsBtn->setEnabled(false);
  saveEditsBtn->setFocusPolicy(STRONG_FOCUS);
    
  dismissBtn = new QPushButton( "Dismiss", buttons_layout, "dismissBtn" );
  dismissBtn->setMinimumSize(dismissBtn->sizeHint());
  connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
  dismissBtn->setDefault(true);
  
  this->setMinimumSize(this->sizeHint());
  connect(name_field, SIGNAL(textChanged(const QString &)), this, SLOT(allowSaveAs()));
}
#else
DataBrowseWindow::DataBrowseWindow(QdObject_var dobj, QString WLcontext,
				   QWidget *parent, const char *name ) 
	: QMainWindow(parent,name)      
{

	QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);

	this->setPalette(qApp->palette());

	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
   // turn off save buttons until after objects set
	disallowSaves = true;
	
	browserVB=new QWidget(this);

	QVBoxLayout * browserVBLayout=new QVBoxLayout;
	
	// Name panel
	
	name_panel=new QGroupBox;
	QHBoxLayout * name_panel_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Name", name_panel);
	name_panel_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->acceptText = true;
	name_field->setEnabled(true);
	connect(name_field, SIGNAL(textChanged(const QString &)), this, SLOT(allowSaveAs()));
	name_panel_layout->addWidget(name_field);
	
	name_panel->setLayout(name_panel_layout);
	browserVBLayout->addWidget(name_panel);
	

	// Record number panel
	
	recSlider=new QGroupBox;
	QHBoxLayout * recSliderLayout=new QHBoxLayout;
	
	QToolButton * leftEnd = new QToolButton;
	leftEnd->setText("|");
	leftEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(leftEnd);
	
	QToolButton * leftSlide = new QToolButton( Qt::LeftArrow, recSlider, "");
	leftSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(leftSlide);
	
	recBar = new QSlider(Qt::Horizontal, recSlider, "recBar");
	recSliderLayout->addWidget(recBar);
	
	QToolButton * rightSlide = new QToolButton( Qt::RightArrow, recSlider, "");
	rightSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(rightSlide);
	
	QToolButton * rightEnd = new QToolButton( recSlider, "rightEnd" );
	rightEnd->setText("|");
	rightEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(rightEnd);
	
	recField = new QguWLinput(recSlider,"recField");
	recField->acceptInteger = true;
	recField->setEnabled(true);
	recField->setFixedWidth(60);
	recField->setAlignment(Qt::AlignHCenter);
	recSliderLayout->addWidget(recField);
	
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
	connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
	connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(adjustRec(int)) );
	connect( recField, SIGNAL(validated_OK()), this, SLOT(inputRec()) );
	
	recSlider->setLayout(recSliderLayout);
	browserVBLayout->addWidget(recSlider);
	
	
	// Object view widget
	setObject(dobj, WLcontext);
	browserVBLayout->addWidget(browser);
	

	// Buttons layout
	
	buttons_groupbox=new QGroupBox;
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveAsBtn = new QPushButton( "Save as...");
	saveAsBtn->setMinimumSize(saveAsBtn->sizeHint());
	connect( saveAsBtn, SIGNAL(clicked()), this, SLOT(saveAs()) );
	saveAsBtn->setEnabled(false);
	saveAsBtn->setFocusPolicy(STRONG_FOCUS);
	buttons_layout->addWidget(saveAsBtn);
	
	saveEditsBtn = new QPushButton( "Save Edits");
	saveEditsBtn->setMinimumSize(saveEditsBtn->sizeHint());
	connect( saveEditsBtn, SIGNAL(clicked()), this, SLOT(saveEditsAndClose()) );
	saveEditsBtn->setEnabled(false);
	saveEditsBtn->setFocusPolicy(STRONG_FOCUS);
	buttons_layout->addWidget(saveEditsBtn);
	
	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	dismissBtn->setDefault(true);
	buttons_layout->addWidget(dismissBtn);
	
	buttons_groupbox->setLayout(buttons_layout);
	browserVBLayout->addWidget(buttons_groupbox);
	
	//Menu
	
	editMenu = new QMenu( this );
	menuBar()->insertItem( "  Edit ", editMenu );
	editMenu->insertItem("New Metadata", browser, SLOT( newXref() ), Qt::CTRL+Qt::Key_M );
	editMenu->insertItem("Attach Metadata", browser, SLOT( attachXref() ), Qt::CTRL+Qt::Key_A );
	editMenu->insertItem("Add time tags", browser, SLOT( addTimeTags() ));
	editMenu->insertSeparator();
	editMenu->insertItem("Delete Metadata", browser, SLOT( deleteXref() ), Qt::CTRL+Qt::Key_X );
   
   
	menuBar()->insertSeparator();
	QMenu * help = new QMenu( this );
	menuBar()->insertItem( "  Help ", help );
	help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->insertSeparator();
	help->insertItem("...Data Import", this, SLOT( launchManBrowser() ));
	help->insertSeparator();
	help->insertItem("Module Version", this, SLOT( launchVersion() ));
	
	//Done
	
	browserVB->setLayout(browserVBLayout);
	
	setCentralWidget(browserVB);

// 	this->setMinimumSize(this->sizeHint());
}
#endif

void DataBrowseWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveAsBtn->isDefault()  ) {
	     if( saveEditsBtn->isEnabled()) {
		   saveAsBtn->setDefault(false);
		   saveEditsBtn->setDefault(true);
		}
		else{
		   saveAsBtn->setDefault(false);
		   dismissBtn->setDefault(true);
		}
	  }
	  else if(saveEditsBtn->isDefault() ) {
		 saveEditsBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
	     if(saveAsBtn->isEnabled()){
		   dismissBtn->setDefault(false);
		   saveAsBtn->setDefault(true);
		 }
		 else if(saveEditsBtn->isEnabled()){
		   dismissBtn->setDefault(false);
		   saveEditsBtn->setDefault(true);
		 }
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveAsBtn->isDefault()) saveAs();
	 else if (saveEditsBtn->isDefault()) saveEditsAndClose();
	 else if (dismissBtn->isDefault()) dismiss();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}


DataBrowseWindow::~DataBrowseWindow(){

 if(QuSlotDebug())
 {
  printf("DataBrowseWindow::~DataBrowseWindow %ld\n", this);
  fflush(stdout);
 }
 
 
//  delete recField;
  if(browser != 0) delete browser;
}

void DataBrowseWindow::dismiss(){
  
  close();
}

void DataBrowseWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::launchManual %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("index.html");
}

void DataBrowseWindow::launchManBrowser(){ 

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::launchManBrowser %ld\n", this);
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}


void DataBrowseWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::launchVersion %ld\n", this);
	fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Data Object Browser  ";
  text += QString(QBU_VERSION);
  QMessageBox::information( this, "QSAS information", text.latin1(), "OK" ); 

}

void DataBrowseWindow::incRec(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::incRec %ld\n", this);
	fflush(stdout);
 }
 
 if(browser->activeEditor->record < browser->activeEditor->maxRec){
    browser->activeEditor->record +=1;
 }
 else{
    browser->activeEditor->record = browser->activeEditor->maxRec;
 }
 recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::decRec(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::decRec %ld\n", this);
	fflush(stdout);
 }
 
  if(browser->activeEditor->record > 0){
    browser->activeEditor->record -= 1;
  }
  else{
    browser->activeEditor->record = 0;
  }
  recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::lastRec(){

 if(QuSlotDebug())
 {
  printf("DataBrowseWindow::lastRec %ld\n", this);
  fflush(stdout);
 }
 
 
    browser->activeEditor->record = browser->activeEditor->maxRec;
    recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::firstRec(){

 if(QuSlotDebug())
 {
  printf("DataBrowseWindow::firstRec %ld\n", this);
  fflush(stdout);
 }
 
 
    browser->activeEditor->record = 0;
    recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::inputRec(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::inputRec %ld\n", this);
	fflush(stdout);
 }
 
    if (recField->edited()) {
      bool ok;
      int rec = recField->text().toInt(&ok);
      if(ok){
        if(rec < 0) rec = 0;
	    if(rec > browser->activeEditor->maxRec)  rec = browser->activeEditor->maxRec;
        recField->setText(tr("%1").arg(rec)); // reset field in case changed
        browser->activeEditor->record = rec;
	    recBar->setValue(rec);
        browser->activeEditor->updateForRec();
      }
    }
}
 
void DataBrowseWindow::adjustRec(int rec){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::adjustRec %ld %d\n", this, rec);
	fflush(stdout);
 }
 
 
    browser->activeEditor->record = rec;
    recField->setText(tr("%1").arg(rec));
    browser->activeEditor->updateForRec();
  
}

void DataBrowseWindow::allowSave(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::allowSave %ld\n", this);
	fflush(stdout);
 }
 
  if(disallowSaves) return;
 
  if( browser->activeEditor == 0) return;
  
  this->saveEditsBtn->setEnabled(true);
  this->saveAsBtn->setDefault(false);
  this->dismissBtn->setDefault(false);
  this->saveEditsBtn->setDefault(true);
  
}

void DataBrowseWindow::allowTimeSave(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::allowTimeSave %ld\n", this);
	fflush(stdout);
 }
 
  if(disallowSaves) return;

  if( browser->activeEditor == 0) return;

  this->saveEditsBtn->setEnabled(true);
  this->saveAsBtn->setDefault(false);
  this->dismissBtn->setDefault(false);
  this->saveEditsBtn->setDefault(true);
 
}

void DataBrowseWindow::allowSaveAs(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::allowSaveAs %ld\n", this);
	fflush(stdout);
 }
 
 
  if(disallowSaves) return;
  this->saveAsBtn->setEnabled(true);
  this->dismissBtn->setDefault(false);
  this->saveEditsBtn->setDefault(false);
  this->saveAsBtn->setDefault(true);
}

void DataBrowseWindow::saveEditsAndClose(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::saveEditsAndClose %ld\n", this);
	fflush(stdout);
 }

  if( this->saveEdits()){
    close();
  }
}


void DataBrowseWindow::dragEnterEvent(QDragEnterEvent *event)
{ 
    if(QuSlotDebug()){ printf("DataBrowseWindow::dragEnterEvent %ld\n", this); fflush(stdout); }
    this->setActiveWindow();
    this->raise();
}

bool DataBrowseWindow::saveEdits(){

   if(QuSlotDebug())
   {
		printf("DataBrowseWindow::saveEdits %ld\n", this);
		fflush(stdout);
	}

	if(name_field->text().isEmpty()) {
		cout << "empty save" << endl;
		msgBar->message("Name field is empty");
		return false;
	}
	
	QdObject_var new_obj = browser->getUpdate();  
 
	if( parentObject.is_nil() ) {
		// object from working list
		QguListItem* item = qsasWindow->WorkingList->getItemByName(editObjectName);
		if(originalName != name_field->text()) {
			// new name for object
      
			// ensure new name is not already used
      
			// find original object to be renamed
			if( qsasWindow->WorkingList->nameUsedAtWLroot(name_field->text()) ){
				msgBar->message("New name already used");
				return false;
			}
      
			QString newName = QString(editObjectName.latin1());  
			int pos = newName.findRev('/'); 
			newName.truncate(pos+1);
			newName += name_field->text();
      
			if(item == 0) {
				QuMsgBar("Cannot find object on WL", editObjectName.latin1(), true);
				return true;
			}
			item->setText(Obj_Name_, name_field->text());
			item->refreshWLcontext();
			item->setSelected(true);
			editObjectName = newName;
		}
 
		if(item == 0)
		{
			int pos=editObjectName.findRev('/');
			QString list(editObjectName);
			list.truncate(pos);
			QString objectName(editObjectName);
			objectName.remove(0,pos+1);
			QwuiPutOnWL(new_obj,objectName,list);
			emit objChanged(editObjectName);
		}
		else
		{	    
			QuReplaceObject(editObjectName, new_obj);
			emit objChanged(editObjectName);
		}
	}
	else {
		// xref object
		if(originalName != name_field->text()) {   
			// rename first
			parentObject->rename_xref(string(originalName.latin1()), string(name_field->text().latin1()) );
			originalName = name_field->text();
		}
		parentObject->change_xref(originalName.latin1(), new_obj);
		emit objChanged(originalName);
	}

	qsasWindow->WorkingList->emitListModified();
	msgBar->message("----   Changes Saved");
	dismissBtn->setText("Done");
	return true;
}

void DataBrowseWindow::saveAs(){

 if(QuSlotDebug())
 {
	printf("DataBrowseWindow::saveAs %ld\n", this);
	fflush(stdout);
 }
 
 
   QString newName;
   
   QdObject_var new_obj = browser->getUpdate();
   newName = name_field->text();
   if(newName.isEmpty()) {
     msgBar->message("Name Field is empty");
     return;
   }
   
   if( parentObject.is_nil() ) {
  
     // put on WL somewhere
     
     QdTimeInterval_var t_out = QdTimeInterval_var::narrow(new_obj);

     if(qsasWindow->WorkingList->nameUsedAtWLroot(newName)) {
	  msgBar->message("Name Already Used");
          return;
     }  
     
     if ( !t_out.is_nil() ){
        QString context=QwuiPutOnWL( new_obj, newName.latin1(), "Time Intervals");
        emit objChanged(context);
     } 
     else if( new_obj->sequence_size() <= 1 ){
        QString context=QwuiPutOnWL( new_obj, newName.latin1(), "Constants");
        emit objChanged(context);
     }
     else{
        QString context=QwuiPutOnWL( new_obj, newName.latin1(), "Data");
        emit objChanged(context);
     }
       
   }
   else {
    parentObject->change_xref(newName.latin1(), new_obj);
    emit objChanged(newName);
  }

   close();
}

void DataBrowseWindow::setObject(QdObject_var dobj, QString WLcontext){


  editObjectName = WLcontext;
  
  browser = new QbObjViewer(this, dobj, browserVB, "browser");
  if( browser == 0 || browser->activeEditor == 0){
    return;
  }
  
  // ask qsas main window to ask qda to open a file (for Windows port)
  connect(browser, SIGNAL(launchQdaWindow(const char *)), qsasWindow, SIGNAL(openQdaWindow(const char *)));
  
  if( dobj.is_nil() ){
    msgBar->message("Nil Object");
    return;
  }
    
  msgBar->message(WLcontext);
  QuMsgBar(QString(""), false);
  
  
  // initialise slider to first record
 browser->activeEditor->record = 0;
 recField->setText("0");
  
  recBar->setRange(0, browser->activeEditor->maxRec);
  int step = browser->activeEditor->maxRec / 10;
  if (step < 1) step = 1;
  recBar->setPageStep(step);

  if(browser->activeEditor->maxRec > 0)  recSlider->setEnabled(true);
  else recSlider->setEnabled(false);

  // set Xrefs
  vector<string> xrefs;
  dobj->list_xref_names(xrefs);
  for(int i=0; i<xrefs.size(); i++){
    QdObject_var xref_obj=dobj->get_xref(xrefs[i]);
    xrefListItem * newitem = new xrefListItem(xref_obj, 
                              browser->activeEditor->edited_var, 
                              browser->xrefList, QString(xrefs[i].c_str()));
  }

//   this->resize(this->minimumSizeHint());

  if(browser->activeEditor != 0){
    connect(browser->activeEditor, SIGNAL(saveNow()), this, SLOT(saveEdits()));
    connect(browser->activeEditor, SIGNAL(editorHasInput()), this, SLOT(allowSave()));
    connect(browser->activeEditor, SIGNAL(editorHasTimeInput()), this, SLOT(allowTimeSave()));
  }

}



// generic viewer widget class

#if QT_VERSION < 0x040000
QbObjViewer::QbObjViewer(DataBrowseWindow * pWindow, QdObject_var dobj, QWidget *parent, const char *name)
  : QHGroupBox(parent,name)
{
	QdObject_var nilObj;
	activeEditor = 0;
	parentBrowser = pWindow;
	
	QHBoxLayout * viewerLayout=new QHBoxLayout;
	if(dobj.is_nil())
	{
		std::cout << "Nil " << std::endl;
	}
	QdTimeInterval_var TI_in = QdTimeInterval_var::narrow(dobj);
	QdTimeIntervalSeq_var TIS_in = QdTimeIntervalSeq_var::narrow(dobj);
	QdTimeSeq_var Ttag_in =  QdTimeSeq_var::narrow (dobj);
	QdRScalar_var S_in = QdRScalar_var::narrow(dobj);
	QdTime_var T_in = QdTime_var::narrow(dobj);
	QdRScalarSeq_var Sseq_in = QdRScalarSeq_var::narrow(dobj);
	QdRMatrix_var M_in = QdRMatrix_var::narrow(dobj);
	QdRMatrixSeq_var Mseq_in = QdRMatrixSeq_var::narrow(dobj);
	QdString_var Str_in = QdString_var::narrow(dobj);
	QdStringSeq_var Strseq_in = QdStringSeq_var::narrow(dobj);
	QdSMatrix_var SM_in = QdSMatrix_var::narrow(dobj);
	QdSMatrixSeq_var SMseq_in = QdSMatrixSeq_var::narrow(dobj);

 // is this a time interval object?
  
	if( !TI_in.is_nil() ){
		TI_window = new QbTimeIntervalEditor(TI_in, this, "Time Interval");
		TI_window->adjustSize();
		TI_window->show();
		activeEditor = TI_window;
		viewerLayout->addWidget(TI_window);
		activeEditor->edited_var=TI_window->_time_interval_var;
	}
  
   // is this a time interval sequence object?
  
	else if( !TIS_in.is_nil() ){
		TIS_window = new QbEventTableEditor(TIS_in, this, "Event Table");
		TIS_window->adjustSize();
		TIS_window->show();
		activeEditor = TIS_window;
		viewerLayout->addWidget(TIS_window);
		activeEditor->edited_var=TIS_window->_event_table_var;
	}

  
	else if(!Ttag_in.is_nil()){
		Ttag_window = new QbTtagEditor(Ttag_in, this, "Time Line");
		Ttag_window->show();
		activeEditor = Ttag_window;
		viewerLayout->addWidget(Ttag_window);
		activeEditor->edited_var = Ttag_window->Ttag_var;
	}

  // is this a scalar sequence object?
	else if( !Sseq_in.is_nil() ){
    // scalar sequence viewer  
		Sseq_window = new QbScalarSeqEditor(Sseq_in, this, "Sequence");

		Sseq_window->show();
		activeEditor = Sseq_window;
		viewerLayout->addWidget(Sseq_window);
		activeEditor->edited_var = Sseq_window->Sseq_var;

	}

   // is this a Matrix sequence object?
	else if( !Mseq_in.is_nil() ){
		Mseq_window = new QbMatrixSeqEditor(Mseq_in, this, "MSequence");
    
		Mseq_window->show();
		activeEditor = Mseq_window;
		viewerLayout->addWidget(Mseq_window);
		activeEditor->edited_var = Mseq_window->Mseq_var;
	}
  
  // is this a text const object?
	else if( !Str_in.is_nil() ){
		Str_window = new QbTextEditor(Str_in, this, "Text");
    
		Str_window->show();
		activeEditor = Str_window;
		viewerLayout->addWidget(Str_window);
		activeEditor->edited_var = Str_window->Str_var;
	}
  
  // is this a text sequence object?
	else if( !Strseq_in.is_nil() ){
		Strseq_window = new QbTextSeqEditor(Strseq_in, this, "TextSequence");
    
		Strseq_window->show();
		activeEditor = Strseq_window;
		viewerLayout->addWidget(Strseq_window);
		activeEditor->edited_var = Strseq_window->Strseq_var;
	}
 
  // is this a String Matrix sequence object?
	else if( !SMseq_in.is_nil() ){
		SMseq_window = new QbMatrixSeqEditor(SMseq_in, this, "StringMatrixSequence");
    
		SMseq_window->show();
		activeEditor = SMseq_window;
		viewerLayout->addWidget(SMseq_window);
		activeEditor->edited_var = SMseq_window->Mseq_var;
	}
  // is this a scalar const object?
	else if( !S_in.is_nil() ){
		S_window = new QbScalarEditor(S_in, this, "Constant");
		S_window->show();
		activeEditor = S_window;
		viewerLayout->addWidget(S_window);
		activeEditor->edited_var = S_window->S_var;
	}
  
  // is this a time const object?
	else if( !T_in.is_nil() ){
		T_window = new QbTimeObjEditor(T_in, this, "Constant");

		T_window->show();
		activeEditor = T_window;
		viewerLayout->addWidget(T_window);
		activeEditor->edited_var = T_window->T_var;
	}
  
  // Is this a String Matrix?
	else if( !SM_in.is_nil() ){
		SM_window = new QbMatrixEditor(SM_in, this, "StringMatrix");

		SM_window->show();
		activeEditor = SM_window;
		viewerLayout->addWidget(SM_window);
		activeEditor->edited_var = SM_window->M_var;
	}
  
  // is this a Matrix object?
	else if( !M_in.is_nil() ){
		M_window = new QbMatrixEditor(M_in, this, "Matrix");

		M_window->show();
		activeEditor = M_window;
		viewerLayout->addWidget(M_window);
		activeEditor->edited_var = M_window->M_var;
	}

	if (activeEditor == 0) {
		QuMsgBar("object type not supported", true);
		return;
	}
   
    QVBox *Attribute_box = new QVBox(this, "AttributeBox");
    Attribute_box->setSpacing(3);

    xrefList = new xrefListView(Attribute_box, "Attributes");
    xrefList->setAcceptDrops(true);
    xrefList->addColumn("            Attributes           ");
    xrefList->setMinimumHeight(200);
    xrefList->setMinimumWidth(160);
    xrefList->viewer = this;

    connect(xrefList, SIGNAL(doubleClicked(QListViewItem*)), 
            this, SLOT(xrefEdit(QListViewItem*)) );

    parentsBtn = new QPushButton( " View Source Files... ", Attribute_box, "parentsBtn" );
    parentsBtn->setMinimumSize(parentsBtn->sizeHint());
    connect( parentsBtn, SIGNAL(clicked()), this, SLOT(openParentFile()) );
    
    if(!TI_in.is_nil() || !TIS_in.is_nil())
    {
	    Attribute_box->hide();
    }
    
    fileList = new QPopupMenu(this, "FileListPopup");
    adjustSize();
}
#else
QbObjViewer::QbObjViewer(DataBrowseWindow * pWindow, QdObject_var dobj, QWidget *parent, const char *name)
	: QGroupBox(parent,name)
{
	QdObject_var nilObj;
	activeEditor = 0;
	parentBrowser = pWindow;
	
	QHBoxLayout * viewerLayout=new QHBoxLayout;
	
	QdTimeInterval_var TI_in = QdTimeInterval_var::narrow(dobj);
	QdTimeIntervalSeq_var TIS_in = QdTimeIntervalSeq_var::narrow(dobj);
	QdTimeSeq_var Ttag_in =  QdTimeSeq_var::narrow (dobj);
	QdRScalar_var S_in = QdRScalar_var::narrow(dobj);
	QdTime_var T_in = QdTime_var::narrow(dobj);
	QdRScalarSeq_var Sseq_in = QdRScalarSeq_var::narrow(dobj);
	QdRMatrix_var M_in = QdRMatrix_var::narrow(dobj);
	QdRMatrixSeq_var Mseq_in = QdRMatrixSeq_var::narrow(dobj);
	QdString_var Str_in = QdString_var::narrow(dobj);
	QdStringSeq_var Strseq_in = QdStringSeq_var::narrow(dobj);
	QdSMatrix_var SM_in = QdSMatrix_var::narrow(dobj);
	QdSMatrixSeq_var SMseq_in = QdSMatrixSeq_var::narrow(dobj);
  
	// is this a time interval object?
	if( !TI_in.is_nil() ){
		TI_window = new QbTimeIntervalEditor(TI_in, this, "Time Interval");
		TI_window->adjustSize();
		TI_window->show();
		activeEditor = TI_window;
		viewerLayout->addWidget(TI_window);
		activeEditor->edited_var=TI_window->_time_interval_var;
	}
  
	// is this a time interval sequence object?  
	else if( !TIS_in.is_nil() ){
		TIS_window = new QbEventTableEditor(TIS_in, this, "Event Table");
		TIS_window->adjustSize();
		TIS_window->show();
		activeEditor = TIS_window;
		viewerLayout->addWidget(TIS_window);
		activeEditor->edited_var=TIS_window->_event_table_var;
	}
  
	else if(!Ttag_in.is_nil()){
		Ttag_window = new QbTtagEditor(Ttag_in, this, "Time Line");
		Ttag_window->show();
		activeEditor = Ttag_window;
		viewerLayout->addWidget(Ttag_window);
		activeEditor->edited_var = Ttag_window->Ttag_var;
	}
  
	// is this a time const object?
	else if( !T_in.is_nil() ){
		T_window = new QbTimeObjEditor(T_in, this, "Constant");

		T_window->show();
		activeEditor = T_window;
		viewerLayout->addWidget(T_window);
		activeEditor->edited_var = T_window->T_var;
	}
  
	// is this a scalar sequence object?
	else if( !Sseq_in.is_nil() ){
		Sseq_window = new QbScalarSeqEditor(Sseq_in, this, "Sequence");

		Sseq_window->show();
		activeEditor = Sseq_window;
		viewerLayout->addWidget(Sseq_window);
		activeEditor->edited_var = Sseq_window->Sseq_var;

	}

	// is this a scalar const object?
	else if( !S_in.is_nil() ){
		S_window = new QbScalarEditor(S_in, this, "Constant");
		S_window->show();
		activeEditor = S_window;
		viewerLayout->addWidget(S_window);
		activeEditor->edited_var = S_window->S_var;
	}

	// is this a Matrix sequence object?
	else if( !Mseq_in.is_nil() ){
		Mseq_window = new QbMatrixSeqEditor(Mseq_in, this, "MSequence");
    
		Mseq_window->show();
		activeEditor = Mseq_window;
		viewerLayout->addWidget(Mseq_window);
		activeEditor->edited_var = Mseq_window->Mseq_var;
	}
  
	// is this a Matrix object?
	else if( !M_in.is_nil() ){
		M_window = new QbMatrixEditor(M_in, this, "Matrix");

		M_window->show();
		activeEditor = M_window;
		viewerLayout->addWidget(M_window);
		activeEditor->edited_var = M_window->M_var;
	}
  
	// is this a text sequence object?
	else if( !Strseq_in.is_nil() ){
		Strseq_window = new QbTextSeqEditor(Strseq_in, this, "TextSequence");
    
		Strseq_window->show();
		activeEditor = Strseq_window;
		viewerLayout->addWidget(Strseq_window);
		activeEditor->edited_var = Strseq_window->Strseq_var;
	}
  
	// is this a text const object?
	else if( !Str_in.is_nil() ){
		Str_window = new QbTextEditor(Str_in, this, "Text");
    
		Str_window->show();
		activeEditor = Str_window;
		viewerLayout->addWidget(Str_window);
		activeEditor->edited_var = Str_window->Str_var;
	}
 
	// is this a String Matrix sequence object?
	else if( !SMseq_in.is_nil() ){
		SMseq_window = new QbMatrixSeqEditor(SMseq_in, this, "StringMatrixSequence");
    
		SMseq_window->show();
		activeEditor = SMseq_window;
		viewerLayout->addWidget(SMseq_window);
		activeEditor->edited_var = SMseq_window->Mseq_var;
	}
  
	// Is this a String Matrix?
	else if( !SM_in.is_nil() ){
		SM_window = new QbMatrixEditor(SM_in, this, "StringMatrix");

		SM_window->show();
		activeEditor = SM_window;
		viewerLayout->addWidget(SM_window);
		activeEditor->edited_var = SM_window->M_var;
	}
	  
	if (activeEditor == 0) {
		QuMsgBar("object type not supported", true);
		return;
	}

	QWidget *Attribute_box = new QWidget(this, "AttributeBox");
	QVBoxLayout * Attribute_box_layout=new QVBoxLayout;
	Attribute_box_layout->setSpacing(3);

	xrefList = new xrefListView(Attribute_box, "Attributes");
	xrefList->setAcceptDrops(true);
	xrefList->addColumn("            Attributes           ");
	xrefList->setMinimumHeight(200);
	xrefList->setMinimumWidth(160);
	xrefList->viewer = this;
	connect(xrefList, SIGNAL(doubleClicked(Q3ListViewItem*)), 
		this, SLOT(xrefEdit(Q3ListViewItem*)) );
  

	parentsBtn = new QPushButton( " View Source Files... ", Attribute_box, "parentsBtn" );
	parentsBtn->setMinimumSize(parentsBtn->sizeHint());
	connect( parentsBtn, SIGNAL(clicked()), this, SLOT(openParentFile()) );

	Attribute_box_layout->addWidget(xrefList);
	Attribute_box_layout->addWidget(parentsBtn);
	Attribute_box->setLayout(Attribute_box_layout);
    
	viewerLayout->addWidget(Attribute_box);
	
	if(!TI_in.is_nil() || !TIS_in.is_nil())
	{
		Attribute_box->hide();
	}
    
	fileList = new QMenu(this);

	setLayout(viewerLayout);
	adjustSize();
}
#endif


void QbObjViewer::refreshList(){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::refreshList %ld\n", this);
	fflush(stdout);
 }

    xrefList->clear();
   if( activeEditor->edited_var.is_nil() ) return;

   // reset Xrefs
   vector<string> xrefs;
   activeEditor->edited_var->list_xref_names(xrefs);
   for(int i=0; i<xrefs.size(); i++){
     QdObject_var xref_obj=activeEditor->edited_var->get_xref(xrefs[i]);
     xrefListItem * newitem = new xrefListItem(xref_obj, activeEditor->edited_var, xrefList, QString(xrefs[i].c_str()));
   }
   
}

QdObject_var QbObjViewer::getUpdate(){
  
  return activeEditor->edited_var;
  
}

void QbObjViewer::xrefEdit ( Q_LIST_VIEW_ITEM *item ){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::xrefEdit %ld, %ld\n", this, item);
	fflush(stdout);
 }
 

  DataBrowseWindow *QBROWSER_widget_xref=0;
    
  if(item == 0) return;
  
  
  QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
                  0, item->text(Obj_Name_).latin1(), 
		  ((xrefListItem*)item)->xref_obj, 
		  QString(item->text(Obj_Name_).latin1()));

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  

  this->parentBrowser->saveEditsBtn->setEnabled(true); // allow saves so that xref can be renamed
   
  if (QBROWSER_widget_xref == 0 ) return;
  
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this,  SLOT(refreshList()) );
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this->activeEditor,  SLOT(gotInput()) );
		    
}

void QbObjViewer::addTimeTags(){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::addTimeTags %ld\n", this);
	fflush(stdout);
 }
 
 
  
  if(activeEditor == 0 || activeEditor->edited_var.is_nil() ) {
    QuMsgBar("Object does not exist", true);
    return;
  }
  
  if(activeEditor->edited_var->xref_exists(TIME_TAGS) ){
    QuMsgBar("Time Tags already exist", true);
    return;  
  }
  
  int n_recs = activeEditor->edited_var->sequence_size();
  
  QdTimeSeq_var T_var = new QdTimeSeq(n_recs, QDOS::Time("2001-09-04 00:00:00"));
    
  activeEditor->timeBox->show();
#if QT_VERSION < 0x040000
  activeEditor->timeField->setText( T_var[0].iso().c_str() );
#else
  activeEditor->timeField->setTime( T_var[0] );
#endif
  activeEditor->edited_var->change_xref(TIME_TAGS, T_var);

}

void QbObjViewer::openParentFile(){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::openParentFile %ld\n", this);
	fflush(stdout);
 }
 
  
  QdStringSeq_var parentsXref;
  if(activeEditor == 0) return;
  
  if( (activeEditor->edited_var).is_nil() ) {
    QuMsgBar("No variable edited", true);
    return;
  }
  try{
    parentsXref = QdStringSeq_var::narrow(activeEditor->edited_var->get_xref("Parents"));
  }
  catch(Exception &e){
    QuMsgBar( e.msg().c_str(), true );
    return;
  }
  
  if ( parentsXref.is_nil() ) return;
  
  fileList->clear();
  int listSize = parentsXref->sequence_size();
  for (int i=0; i<listSize; i++){
    fileList->insertItem( QString(parentsXref[i].c_str()), i);
  }
  fileList->insertSeparator( listSize);
  fileList->insertItem( QString("Cancel"), listSize+1);

#if QT_VERSION < 0x040000
  int IDinList = fileList->exec(QCursor::pos());
  if( IDinList >= listSize || IDinList < 0) return;
  
  QString file = fileList->text(IDinList);
#else
  QAction* act=fileList->exec(QCursor::pos());
  if(act==NULL) return;
  if(act->text()==QString("Cancel")) return;
  QString file=act->text();
#endif


  emit launchQdaWindow(file.latin1());

}


void QbObjViewer::attachXref()
{ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::attachXref %ld\n", this);
	fflush(stdout);
 }
  
  QbuAttachWindow * attachWindow = new QbuAttachWindow(this, "attachXrefChooser");
  attachWindow->dobj = activeEditor->edited_var;
  attachWindow->thisViewer = this;
  attachWindow->show();
}


void QbObjViewer::deleteXref()
{ 

  if(QuSlotDebug())
  {
	printf("QbObjViewer::deleteXref %ld\n", this);
	fflush(stdout);
  }
  
  Q_LIST_VIEW_ITEM * Item = this->xrefList->selectedItem();
  if( Item == 0) return;
  try {
     activeEditor->edited_var->delete_xref(Item->text(0).latin1() );
  }
  catch(Exception &e){
       QuMsgBar(e.msg().c_str(), false);   
       return;
  }
  
  refreshList();
  this->activeEditor->gotInput();
 
}

void QbObjViewer::newXref()
{ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::newXref %ld\n", this);
	fflush(stdout);
 }
 

   static QPushButton *Sbtn;
   static QPushButton *Mbtn;
   static QPushButton *Vbtn;
   static QPushButton *tbtn;
   static QPushButton *Tbtn;
   static QPushButton *Txtbtn;
   static QPushButton *TxtArrbtn;
   static Q_V_BOX * typeBox;
   
     objTypeSelector = new QDialog(0, "new object type?");
     typeBox = new Q_V_BOX(objTypeSelector, "typeBox");
#if QT_VERSION < 0x040000
     typeBox->setMargin(QSAS_profiles->current()->GUImargins);
     typeBox->setSpacing(QSAS_profiles->current()->GUImargins);

     Txtbtn = new QPushButton(" Text ", typeBox, "Txtbtn");
     TxtArrbtn = new QPushButton(" Text Array", typeBox, "TxtArrbtn");
     Tbtn = new QPushButton(" Time Object ", typeBox, "Tbtn");
     tbtn = new QPushButton(" Time Interval  ", typeBox, "tbtn");
     Sbtn = new QPushButton(" Scalar Constant  ", typeBox, "Sbtn");
     Mbtn = new QPushButton(" Matrix Constant  ", typeBox, "Mbtn");
     Vbtn = new QPushButton(" Vector Constant  ", typeBox, "Vbtn");
     connect(Txtbtn,SIGNAL(clicked()),this,SLOT(createText()));
     connect(TxtArrbtn,SIGNAL(clicked()),this,SLOT(createTextArr()));
     connect(Tbtn,SIGNAL(clicked()),this,SLOT(createTime()));
     connect(tbtn,SIGNAL(clicked()),this,SLOT(createTInterval()));
     connect(Sbtn,SIGNAL(clicked()),this,SLOT(createConstant()));
     connect(Mbtn,SIGNAL(clicked()),this,SLOT(createArray()));
     connect(Vbtn,SIGNAL(clicked()),this,SLOT(createVConstant()));
     typeBox->adjustSize();
#else
     QVBoxLayout * typeBoxLayout=new QVBoxLayout;
     typeBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
     typeBoxLayout->setSpacing(QSAS_profiles->current()->GUImargins);

     Txtbtn = new QPushButton(" Text ", typeBox, "Txtbtn"); typeBoxLayout->addWidget(Txtbtn);
     TxtArrbtn = new QPushButton(" Text Array", typeBox, "TxtArrbtn"); typeBoxLayout->addWidget(TxtArrbtn);
     Tbtn = new QPushButton(" Time Object ", typeBox, "Tbtn"); typeBoxLayout->addWidget(Tbtn);
     tbtn = new QPushButton(" Time Interval  ", typeBox, "tbtn");     typeBoxLayout->addWidget(tbtn);
     Sbtn = new QPushButton(" Scalar Constant  ", typeBox, "Sbtn"); typeBoxLayout->addWidget(Sbtn);
     Mbtn = new QPushButton(" Matrix Constant  ", typeBox, "Mbtn"); typeBoxLayout->addWidget(Mbtn);
     Vbtn = new QPushButton(" Vector Constant  ", typeBox, "Vbtn"); typeBoxLayout->addWidget(Vbtn);
     connect(Txtbtn,SIGNAL(clicked()),this,SLOT(createText()));
     connect(TxtArrbtn,SIGNAL(clicked()),this,SLOT(createTextArr()));
     connect(Tbtn,SIGNAL(clicked()),this,SLOT(createTime()));
     connect(tbtn,SIGNAL(clicked()),this,SLOT(createTInterval()));
     connect(Sbtn,SIGNAL(clicked()),this,SLOT(createConstant()));
     connect(Mbtn,SIGNAL(clicked()),this,SLOT(createArray()));
     connect(Vbtn,SIGNAL(clicked()),this,SLOT(createVConstant()));
     typeBox->setLayout(typeBoxLayout);
     typeBox->adjustSize();
#endif
     
  objTypeSelector->show();  
}

void QbObjViewer::createText ( ){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createText %ld\n", this);
	fflush(stdout);
 }
 
 
  DataBrowseWindow *QBROWSER_widget_xref;
  QdObject_var dobj;  
  QdString_var sobj = new QdString("");
  
  dobj = sobj;

  objTypeSelector->close();

  QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
                  0, "new", dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
  if(QBROWSER_widget_xref == 0 ) return;
  
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this->activeEditor, SLOT(gotInput()) );

}


void QbObjViewer::createTime(){ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createTime %ld\n", this);
	fflush(stdout);
 }
 
  QdTime_var dobj;
  DataBrowseWindow *QBROWSER_widget_xref;
  
  dobj = new QdTime("2001-09-05 00:00:00.000");
  objTypeSelector->close();
  
  QdString_var SI_ptr = new QdString("1>s");
  dobj->change_xref(SI_CONVERSION, SI_ptr);
  QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
                  0, "newTime", dobj, "newTime");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
  if(QBROWSER_widget_xref == 0 ) return;
  
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this->activeEditor, SLOT(gotInput()) );
  
}


void QbObjViewer::createTInterval ( ){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createTInterval %ld\n", this);
	fflush(stdout);
 }
 
 
  printf("type not yet supported\n");
  fflush(stdout);
  objTypeSelector->close();

}


void QbObjViewer::createVConstant ( ){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createVConstant %ld\n", this);
	fflush(stdout);
 }
 
 
  Dim mDim = Dim(3);
  
  QdRMatrix_var dobj = new QdRMatrix(mDim, 0);
  QdString_var Frame_ptr = new QdString("vector>gse_xyz");
  dobj->change_xref(FRAME, Frame_ptr);
  
  DataBrowseWindow *QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
                  0, "newVector", dobj, "newVector");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
  if(QBROWSER_widget_xref == 0 ) return;
  
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this->activeEditor, SLOT(gotInput()) );
 
  objTypeSelector->close();

}

void QbObjViewer::createArray(){ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createArray %ld\n", this);
	fflush(stdout);
 }
   
  objTypeSelector->close();
  
  arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


  arrTypeSelector->arrType = QB_DOUBLE;
  arrTypeSelector->show();  
}



const char * QbObjViewer::objName(){ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::objName %ld\n", this);
	fflush(stdout);
 }
   
  return parentBrowser->name_field->text().latin1();
  
}


void QbObjViewer::createTextArr(){ 

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createTextArr %ld\n", this);
	fflush(stdout);
 }
   
  objTypeSelector->close();
  
  arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


  arrTypeSelector->arrType = QB_STRING;
  arrTypeSelector->show();  
}


void QbObjViewer::createConstant ( ){

 if(QuSlotDebug())
 {
	printf("QbObjViewer::createConstant %ld\n", this);
	fflush(stdout);
 }
 
 
   DataBrowseWindow *QBROWSER_widget_xref;
  QdRScalar_var dobj;  
  dobj = new QdRScalar(0.0);

  objTypeSelector->close();

  QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
                  0, "new",dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
  if (QBROWSER_widget_xref == 0) return;
  
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString &)), this->activeEditor, SLOT(gotInput()) );

}


DataBrowseWindow * QbuLaunchBrowse(QdObject_var source, 
                     QWidget *parent, const char * objectName, 
                     QdObject_var dobj, QString WLcontext)
{
  DataBrowseWindow *QBROWSER_widget;
      
  QBROWSER_widget = new DataBrowseWindow(dobj, WLcontext, parent, "editor");

  QBROWSER_widget->originalName = objectName;
  QBROWSER_widget->name_field->setText(objectName);
  if( QBROWSER_widget->browser == 0 
   || QBROWSER_widget->browser->activeEditor == 0) {
      QBROWSER_widget->close();
      return 0;
  }

  QBROWSER_widget->parentObject = source;  

  // allow save buttons to be turned on by changes now content is set
  QBROWSER_widget->disallowSaves = false;

  if( source.is_nil() ){
     QBROWSER_widget->setCaption("Object Editor");
     QBROWSER_widget->browser->parentsBtn->setEnabled(true);
     QBROWSER_widget->move(30,30);
  }
  else{
     // source means launched from a xref on another edit window
     QBROWSER_widget->setCaption("Metadata Editor");
     QBROWSER_widget->browser->parentsBtn->setEnabled(false);
     QBROWSER_widget->saveEditsBtn->setEnabled(true);
     QBROWSER_widget->move(30,250);
  }
  
  QBROWSER_widget->show();
 
  return QBROWSER_widget;
 
}

// class xrefListItem

  xrefListItem::xrefListItem(QdObject_var xobj, QdObject_var dobj,
                             Q_LIST_VIEW *parent, QString name) 
  : Q_LIST_VIEW_ITEM(parent,name)
{
  xref_obj = xobj;

}

// class xrefListView

  xrefListView::xrefListView( QWidget *parent, char * name ) 
  : Q_LIST_VIEW(parent,name)
{
   setAcceptDrops(true);
   setFocusPolicy(WHEEL_FOCUS);
 
}

void xrefListView::dragEnterEvent(QDragEnterEvent* event)
{
#if QT_VERSION < 0x040000
    event->accept( QTextDrag::canDecode(event) );
#else
    if (event->mimeData()->hasText()) event->acceptProposedAction();
#endif
}

void xrefListView::dropEvent(QDropEvent *event)
{
  QString txt;
  Q_LIST_VIEW_ITEM *newItem;

 if(QuSlotDebug())
  cout << "xrefListView::dropEvent " << endl;
 
  
  if (dragList.size() == 0)  return;
 
  if ((Q_LIST_VIEW *)this == (Q_LIST_VIEW *) dragFromList ) return; // cannot drag within this list

#if QT_VERSION < 0x040000
  if ( QTextDrag::decode(event, txt) )
#else
	  if (event->mimeData()->hasText())
#endif
  {
#if QT_VERSION >= 0x040000
	  txt=event->mimeData()->text();
#endif
    for (int i=0; i<dragList.size(); i++ ){   
       if ( dragList[i]->isLocked() ){
          QuMsgBar("A drag item is locked", false);
		continue;
	  }
	   
       QguListItem * item = dragList[i];
	  QdObject_var xref_obj = QuGetObjByName(item->getWLcontext());
	  
       if (strcmp(viewer->objName(), item->text(Obj_Name_).latin1() ) == 0 ) continue; // stop drop of obj on its own xref list
	  
       if(xref_obj.is_nil()) continue;

       try {
          viewer->activeEditor->edited_var->change_xref(item->text(Obj_Name_).latin1(), xref_obj);
       }
       catch(Exception &e){
          QuMsgBar(e.msg().c_str(), false);   
          return;
       }

    } // end for
    
    dragList.clear();
 
    viewer->refreshList();
    

  } // end if on decode event 
}

QbuAttachWindow::QbuAttachWindow(QbObjViewer *parent, const char *name)
   :  QDialog(parent,name)
#if QT_VERSION < 0x040000
{

  QVBoxLayout * centreBox;
  QVGroupBox * typeBox;
  QHBox * btnBox;

  QLabel *slotLabel; 
  QLabel *slotSpace; 
  QLabel *ObjSlotLabel; 
  QLabel *infoSlot; 
  QPushButton *cancelBtn;
  QPushButton *acceptBtn;
  
  centreBox = new QVBoxLayout(this, 4, -1, "centreBox");
  centreBox->setAutoAdd(TRUE);
  typeBox = new QVGroupBox(" Choose Name and Object " , this, "typeBox");
  typeBox->setMinimumWidth(250);
  typeBox->setMargin(QSAS_profiles->current()->GUImargins);
       
  
  slotLabel = new QLabel(typeBox, "slotLabel");
  slotLabel->setText(" Metadata Name ");
  
  name_field  = new QguWLinput(typeBox, "name_field");
  name_field->acceptText = true;
  name_field->setText("new");
  name_field->setMinimumWidth(150);
  
  
  slotSpace = new QLabel(typeBox, "slotSpace");
  slotSpace->setText("  ");
  
  ObjSlotLabel = new QLabel(typeBox, "ObjSlotLabel");
  ObjSlotLabel->setText(" Drop object to attach ");
  
  object_field  = new QguWLinput(typeBox, "object_field");
  object_field->acceptObject = true;
  object_field->setMinimumWidth(150);
  
  infoSlot = new QLabel(typeBox, "infoSlot");
  infoSlot->setText("( Dragging straight to metadata List also works )");
  
  connect(object_field,SIGNAL(validated_OK()),this, SLOT(updateName()));

  btnBox = new QHBox(this, "btnBox");
  btnBox->setSpacing(4);

  cancelBtn = new QPushButton("Cancel", btnBox, "cancelBtn");
  acceptBtn = new QPushButton("Create...", btnBox, "acceptBtn");

  connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
  connect(acceptBtn,SIGNAL(clicked()),this,SLOT(attachObject()));

  typeBox->adjustSize();
  
}
#else
{

	QVBoxLayout * centreBox;
	QGroupBox * typeBox;
	QWidget * btnBox;
	QLabel *slotLabel; 
	QLabel *slotSpace; 
	QLabel *ObjSlotLabel; 
	QLabel *infoSlot; 
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;
  
	centreBox = new QVBoxLayout(this, 4, -1, "centreBox");
	centreBox->setAutoAdd(TRUE);
	typeBox = new QGroupBox(" Choose Name and Object " , this);
	QVBoxLayout * typeBoxLayout=new QVBoxLayout;
	typeBox->setMinimumWidth(250);
	typeBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
       
  
	slotLabel = new QLabel(typeBox, "slotLabel");
	slotLabel->setText(" Metadata Name ");
	typeBoxLayout->addWidget(slotLabel);
  
	name_field  = new QguWLinput(typeBox, "name_field");
	name_field->acceptText = true;
	name_field->setText("new");
	name_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(name_field);
  
	slotSpace = new QLabel(typeBox, "slotSpace");
	slotSpace->setText("  ");
	typeBoxLayout->addWidget(slotSpace);
  
	ObjSlotLabel = new QLabel(typeBox, "ObjSlotLabel");
	ObjSlotLabel->setText(" Drop object to attach ");
	typeBoxLayout->addWidget(ObjSlotLabel);
  
	object_field  = new QguWLinput(typeBox, "object_field");
	object_field->acceptObject = true;
	object_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(object_field);
  
	infoSlot = new QLabel(typeBox, "infoSlot");
	infoSlot->setText("( Dragging straight to metadata List also works )");
	typeBoxLayout->addWidget(infoSlot);
  
	connect(object_field,SIGNAL(validated_OK()),this, SLOT(updateName()));

	btnBox = new QWidget(this, "btnBox");
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);
	btnLayout->setSpacing(QSAS_profiles->current()->GUImargins);
#if QT_VERSION < 0x040300
	btnLayout->setMargin(0);
#else
	btnLayout->setContentsMargins(0,0,0,0);
#endif

	cancelBtn = new QPushButton("Cancel", btnBox, "cancelBtn");
	acceptBtn = new QPushButton("Create...", btnBox, "acceptBtn");

	btnBox->layout()->addWidget(cancelBtn);
	btnBox->layout()->addWidget(acceptBtn);

	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()),this,SLOT(attachObject()));
	
	typeBox->setLayout(typeBoxLayout);
	typeBox->adjustSize();
  
}
#endif
  
void QbuAttachWindow::updateName(){
	  
     QString newText = QString(object_field->textObject().latin1());
	int ptr = newText.findRev( "/");
	if (ptr >= 0) newText.remove(0, ptr+1);
     
	// disallow drop of itself into list
     if (strcmp(thisViewer->objName(), newText ) == 0 ) {
	  QuMsgBar(" drop of object as its own xref", true);
	  object_field->setText("self-link");
	  object_field->reValidate();
	} // stop drop of obj on its own xref list
     else{
	   name_field->setText(newText);
	   name_field->reValidate();
	}
  }
  
void  QbuAttachWindow::attachObject(){
    try{
       dobj->change_xref(name_field->text().latin1(), object_field->getSlotObject() );
    }
    catch (Exception &e){
       QuMsgBar(QString(e.msg().c_str()), true);
    }

    thisViewer->refreshList();
    thisViewer->activeEditor->gotInput();
    this->close();

  }

arrDialog::arrDialog(QWidget *parent, const char *name, QbObjViewer* browser, bool modal)
   :  QDialog(parent,name,modal)
#if QT_VERSION < 0x040000
{
  parentBrowser = browser;
  dimSlot = 0;
  QVBoxLayout * centreBox;
  QVGroupBox * dimBox;
  QHBox * btnBox;

  QLabel *slotLabel;
  QPushButton *cancelBtn;
  QPushButton *acceptBtn;

  centreBox = new QVBoxLayout(this, 4, -1, "centreBox");
  centreBox->setAutoAdd(TRUE);
  dimBox = new QVGroupBox(" Choose Array Dimensions " , this, "dimBox");
  dimBox->setMinimumWidth(250);
  dimBox->setMargin(QSAS_profiles->current()->GUImargins);
  dimSlot = new QguWLinput(dimBox, "dimSlot");
  dimSlot->acceptText = true;
  dimSlot->acceptArrayObject = true;

  slotLabel = new QLabel(dimBox, "slotLabel");
  slotLabel->setText("drop object with dimensions \n    or \ntype in array sizes, e.g. 3, 16, 36  or  3, 3");

  btnBox = new QHBox(this, "btnBox");
  cancelBtn = new QPushButton("Cancel", btnBox, "cancelBtn");
  acceptBtn = new QPushButton("Create...", btnBox, "acceptBtn");

  connect(cancelBtn,SIGNAL(clicked()), this, SLOT(close()));
  connect(acceptBtn,SIGNAL(clicked()), this, SLOT(editArray()));

  dimBox->adjustSize();
}
#else
{
	parentBrowser = browser;
	dimSlot = 0;
	QVBoxLayout * centreBox;
	QGroupBox * dimBox;

	QWidget * btnBox;

	QLabel *slotLabel;
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;

	centreBox = new QVBoxLayout(this, 4, -1, "centreBox");
	centreBox->setAutoAdd(TRUE);
	dimBox = new QGroupBox(" Choose Array Dimensions ", this);
	QVBoxLayout * dimBoxLayout=new QVBoxLayout;
	dimBox->setMinimumWidth(250);
	dimBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
	
	dimSlot = new QguWLinput(dimBox, "dimSlot");
	dimSlot->acceptText = true;
	dimSlot->acceptArrayObject = true;
	dimBoxLayout->addWidget(dimSlot);

	slotLabel = new QLabel(dimBox, "slotLabel");
	slotLabel->setText("drop object with dimensions \n    or \ntype in array sizes, e.g. 3, 16, 36  or  3, 3");
	dimBoxLayout->addWidget(slotLabel);
	
	btnBox = new QWidget(this, "btnBox");
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);
#if QT_VERSION < 0x040300
	btnLayout->setMargin(0);
#else
	btnLayout->setContentsMargins(0,0,0,0);
#endif
	cancelBtn = new QPushButton("Cancel", btnBox, "cancelBtn");
	acceptBtn = new QPushButton("Create...", btnBox, "acceptBtn");
	btnLayout->addWidget(cancelBtn);
	btnLayout->addWidget(acceptBtn);

	connect(cancelBtn,SIGNAL(clicked()), this, SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()), this, SLOT(editArray()));

	dimBox->setLayout(dimBoxLayout);
	dimBox->adjustSize();
}
#endif

// ---------------------------------------------------

void arrDialog::editArray(){ 

 if(QuSlotDebug())
 {
	printf("arrDialog::editArray %ld\n", this);
	fflush(stdout);
 }
 
  QdObject_var target = dimSlot->getSlotObject();
  if( target.is_nil() ){
    QuMsgBar("Bad Dimension Input", true);
    return;
  }
  
  QdRMatrix_var m_obj = QdRMatrix_var::narrow(target);
  QdRMatrixSeq_var ms_obj = QdRMatrixSeq_var::narrow(target);
  QdString_var str_obj = QdString_var::narrow(target);
  Dim mDim;
  if( !m_obj.is_nil() ) {
     mDim = m_obj->dimdata();
  }
  else if (!ms_obj.is_nil()){
     mDim = ms_obj[0].dimdata();
    
  }
  else if( !str_obj.is_nil()) {
     QString dimStr = QString( str_obj->c_str()) ;
	int rankM = 0;
	vector <int> dimList;
	int ptr = dimStr.find(",");
	QString token = QString(dimStr.latin1());
	if( ptr > 0) { // last entry has np comma
	  token.truncate(ptr);
	  dimStr.remove(0, ptr+1);
	}
	else dimStr = QString("stop");
	
	bool OK = true;
	while (OK){
	   // handle 5 dims for now
	   int sizeN = token.toInt(&OK);
	   if (!OK) break;
	   dimList.push_back(sizeN);
	   rankM++;
	   ptr = dimStr.find(",");
	   token = QString(dimStr.latin1());
	   if( ptr > 0) {	// last entry has no comma
	     token.truncate(ptr);
	     dimStr.remove(0, ptr+1);
        }
	   else dimStr = QString("stop");
	}
	
	if(rankM == 0){
       QuMsgBar("Bad Dimension Input string", true);
       return;
	}
	
	  mDim = Dim(dimList);

  }
  else{
    QuMsgBar("Input not matrix type or text", true);
    return;
  }
  
  DataBrowseWindow *QBROWSER_widget;
	
  if (arrType == QB_DOUBLE){ 
  
     QdRMatrix_var dobj = new QdRMatrix(mDim, 0);
     QString context;
     QdObject_var parentObj;
     if( parentBrowser == 0 ) context = QwuiPutOnWL(dobj, "newArray", "Constants");
     else {
        context = QString("newArray");
        parentObj = parentBrowser->activeEditor->edited_var;
     }
     
     QdString_var SI_ptr = new QdString("1>?");
     dobj->change_xref(SI_CONVERSION, SI_ptr);
  
     QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "newArray", dobj, context);
 
  }
  else if (arrType == QB_STRING){ 
  
     QdSMatrix_var dobj = new QdSMatrix(mDim, " ");

     QString context;
     QdObject_var parentObj;
     if( parentBrowser == 0 ) context = QwuiPutOnWL(dobj, "newTextArray", "Constants");
     else {
        context = QString("newTextArray");
	parentObj = parentBrowser->activeEditor->edited_var;
     }
  
     QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "newTextArray", dobj, context);

  }
    
   // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed

  if(parentBrowser != 0){
     if (QBROWSER_widget == 0) return;
  
     connect(QBROWSER_widget, SIGNAL(objChanged(const QString&)), parentBrowser,  SLOT(refreshList()) );
     connect(QBROWSER_widget, SIGNAL(objChanged(const QString &)), parentBrowser->activeEditor,  SLOT(gotInput()) );

  
  }
  
  this->close();

}

