#include "GuiConfig.h"

GuiConfig::GuiConfig()
{
#if QT_VERSION < 0x040000
	QBoxLayout * v_layout=new QBoxLayout(this, QBoxLayout::TopToBottom);
#else
	QBoxLayout * v_layout=new QBoxLayout(QBoxLayout::TopToBottom, this);
#endif
	
	v_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	v_layout->setMargin(QSAS_profiles->current()->GUImargins);
	
	QBoxLayout * h_layout_1=new QBoxLayout(QBoxLayout::LeftToRight);
	QBoxLayout * h_layout_2=new QBoxLayout(QBoxLayout::LeftToRight);
	QBoxLayout * h_layout_3=new QBoxLayout(QBoxLayout::LeftToRight);
	QBoxLayout * h_layout_4=new QBoxLayout(QBoxLayout::LeftToRight);
	QBoxLayout * h_layout_5=new QBoxLayout(QBoxLayout::LeftToRight);

	h_layout_1->setSpacing(QSAS_profiles->current()->GUImargins);
	h_layout_1->setMargin(QSAS_profiles->current()->GUImargins);
	
	h_layout_2->setSpacing(QSAS_profiles->current()->GUImargins);
	h_layout_2->setMargin(QSAS_profiles->current()->GUImargins);
	
	h_layout_3->setSpacing(QSAS_profiles->current()->GUImargins);
	h_layout_3->setMargin(QSAS_profiles->current()->GUImargins);
	
	h_layout_4->setSpacing(QSAS_profiles->current()->GUImargins);
	h_layout_4->setMargin(QSAS_profiles->current()->GUImargins);
	
	h_layout_5->setSpacing(QSAS_profiles->current()->GUImargins);
	h_layout_5->setMargin(QSAS_profiles->current()->GUImargins);
	
	
	QLabel * tempLabel=new QLabel("Default main window height (in characters)*: ", this);
#if QT_VERSION >= 0x040000
	tempLabel->setWordWrap(true);
#endif
	h_layout_1->addWidget(tempLabel);
	
	heightLine=new QLineEdit(this);
	heightLine->setValidator(new QIntValidator(10, 100, heightLine));
	h_layout_1->addWidget(heightLine);
	
	tempLabel=new QLabel("Default size for the plot interface *", this);
#if QT_VERSION >= 0x040000
	tempLabel->setWordWrap(true);
#endif
	h_layout_2->addWidget(tempLabel);
	
	QpuiSizeLine=new QLineEdit(this);
	QpuiSizeLine->setEnabled(false);
	h_layout_2->addWidget(QpuiSizeLine);
	
	QpuiSizeButton=new QPushButton("Get current size", this);
	h_layout_2->addWidget(QpuiSizeButton);
	connect(QpuiSizeButton, SIGNAL(clicked()), this, SLOT(getQpuiSize()));
	
	tempLabel=new QLabel("Font size in pixels: ", this);
#if QT_VERSION >= 0x040000
	tempLabel->setWordWrap(true);
#endif
	h_layout_3->addWidget(tempLabel);
	
	fontSizeBox=new QSpinBox(this);
#if QT_VERSION < 0x040000
	fontSizeBox->setMinValue(4);
	fontSizeBox->setMaxValue(20);
#else
	fontSizeBox->setMinimum(4);
	fontSizeBox->setMaximum(20);
#endif
	fontSizeBox->setSpecialValueText("Default");
	h_layout_3->addWidget(fontSizeBox);
	
	tempLabel=new QLabel("Compact GUI* ", this);
#if QT_VERSION >= 0x040000
	tempLabel->setWordWrap(true);
#endif
	h_layout_4->addWidget(tempLabel);

	compactUIBox=new QComboBox(this);
	compactUIBox->insertItem("Off");
	compactUIBox->insertItem("On");
	h_layout_4->addWidget(compactUIBox);
	
	tempLabel=new QLabel("Interactive Interval Editor Button ", this);
#if QT_VERSION >= 0x040000
	tempLabel->setWordWrap(true);
#endif
	h_layout_5->addWidget(tempLabel);

	timeIntervalButtonBox=new QComboBox(this);
	timeIntervalButtonBox->insertItem("Left or Right click");
	timeIntervalButtonBox->insertItem("Right click only");
	h_layout_5->addWidget(timeIntervalButtonBox);
	
	v_layout->addLayout(h_layout_1);
	v_layout->addLayout(h_layout_2);
	v_layout->addLayout(h_layout_3);
	v_layout->addLayout(h_layout_4);
	v_layout->addLayout(h_layout_5);
	QPUI_widget=NULL;
}

void GuiConfig::getQpuiSize()
{
	if(QPUI_widget!=NULL)
	{
		QSize size=QPUI_widget->size();
		qpuiHeight=size.height();
		qpuiWidth=size.width();
		QpuiSizeLine->setText(QString("%1 x %2").arg(qpuiWidth).arg(qpuiHeight));
	}
}

void GuiConfig::updateProfile(QProfile* prof)
{
	prof->Height=heightLine->text().toInt();
	prof->QpuiHeight=qpuiHeight;
	prof->QpuiWidth=qpuiWidth;
	prof->fontSize=fontSizeBox->value();
	if(prof->fontSize<5) prof->fontSize=0;
	
#if QT_VERSION < 0x040000
	switch(compactUIBox->currentItem())
#else
	switch(compactUIBox->currentIndex())
#endif
	{
		case 1:
			prof->GUImargins=1;
			break;
		case 0:
		default:
			prof->GUImargins=4;
			break;
	}
	
#if QT_VERSION < 0x040000
	switch(timeIntervalButtonBox->currentItem())
#else
	switch(timeIntervalButtonBox->currentIndex())
#endif
	{
		case 1:
			prof->timeIntervalButton=1;
			break;
		case 0:
		default:
			prof->timeIntervalButton=0;
			break;
	}	
}

void GuiConfig::setSelectedProfile(QProfile * p)
{
	heightLine->setText(QString().setNum(p->Height));
	qpuiHeight=p->QpuiHeight;
	qpuiWidth=p->QpuiWidth;
	QpuiSizeLine->setText(QString("%1 x %2").arg(qpuiWidth).arg(qpuiHeight));
	if(p->fontSize<5)
	{
		fontSizeBox->setValue(4);
	}
	else
	{
		fontSizeBox->setValue(p->fontSize);
	}
	
#if QT_VERSION < 0x040000
	switch(p->GUImargins)
	{
		case 1:
			compactUIBox->setCurrentItem(1);
			break;
		default:
			compactUIBox->setCurrentItem(0);
			break;
	}
	switch(p->timeIntervalButton)
	{
		case 1:
			timeIntervalButtonBox->setCurrentItem(1);
			break;
		default:
			timeIntervalButtonBox->setCurrentItem(0);
			break;
	}
#else
	switch(p->GUImargins)
	{
		case 1:
			compactUIBox->setCurrentIndex(1);
			break;
		default:
			compactUIBox->setCurrentIndex(0);
			break;
	}
	switch(p->timeIntervalButton)
	{
		case 1:
			timeIntervalButtonBox->setCurrentIndex(1);
			break;
		default:
			timeIntervalButtonBox->setCurrentIndex(0);
			break;
	}
#endif
}
