#include "qt_version.h"
#include <qobject.h>

#if QT_VERSION < 0x040000
#include "PlotConfig2.h"
#else
#include "PlotConfig2_qt4.h"
#endif

PlotConfig2::PlotConfig2()
{
#if QT_VERSION < 0x040000
	QBoxLayout * mainLayout=new QBoxLayout(this, QBoxLayout::TopToBottom);
	
	mainLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	mainLayout->setMargin(QSAS_profiles->current()->GUImargins);
	
	QGroupBox * fontsGroup=new QGroupBox(5, Qt::Vertical, "Fonts in plots", this);
	
	fontsGroup->setInsideSpacing(QSAS_profiles->current()->GUImargins);
	fontsGroup->setInsideMargin(QSAS_profiles->current()->GUImargins);
	
	QLabel * warning=new QLabel("Leave blank for default fonts.\nFont settings are local and linked to profiles, thus they are not saved with the plots.\nPDF and EPS plots embed the chosen fonts.", fontsGroup);
	
	QWidget * sansWidget=new QWidget(fontsGroup);
	QHBoxLayout * sansLayout=new QHBoxLayout(sansWidget);
	sansLayout->setAutoAdd(true);
	new QLabel("Sans font ", sansWidget);
	sansFontEdit=new QLineEdit(sansWidget);
	QPushButton * sansFontButton=new QPushButton("Select...", sansWidget);
	connect(sansFontButton, SIGNAL(clicked()), this, SLOT(changeSansFont()));
	
	QWidget * serifWidget=new QWidget(fontsGroup);
	QHBoxLayout * serifLayout=new QHBoxLayout(serifWidget);
	serifLayout->setAutoAdd(true);
	new QLabel("Serif font ", serifWidget);
	serifFontEdit=new QLineEdit(serifWidget);
	QPushButton * serifFontButton=new QPushButton("Select...", serifWidget);
	connect(serifFontButton, SIGNAL(clicked()), this, SLOT(changeSerifFont()));
	
	QWidget * monotypeWidget=new QWidget(fontsGroup);
	QHBoxLayout * monotypeLayout=new QHBoxLayout(monotypeWidget);
	monotypeLayout->setAutoAdd(true);
	new QLabel("Monotype font ", monotypeWidget);
	monotypeFontEdit=new QLineEdit(monotypeWidget);
	QPushButton * monotypeFontButton=new QPushButton("Select...", monotypeWidget);
	connect(monotypeFontButton, SIGNAL(clicked()), this, SLOT(changeMonotypeFont()));
	
	QWidget * decorativeWidget=new QWidget(fontsGroup);
	QHBoxLayout * decorativeLayout=new QHBoxLayout(decorativeWidget);
	decorativeLayout->setAutoAdd(true);
	new QLabel("Decorative font ", decorativeWidget);
	decorativeFontEdit=new QLineEdit(decorativeWidget);
	QPushButton * decorativeFontButton=new QPushButton("Select...", decorativeWidget);
	connect(decorativeFontButton, SIGNAL(clicked()), this, SLOT(changeDecorativeFont()));

	
	mainLayout->addWidget(fontsGroup);
	
	
	QGroupBox * colourTableGroup=new QGroupBox(1, Qt::Horizontal, "Custom colour table", this);
	
	colourTableGroup->setInsideSpacing(QSAS_profiles->current()->GUImargins);
	colourTableGroup->setInsideMargin(QSAS_profiles->current()->GUImargins);
	
	new QLabel("See online help for syntax. Undefined behaviour for wrong syntaxes.", colourTableGroup);
	colourTableEdit=new QLineEdit(colourTableGroup);

	mainLayout->addWidget(colourTableGroup);
	
	QGroupBox * plotEltsGroup=new QGroupBox(1, Qt::Horizontal, "Defaults for Plot Elements", this);
	
	plotEltsGroup->setInsideSpacing(QSAS_profiles->current()->GUImargins);
	plotEltsGroup->setInsideMargin(QSAS_profiles->current()->GUImargins);
	
	new QLabel("Default line width.", plotEltsGroup);
	lineWidthBox=new QSpinBox(1, 10, 1, plotEltsGroup);

    new QLabel("Gap threshold (times smallest interval between data points).", plotEltsGroup);
    gapToleranceBox=new QLineEdit(plotEltsGroup);
    gapToleranceBox->setValidator(new QDoubleValidator(gapToleranceBox));
    
	mainLayout->addWidget(plotEltsGroup);
	
#else // Qt4
	QGridLayout * mainLayout=new QGridLayout(this);
	
	
	QGroupBox * fontsGroup=new QGroupBox("Fonts in plots", this);
	QGridLayout* fontsLayout=new QGridLayout;
	fontsLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	QLabel * warning=new QLabel("Selected profile determines the font used in a screen plot. PDF and EPS plots embed the chosen fonts.", fontsGroup);
	warning->setWordWrap(true);
	fontsLayout->addWidget(warning, 0, 0, 1, 3);
	
	fontsLayout->addWidget(new QLabel("Sans font", fontsGroup), 1, 0);
	fontsLayout->addWidget(new QLabel("Serif font", fontsGroup), 2, 0);
	fontsLayout->addWidget(new QLabel("Monotype font", fontsGroup), 3, 0);
	fontsLayout->addWidget(new QLabel("Decorative font", fontsGroup), 4, 0);
	
	sansFontCheck=new QCheckBox("Default", fontsGroup);
	fontsLayout->addWidget(sansFontCheck, 1, 1);
	connect(sansFontCheck, SIGNAL(stateChanged(int)), this, SLOT(sansCheckChanged()));
	sansFontCombo=new QFontComboBox(fontsGroup);
	fontsLayout->addWidget(sansFontCombo, 1, 2);
	
	serifFontCheck=new QCheckBox("Default", fontsGroup);
	fontsLayout->addWidget(serifFontCheck, 2, 1);
	connect(serifFontCheck, SIGNAL(stateChanged(int)), this, SLOT(serifCheckChanged()));
	serifFontCombo=new QFontComboBox(fontsGroup);
	fontsLayout->addWidget(serifFontCombo, 2, 2);
	
	monotypeFontCheck=new QCheckBox("Default", fontsGroup);
	fontsLayout->addWidget(monotypeFontCheck, 3, 1);
	connect(monotypeFontCheck, SIGNAL(stateChanged(int)), this, SLOT(monotypeCheckChanged()));
	monotypeFontCombo=new QFontComboBox(fontsGroup);
	fontsLayout->addWidget(monotypeFontCombo, 3, 2);
	
	decorativeFontCheck=new QCheckBox("Default", fontsGroup);
	fontsLayout->addWidget(decorativeFontCheck, 4, 1);
	connect(decorativeFontCheck, SIGNAL(stateChanged(int)), this, SLOT(decorativeCheckChanged()));
	decorativeFontCombo=new QFontComboBox(fontsGroup);
	fontsLayout->addWidget(decorativeFontCombo, 4, 2);
	
	fontsGroup->setLayout(fontsLayout);
	
	mainLayout->addWidget(fontsGroup, 0, 0);
	
#if QT_VERSION >= 0x040300
	svgFontsCheck=new QCheckBox("Vectorise fonts in SVG, EPS or PDF export", this);
	mainLayout->addWidget(svgFontsCheck, 1, 0);
#endif
	
	QGroupBox * colourTableGroup=new QGroupBox("Custom colour table", this);
	QVBoxLayout * colourTableLayout=new QVBoxLayout;
	colourTableLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	colourTableLayout->addWidget(new QLabel("See online help for syntax (""Profiles"" page). Undefined behaviour for wrong syntaxes."));
	colourTableEdit=new QLineEdit(this);
	colourTableLayout->addWidget(colourTableEdit);
	
	colourTableGroup->setLayout(colourTableLayout);
	
	mainLayout->addWidget(colourTableGroup);
	
	
	QGroupBox * plotEltsGroup=new QGroupBox("Defaults for Plot Elements", this);
	QVBoxLayout * plotEltsLayout=new QVBoxLayout;
	plotEltsLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	plotEltsLayout->addWidget(new QLabel("Default line width."));
	lineWidthBox=new QDoubleSpinBox(this);
	lineWidthBox->setRange(0.5, 10.);
	lineWidthBox->setDecimals(2);
	lineWidthBox->setSingleStep(0.1);
	plotEltsLayout->addWidget(lineWidthBox);
	
	plotEltsLayout->addWidget(new QLabel("Base font size (for labels, +0.3 for titles, +0.1 for subtitles)."));
	baseFontSizeBox=new QDoubleSpinBox(this);
	baseFontSizeBox->setRange(0.1, 10.);
	baseFontSizeBox->setDecimals(2);
	baseFontSizeBox->setSingleStep(0.1);
	plotEltsLayout->addWidget(baseFontSizeBox);

    plotEltsLayout->addWidget(new QLabel("Gap threshold (times smallest interval between data points)."));
    gapToleranceBox=new QDoubleSpinBox(this);
    gapToleranceBox->setRange(0.1, 100.);
    gapToleranceBox->setDecimals(2);
    gapToleranceBox->setSingleStep(0.2);
    plotEltsLayout->addWidget(gapToleranceBox);
	
	plotEltsGroup->setLayout(plotEltsLayout);
	
	mainLayout->addWidget(plotEltsGroup);
#endif
}





#if QT_VERSION < 0x040000
void PlotConfig2::updateProfile(QProfile * p)
{
	p->SansFont=sansFontEdit->text();
	p->SerifFont=serifFontEdit->text();
	p->MonotypeFont=monotypeFontEdit->text();
	p->DecorativeFont=decorativeFontEdit->text();
#if QT_VERSION >= 0x040300
	p->VectoriseSVGFonts=svgFontsCheck->isChecked();
#endif
	p->ColourTable=colourTableEdit->text().replace(' ', "");
	p->LineWidth=lineWidthBox->value();
    p->gapTolerance=gapToleranceBox->text().toDouble();
}

void PlotConfig2::setSelectedProfile(QProfile * p)
{
	sansFontEdit->setText(p->SansFont);
	serifFontEdit->setText(p->SerifFont);
	monotypeFontEdit->setText(p->MonotypeFont);
	decorativeFontEdit->setText(p->DecorativeFont);
#if QT_VERSION >= 0x040300
	svgFontsCheck->setChecked(p->VectoriseSVGFonts);
#endif
	colourTableEdit->setText(p->ColourTable);
	lineWidthBox->setValue(p->LineWidth);
    gapToleranceBox->setText(QString("%1").arg(p->gapTolerance));
}

void PlotConfig2::changeSansFont()
{
	bool ok;
	QFont currentFont;
	currentFont.setStyleHint(QFont::SansSerif);
	currentFont.setFamily(sansFontEdit->text());
	QFont font = QFontDialog::getFont( &ok, currentFont, this);
	if(ok)
	{
		sansFontEdit->setText(font.family());
	}
}

void PlotConfig2::changeSerifFont()
{
	bool ok;
	QFont currentFont;
	currentFont.setStyleHint(QFont::Serif);
	currentFont.setFamily(serifFontEdit->text());
	QFont font = QFontDialog::getFont( &ok, currentFont, this);
	if(ok)
	{
		serifFontEdit->setText(font.family());
	}
}

void PlotConfig2::changeMonotypeFont()
{
	bool ok;
	QFont currentFont;
	currentFont.setStyleHint(QFont::TypeWriter);
	currentFont.setFamily(monotypeFontEdit->text());
	QFont font = QFontDialog::getFont( &ok, currentFont, this);
	if(ok)
	{
		monotypeFontEdit->setText(font.family());
	}
}

void PlotConfig2::changeDecorativeFont()
{
	bool ok;
	QFont currentFont;
	currentFont.setStyleHint(QFont::Decorative);
	currentFont.setFamily(decorativeFontEdit->text());
	QFont font = QFontDialog::getFont( &ok, currentFont, this);
	if(ok)
	{
		decorativeFontEdit->setText(font.family());
	}
}
#else

void PlotConfig2::updateProfile(QProfile * p)
{
// 	p->SansFont=sansFontEdit->text();
	if(sansFontCheck->isChecked())
	{
		p->SansFont="";
	}
	else
	{
		p->SansFont=sansFontCombo->currentFont().family();
	}
	
	if(serifFontCheck->isChecked())
	{
		p->SerifFont="";
	}
	else
	{
		p->SerifFont=serifFontCombo->currentFont().family();
	}

	if(monotypeFontCheck->isChecked())
	{
		p->MonotypeFont="";
	}
	else
	{
		p->MonotypeFont=monotypeFontCombo->currentFont().family();
	}

	if(decorativeFontCheck->isChecked())
	{
		p->DecorativeFont="";
	}
	else
	{
		p->DecorativeFont=decorativeFontCombo->currentFont().family();
	}
	
#if QT_VERSION >= 0x040300
	p->VectoriseSVGFonts=svgFontsCheck->isChecked();
#endif
	p->ColourTable=colourTableEdit->text().replace(' ', "");
	p->LineWidth=lineWidthBox->value();
	p->BaseFontSize=baseFontSizeBox->value();
    p->gapTolerance=gapToleranceBox->value();
}

void PlotConfig2::setSelectedProfile(QProfile * p)
{

	int index=sansFontCombo->findText(p->SansFont);
	QFont font;
	font.setStyleHint(QFont::SansSerif);
	font.setFamily(p->SansFont);
	if(index<0)
	{
		sansFontCheck->setChecked(true);
		sansFontCombo->setEnabled(false);
	}
	else
	{
		sansFontCheck->setChecked(false);
		sansFontCombo->setEnabled(true);
		sansFontCombo->setCurrentIndex(index);
	}
	sansFontCombo->setCurrentFont(font);
	

	font.setStyleHint(QFont::Serif);
	font.setFamily(p->SerifFont);
	index=serifFontCombo->findText(p->SerifFont);
	if(index<0)
	{
		serifFontCheck->setChecked(true);
		serifFontCombo->setEnabled(false);
	}
	else
	{
		serifFontCheck->setChecked(false);
		serifFontCombo->setEnabled(true);
		serifFontCombo->setCurrentIndex(index);
	}
	serifFontCombo->setCurrentFont(font);

	font.setStyleHint(QFont::TypeWriter);
	font.setFamily(p->MonotypeFont);
	index=monotypeFontCombo->findText(p->MonotypeFont);
	if(index<0)
	{
		monotypeFontCheck->setChecked(true);
		monotypeFontCombo->setEnabled(false);
	}
	else
	{
		monotypeFontCheck->setChecked(false);
		monotypeFontCombo->setEnabled(true);
		monotypeFontCombo->setCurrentIndex(index);
	}
	monotypeFontCombo->setCurrentFont(font);

	font.setStyleHint(QFont::Decorative);
	font.setFamily(p->DecorativeFont);
	index=decorativeFontCombo->findText(p->DecorativeFont);
	if(index<0)
	{
		decorativeFontCheck->setChecked(true);
		decorativeFontCombo->setEnabled(false);
	}
	else
	{
		decorativeFontCheck->setChecked(false);
		decorativeFontCombo->setEnabled(true);
		decorativeFontCombo->setCurrentIndex(index);
	}
	decorativeFontCombo->setCurrentFont(font);

#if QT_VERSION >= 0x040300
	svgFontsCheck->setChecked(p->VectoriseSVGFonts);
#endif
	colourTableEdit->setText(p->ColourTable);
	lineWidthBox->setValue(p->LineWidth);

	baseFontSizeBox->setValue(p->BaseFontSize);
    gapToleranceBox->setValue(p->gapTolerance);

}

void PlotConfig2::sansCheckChanged()
{
	sansFontCombo->setEnabled(!sansFontCheck->isChecked());
}

void PlotConfig2::serifCheckChanged()
{
	serifFontCombo->setEnabled(!serifFontCheck->isChecked());
}

void PlotConfig2::monotypeCheckChanged()
{
	monotypeFontCombo->setEnabled(!monotypeFontCheck->isChecked());
}

void PlotConfig2::decorativeCheckChanged()
{
	decorativeFontCombo->setEnabled(!decorativeFontCheck->isChecked());
}
#endif