#include "qprofui.h"
#if QT_VERSION < 0x040000
#include <qmessagebox.h>
#include <qpopupmenu.h>
#else
#include <QMessageBox>
#include <QMenu>
#endif

PresetSelector::PresetSelector(QWidget * parent) : QDialog(parent)
{
	//QWidget* mainWidget=new QWidget(this);

	QGridLayout * layout=new QGridLayout(/*mainWidget*/this);
	
	QComboBox * combo=new QComboBox(/*mainWidget*/this);
	layout->addMultiCellWidget(combo, 0, 0, 0, 1);
	
	std::vector<QString> names;
	
	QSAS_profiles->names(names);
	
	for(int i=0; i<names.size(); ++i)
	{
		if(names[i]!="Default" && names[i]!="Custom")
			combo->insertItem(names[i]);
	}
	
	connect(combo, SIGNAL(activated(const QString&)), this, SLOT(setSelected(const QString&)));
	
	QPushButton * ok=new QPushButton("OK", /*mainWidget*/this);
	layout->addWidget(ok, 1, 0);
	connect(ok, SIGNAL(clicked()), this, SLOT(okPressed()));
	
	QPushButton * cancel=new QPushButton("Cancel", /*mainWidget*/this);
	layout->addWidget(cancel, 1, 1);
	connect(cancel, SIGNAL(clicked()), this, SLOT(cancelPressed()));
	
	selectedPreset=combo->text(0);
	
	setCaption("Select a preset");
	
	resize(250, 150);
}

void PresetSelector::setSelected(const QString& select)
{
	selectedPreset=select;
}

void PresetSelector::okPressed()
{
	done(QDialog::Accepted);
}

void PresetSelector::cancelPressed()
{
	done(QDialog::Rejected);
}

QProfileEditor::QProfileEditor()
{
	
 	QWidget* mainWidget=new QWidget(this);
	this->setCentralWidget(mainWidget);
	QGridLayout * layout=new QGridLayout(mainWidget);
	
	layout->addWidget(new QLabel("Load preset:", mainWidget), 0, 0);

	profilesList=new QComboBox(mainWidget);

	updateProfilesList();

	layout->addWidget(profilesList, 0, 1);
	
	QTabWidget* tabWidget=new QTabWidget(mainWidget);
	
	pathWidget=new PathConfig;
	tabWidget->addTab(pathWidget, "Paths setup");
	pathWidget->setSelectedProfile(QSAS_profiles->current());
	
	plotWidget=new PlotConfig;
	tabWidget->addTab(plotWidget, "Plot (1/2)");
	plotWidget->setSelectedProfile(QSAS_profiles->current());
	
	plotWidget2=new PlotConfig2;
	tabWidget->addTab(plotWidget2, "Plot (2/2)");
	plotWidget2->setSelectedProfile(QSAS_profiles->current());
	
	guiWidget=new GuiConfig;
	tabWidget->addTab(guiWidget, "User Interface");
	guiWidget->setSelectedProfile(QSAS_profiles->current());
	
	miscWidget=new MiscConfig;
	tabWidget->addTab(miscWidget, "Debug");
	miscWidget->setSelectedProfile(QSAS_profiles->current());
	
#if QT_VERSION < 0x040000
	layout->addMultiCellWidget(tabWidget, 1, 1, 0, 2);
	
	layout->addMultiCellWidget(new QLabel("Options marked \"*\" will take effect next time you start QSAS", mainWidget), 2, 2, 0, 2);
#else
	layout->addWidget(tabWidget, 1, 0, 1, 3);
	
	layout->addWidget(new QLabel("Options marked \"*\" will take effect next time you start QSAS", mainWidget), 2, 0, 1, 3);
#endif

	applyButton=new QPushButton("Use these settings", mainWidget);
	layout->addWidget(applyButton, 3, 0);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	
	saveAsButton=new QPushButton("Save as preset", mainWidget);
	layout->addWidget(saveAsButton, 3, 1);
	connect(saveAsButton, SIGNAL(clicked()), this, SLOT(saveAs()));
	
	deleteButton=new QPushButton("Delete...", mainWidget);
	layout->addWidget(deleteButton, 3, 2);
	connect(deleteButton, SIGNAL(clicked()), this, SLOT(deletePreset()));


	setCaption("Profiles Editor");
	
	layout->setRowStretch(0, 1);
	layout->setRowStretch(1, 10);
	layout->setRowStretch(2, 1);
	layout->setRowStretch(3, 1);
	
	resize(600, 600);

	this->show();
}


void QProfileEditor::setSelectedProfile(int index)
{
	QString name=profilesList->text(index);
	QProfile * selected=QSAS_profiles->fromName(name);
	if(selected!=NULL)
	{
		plotWidget->setSelectedProfile(selected);
		plotWidget2->setSelectedProfile(selected);
		pathWidget->setSelectedProfile(selected);
		guiWidget->setSelectedProfile(selected);
		miscWidget->setSelectedProfile(selected);
		
		QSAS_profiles->setCurrent(name);
		emit(changedNames());
	}
}

void QProfileEditor::updateProfilesList()
{
	std::vector<QString> names;
	QSAS_profiles->names(names);
	int selected=0;
	profilesList->clear();
#if QT_VERSION < 0x040000
	for(int i=0; i<names.size(); ++i)
	{
		profilesList->insertItem(names[i]);
		if(names[i]==QSAS_profiles->current()->getName()) selected=i;
	}
	profilesList->setCurrentItem(selected);
#else
	for(int i=0; i<names.size(); ++i)
	{
		profilesList->addItem(names[i]);
		if(names[i]==QSAS_profiles->current()->getName()) selected=i;
	}
	profilesList->setCurrentIndex(selected);
#endif

	connect(profilesList, SIGNAL(activated(int)), this, SLOT(setSelectedProfile(int)));
}

void QProfileEditor::apply()
{
	QProfile * prof=new QProfile("Custom");
	plotWidget->updateProfile(prof);
	plotWidget2->updateProfile(prof);
	pathWidget->updateProfile(prof);
	miscWidget->updateProfile(prof);
	guiWidget->updateProfile(prof);
	
	QSAS_profiles->newCustom(prof);
	
	QSAS_profiles->setCurrent("Custom");
	
	updateProfilesList();
	
	emit(changedNames());
	
	if(QSAS_profiles->current()->fontSize>=5)
	{
		QFont f;
		f.setPixelSize(QSAS_profiles->current()->fontSize);
		qApp->setFont(f);
	}
	else
	{
		qApp->setFont(defaultFont);
	}
	
	close();
}

void QProfileEditor::saveAs()
{
	//Get name,
	bool ok;
#if QT_VERSION < 0x040000
	QString newName=QInputDialog::getText("Name for this preset?", "Name:", QLineEdit::Normal, QSAS_profiles->current()->getName(), &ok, this);
#else
	QString newName=QInputDialog::getText(this, "Name for this preset?", "Name:", QLineEdit::Normal, QSAS_profiles->current()->getName(), &ok);
#endif
	if(ok)
	{
		if(newName=="Custom" || newName=="Default")
		{
			QMessageBox::critical(this, "Save failed", "\"Default\" and \"Custom\" are protected preset names");
			return;
		}
		
		if(QSAS_profiles->current()->getName()=="Custom")
		{
			QSAS_profiles->remove("Custom");
		}
		
		bool alreadyExisting=true;
		QProfile * prof=QSAS_profiles->fromName(newName);
		if(prof==NULL)
		{
			alreadyExisting=false;
			prof=new QProfile(newName);
		}
		
		plotWidget->updateProfile(prof);
		plotWidget2->updateProfile(prof);
		pathWidget->updateProfile(prof);
		miscWidget->updateProfile(prof);
		guiWidget->updateProfile(prof);
		
		if(!alreadyExisting) QSAS_profiles->add(prof);
		
		QSAS_profiles->setCurrent(newName);
	
		updateProfilesList();
	
		emit(changedNames());
		
		if(QSAS_profiles->current()->fontSize>=5)
		{
			QFont f;
			f.setPixelSize(QSAS_profiles->current()->fontSize);
			qApp->setFont(f);
		}
		else
		{
			qApp->setFont(defaultFont);
		}
	}

}

void QProfileEditor::deletePreset()
{
	PresetSelector * selector=new PresetSelector(this);
	
	if(selector->exec())
	{
		QSAS_profiles->remove(selector->selectedPreset);
		plotWidget->setSelectedProfile(QSAS_profiles->current());
		plotWidget2->setSelectedProfile(QSAS_profiles->current());
		pathWidget->setSelectedProfile(QSAS_profiles->current());
		miscWidget->setSelectedProfile(QSAS_profiles->current());
		guiWidget->setSelectedProfile(QSAS_profiles->current());
		
		updateProfilesList();

		emit(changedNames());
	}
}

void QProfileEditor::setQpuiWidget(QpWindow * w)
{
	if(guiWidget!=NULL)
	{
		guiWidget->setQpuiWidget(w);
	}
}
