/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
#include <stdlib.h>

#include <qtooltip.h> 
#include <qfiledialog.h> 
#include <qlayout.h> 
#include <qglobal.h>
#include <qsize.h>
#include <qtextedit.h>
#include <qsplitter.h>

#include "qtpl.h"
#include "qwui.h"
#include "qdutil.h"
#include "qplug_if.h"

#if QT_VERSION < 0x040000
#include "qgu.h"
#else
#include "qgu_qt4.h"
#endif

#ifdef WINDOWS
#include <windows.h>
#else
#include <dlfcn.h>     // dynamic-load stuff
#endif

using namespace QSAS; 

static qplugWindow * activeqplugWindow;
extern qwuiWindow* qsasWindow;
extern QplugArgList* qsasPlugArgs;

// qtplMenu Class
// --------------------
// constructor is in qtpl_parser
#if QT_VERSION < 0x040000
 qtplMenu::qtplMenu(QWidget *parent, const char *name)
    :  QPopupMenu(parent,name)
{  
}
#else
qtplMenu::qtplMenu(QWidget *parent, const char *name)
    :  QMenu(QString(name), parent)
{  
}
#endif

// -----------------------

void qtplMenu::qtplOpen(int id){
 if( this->text(id).contains("&Join", true) == 1 || this->text(id).isEmpty()) return;
 
  QString name = QString(menuPath.latin1()); // force deep copy
          name += "/";           
          name += this->text(id);
	  name += ".qtpl";

  QtplOpenByName(name, this->parentWidget());
  
}

// -----------------------

void qtplMenu::addFolderToMenu(const char *folder, bool first)
{ 
  this->menuPath = QString(qsasWindow->qsas_home); // force deep copy
  this->menuPath += "/qtpl/";
  this->menuPath += folder;
  QDir d(this->menuPath);

  if( !d.exists() ) {
    QuMsgBar("Cannot find qtpl folder", this->menuPath, true);
    return;
  }
  
  for(int i=0; i<d.count(); i++){
    QString name = QString(d[i].latin1());
    int posn = name.findRev(".qtpl");
    if(posn <0 ) continue;
    name.truncate(posn);
    this->insertItem(name);
  }
  
  if(first) connect(this, SIGNAL(activated(int)), this, SLOT(qtplOpen(int)) );
  
  //Same thing for subfolders
  QStringList subFolders=d.entryList(QDir::Dirs/* | QDir::NoDotAndDotDot QT4 only*/);
  for(int i=0; i<subFolders.count(); i++)
  {
      if(!subFolders[i].startsWith("."))
      {
        QDir d2(QString(menuPath+"/"+subFolders[i]));
        QStringList filters;
        QStringList subFolders2=d2.entryList(QString("*.qtpl"), QDir::Files);
        
        if(subFolders2.count()>0)
        {
            qtplMenu * temp=new qtplMenu(this);
            temp->menuPath=menuPath+"/"+subFolders[i];
            this->insertItem(subFolders[i], temp);
            
            for(int j=0; j<subFolders2.count(); j++)
            {
                QString name = QString(subFolders2[j].latin1());
                int posn = name.findRev(".qtpl");
                if(posn <0 ) continue;
                name.truncate(posn);
                temp->insertItem(name);
            }
            if(first) connect(temp, SIGNAL(activated(int)), temp, SLOT(qtplOpen(int)) );
        }
      }
 }
  

}

  // qplugWindow  class
  // -----------------------

 qplugWindow::qplugWindow(QWidget *parent, const char *name, QtplParserFile *qtplFile ) 
  : QguWindow(parent,name)
{
	// copy Mac set brushes - will be solid colours on other platforms
   
  qtplf = qtplFile;
  
  setAcceptDrops(true);
  setFocusPolicy(WHEEL_FOCUS);

#if QT_VERSION < 0x040000
  QWidget::setWFlags(Qt::WDestructiveClose);
#else
  setAttribute(Qt::WA_DeleteOnClose);
#endif
  
  _menu_bar->clear();
#if QT_VERSION < 0x040000
  QPopupMenu* file_menu = new QPopupMenu(this,"file_menu");
#else
  QMenu* file_menu = new QMenu(this);
#endif
  menuBar()->insertItem( " File", file_menu );
  file_menu->clear();
  file_menu->insertItem("&Save", this, SLOT(save()));
  file_menu->insertItem("Save &As...", this, SLOT(saveAs()));
  file_menu->insertSeparator();
  file_menu->insertItem("&Restore", this, SLOT(restore()));

   _menu_bar->insertSeparator();
#if QT_VERSION < 0x040000
   QPopupMenu * help = new QPopupMenu( this );
#else
   QMenu * help = new QMenu( this );
#endif
   menuBar()->insertItem( " Help", help );
   help->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
   help->insertSeparator();
   help->insertItem("...Plugins", this, SLOT( launchManPlugins() ));
   help->insertItem("...This Plugin Help", this, SLOT( launchPluginHelp() ));
   help->insertItem("...This Plugin Documentation", this, SLOT( launchDocPlugin() ));
   help->insertSeparator();
   help->insertItem("Module Version", this, SLOT( launchVersion() ));
      
   
   QSplitter *plugVB = new QSplitter( Qt::Vertical, this, "plugVB" );
   setCentralWidget( plugVB );
   
#ifdef QT_VERSION
#if QT_VERSION <0x030000
   scrollW = new Q_SCROLL_VIEW(plugVB, "scrollW");
   scrollW->setResizePolicy(Q_SCROLL_VIEW::AutoOne);
#else
   scrollW = new Q_SCROLL_VIEW(plugVB);
#if QT_VERSION <0x040000
    scrollW->setResizePolicy(Q_SCROLL_VIEW::AutoOneFit);
#else
   scrollW->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
   scrollW->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
#endif // if QT_VERSION <0x040000
#endif // end of #if
#endif // end of #ifdef
   ioWidget=new QWidget(scrollW->viewport(),"ioWidget");
   ioHLayout=new QHBoxLayout(ioWidget);
  
   inputWidget=new QWidget(ioWidget,"inputWidget");
   inputVLayout=new QVBoxLayout(inputWidget);
   outputWidget=new QWidget(ioWidget,"outputWidget");
   outputVLayout=new QVBoxLayout(outputWidget);

   QplugInit(); //  initialize arguments interface
   
   inputWidget->setMinimumSize(inputWidget->sizeHint());
   outputWidget->setMinimumSize(outputWidget->sizeHint());

   ioHLayout->addWidget(inputWidget);
   ioHLayout->addWidget(outputWidget);

   ioWidget->setMinimumSize(ioWidget->sizeHint());
#if QT_VERSION <0x040000
   scrollW->addChild(ioWidget); 
#else
   scrollW->setWidget(ioWidget); 
#endif
   QWidget *vp=scrollW->viewport();
   vp->setMinimumWidth(QTPL_MINWIDTH);
   vp->adjustSize();
   scrollW->setMinimumSize(min(QTPL_MINWIDTH,vp->width()+5),\
		   min(QTPL_MINHEIGHT,vp->height()+5));

   scrollW->adjustSize();

   textArea = new QTextEdit(plugVB,"textArea");
   textArea->setReadOnly(true);

   QWidget * buttons=new QWidget(plugVB);
   
   QGridLayout * buttons_layout=new QGridLayout(buttons);
//    QWidget * buttons1=new QWidget(plugVB);
//    QWidget * buttons2=new QWidget(plugVB);
// #if QT_VERSION < 0x040000
//    QBoxLayout *buttons_layout1=new QBoxLayout(buttons1, QBoxLayout::LeftToRight);
//    QBoxLayout *buttons_layout2=new QBoxLayout(buttons2, QBoxLayout::LeftToRight);
// #else
//    QBoxLayout *buttons_layout1=new QBoxLayout(QBoxLayout::LeftToRight, buttons1);
//    QBoxLayout *buttons_layout2=new QBoxLayout(QBoxLayout::LeftToRight, buttons2);
// #endif
   
   buttons_layout->setSpacing(QSAS_profiles->current()->GUImargins);
   buttons_layout->setMargin(QSAS_profiles->current()->GUImargins);
//    buttons_layout1->setSpacing(4);
//    buttons_layout1->setMargin(4);
//    buttons_layout2->setSpacing(4);
//    buttons_layout2->setMargin(4);
#if QT_VERSION >=0x040000
	buttons->setLayout(buttons_layout);
//    buttons1->setLayout(buttons_layout1);
//    buttons2->setLayout(buttons_layout2);
#endif
   
   
   runBtn = new QPushButton( "Run", buttons, "runBtn" );
   runBtn->setMinimumSize(runBtn->sizeHint());
   runBtn->setDefault(true);
   connect( runBtn, SIGNAL(clicked()), this, SLOT(runPlugin()) );
   
   defaultBtn = new QPushButton( "Default All", buttons, "defaultBtn" );
   defaultBtn->setMinimumSize(defaultBtn->sizeHint());
   connect( defaultBtn, SIGNAL(clicked()), this, SLOT(defaultAll()) );
   
   buttons_layout->addWidget(runBtn, 0, 0);
   buttons_layout->addWidget(defaultBtn, 0, 1);

   saveBtn = new QPushButton( "Save current output as...", buttons, "saveBtn" );
   saveBtn->setMinimumSize(saveBtn->sizeHint());
   connect( saveBtn, SIGNAL(clicked()), this, SLOT(saveLog()) );
   
   dismissBtn = new QPushButton( "Dismiss", buttons, "dismissBtn" );
   dismissBtn->setMinimumSize(dismissBtn->sizeHint());
   connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );

   buttons_layout->addWidget(saveBtn, 1, 0);
   buttons_layout->addWidget(dismissBtn, 1, 1);
   
  //this->setMinimumSize(this->sizeHint());
  this->fromSession = false;
 
  //this->connect(parent, SIGNAL(saveSessionRequest(QString)),   this, SLOT(saveSession(QString)) );
  //this->connect(parent, SIGNAL(restoreSessionRequest(QString)),   this, SLOT(restoreSession(QString)) );
//  not implemented for plugins yet 
  
}
void qplugWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   runBtn->isDefault()) {
		 runBtn->setDefault(false);
		 defaultBtn->setDefault(true);
	  }
	  else if(defaultBtn->isDefault()) {
		 defaultBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 runBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(runBtn->isDefault()) {
        runPlugin();
	 }
	 else if (defaultBtn->isDefault()) defaultAll();
	 else if (dismissBtn->isDefault()) dismiss();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void qplugWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchManual %ld\n", this);
	fflush(stdout);
 }

   qsasWindow->help_window->manEntry("index.html");
}

void qplugWindow::launchManPlugins(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchManPlugins %ld\n", this);
	fflush(stdout);
 }

   qsasWindow->help_window->manEntry("QSAS_analysis.html");
}

void qplugWindow::launchPluginHelp(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchPluginHelp %ld\n", this);
	fflush(stdout);
 }

  QString text;
  text  = QString(qtplf->description);
  if( text == "" || text == " " ) text = QString("Description Not Available");

// set up scrolling text window
  QMainWindow *db = new QMainWindow( 0, "helpDialog");
  QTextEdit *textArea = new QTextEdit(db);
  textArea->setReadOnly(true);
  textArea->setTextFormat(Qt::PlainText);
  textArea->setText(text);
  
  db->setCentralWidget(textArea);
  db->resize(500,300);  
  
  db->show();


}

void qplugWindow::launchDocPlugin(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchDocPlugin %ld\n", this);
	fflush(stdout);
 }

  // Find html doc for this plugin
  QString search_path = QString(qtplName.latin1());// force deep copy
  int posn = search_path.findRev("/");
  search_path.remove(0, posn+1);
  search_path.prepend("PlugIns/");
  posn = search_path.findRev(".qtpl");
  search_path.truncate(posn);
  search_path += QString(".html");

    qsasWindow->help_window->manEntry(search_path.latin1());

}


void qplugWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("qplugWindow::launchVersion %ld\n", this);
	fflush(stdout);
 }


  QString text;
  text  = QString("Version Numbers...\n");
  text += QString(QSAS_VERSION_ID);
  text += "\n Plugin Template Parser ";
  text += QString(QTPL_VERSION);
  QMessageBox::information( this, "QSAS information", text.latin1(), "OK" );

}


void qplugWindow::dismiss(){

  activeqplugWindow = NULL;
  close();
}

void qplugWindow::defaultAll(){
  activeqplugWindow = this;
  QplugSetDefaults();  
 }

 void qplugWindow::saveAs()
 {
   QString _filter = QString("save (*");
   _filter += SAVE_EXTENSION;
   _filter += QString(")");
   QString fn=QFileDialog::getSaveFileName(_filename, _filter, this, "save", "Select file - Save Plugin Settings");
   if(!fn.isEmpty())
   {
     int pos = fn.findRev(SAVE_EXTENSION);
     if (pos < 0) {
       pos = fn.findRev(".");
       if (pos > 0) fn.truncate(pos);
       fn += SAVE_EXTENSION;
     }
     _filename=fn;
     save();
   }
   else
     status("saving aborted ",10000);

 }
      
 void qplugWindow::saveSession(QString folder){
printf("saving plugin\n");

  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.latin1());
  
  // set session folder and save
     this->_filename = QString(folder.latin1());
     this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;
	save();

  // put file name back
  _filename = QString(fileN.latin1());
}

void qplugWindow::saveLog(){
	if(QuSlotDebug())
	{
		printf("qplugWindow::saveLog %ld\n", this);
		fflush(stdout);
	}

	QString s = QFileDialog::getSaveFileName("", "Text (*.txt)", this, "save file dialog", "Choose a filename to save under" );
	
	if(!s.isEmpty())
	{
		ofstream file;
		file.open(s);
		if(!file.fail())
		{
			file << textArea->text().latin1();
			file.close();
		}
	}
}

      
 void qplugWindow::save()
 { 

 if(QuSlotDebug())
 {
	printf("qplugWindow::save %ld\n", this);
	fflush(stdout);
 }
 

    with = 0;
    
    if(_filename.isEmpty())
    {
       saveAs();
       return;
    }
    
    try
    {
      VXDRostream xdr_os(_filename.latin1());
      save(xdr_os);
    }
    catch(Exception &e)
    {
       status(e.msg().c_str(),10000);   
       return;
    }
    status(_filename+" saved",10000);
    return;

 }


void qplugWindow::save(Vostream& v_os)
{
	string guiID;
	guiID=QSAS_VERSION_ID;
	v_os << guiID;   
  
   v_os << with;
   
    DataNamesTable_var qplug_objects;
    try{
      qplug_objects=\
        QguWindow::_saved_objects->resolve_context(Name("qplug"));
    }
    catch(...)
    {;}  
    if(qplug_objects.is_nil())
    {
      qplug_objects=new DataNamesTable();
      if(!QguWindow::_saved_objects.is_nil())
        QguWindow::_saved_objects->bind_context(Name("qplug"),qplug_objects);
    }
  
   for (int i = 0; i < iSlots.size(); i++) {
     v_os << iSlots[i]->ioSlot->text().latin1(); 
     // find object
    QdObject_var dobj;
 
     if(with == 1){
       bool found = false;
       try{
          qplug_objects->resolve(Name(iSlots[i]->ioSlot->text().latin1()));
          found=true;
       }
       catch(...)
       {;}     
       
       string dobj_type;
       if(!found){
           // put data in save file and object on WL
	   // uses QuGetObjByName rather than getSlotObject as we want the parent
         dobj=QuGetObjByName(iSlots[i]->ioSlot->text().latin1()); 
       }
       if(dobj.is_nil())
         dobj_type = NULL_OBJECT;
       else{
         dobj_type = dobj->object_type();
	 qplug_objects->bindx(Name(iSlots[i]->ioSlot->text().latin1()),"");
       }
       v_os << dobj_type;
       if(!(dobj.is_nil()))
         v_os << dobj;
	 
     }
   } 

   // output slots
   for (int i = 0; i < oSlots.size(); i++) {
     v_os << oSlots[i]->ioSlot->text().latin1(); 
   } 

}

void qplugWindow::restore(Vistream& v_is, bool replace)
{

	// no list view in this GUI so replace is ignored
	string guiID; 
	v_is >> guiID;
	char *ptr;
   
	if( QuSlotDebug() ) printf("QplugWindow::restore set ID =  %s\n", guiID.c_str());
  
	if( strstr(guiID.c_str(), SAVE_EXTENSION.latin1()) == NULL)
	{
		// changed to QSAS_VERSION_ID at 2.3.04
		setRestoreVersion(guiID.c_str());
	}
	else
	{
		setRestoreVersion("QSAS 2.3.03"); // old saveset so ensure old Time class is used
	}
	
	v_is >> with;
  
	DataNamesTable_var qplug_objects;
    try{
		qplug_objects=QguWindow::_restored_objects->resolve_context(Name("qplug"));
    }
    catch(...)
    {;}
	
    if(qplug_objects.is_nil())
    {
		qplug_objects=new DataNamesTable();
		if(!QguWindow::_restored_objects.is_nil())
			QguWindow::_restored_objects->bind_context(Name("qplug"),qplug_objects);
    }

	for (int i = 0; i < iSlots.size(); i++) {
		string restore_str;
		v_is >>  restore_str;
		if(with){
			string dobj_type;
			v_is >> dobj_type;
			if(dobj_type != NULL_OBJECT){
				QdObject_var dobj=QdObject::create(dobj_type);
				v_is >> dobj;
				Name dobj_name = restore_str;
				dobj_name.delete_component(0);
				string name_old = restore_str;
				restore_str = QwuiPutOnWL(dobj,name_old.c_str(),"Data").latin1();
				// ensure name on slot is same as WL
				try{
					qplug_objects->unbind(Name(name_old));
				}
				catch(...){;}
				try{
					qplug_objects->bindx(Name(name_old),restore_str);
				}
				catch(...){;}
			}
			else{
				// NULL OBJECT so this has been restored already
				string name_new;
				try{
					name_new=qplug_objects->resolve(Name(restore_str));
				}
				catch(...){;}
				if(name_new != "") restore_str = name_new;      
			}
		}
		iSlots[i]->ioSlot->setText(QString(restore_str.c_str()));
		iSlots[i]->ioSlot->reValidate();
	} 

	for (int i = 0; i < oSlots.size(); i++) {
		string restore_str;
		v_is >>  restore_str;
		oSlots[i]->ioSlot->setText(QString(restore_str.c_str()));
	} 

}

void qplugWindow::runPlugin(){
  activeqplugWindow = this;
      
//   textArea->clear();
  QplugAppendTextDisplay ("\n Running... "); 
    
  // buffer for argument-list
  QplugArgList      qplug_args;
 
  QdObject_var do_ptr;    // for output data objects

#ifdef WINDOWS
  HINSTANCE handle;
  typedef QplugReturnStatus (__cdecl *MYPROC)(QplugArgList*);
  MYPROC fptr;
#else
  void *handle = NULL;                      // for dynamic load
    QplugReturnStatus (*fptr)(QplugArgList *arglist)=NULL;
#endif
  QplugReturnStatus  f_status;


  //  Check there is a template/plug-in loaded to run,
  //  and return promptly if there is not.

  if (this->qtplf == NULL || !(this->qtplf->openedOK))  {
    QplugAppendTextDisplay("\nERROR: no plug-in template loaded.\n");
    return;
  }

  // start with a clean empty list
  // create and init a null ptr for each plug-in-argument

  qplug_args.clear();

  // work through input args, type-checking or creating-from-value 
  bool OK = QplugBuildInputArgsList(&qplug_args);
   
  if ( !OK )  {
    QplugAppendTextDisplay ("\nQplug> ERROR: Not attempting plug-in load because of arg error.\n");
    return;
  }

  // Added by AJA for Arithmetic

  for (int o_count = 0; o_count < oSlots.size(); o_count++) {
       SetSlotText(o_count, oSlots[o_count]->ioSlot->text().latin1()); 
       oSlots[o_count]->argIndex  =  iSlots.size() + o_count;                         
  }  

  //  built input arg list, attempt plug-in load

  handle = NULL;

  // Try to find entry point and the full path to .so used
  
  QplugGetEntryName();     


       // dynamic load of plugin
    
       QplugAppendTextDisplay ("Qplug> using entry name... ");     
       QplugAppendTextDisplay (try_entry_name.latin1());     
       QplugAppendTextDisplay ("\n");     
      
#ifdef WINDOWS
      handle=LoadLibrary(PathAndNameSO.latin1());
	   if( !handle) {
      
          QplugAppendTextDisplay ("Dynamic load ERROR: LoadLibrary returned null, abandoning dynamic-load for\n");
          QplugAppendTextDisplay(qtplf->object_file.latin1());
          QplugAppendTextDisplay(" with path ");
          QplugAppendTextDisplay(PathAndNameSO.latin1());
          QplugAppendTextDisplay("\n");
          QplugAppendTextDisplay("LoadLibrary error... ");
          return;
        }
		
		if ( (fptr = (MYPROC) GetProcAddress(handle, try_entry_name.latin1())/*(QplugReturnStatus_e (*)(QplugArgList *)) GetProcAddress(handle, try_entry_name.latin1())*/) == NULL )  {
      
          QplugAppendTextDisplay ("plugin entry point ERROR: GetProcAddress lookup failed, dlerror contents are:\n  ");
          QplugAppendTextDisplay("\n");
		  if( !FreeLibrary(handle)) QplugAppendTextDisplay("Error freeing the library");
          return;
        }
#else
       handle = dlopen(PathAndNameSO.latin1(), RTLD_LAZY);
	   if( handle == NULL ) {
          QplugAppendTextDisplay ("Dynamic load ERROR: dlopen returned null, abandoning dynamic-load for\a\n");
          QplugAppendTextDisplay(qtplf->object_file.latin1());
          QplugAppendTextDisplay(" with path ");
          QplugAppendTextDisplay(PathAndNameSO.latin1());
          QplugAppendTextDisplay("\n");
          QplugAppendTextDisplay("dlopen error... ");
          QplugAppendTextDisplay(dlerror());
          return;
        }
		
		 if ( (fptr = (QplugReturnStatus_e (*)(QplugArgList *)) dlsym(handle, try_entry_name.latin1())) == NULL )  {
      
          QplugAppendTextDisplay ("plugin entry point ERROR: dlsym lookup failed, dlerror contents are:\n  ");
          QplugAppendTextDisplay(dlerror());
          QplugAppendTextDisplay("\n");
          if( dlclose(handle) != 0 ) QplugAppendTextDisplay(dlerror());
          return;
        }
#endif

      
        try{
            f_status = (*fptr)(&qplug_args);  // ATTEMPT DYNAMIC FUNCTION CALL 
        }
        catch(Exception &e){
            e.print_msg();
            f_status = QPLUG_FAILURE;
        }

     
   switch (f_status)  {

        case QPLUG_WARNING:  // print warning, then fall thru QPLUG_SUCCESS case to put output args on working list
		{
              QplugAppendTextDisplay  ("Qplug> WARNING: plug-in returned warning\n");
              QplugAppendTextDisplay  ("      use the output data objects at your own risk!\n");
		}
        case QPLUG_SUCCESS:
		{
              // put output data-objects on working list
			int nReturned=0;
			for (int o_count = 0; o_count < oSlots.size(); o_count++) {
				int indx = oSlots[o_count]->argIndex;
	      
				// don't crash if plugin doesn't put output on arg list.
				if ( indx >= qplug_args.size() ) continue;
	      
					if ( qplug_args[indx] != NULL) {
					nReturned++;
						do_ptr = (QdObject_var) qplug_args[indx];
						if (do_ptr.is_nil() ) continue; // skip null returns
	       
						char *usrStr = GetSlotText(o_count);  // returned from plug-in   	  
						QdTimeInterval_var ti=QdTimeInterval_var::narrow(do_ptr);
						if ( ti.is_nil() ){
							QwuiPutOnWL(do_ptr, usrStr, "Data").latin1();
						}
						else {
							QwuiPutOnWL(do_ptr, usrStr, "Time Intervals").latin1();
						}

					}  // END of test for non-null output

				}  // END of for loop for output args
			if(nReturned == 1) QplugAppendTextDisplay  ("Qplug> Placing output data object on working list...");
			else if(nReturned > 1)  QplugAppendTextDisplay  ("Qplug> Placing output data objects on working list...");

           QplugAppendTextDisplay("finished.\n");

           break;

		}
        case QPLUG_FAILURE:  // print error, don't touch output args
		{
            QplugAppendTextDisplay  ("Qplug> ERROR: plug-in returned failure.\n");

            break;

		}
        default:  // print panic error and don't touch output args!
		{
            QplugAppendTextDisplay ("Qplug> PANIC: plug-in returned unrecognised status code.\n");
            QplugAppendTextDisplay ("      Check that plug-in returns QplugReturnStatus_e\n");
            QplugAppendTextDisplay ("      No output data objects generated.\n");

            break;
		}
  }  // END of switch on f_status

#ifdef WINDOWS		
        //if( FreeLibrary(handle)==0) QplugAppendTextDisplay("Error freeing the library"); //Crashes... Is there some work to do on how to build the plugins dlls?
#else
		if( dlclose(handle) != 0 ) QplugAppendTextDisplay(dlerror());
#endif
      
   // end dynamic load
  
  return;

}


bool qplugWindow::QplugBuildInputArgsList (QplugArgList *arg_list){

  bool OK;
  
  // Work through Qplug input args, type-checking or creating-from-value 

  for (int i = 0; i < iSlots.size(); i++) {
    // keep track of it's index in the arg list
    iSlots[i]->argIndex = i;

    if (iSlots[i]->type == "STRING")  // copy the string value from the widget
    {
         QdString_var ptr =  new QdString(iSlots[i]->ioSlot->text().latin1());
         arg_list->push_back((QdObject_var)ptr);
    }
    else{

       // ask slot for object
       QdObject_var slotObj = (QdObject_var) iSlots[i]->ioSlot->getSlotObject();
       if (slotObj.is_nil() ) {
         QplugAppendTextDisplay("Incompatible input value for type... ");
         QplugAppendTextDisplay( iSlots[i]->type.latin1());         
		 QplugAppendTextDisplay (iSlots[i]->ioSlot->text().latin1());
         return false;
       }
       arg_list->push_back( slotObj);
    }


  }  // END of for loop through input_args


  return true;

}


void qplugWindow::QplugInit (){

  if(!qtplf || !(qtplf->openedOK))
    return;

  //  Work through the function argument list and create the slots  
  int nOutputs=0;
  for (int i=0; i<qtplf->qtpl_vars.size(); i++) {

    QtplParserVar *pVar = qtplf->qtpl_vars[i];
    if(pVar == NULL) continue;
    
    QString title = pVar->public_name;
    QString text  = pVar->description;

      if(pVar->ioType == "INPUT") 
      {
        QplugSlot *next = new QplugSlot(title, text, pVar->var_type, inputWidget);
	    if ( !(pVar->default_value.isEmpty()) )
	      next->defaultValue = QString(pVar->default_value.latin1());
	    else
	      next->defaultValue = QString("");

	    inputVLayout->addWidget(next);
	  
	    iSlots.push_back(next);
      }
      else if(pVar->ioType == "OUTPUT"){
		nOutputs++;
        QplugSlot *next = new QplugSlot(title, text, QString("NAME"), outputWidget);
	    if ( !(pVar->default_value.isEmpty()) )
	      next->defaultValue = QString(pVar->default_value.latin1());
	    else
	      next->defaultValue = QString("");

	    outputVLayout->addWidget(next);

	    oSlots.push_back(next);
      }
      else{
        QplugAppendTextDisplay("ERROR: unknown argument type for: ");
        QplugAppendTextDisplay(pVar->public_name.latin1());
        QplugAppendTextDisplay("\n");
      }

  }  // END of loop in qtplf args

if(nOutputs == 0)
{
	QplugSlot *dummy = new QplugSlot("No output objects", " See text window \n New display and/or attaches results as attributes", QString("NAME"), outputWidget);
	dummy->defaultValue = QString("");
	outputVLayout->addWidget(dummy);
	dummy->setDisabled(true);

}
   /* Set default values for args that have them */

  QplugSetDefaults();

}


void qplugWindow::QplugSetDefaults (){
  
  //  Set all arguments to their default values.  

  for (int i=0; i <iSlots.size() ; i++){
     iSlots[i]->ioSlot->setText(iSlots[i]->defaultValue);
     iSlots[i]->ioSlot->validateInput(iSlots[i]->ioSlot->text());
  }
  
  for (int i=0; i <oSlots.size() ; i++){
     oSlots[i]->ioSlot->setText(oSlots[i]->defaultValue);
     oSlots[i]->ioSlot->validateInput(oSlots[i]->ioSlot->text());
  }
  
  return;
}

void qplugWindow::QplugGetEntryName (){

  FILE *fp;
  static char * str_NULL = "";
 
  // find path to loaded .qtpl file as a possible location for .so
  QString qtplPath = QString(qtplName.latin1()); // deep copy
  
  int posn = qtplPath.findRev('/');
  if(posn >= 0)  qtplPath.truncate(posn);
  else           qtplPath = "./";
  
  
  QString file_stem = QString(qtplf->object_file.latin1());
  posn = file_stem.findRev('.');
  if(posn >= 0)  file_stem.truncate(posn);

  QString user_lib = QSAS_profiles->current()->UserLibPath;
  
  QString search_path = QString("");		
  if( (qtplf->object_file)[0] != '/') 
      search_path = QString("./");		

  search_path += qtplf->object_file;
  QString windowsPath = QString(qtplf->object_file.latin1());
  int ptr = windowsPath.find(".so");
  windowsPath.replace(ptr, 3, ".dll");

  fp = fopen(search_path.latin1(), "r");
  if ( fp == NULL ){
       search_path = qtplPath; 
       search_path += "/"; 
       search_path += qtplf->object_file;
       fp = fopen(search_path.latin1(), "r");
  }
  if ( fp == NULL && strcmp(user_lib, str_NULL) != 0){
       search_path = user_lib; 
       search_path += "/";
       search_path += qtplf->object_file;
       fp = fopen(search_path.latin1(), "r");
  }
  if ( fp == NULL){
       search_path = QString(qsasWindow->qsas_home); 
       search_path += "/lib/";
       search_path += qtplf->object_file;
       fp = fopen(search_path, "r");
  }
  // try for Windows .dl file
  if ( fp == NULL ){
       search_path = qtplPath; 
       search_path += "/"; 
       search_path += windowsPath;
       fp = fopen(search_path.latin1(), "r");
  }
  if ( fp == NULL && strcmp(user_lib, str_NULL) != 0){
       search_path = user_lib; 
       search_path += "/";
       search_path += windowsPath;
       fp = fopen(search_path.latin1(), "r");
  }
  if ( fp == NULL){
       search_path = QString(qsasWindow->qsas_home); 
       search_path += "/lib/";
       search_path += windowsPath;
       fp = fopen(search_path, "r");
  }

  if ( fp == NULL)  {
    QplugAppendTextDisplay(qtplf->object_file.latin1());
    QplugAppendTextDisplay(" < cannot locate dynamic library  \n");
    
    try_entry_name = "";
    return; 
  }
      
  // We have the correct path if we get here. 

  // keep full path to .so for dynamic load

  PathAndNameSO = QString(search_path.latin1());  // deep copy

  
  // Close file and use system call to find entry point
    
  fclose(fp);
  
  try_entry_name = QString(qtplf->entry_name.latin1());
  
  return;
}



// QplugSlot class
//-----------------

 QplugSlot::QplugSlot(const QString &title, const QString &text, 
                      QString varType,
                      QWidget *parent, const char *name)
    :  Q_V_GROUP_BOX(title, parent, name)
{ 

  type = varType;
  
  ioSlot = new QguWLinput(this, "ioSlot");
  ioSlot->setEnabled(TRUE);
  ioSlot->setMinimumWidth( 300 );
#if QT_VERSION >= 0x040000
  QVBoxLayout * mainLayout=new QVBoxLayout;
  
  mainLayout->addWidget(ioSlot);
#endif


  QLabel *label = new QLabel(text, this, "ioLabel");
  label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
#if QT_VERSION >= 0x040000
  mainLayout->addWidget(label);
  
  setLayout(mainLayout);
#endif
  

         if(type == "NAME"){
            toolTipText = "NAME OF OUTPUT OBJECT FOR WORKING LIST";
 	        ioSlot->acceptText = true;
         }
         else if(type == "ANY_DATA_OBJECT"){
            toolTipText = "ANY DATA OBJECT";
            ioSlot->acceptNumber = true;
            ioSlot->acceptObjectPlus = true;
            ioSlot->acceptTimeSubset = true;
            ioSlot->acceptTimeSlice = true;
         }
         else if(type == "INTEGER"){
            toolTipText = "INTEGER";
            ioSlot->acceptInteger = true;
            ioSlot->acceptScalarObject = true;
            ioSlot->acceptTimeSlice = true;
         }
         else if(type == "FLOAT"){
            toolTipText = "FLOAT";
            ioSlot->acceptNumber = true;
            ioSlot->acceptScalarObject = true;
            ioSlot->acceptTimeSlice = true;
         }
         else if(type == "VECTOR"){
            toolTipText = "VECTOR";
            ioSlot->acceptVectorObject = true;
            ioSlot->acceptTimeSlice = true;
         }
         else if(type == "MATRIX"){
            toolTipText = "MATRIX";
            ioSlot->acceptArrayObject = true;
            ioSlot->acceptTimeSlice = true;
         }
         else if(type == "STRING"){
            toolTipText = "STRING";
            ioSlot->acceptText = true;
         }
         else if(type == "TIMETAGS"){
            toolTipText = "TIMETAGS";
            ioSlot->acceptTTagObject = true;
         }
         else if(type == "TIME_INTERVAL"){
            toolTipText = "TIME INTERVAL";
            ioSlot->acceptDuration = true;
            ioSlot->acceptTIvlObject = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "SCALAR_SERIES"){
            toolTipText = "SCALAR DATA SERIES";
            ioSlot->acceptScalarObject = true;
            ioSlot->acceptTimeSlice = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "VECTOR_SERIES"){
            toolTipText = "VECTOR DATA SERIES";
            ioSlot->acceptVectorObject = true;
            ioSlot->acceptTimeSlice = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "MATRIX_SERIES"){
            toolTipText = "MATRIX DATA SERIES";
            ioSlot->acceptArrayObject = true;
            ioSlot->acceptTimeSlice = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "ANY_TIME_SERIES"){
            toolTipText = "ANY TIME SERIES";
            ioSlot->acceptObjectPlus_TSonly = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "SCALAR_TIME_SERIES"){
            toolTipText = "SCALAR TIME SERIES";
            ioSlot->acceptScalarObject_TSonly = true;
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "VECTOR_TIME_SERIES"){
            toolTipText = "VECTOR TIME SERIES";
            ioSlot->acceptVectorObject_TSonly = true; 
            ioSlot->acceptTimeSubset = true;
         }
         else if(type == "MATRIX_TIME_SERIES"){
            toolTipText = "MATRIX TIME SERIES";
            ioSlot->acceptArrayObject_TSonly = true;  
            ioSlot->acceptTimeSubset = true;
         }
         else
            toolTipText = "?? unknown data type ??";
        
	QToolTip::add(this, toolTipText);
}



// functions
// -----------


void QtplOpenByName(QString qtplName, QWidget * parent){
  
  qplugWindow *qplug_widget;
  QtplParserFile *qtplFile;
  QString title;
  
  activeqplugWindow = NULL;
  
  if (qtplName.isEmpty() ){
    QuMsgBar("no template specified", false);
    return;
  }
  
   // attempt to open and parse template file... 
     
  qtplFile = new QtplParserFile(qtplName);
  
  if (qtplFile->openedOK) {
    
   // create empty qplugWindow widget
    qplug_widget = new qplugWindow(parent, "pluginWindow", qtplFile);
    activeqplugWindow = qplug_widget;
    
    qplug_widget->qtplName = QString(qtplName.latin1());

    // set title bar
    
    title = QString(qtplFile->public_name.latin1());
    qplug_widget->setCaption(title);
	title.replace(" ","_");
    qplug_widget->_filename += QString("/saveSet");
    qplug_widget->SAVE_EXTENSION = QString(".qs_");
    qplug_widget->SAVE_EXTENSION += title;
    qplug_widget->_filename += qplug_widget->SAVE_EXTENSION;
    qplug_widget->_saveset_filter=QString("plugin saveset (*.qs_%1)").arg(title);
    qplug_widget->move(qsasGUISize().width(),30);
    qplug_widget->show();
  }

  else  {   //  failed to load new plug-in
    QuMsgBar("Cannot load template ", qtplName, true);
  }

}


static char * slot_text[N_MAX_QPLUG_OUTPUT_ARGS];

char * GetSlotText(int n_slot){
  char * strNULL = "";
  
  if (slot_text[n_slot] == NULL) return strNULL;
  return slot_text[n_slot];
}

void SetSlotText(int n_slot, const char * text){      
      if(slot_text[n_slot] != NULL) free(slot_text[n_slot]);
      if(text[0] != '\0')  slot_text[n_slot] = QplugNewSTR(text);
      else slot_text[n_slot] = NULL;
      return;
}



char * QplugNewSTR(const char * in_str)
{
  char *new_str;
  if(in_str != NULL){
    new_str = (char *) malloc(strlen(in_str)+1);
    strcpy(new_str, in_str);
  }
  else {
    new_str = (char *) malloc(1);
    new_str[0] = '\0';
  }

  return &(new_str[0]);
}


void QplugAppendTextDisplay (const char *text)
{
    if(activeqplugWindow == NULL) {
      cout <<  text ;
      QuMsgBar("see console... ", false);
      return;
    }
#if QT_VERSION < 0x040000
    activeqplugWindow->textArea->append(text);
#else
    QString t(text);
    if(t.endsWith("\n")) t=t.left(t.length()-1); // because append() adds a \n
    activeqplugWindow->textArea->append(t);
#endif
	qApp->processEvents(); // do it now

    
    return;
}


