#include "IntervalDialog.h"

#if QT_VERSION < 0x040000
IntervalDialog::IntervalDialog(QWidget * parent) :
	QMainWindow(parent, "", Qt::WStyle_DialogBorder|Qt::WStyle_Tool|Qt::WStyle_Minimize|Qt::WStyle_Dialog)
{
	QWidget * mainWidget=new QWidget(this);
	
	QGridLayout * mainLayout=new QGridLayout(mainWidget, 3, 1);
	
	slider=new IntervalSlider(mainWidget);
	mainLayout->addWidget(slider, 0, 0);
	
	// Time axis
	
	tAxisWidget=new QWidget(mainWidget);
	QHBoxLayout * timeLayout=new QHBoxLayout(tAxisWidget);
	timeLayout->addWidget(new QLabel("From:", tAxisWidget));
	
	startTimeWidget=new QbDateTimeWidget(tAxisWidget);
	connect(startTimeWidget, SIGNAL(valueChanged()), this, SLOT(updateSlider()));
	timeLayout->addWidget(startTimeWidget);
	
	timeLayout->addWidget(new QLabel("To:", tAxisWidget));
	endTimeWidget=new QbDateTimeWidget(tAxisWidget);
	connect(endTimeWidget, SIGNAL(valueChanged()), this, SLOT(updateSlider()));
	timeLayout->addWidget(endTimeWidget);
	
	mainLayout->addWidget(tAxisWidget, 1, 0);
	
	// X axis
	
	xAxisWidget=new QWidget(mainWidget);
	QHBoxLayout * xLayout=new QHBoxLayout(xAxisWidget);
	xLayout->addWidget(new QLabel("From:", xAxisWidget));
	
	startXWidget=new QguWLinput(xAxisWidget, "startX");
	startXWidget->acceptNumber = true;
	connect(startXWidget, SIGNAL(validated_OK()), this, SLOT(updateSlider()));
	xLayout->addWidget(startXWidget);
	
	xLayout->addWidget(new QLabel("To:", xAxisWidget));
	endXWidget=new QguWLinput(xAxisWidget, "endX");
	endXWidget->acceptNumber = true;
	connect(endXWidget, SIGNAL(validated_OK()), this, SLOT(updateSlider()));
	xLayout->addWidget(endXWidget);
	
	mainLayout->addWidget(xAxisWidget, 1, 0);
	
	// Buttons
	
	QWidget * buttonsWidget=new QWidget(mainWidget);
	QHBoxLayout * buttonsLayout=new QHBoxLayout(buttonsWidget);
	
	QPushButton * resetButton=new QPushButton("Reset", buttonsWidget);
	connect(resetButton, SIGNAL(clicked()), this, SLOT(reset()));
	buttonsLayout->addWidget(resetButton);
	
	QPushButton * dismissButton=new QPushButton("Dismiss", buttonsWidget);
	connect(dismissButton, SIGNAL(clicked()), this, SLOT(dismiss()));
	buttonsLayout->addWidget(dismissButton);
	
	QPushButton * applyButton=new QPushButton("Apply", buttonsWidget);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	buttonsLayout->addWidget(applyButton);
	
	mainLayout->addWidget(buttonsWidget, 2, 0);
	
	setCentralWidget(mainWidget);
	
	connect(slider, SIGNAL(changed(double, double)), this, SLOT(updateFields(double, double)));
	
	hide();
}
#else
IntervalDialog::IntervalDialog(QWidget * parent) :
	QMainWindow(parent, Qt::Tool)
{
	QWidget * mainWidget=new QWidget(this);
	
	QGridLayout * mainLayout=new QGridLayout;
	
	slider=new IntervalSlider;
	mainLayout->addWidget(slider, 0, 0);
	
	// Time axis
	
	tAxisWidget=new QWidget(mainWidget);
	QHBoxLayout * timeLayout=new QHBoxLayout;
	timeLayout->addWidget(new QLabel("From:"));
	
	startTimeWidget=new QbDateTimeWidget(tAxisWidget);
	timeLayout->addWidget(startTimeWidget);
	connect(startTimeWidget, SIGNAL(valueChanged()), this, SLOT(updateSlider()));
	
	timeLayout->addWidget(new QLabel("To:"));
	endTimeWidget=new QbDateTimeWidget(tAxisWidget);
	timeLayout->addWidget(endTimeWidget);
	connect(endTimeWidget, SIGNAL(valueChanged()), this, SLOT(updateSlider()));
	
	tAxisWidget->setLayout(timeLayout);
	mainLayout->addWidget(tAxisWidget, 1, 0);
	
	// X axis
	
	xAxisWidget=new QWidget(mainWidget);
	QHBoxLayout * xLayout=new QHBoxLayout;
	xLayout->addWidget(new QLabel("From:"));
	
	startXWidget=new QguWLinput(xAxisWidget, "startX");
	startXWidget->acceptNumber = true;
	connect(startXWidget, SIGNAL(validated_OK()), this, SLOT(updateSlider()));
	xLayout->addWidget(startXWidget);
	
	xLayout->addWidget(new QLabel("To:"));
	endXWidget=new QguWLinput(xAxisWidget, "endX");
	endXWidget->acceptNumber = true;
	connect(endXWidget, SIGNAL(validated_OK()), this, SLOT(updateSlider()));
	xLayout->addWidget(endXWidget);
	
	xAxisWidget->setLayout(xLayout);
	mainLayout->addWidget(xAxisWidget, 1, 0);
	
	// Buttons
	
	QWidget * buttonsWidget=new QWidget(mainWidget);
	QHBoxLayout * buttonsLayout=new QHBoxLayout;
	
	QPushButton * resetButton=new QPushButton("Reset", buttonsWidget);
	connect(resetButton, SIGNAL(clicked()), this, SLOT(reset()));
	buttonsLayout->addWidget(resetButton);
	
	QPushButton * dismissButton=new QPushButton("Dismiss", buttonsWidget);
	connect(dismissButton, SIGNAL(clicked()), this, SLOT(dismiss()));
	buttonsLayout->addWidget(dismissButton);
	
	QPushButton * applyButton=new QPushButton("Apply", buttonsWidget);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	buttonsLayout->addWidget(applyButton);
	
	buttonsWidget->setLayout(buttonsLayout);
	mainLayout->addWidget(buttonsWidget, 2, 0);
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	connect(slider, SIGNAL(changed(double, double)), this, SLOT(updateFields(double, double)));
	
	setVisible(false);
}
#endif

void IntervalDialog::setAsTimeInterval(double tmin, double tmax, double currentTmin, double currentTmax)
{
#if QT_VERSION < 0x040000
	xAxisWidget->hide();
	tAxisWidget->show();
#else
	xAxisWidget->setVisible(false);
	tAxisWidget->setVisible(true);
#endif
	
	QDOS::Time start, end;
	start.set_time_rep(currentTmin);
	end.set_time_rep(currentTmax);
	startTimeWidget->setTime(start);
	endTimeWidget->setTime(end);
	
	slider->reset(tmin, tmax, currentTmin, currentTmax);
	adjustSize();
}

void IntervalDialog::setAsXInterval(double tmin, double tmax, double currentTmin, double currentTmax)
{
#if QT_VERSION < 0x040000
	xAxisWidget->show();
	tAxisWidget->hide();
#else
	xAxisWidget->setVisible(true);
	tAxisWidget->setVisible(false);
#endif
	
	startXWidget->setText(double_to_string(currentTmin).c_str());
	endXWidget->setText(double_to_string(currentTmax).c_str());
	
	slider->reset(tmin, tmax, currentTmin, currentTmax);
	adjustSize();
}

void IntervalDialog::apply()
{
	if(tAxisWidget->isVisible())
	{
		emit rescale(startTimeWidget->value(), endTimeWidget->value());
	}
	else if(xAxisWidget->isVisible())
	{
		emit rescale(startXWidget->value(), endXWidget->value());
	}
}

void IntervalDialog::dismiss()
{
#if QT_VERSION < 0x040000
	hide();
#else
	setVisible(false);
#endif
}

void IntervalDialog::reset()
{
	startXWidget->setText(double_to_string(slider->min).c_str());
	endXWidget->setText(double_to_string(slider->max).c_str());
	
	QDOS::Time start, end;
	start.set_time_rep(slider->min);
	end.set_time_rep(slider->max);
	startTimeWidget->setTime(start);
	endTimeWidget->setTime(end);
	
	slider->reset(slider->min, slider->max, slider->min, slider->max);
}

void IntervalDialog::updateFields(double start, double end)
{
	QDOS::Time s, e;
	s.set_time_rep(start);
	e.set_time_rep(end);
	startTimeWidget->setTime(s);
	endTimeWidget->setTime(e);
	
	startXWidget->setText(double_to_string(start).c_str());
	endXWidget->setText(double_to_string(end).c_str());
}

void IntervalDialog::updateSlider()
{
	slider->blockSignals(true);
	if(tAxisWidget->isVisible())
	{
		slider->setValue(startTimeWidget->value(), endTimeWidget->value());
	}
	else if(xAxisWidget->isVisible())
	{
		slider->setValue(startXWidget->text().toDouble(), endXWidget->text().toDouble());
	}
	slider->blockSignals(false);
	slider->update();
}
