// qpui_frame.cc
//
// Plot Interface Implementation - Frame Object
//
//  Tony Allen
//  Jan 2008

#include "qpui.h"

extern QPalette QPUI_normal_palette;
extern int qsas_restore_release;

//     QpFrameItem CLASS
//     ----------------------

QpFrameItem::QpFrameItem(QpPageItem *parent, QString name, QpFrameItem *after, QpFrameType frame_type):
    QpListItem(parent,name,after,ListTypeGroup,"")
{
	// Constructor
  	
	I_Page = parent;
	I_Type = QP_FRAME;
	_frameType = frame_type;
	
   // initialise options tab widgets
   	PropertiesTab = new QpPropertiesTab(tabWidget, "PropertiesTab", this);
	PropertiesTab->hide();
	
   	DisplayTab = new QpDisplayTab(tabWidget, "DisplayTab", this);
	DisplayTab->hide();
	
	TitleTab = new QpTitleTab(tabWidget, "TitleTab", this);
	TitleTab->hide();
	
	tAxisTab = new QpTAxisTab(tabWidget,  "tAxisTab", this);
	tAxisTab->hide();
	
	xAxisTab = new QpXAxisTab(tabWidget, "xAxisTab", this);
	xAxisTab->hide();

	if(frame_type == QP_TS) xtAxisTab = tAxisTab;
	else xtAxisTab = xAxisTab;
}

// -------------------------------------------

QpFrameItem::~QpFrameItem()
{
	// Destructor
	// empty frame of panel items
	QpPanelItem * child = dynamic_cast<QpPanelItem*>(this->firstChild());
	while(child)
	{
		QpPanelItem * next = dynamic_cast<QpPanelItem*>(child->nextSibling());
		delete child;
		child = next;
	}

	delete PropertiesTab;
	delete DisplayTab;
	delete TitleTab;
	delete tAxisTab;
	delete xAxisTab;
}
// ----------------------------------

void QpFrameItem::setDefaultOptions()
{
	TitleTab->_title_fld->setText("");
	TitleTab->_subtitle_fld->setText("");
	TitleTab->_autotitle_fld->setChecked(true);
	TitleTab->_title_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_TITLE_CHAR_SIZE_ADD);
	TitleTab->_title_disp_fld->setValue(QPUI_TITLE_DISP);
	TitleTab->_subtitle_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_SUBTITLE_CHAR_SIZE_ADD);
	TitleTab->_subtitle_disp_fld->setValue(QPUI_SUBTITLE_DISP);
	TitleTab->_title_just_fld->setCurrentItem(QPUI_HCENTRE);
// 	TitleTab->_title_font_fld->setCurrentItem(QPUI_NORMAL); //TODO
	
	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QColorGroup::ButtonText, Qt::black);
	line_palette.setColor(QPalette::Inactive, QColorGroup::ButtonText, Qt::black);
	TitleTab->_title_colour_fld->setPalette(line_palette);
    _title_colour = Qt::black;
	
	DisplayTab->_line_width_fld->setValue(1.);
	DisplayTab->_ticks_length_fld->setValue(QPUI_TICKS_LENGTH);
	
	DisplayTab->_line_colour_fld->setPalette(line_palette);
    _line_colour = Qt::black;
	
	DisplayTab->_show_top_axis_fld->setChecked(true);
	DisplayTab->_show_bottom_axis_fld->setChecked(true);
	DisplayTab->_show_left_axis_fld->setChecked(true);
	DisplayTab->_show_right_axis_fld->setChecked(true);
	DisplayTab->_show_top_ticks_labels_fld->setChecked(false);
	DisplayTab->_show_bottom_ticks_labels_fld->setChecked(true);
	DisplayTab->_show_left_ticks_labels_fld->setChecked(true);
	DisplayTab->_show_right_ticks_labels_fld->setChecked(false);
	DisplayTab->_show_top_labels_fld->setChecked(false);
	DisplayTab->_show_bottom_labels_fld->setChecked(true);
	DisplayTab->_show_left_labels_fld->setChecked(true);
	DisplayTab->_show_right_labels_fld->setChecked(false);
	DisplayTab->_show_xiszero_fld->setChecked(false);
	DisplayTab->_show_yiszero_fld->setChecked(false);

	xAxisTab->_axis_linlog_fld->setCurrentItem(QP_LINEAR);
	
	xtAxisTab->_axis_from_xy_fld->setText("");	
	xtAxisTab->_axis_to_xy_fld->setText("");

	xtAxisTab->_label_fld->setText("");
		
	xtAxisTab->_axis_autoscale_fld->setCurrentItem(QP_FROM_META);
	xtAxisTab->_axis_major_ticks_num_fld->setValue(0);
	xtAxisTab->_axis_minor_ticks_num_fld->setValue(0);
	xtAxisTab->_axis_major_tickmarks_fld->setCurrentItem(QPUI_IN);
	xtAxisTab->_axis_minor_tickmarks_fld->setCurrentItem(QPUI_IN);
	xtAxisTab->_axis_autolabel_fld->setChecked(true);
	xtAxisTab->_label_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	xtAxisTab->_label_disp_fld->setValue(QPUI_LABELS_DISP);
	xtAxisTab->_label_rotation_fld->setValue(0);
// 	xtAxisTab->_label_font_fld->setCurrentItem(QPUI_NORMAL); //TODO
	xtAxisTab->_label_colour_fld->setPalette(line_palette);
    xtAxisTab->_label_colour = Qt::black;


}

// ------------------------------------

void QpFrameItem::updateAxes()
{
	if(QuSlotDebug()) cout << "QpFrameItem::updateAxes " << endl;
	
	// Note range done first to ensure _xdo exists for other calls
	
	xtAxisTab->setAxisMinMax(); // always set axis, if user set will just ensure yobj is there
	if( xtAxisTab->_axis_autolabel_fld->isChecked()) xtAxisTab->setAutoLabelText();
	
	// with x or t axis set we must update the y axis
	QpPanelItem * panel = dynamic_cast <QpPanelItem*>(this->firstChild());
	while(panel)
	{
		panel->updateAxes();
		panel = dynamic_cast <QpPanelItem*>(panel->nextSibling());
	}
	
	// set Title
	if( TitleTab->_autotitle_fld->isChecked())
	{
		string title = getTitle();
		TitleTab->_title_fld->clearFocus(); // ensure field doesn't try to turn autotitle off
		TitleTab->_title_fld->setText(title.c_str());
	}

}


// -------------------------------------------

void QpFrameItem::showTabs()
{
	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--) 
	{
		if(tabWidget->page(i))
		{
			tabWidget->page(i)->hide();
			tabWidget->removePage(this->tabWidget->page(i));
		}
	}

	if (frameType() == QP_TS) 
	{
		tabWidget->addTab(tAxisTab, "T Axis");
		xtAxisTab=tAxisTab;
	}
	else 
	{
		tabWidget->addTab(xAxisTab, "X Axis");
		xtAxisTab=xAxisTab;
	}
	tabWidget->addTab(PropertiesTab, "Position");
	
	tabWidget->addTab(DisplayTab, "Frame");
	
	tabWidget->addTab(TitleTab, "Title");
	
	// value return is always shown
	tabWidget->addTab(I_Page->ValueReturnTab, "Value Return");
	
	if(this->I_ListView->I_LastTab == QP_POSITION_TAB)  tabWidget->showPage(PropertiesTab);
	else if(this->I_ListView->I_LastTab == QP_FRAME_TAB)  tabWidget->showPage(DisplayTab);
	else if(this->I_ListView->I_LastTab == QP_TITLE_TAB)  tabWidget->showPage(TitleTab);
	else tabWidget->showPage(xtAxisTab);

	
}

// -------------------------------------------

// void QpFrameItem::postPlot()
// {
// 	// post plots Frame (i.e. called AFTER plot component has been plotted)
// 	// calls panels to plot the axes, but needs to distinguish top and bottom panels
// 	
// 	QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(this->firstChild());
// 	bool first=true;
// 	while(panelItem)
// 	{
// 		QpPanelItem *nextItem = dynamic_cast <QpPanelItem*>( panelItem->nextSibling() );
// 		bool show_top_axis=false;
// 		bool show_bottom_axis=false;
// 		if(first) show_top_axis = true; // first panel is at top of frame
// 		if(nextItem == 0) show_bottom_axis = true; // last panel is at bottom of frame
// 		panelItem->postPlot(show_top_axis, show_bottom_axis);
// 		panelItem = nextItem;
// 	}
// }

// -------------------------------------------

int QpFrameItem::numberInStack()
{
	// returns number of stack sub panels in this frame
	int cnt = 0;
	
  	QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(this->firstChild());
	while(panelItem)
	{
		cnt++;
		panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
	}
	return cnt;
}

// -------------------------------------------------

string QpFrameItem::getTitle()
{
	if ( QuSlotDebug() )
	{
		printf( "QpFrameItem::getTitle \n" );
		fflush(stdout);
	}

	string title="";
	// get frame title from panel(s)
	
	QpPanelItem * panelItem = dynamic_cast<QpPanelItem*>(this->firstChild());
  
	while(panelItem)
	{
		string item_title = panelItem->getTitle();
		
		if(title != item_title) 
		{
			if(title != "" ) title += "/";
			title += item_title;
		}
		panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
	}
	title = SafePGPString(title.c_str()); // truncate to be safe length for pgplot
	return title;
}

// -------------------------------------------------

bool QpFrameItem::getXmetaRange(double &from, double &to)
{
	double xmin = DBL_MAX;
	double xmax = -DBL_MAX;
	
	bool rangeFound = false;
	int isLog=0;
	
	// get union of ranges of panels
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	
	while(panelItem)
	{
		double  item_from,  item_to;
		// if any object has scalemin/max, use it
		if(panelItem->getXmetaRange( item_from,  item_to) ) 
		{
			rangeFound = true;
			xmin = min(xmin, item_from);
			xmax = max(xmax, item_to);
		}
		// get Scaletype accumulation after range to ensure objects exist
		if(isLog == 0) isLog = panelItem->getXmetaIsLog(); // only update until one plot is log
		
		panelItem = dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	if(rangeFound)
	{
		from = xmin;
		to = xmax;
	}
	
	if(frameType() == QP_XY)
	{	
		if(isLog == 1 ) xAxisTab->_axis_linlog_fld->setCurrentItem(QP_LOG);
		else  xAxisTab->_axis_linlog_fld->setCurrentItem(QP_LINEAR);
	}
	
	return rangeFound;
}

// -------------------------------------------------

bool QpFrameItem::getXRange(double &from, double &to)
{
	double xmin = DBL_MAX;
	double xmax = -DBL_MAX;
	
	bool rangeFound = false;
	// get union of ranges of panels
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	
	while(panelItem)
	{
		double  item_from,  item_to;
		if(panelItem->getXRange( item_from,  item_to) )
		{
			rangeFound = true;
			xmin = min(xmin, item_from);
			xmax = max(xmax, item_to);
		}		
		panelItem = dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	from = xmin;
	to = xmax;
	return rangeFound;
}
// -------------------------------------------------

void QpFrameItem::getYObjects()
{
	
	// tell all panels
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	
	while(panelItem)
	{
		panelItem->getYObjects();
		panelItem = dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	
	return;
}
// ------------------------------------

void QpFrameItem::setPanelProperties(double loffset, double width, double toffset, double height)
{
	if(QuSlotDebug())
	{
		printf("QpFrameItem::setPanelProperties L=%g, W=%g, T=%g, H=%g\n", loffset, width, toffset, height);
		fflush(stdout);
	}

		this->PropertiesTab->_loffset_fld->setValue(loffset);
		this->PropertiesTab->_width_fld->setValue(width);
		this->PropertiesTab->_toffset_fld->setValue(toffset);
		this->PropertiesTab->_height_fld->setValue(height);
}
// ------------------------------------

bool QpFrameItem::containsSpectrum()
{
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	while(panelItem)
	{
		QpPlotItem *plotItem =  dynamic_cast <QpPlotItem*>(panelItem->firstChild());
		while(plotItem)
		{
			QpSpectrumItem * spectrumItem = dynamic_cast <QpSpectrumItem*>(plotItem);
			if(spectrumItem) return true;
			plotItem =  dynamic_cast <QpPlotItem*>(plotItem->nextSibling());
		}
		panelItem = dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	return false;
}

// ------------------------------------

QpuiReturnStatus QpFrameItem::plot(PlotWindow* win)
{
	// plots contents of each panel in frame
	bool hasSpectrum = I_Page->containsSpectrum();
	
	// count number of panels in frame
	int stackNum = numberInStack();
	int num = 0;
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	
	while(panelItem)
	{
		double height  = PropertiesTab->_height_fld->value() / stackNum;
		double toffset = PropertiesTab->_toffset_fld->value() + num * height;
		double loffset = PropertiesTab->_loffset_fld->value();
		double width   = PropertiesTab->_width_fld->value();
		if(hasSpectrum) width *= 0.9;  // allow space for colour scale
		if(panelItem->yAxisTab->_axis_autolabel_fld->isChecked())
		{
			string _label="";
			panelItem->getLabel(Y_AXIS, _label);
			panelItem->yAxisTab->_label_fld->setText(_label.c_str());
		}

		panelItem->plot(height,  width, toffset, loffset, win);

		num++;
		panelItem = dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	
	return QPUI_SUCCESS;
}

// ---------------------------

int QpFrameItem::panelNum(QpPanelItem *panelItem)
{
	int num=0;
	QpPanelItem *item = dynamic_cast<QpPanelItem*>(this->firstChild());
	while(item)
	{
		if(item == panelItem) return num;
		num++;
		item = dynamic_cast<QpPanelItem*>(item->nextSibling());
	}
	return 0; // just return first panel to be safe
}

// ---------------------------

void QpFrameItem::changeFrame(int frameType)
{
	// Tells panels to reset data tabs for appropriate inputs
	// NOTE: X and t axis tabs are changed by updateOnSelection()
	
	this->_frameType = (QpFrameType)frameType;
	
	// Now tell all contained plots they are now of new type TS or XY
	
	QpPanelItem *panelItem = dynamic_cast <QpPanelItem*>(this->firstChild());
	while(panelItem)
	{
		QpPlotItem *item = dynamic_cast <QpPlotItem*>(panelItem->firstChild());
		while(item)
		{
			item->changeFrameType(frameType);  // will be QP_TS or QP_XY
			item =  dynamic_cast <QpPlotItem*>(item->nextSibling());
		}
		panelItem =  dynamic_cast <QpPanelItem*>(panelItem->nextSibling());
	}
	
	if (this->frameType() == QP_TS) this->xtAxisTab=tAxisTab;
	else xtAxisTab=xAxisTab;
	
	// may need to reset axis ranges
	this->updateAxes();
}  
	


Vistream& operator>>(Vistream& v_is, QpFrameItem& frame)
{
	if(QuSlotDebug()) cout <<  "QpFrameItem restore (>>)" << endl;

	// streams from virtual input stream v_is into item

	QpListItem& list_item = frame;
	v_is >> list_item;
	
	int frameType;
	v_is >> frameType;
	frame._frameType = (QpFrameType) frameType;

	float _loffset;
	v_is >> _loffset;
	frame.PropertiesTab->_loffset_fld->setValue(_loffset);
	
	float _width;
	v_is >> _width;
	frame.PropertiesTab->_width_fld->setValue(_width);
	
	float _toffset;
	v_is >> _toffset;
	frame.PropertiesTab->_toffset_fld->setValue(_toffset);
	
	float _height;
	v_is >> _height;
	frame.PropertiesTab->_height_fld->setValue(_height);
	
	bool _aspect_ratio;
	v_is >> _aspect_ratio;
	frame.PropertiesTab->_aspect_ratio_fld->setChecked(_aspect_ratio);
		
	float _line_width;
	v_is >> _line_width;
	frame.DisplayTab->_line_width_fld->setValue(_line_width);
	
	float _ticks_length;
	v_is >> _ticks_length;
	frame.DisplayTab->_ticks_length_fld->setValue(_ticks_length);
	
	float _ticks_char_size;
	v_is >> _ticks_char_size;
	frame.DisplayTab->_ticks_char_size_fld->setValue(_ticks_char_size);
	
	int r, g, b;
	v_is >> r;
	v_is >> g;
	v_is >> b;
	frame._line_colour = QColor(r, g, b);
		
	bool _show_top_axis;
	v_is >> _show_top_axis;
	frame.DisplayTab->_show_top_axis_fld->setChecked(_show_top_axis);
		
	bool _show_bottom_axis;
	v_is >> _show_bottom_axis;
	frame.DisplayTab->_show_bottom_axis_fld->setChecked(_show_bottom_axis);
		
	bool _show_left_axis;
	v_is >> _show_left_axis;
	frame.DisplayTab->_show_left_axis_fld->setChecked(_show_left_axis);
		
	bool _show_right_axis;
	v_is >> _show_right_axis;
	frame.DisplayTab->_show_right_axis_fld->setChecked(_show_right_axis);
	
	bool _show_lr_alternating_axis;
	v_is >> _show_lr_alternating_axis;
	frame.DisplayTab->_show_lr_alternating_axis_fld->setChecked(_show_lr_alternating_axis);
	
	bool _show_rl_alternating_axis;
	v_is >> _show_rl_alternating_axis;
	frame.DisplayTab->_show_rl_alternating_axis_fld->setChecked(_show_rl_alternating_axis);
		
	bool _show_top_ticks_labels;
	v_is >> _show_top_ticks_labels;
	frame.DisplayTab->_show_top_ticks_labels_fld->setChecked(_show_top_ticks_labels);
		
	bool _show_bottom_ticks_labels;
	v_is >> _show_bottom_ticks_labels;
	frame.DisplayTab->_show_bottom_ticks_labels_fld->setChecked(_show_bottom_ticks_labels);
		
	bool _show_left_ticks_labels;
	v_is >> _show_left_ticks_labels;
	frame.DisplayTab->_show_left_ticks_labels_fld->setChecked(_show_left_ticks_labels);
		
	bool _show_right_ticks_labels;
	v_is >> _show_right_ticks_labels;
	frame.DisplayTab->_show_right_ticks_labels_fld->setChecked(_show_right_ticks_labels);
	
	bool _show_lr_alternating_ticks_labels;
	v_is >> _show_lr_alternating_ticks_labels;
	frame.DisplayTab->_show_lr_alternating_ticks_labels_fld->setChecked(_show_lr_alternating_ticks_labels);
	
	bool _show_rl_alternating_ticks_labels;
	v_is >> _show_rl_alternating_ticks_labels;
	frame.DisplayTab->_show_rl_alternating_ticks_labels_fld->setChecked(_show_rl_alternating_ticks_labels);
		
	bool _show_top_labels;
	v_is >> _show_top_labels;
	frame.DisplayTab->_show_top_labels_fld->setChecked(_show_top_labels);
		
	bool _show_bottom_labels;
	v_is >> _show_bottom_labels;
	frame.DisplayTab->_show_bottom_labels_fld->setChecked(_show_bottom_labels);
		
	bool _show_left_labels;
	v_is >> _show_left_labels;
	frame.DisplayTab->_show_left_labels_fld->setChecked(_show_left_labels);
		
	bool _show_right_labels;
	v_is >> _show_right_labels;
	frame.DisplayTab->_show_right_labels_fld->setChecked(_show_right_labels);
	
	bool _show_lr_alternating_labels;
	v_is >> _show_lr_alternating_labels;
	frame.DisplayTab->_show_lr_alternating_labels_fld->setChecked(_show_lr_alternating_labels);
	
	bool _show_rl_alternating_labels;
	v_is >> _show_rl_alternating_labels;
	frame.DisplayTab->_show_rl_alternating_labels_fld->setChecked(_show_rl_alternating_labels);
		
	bool _show_xiszero;
	v_is >> _show_xiszero;
	frame.DisplayTab->_show_xiszero_fld->setChecked(_show_xiszero);
		
	bool _show_yiszero;
	v_is >> _show_yiszero;
	frame.DisplayTab->_show_yiszero_fld->setChecked(_show_yiszero);

	bool _autotitle;
	v_is >> _autotitle;
	frame.TitleTab->_autotitle_fld->setChecked(_autotitle);

	string _title;
	v_is >> _title;
	frame.TitleTab->_title_fld->setText(_title.c_str());
	
	float _title_char_size;
	v_is >> _title_char_size;
	frame.TitleTab->_title_char_size_fld->setValue(_title_char_size);
	
	float _title_disp;
	v_is >> _title_disp;
	frame.TitleTab->_title_disp_fld->setValue(_title_disp);
	
	string _subtitle;
	v_is >> _subtitle;
	frame.TitleTab->_subtitle_fld->setText(_subtitle.c_str());
	
	float _subtitle_char_size;
	v_is >> _subtitle_char_size;
	frame.TitleTab->_subtitle_char_size_fld->setValue(_subtitle_char_size);
	
	float _subtitle_disp;
	v_is >> _subtitle_disp;
	frame.TitleTab->_subtitle_disp_fld->setValue(_subtitle_disp);

	int _title_just;
	v_is >> _title_just;
	frame.TitleTab->_title_just_fld->setCurrentItem(_title_just);
	
	if( qsas_restore_release >= 20306)
	{
		PLINT family, style, weight;
		v_is >> family;
		v_is >> style;
		v_is >> weight;
		switch(family)
		{
			case PL_FCI_SERIF: frame.TitleTab->_title_font_fld->setSerifFont(); break;
			case PL_FCI_MONO: frame.TitleTab->_title_font_fld->setMonotypeFont(); break;
			case PL_FCI_SCRIPT: frame.TitleTab->_title_font_fld->setDecorativeFont(); break;
			default: frame.TitleTab->_title_font_fld->setSansFont(); break;
		}
		frame.TitleTab->_title_font_fld->setItalic(style==PL_FCI_ITALIC);
		frame.TitleTab->_title_font_fld->setBold(weight==PL_FCI_BOLD);
	}
	else
	{
		int _title_font;
		v_is >> _title_font;
		switch(_title_font)
		{
			case 1: // Roman
				frame.TitleTab->_title_font_fld->setSerifFont();
				frame.TitleTab->_title_font_fld->setItalic(false);
				frame.TitleTab->_title_font_fld->setBold(false);
				break;
				
			case 2: // Italic
				frame.TitleTab->_title_font_fld->setSerifFont();
				frame.TitleTab->_title_font_fld->setItalic(true);
				frame.TitleTab->_title_font_fld->setBold(false);
				break;
				
			default: // Normal
				frame.TitleTab->_title_font_fld->setSansFont();
				frame.TitleTab->_title_font_fld->setItalic(false);
				frame.TitleTab->_title_font_fld->setBold(false);
				break;
		}
	}
	
	v_is >> r;
	v_is >> g;
	v_is >> b;
	frame._title_colour = QColor(r, g, b);
	
	QpAxisTab *axisTab;
	
	if(frame._frameType == QP_TS){
		axisTab = frame.tAxisTab;
		
		v_is >> *axisTab; // do bits common to all axes
			
		string t_data;
		v_is >> t_data;
		frame.tAxisTab->_t_data_fld->setText(t_data.c_str());
		frame.xtAxisTab=frame.tAxisTab;	
	}
	else
	{
		axisTab = frame.xAxisTab;
	
		v_is >> *axisTab; // do bits common to all axes
		
		int linlog;
		v_is >> linlog;
		frame.xAxisTab->_axis_linlog_fld->setCurrentItem(linlog);
		frame.xtAxisTab=frame.xAxisTab;	
	}
	
	int cnt;
	v_is >> cnt;

	for(int i=0; i<cnt; i++)
	{	
		QpPanelItem *panelItem = new QpPanelItem(&frame, "", (QpPanelItem*)frame.lastChild());

		v_is >> *panelItem;
	}
    

	return v_is;
}

// --------------------------------------

Vostream& operator<<(Vostream& v_os, QpFrameItem& frame)
{
	if(QuSlotDebug()) cout << "QpFrameItem save (<<)" << endl;

	// streams frame to virtual output stream v_os

	QpListItem& list_item = frame;
	v_os << list_item;

	v_os << (int) frame._frameType;

	v_os << (float)frame.PropertiesTab->_loffset_fld->value();
	v_os << (float)frame.PropertiesTab->_width_fld->value();
	v_os << (float)frame.PropertiesTab->_toffset_fld->value();
	v_os << (float)frame.PropertiesTab->_height_fld->value();

	bool _aspect_ratio = frame.PropertiesTab->_aspect_ratio_fld->isChecked();
	v_os << _aspect_ratio;
	v_os << (float)frame.DisplayTab->_line_width_fld->value();
	v_os << (float)frame.DisplayTab->_ticks_length_fld->value();
	v_os << (float)frame.DisplayTab->_ticks_char_size_fld->value();
	v_os << frame._line_colour.red();
	v_os << frame._line_colour.green();
	v_os << frame._line_colour.blue();
	bool _show_top_axis = frame.DisplayTab->_show_top_axis_fld->isChecked();
	v_os << _show_top_axis;
	bool _show_bottom_axis = frame.DisplayTab->_show_bottom_axis_fld->isChecked();
	v_os << _show_bottom_axis;
	bool _show_left_axis = frame.DisplayTab->_show_left_axis_fld->isChecked();
	v_os << _show_left_axis;
	bool _show_right_axis = frame.DisplayTab->_show_right_axis_fld->isChecked();
	v_os << _show_right_axis;
	bool _show_lr_alternating_axis = frame.DisplayTab->_show_lr_alternating_axis_fld->isChecked();
	v_os << _show_lr_alternating_axis;
	bool _show_rl_alternating_axis = frame.DisplayTab->_show_rl_alternating_axis_fld->isChecked();
	v_os << _show_rl_alternating_axis;
	bool _show_top_ticks_labels = frame.DisplayTab->_show_top_ticks_labels_fld->isChecked();
	v_os << _show_top_ticks_labels;
	bool _show_bottom_ticks_labels = frame.DisplayTab->_show_bottom_ticks_labels_fld->isChecked();
	v_os << _show_bottom_ticks_labels;
	bool _show_left_ticks_labels = frame.DisplayTab->_show_left_ticks_labels_fld->isChecked();
	v_os << _show_left_ticks_labels;
	bool _show_right_ticks_labels = frame.DisplayTab->_show_right_ticks_labels_fld->isChecked();
	v_os << _show_right_ticks_labels;
	bool _show_lr_alternating_ticks_labels = frame.DisplayTab->_show_lr_alternating_ticks_labels_fld->isChecked();
	v_os << _show_lr_alternating_ticks_labels;
	bool _show_rl_alternating_ticks_labels = frame.DisplayTab->_show_rl_alternating_ticks_labels_fld->isChecked();
	v_os << _show_rl_alternating_ticks_labels;
	bool _show_top_labels = frame.DisplayTab->_show_top_labels_fld->isChecked();
	v_os << _show_top_labels;
	bool _show_bottom_labels = frame.DisplayTab->_show_bottom_labels_fld->isChecked();
	v_os << _show_bottom_labels;
	bool _show_left_labels = frame.DisplayTab->_show_left_labels_fld->isChecked();
	v_os << _show_left_labels;
	bool _show_right_labels = frame.DisplayTab->_show_right_labels_fld->isChecked();
	v_os << _show_right_labels;
	bool _show_lr_alternating_labels = frame.DisplayTab->_show_lr_alternating_labels_fld->isChecked();
	v_os << _show_lr_alternating_labels;
	bool _show_rl_alternating_labels = frame.DisplayTab->_show_rl_alternating_labels_fld->isChecked();
	v_os << _show_rl_alternating_labels;
	bool _show_xiszero = frame.DisplayTab->_show_xiszero_fld->isChecked();
	v_os << _show_xiszero;
	bool _show_yiszero = frame.DisplayTab->_show_yiszero_fld->isChecked();
	v_os << _show_yiszero;
	bool _autotitle = frame.TitleTab->_autotitle_fld->isChecked();
	v_os << _autotitle;
	string _title = frame.TitleTab->_title_fld->text().latin1();
	v_os << _title;
	float _title_char_size = frame.TitleTab->_title_char_size_fld->value();
	v_os << _title_char_size;
	float _title_disp = frame.TitleTab->_title_disp_fld->value();
	v_os << _title_disp;
	string _subtitle = frame.TitleTab->_subtitle_fld->text().latin1();
	v_os << _subtitle;
	float _subtitle_char_size = frame.TitleTab->_subtitle_char_size_fld->value();
	v_os << _subtitle_char_size;
	float _subtitle_disp = frame.TitleTab->_subtitle_disp_fld->value();
	v_os << _subtitle_disp;
	
	v_os << frame.TitleTab->_title_just_fld->currentItem();
	PLINT family, style, weight;
	frame.TitleTab->_title_font_fld->getFontParams(family, style, weight);
	v_os << family;
	v_os << style;
	v_os << weight;
	
	v_os << frame._title_colour.red();
	v_os << frame._title_colour.green();
	v_os << frame._title_colour.blue();


	QpAxisTab *axisTab;
	
	if(frame._frameType == QP_TS){
		axisTab = frame.tAxisTab;
	
		v_os << *axisTab; // do bits common to all axes

		v_os << string(frame.tAxisTab->_t_data_fld->text().latin1());
	}
	else
	{
		axisTab = frame.xAxisTab;
		
		v_os << *axisTab; // do bits common to all axes
		
		v_os << (int) frame.xAxisTab->_axis_linlog_fld->currentItem();
	}

	int cnt = frame.numberInStack();
	v_os << cnt;
	
	QpPanelItem *panelItem = dynamic_cast<QpPanelItem*>(frame.firstChild());

	for(int i=0; i<cnt; i++)
	{
		v_os <<  *panelItem;
		panelItem = dynamic_cast<QpPanelItem*>(panelItem->nextSibling());
	}

	return v_os;
}

