// qpui_globals.cc
//
// Plot Interface Implementation - Global functions and variables
//
// Janet Barnes
//
// July 2002

#include "qpui.h"
#if QT_VERSION < 0x040000
#include <qdatetime.h>
#else
#include <QDateTime>
#endif

QpWindow *QPUI_widget=static_cast<QpWindow*>(NULL);

// global variable for keeping track of colour indices used
int QPUI_colour_index;

// global variables for storing colour palettes for showing
// active and inactive fields
QPalette QPUI_normal_palette;
QPalette QPUI_different_palette;


FontButton::FontButton(QWidget* parent, int family, bool italic, bool bold):
	QPushButton(parent)
{
	this->family=family;
	this->italic=italic;
	this->bold=bold;
	
#if QT_VERSION >= 0x040000
	
	menu=new QMenu(this);
	
	sansAction=menu->addAction("Sans font", this, SLOT(setSansFont())); sansAction->setCheckable(true); sansAction->setChecked(family==PL_FCI_SANS);
	serifAction=menu->addAction("Serif font", this, SLOT(setSerifFont())); serifAction->setCheckable(true); serifAction->setChecked(family==PL_FCI_SERIF);
	monotypeAction=menu->addAction("Monotype font", this, SLOT(setMonotypeFont())); monotypeAction->setCheckable(true); monotypeAction->setChecked(family==PL_FCI_MONO);
	decorativeAction=menu->addAction("Decorative font", this, SLOT(setDecorativeFont())); decorativeAction->setCheckable(true); decorativeAction->setChecked(family==PL_FCI_SCRIPT);
	
	menu->addSeparator();
	
	italicAction=menu->addAction("Italic", this, SLOT(checkItalic())); italicAction->setCheckable(true); italicAction->setChecked(italic);
	boldAction=menu->addAction("Bold", this, SLOT(checkBold())); boldAction->setCheckable(true); boldAction->setChecked(bold);
	
	setText("Font");
	
	setMenu(menu);
	
#else
	menu=new QPopupMenu(this);
	menu->setCheckable(true);
	sansId=menu->insertItem("Sans font", this, SLOT(setSansFont())); menu->setItemChecked(sansId, family==PL_FCI_SANS);
	serifId=menu->insertItem("Serif font", this, SLOT(setSerifFont())); menu->setItemChecked(serifId, family==PL_FCI_SERIF);
	monotypeId=menu->insertItem("Monotype font", this, SLOT(setMonotypeFont())); menu->setItemChecked(monotypeId, family==PL_FCI_MONO);
	decorativeId=menu->insertItem("Decorative font", this, SLOT(setDecorativeFont())); menu->setItemChecked(decorativeId, family==PL_FCI_SCRIPT);
	
	menu->insertSeparator();
	
	italicId=menu->insertItem("Italic", this, SLOT(checkItalic())); menu->setItemChecked(italicId, italic);
	boldId=menu->insertItem("Bold", this, SLOT(checkBold())); menu->setItemChecked(boldId, bold);
	
	setText("Font");
	
#endif
	
	update();
}

FontButton::~FontButton()
{
	delete menu;
}

#if QT_VERSION < 0x040000
void FontButton::mouseReleaseEvent(QMouseEvent* e)
{
	menu->exec(e->globalPos());
	QPushButton::mouseReleaseEvent(e);
}
#endif

void FontButton::getFontParams(PLINT& family, PLINT& style, PLINT& weight)
{
	family=this->family;	
	style=(italic?PL_FCI_ITALIC:PL_FCI_UPRIGHT);
	weight=(bold?PL_FCI_BOLD:PL_FCI_MEDIUM);
}

void FontButton::update()
{
	QFont f=font();
	
	switch(family)
	{
		case PL_FCI_SERIF:
			f.setStyleHint(QFont::Serif);
			f.setFamily(QSAS_profiles->current()->SerifFont);
			break;
		case PL_FCI_MONO:
			f.setStyleHint(QFont::TypeWriter);
			f.setFamily(QSAS_profiles->current()->MonotypeFont);
			break;
		case PL_FCI_SCRIPT:
			f.setStyleHint(QFont::Decorative);
			f.setFamily(QSAS_profiles->current()->DecorativeFont);
			break;
		default:
			f.setStyleHint(QFont::SansSerif);
			f.setFamily(QSAS_profiles->current()->SansFont);
			break;
	}
	
	f.setBold(bold);
	f.setItalic(italic);
	
	setFont(f);
}

void FontButton::setSansFont()
{
	family=PL_FCI_SANS;
#if QT_VERSION >= 0x040000
	sansAction->setChecked(true);
	serifAction->setChecked(false);
	monotypeAction->setChecked(false);
	decorativeAction->setChecked(false);
#else
	menu->setItemChecked(sansId, true);
	menu->setItemChecked(serifId, false);
	menu->setItemChecked(monotypeId, false);
	menu->setItemChecked(decorativeId, false);
#endif
	update();
	emit familyChanged(family);
}

void FontButton::setSerifFont()
{
	family=PL_FCI_SERIF;
#if QT_VERSION >= 0x040000
	sansAction->setChecked(false);
	serifAction->setChecked(true);
	monotypeAction->setChecked(false);
	decorativeAction->setChecked(false);
#else
	menu->setItemChecked(sansId, false);
	menu->setItemChecked(serifId, true);
	menu->setItemChecked(monotypeId, false);
	menu->setItemChecked(decorativeId, false);
#endif
	update();
	emit familyChanged(family);
}

void FontButton::setMonotypeFont()
{
	family=PL_FCI_MONO;
#if QT_VERSION >= 0x040000
	sansAction->setChecked(false);
	serifAction->setChecked(false);
	monotypeAction->setChecked(true);
	decorativeAction->setChecked(false);
#else
	menu->setItemChecked(sansId, false);
	menu->setItemChecked(serifId, false);
	menu->setItemChecked(monotypeId, true);
	menu->setItemChecked(decorativeId, false);
#endif
	update();
	emit familyChanged(family);
}

void FontButton::setDecorativeFont()
{
	family=PL_FCI_SCRIPT;
#if QT_VERSION >= 0x040000
	sansAction->setChecked(false);
	serifAction->setChecked(false);
	monotypeAction->setChecked(false);
	decorativeAction->setChecked(true);
#else
	menu->setItemChecked(sansId, false);
	menu->setItemChecked(serifId, false);
	menu->setItemChecked(monotypeId, false);
	menu->setItemChecked(decorativeId, true);
#endif
	update();
	emit familyChanged(family);
}

void FontButton::setItalic(bool i)
{
	italic=i;
#if QT_VERSION >= 0x040000
	italicAction->setChecked(i);
#else
	menu->setItemChecked(italicId, i);
#endif
	update();
	emit styleChanged(i);
}
void FontButton::checkItalic()
{
	setItalic(!italic);
}

void FontButton::setBold(bool i)
{
	bold=i;
#if QT_VERSION >= 0x040000
	boldAction->setChecked(i);
#else
	menu->setItemChecked(boldId, i);
#endif
	update();
	emit weightChanged(i);
}
void FontButton::checkBold()
{
	setBold(!bold);
}







// Function generating the dash styles
// ----------------------
void setDashStyle(QpLineStyle style) //QP_NOLINE should be catched before calling this
{
	PLINT * mark;
	PLINT * space;
    
	switch(style)
	{
		case QP_SOLID:
			mark=new PLINT;
			*mark=0;
			space=new PLINT;
			*space=0;
			plstyl(0, mark, space);
			delete mark;
			delete space;
			break;
		
		case QP_DASHED:
			mark=new PLINT;
			*mark=30000;
			space=new PLINT;
			*space=30000;
			plstyl(1, mark, space);
			delete mark;
			delete space;
			break;
		
		case QP_DOTDASH:
			mark=new PLINT[2];
			mark[0]=4000; mark[1]=30000;
			space=new PLINT[2];
			space[0]=20000; space[1]=20000;
			plstyl(2, mark, space);
			delete[] mark;
			delete[] space;
			break;
		
		case QP_DOTTED:
			mark=new PLINT;
			*mark=4000;
			space=new PLINT;
			*space=20000;
			plstyl(1, mark, space);
			delete mark;
			delete space;
			break;
		
		case QP_DASHDOTDOTDOT:
			mark=new PLINT[4];
			mark[0]=30000; mark[1]=4000; mark[2]=4000; mark[3]=4000;
			space=new PLINT[4];
			space[0]=20000; space[1]=20000; space[2]=20000; space[3]=20000;
			plstyl(4, mark, space);
			delete[] mark;
			delete[] space;
		break;
		
		default:
			break;
	}
}

PLFLT getCharHeight()
{
	PLFLT def, cur;
	
	plgchr(&def, &cur);
	
	// cur is given in mm. We specified the device as 4pix/mm, for a 32768 (PIXELS_Y) pix height
	
	return cur*4./32768.;
}

int getPGPColourIndex(const QColor& c, bool reset)
{
	PLINT iR,iG,iB;
	static int nextAvailable=16;
	static vector <int> userR(40);
	static vector <int> userG(40);
	static vector <int> userB(40);
	
	if(reset)
	{
		userR.clear();
		userG.clear();
		userB.clear();
		nextAvailable=16;
		for(register int i=0;i<16;++i)
		{
			// set up local integer copy of fixed pgplot table
			plgcol0(i,&iR,&iG,&iB);
			userR[i]=iR;
			userG[i]=iG;
			userB[i]=iB;
		}
		return 0;
	}
		
	// find PGPlot colour index closest to this colour
	int R=c.red();
	int G=c.green();
	int B=c.blue();
	int min_diff=200000; // bigger than 255*255*3
	int colIndex=0;
		
	int indexNearest=0;
		
	for(register int i=0;i<nextAvailable;i++)
	{
		int diff=(R-userR[i])*(R-userR[i]) + (G-userG[i])*(G-userG[i]) + (B-userB[i])*(B-userB[i]);
		if(diff==0)
		{
			return i; // found in existing table
		}
		else if(diff<min_diff)
		{
			min_diff = diff;
			indexNearest=i;
		}
	}
		
	if( nextAvailable < 40)
	{
		// create new colour and set in local copy
		
		userR[nextAvailable] = R;
		userG[nextAvailable] = G;
		userB[nextAvailable] = B;
		colIndex = nextAvailable;
		nextAvailable++;
	}
	else
	{
		// use nearest colour
		return indexNearest;
	}
	
	// set up colour table as new colour added
	plscmap0n(colIndex+1);
	plscol0(colIndex, R, G, B);
	
	return colIndex;
}

void print_iden() // replaces cpgiden in pgplot
{
	PLFLT xmin, xmax, ymin, ymax;
	PLFLT chrsz1, chrsz2;
	
	plgvpd(&xmin, &xmax, &ymin, &ymax);
	
	plvpor(0., 1., 0., 1.);
	
	plcol0(getPGPColourIndex(QColor(0, 0, 0), false));
	plgchr(&chrsz1, &chrsz2);
	plschr(QPUI_CHAR_HEIGHT, 0.55);
	plsfont(PL_FCI_SANS, PL_FCI_UPRIGHT, PL_FCI_MEDIUM);
	
	QString format("dd-MMM-yyyy hh:mm");
	
	QString date=QDateTime::currentDateTime().toString(format);
	QString user(getenv("USER"));
	
	plmtex("b", -0.7, 0.99, 1., (user+QString(" ")+date).latin1());
	
	plschr(chrsz1, chrsz2);
	plvpor(xmin, xmax, ymin, ymax);
}

PLINT symbolIndexConversion(int index) // Index in combo box to plplot index
{
	switch(index)
	{
		case 1:
			return (int)QPUI_POINT;
			break;
		case 2:
			return (int)QPUI_PLUS;
			break;
		case 3:
			return (int)QPUI_ASTERISK;
			break;
		case 4:
			return (int)QPUI_CIRCLE;
			break;
		case 5:
			return (int)QPUI_CROSS;
			break;
		case 6:
			return (int)QPUI_SQUARE;
			break;
		case 7:
			return (int)QPUI_TRIANGLE;
			break;
		case 8:
			return (int)QPUI_CIRCLED_CROSS;
			break;
		case 9:
			return (int)QPUI_CIRCLED_POINT;
			break;
		case 10:
			return (int)QPUI_LENS_FLARE;
			break;
		case 11:
			return (int)QPUI_DIAMOND;
			break;
		case 12:
			return (int)QPUI_STAR;
			break;
		case 13:
			return (int)QPUI_WIDECROSS;
			break;
		case 14:
			return (int)QPUI_FILLEDSQUARE;
			break;
		case 15:
			return (int)QPUI_FILLEDCIRCLE;
			break;
		case 16:
			return (int)QPUI_FILLEDSTAR;
			break;
		case 0:
		default:
			return (int)QPUI_NOSYMBOL;
			break;
	}	
}

double SafeLog10(double d)
{
	// return "safe" log value

	return (d < 1.0e-33)?-33:log10(d);
}

string SafePGPString(const string& str)
{
	// return "safe" string that can be correctly displayed by pgplot
	// Also ensures it is not too long as pgplot can crash with long strings
	
	if(str=="") return str;
	
	if(QuSlotDebug())
	{
		printf("SafePGPString %s\n", str.c_str());
		fflush(stdout);
	}

	string safe_str(str);
	size_t pos;
// 	while((pos=safe_str.find('\136'))!=string::npos) safe_str[pos]='\036';
 
	if(safe_str.length() > 178){
		safe_str.replace(175, safe_str.length() -175, "");
		safe_str += "...";
	}
	
	return safe_str;
}


