// qpui_legend_options.cc
//
// Plot Interface Implementation - Legend Options
//
// Janet Barnes
// November 2004
//
// Tony Allen
// April 2008

#include "qpui.h"

#include <qtooltip.h>
#include <qcolordialog.h>

extern QPalette QPUI_normal_palette;

//     QpLegendTab CLASS
//     ---------------------

#if QT_VERSION < 0x040000
QpLegendTab::QpLegendTab(QTabWidget *tabWidget,  QString name, QpPanelItem * panel):
    QpTab(tabWidget, name, QP_LEGEND_TAB)
{
	// Comstructor

	// keep track of item which owns this tab
	I_Panel = panel; 

	QVBoxLayout *legend_top_layout = new QVBoxLayout(this);
	legend_top_layout->setSpacing(4);

	QGroupBox *legend_properties_groupbox = new QGroupBox(1, Qt::Vertical, this, "legend_properties_groupbox");

	legend_properties_groupbox->setTitle("Properties");

	QWidget* legend_properties_widget = new QWidget(legend_properties_groupbox, "legend_properties_widget");

	QVBoxLayout *legend_properties_layout = new QVBoxLayout(legend_properties_widget);
	legend_properties_layout->setSpacing(4);

	_show_legend_fld = new QCheckBox("Show legend", legend_properties_widget, "show_legend");
	_show_legend_fld->setChecked(false);
	legend_properties_layout->addWidget(_show_legend_fld, 0, Qt::AlignLeft);

	connect( _show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend(bool)) );

	legend_box_groupbox = new QGroupBox(1, Qt::Vertical, legend_properties_widget, "legend_box_groupbox");
	legend_box_groupbox->setTitle("Box");

	QWidget * legend_box_widget = new QWidget(legend_box_groupbox, "legend_box_widget");

	QGridLayout *legend_box_layout = new QGridLayout(legend_box_widget, 7, 5);
	legend_box_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_box_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_border_label = new QLabel("Border: ", legend_box_widget, "legend_border_label");
	legend_box_layout->addWidget(legend_border_label, 1, 0, Qt::AlignRight);
	_legend_border_fld = new QComboBox(false, legend_box_widget, "legend_border");
	_legend_border_fld->insertItem("No border", QPUI_NO_BORDER);
	_legend_border_fld->insertItem("Line", QPUI_LINE);
	QFontMetrics legend_border_fm(_legend_border_fld->font());
	int legend_border_w = legend_border_fm.width("No border");
	int legend_border_h=legend_border_fm.height();
	_legend_border_fld->setMinimumSize(legend_border_w, legend_border_h);
#if QT_VERSION < 0x040000
	_legend_border_fld->listBox()->setMinimumSize(legend_border_w, legend_border_h);
#else
        _legend_border_fld->setMinimumSize(legend_border_w, legend_border_h);
#endif
	QToolTip::add( _legend_border_fld, "Select style of legend border");
	legend_box_layout->addWidget(_legend_border_fld, 1, 1, Qt::AlignLeft);
	_legend_border_fld->setCurrentItem(QPUI_LINE);
	connect( _legend_border_fld, SIGNAL(activated(int)), this, SLOT(setLegendBorder(int)) );

	QLabel* legend_background_label = new QLabel("Background: ", legend_box_widget, "legend_background_label");
	legend_box_layout->addWidget(legend_background_label, 1, 2, Qt::AlignRight);
	_legend_background_fld = new QComboBox(false, legend_box_widget, "legend_background");
	_legend_background_fld->insertItem("Transparent", QPUI_TRANSPARENT);
	_legend_background_fld->insertItem("Opaque", QPUI_OPAQUE);
	QFontMetrics legend_background_fm(_legend_background_fld->font());
	int legend_background_w = legend_background_fm.width("Transparent");
	int legend_background_h = legend_background_fm.height();
	_legend_background_fld->setMinimumSize(legend_background_w, legend_background_h);
#if QT_VERSION < 0x040000
	_legend_background_fld->listBox()->setMinimumSize(legend_background_w, legend_background_h);
#else
        _legend_background_fld->setMinimumSize(legend_background_w, legend_background_h);
#endif
	QToolTip::add( _legend_background_fld, "Select legend background");
	legend_box_layout->addWidget(_legend_background_fld, 1, 3, Qt::AlignLeft);

	connect( _legend_background_fld, SIGNAL(activated(int)), this, SLOT(setLegendBackground(int)) );

	QLabel* num_rows_label = new QLabel("Number of rows: ", legend_box_widget, "num_rows_label");
	legend_box_layout->addWidget(num_rows_label, 3, 0, Qt::AlignRight);
	_legend_num_rows_fld = new QSpinBox(legend_box_widget,"legend_num_rows");
	_legend_num_rows_fld->setMinValue(1);
	_legend_num_rows_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_rows_fld, 3, 1, Qt::AlignLeft);

	connect(_legend_num_rows_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendNumRows(int)));

	QLabel* num_cols_label = new QLabel("Number of cols: ", legend_box_widget, "num_cols_label");
	legend_box_layout->addWidget(num_cols_label, 3, 2, Qt::AlignRight);
	_legend_num_cols_fld = new QSpinBox(legend_box_widget, "legend_num_cols");
	_legend_num_cols_fld->setMinValue(1);
	_legend_num_cols_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_cols_fld, 3, 3, Qt::AlignLeft);

	connect(_legend_num_cols_fld,SIGNAL(valueChanged(int)), this, SLOT(setLegendNumCols(int)));

	QLabel* legend_width_label = new QLabel("Width fraction: ", legend_box_widget, "legend_width_label");
	legend_box_layout->addWidget(legend_width_label, 5, 0, Qt::AlignRight);
	_legend_width_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_widget, "legend_width");
	_legend_width_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_width_fld, 5, 1, Qt::AlignLeft);
	_legend_width_fld->setValue(QPUI_LEGEND_WIDTH);

	connect(_legend_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendWidth(double)));

	QLabel* legend_height_label = new QLabel("Height fraction: ", legend_box_widget,"legend_height_label");
	legend_box_layout->addWidget(legend_height_label, 5, 2, Qt::AlignRight);
	_legend_height_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_widget, "legend_height");
	_legend_height_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_height_fld, 5, 3, Qt::AlignLeft);
	_legend_height_fld->setValue(QPUI_LEGEND_HEIGHT);

	connect(_legend_height_fld,SIGNAL(valueChanged(double)), this, SLOT(setLegendHeight(double)));

	legend_box_layout->addRowSpacing(0,0);
	legend_box_layout->addRowSpacing(1, _legend_border_fld->height());
	legend_box_layout->addRowSpacing(2,0);
	legend_box_layout->addRowSpacing(3,_legend_num_rows_fld->height());
	legend_box_layout->addRowSpacing(4,0);
	legend_box_layout->addRowSpacing(5,_legend_width_fld->height());
	legend_box_layout->addRowSpacing(6,0);

	legend_box_layout->setRowStretch(0,0);
	legend_box_layout->setRowStretch(1,1);
	legend_box_layout->setRowStretch(2,0);
	legend_box_layout->setRowStretch(3,1);
	legend_box_layout->setRowStretch(4,0);
	legend_box_layout->setRowStretch(5,1);
	legend_box_layout->setRowStretch(6,0);

	legend_box_layout->setColStretch(0,0);
	legend_box_layout->setColStretch(1,0);
	legend_box_layout->setColStretch(2,0);
	legend_box_layout->setColStretch(3,0);
	legend_box_layout->setColStretch(4,1);

	legend_properties_layout->addWidget(legend_box_groupbox);
	legend_properties_layout->addSpacing(4);
	legend_box_groupbox->setEnabled(false);

	legend_text_groupbox = new QGroupBox(1, Qt::Vertical, legend_properties_widget, "legend_text_groupbox");
	legend_text_groupbox->setTitle("Text");

	QWidget* legend_text_widget = new QWidget(legend_text_groupbox, "legend_text_widget");

	QGridLayout *legend_text_layout = new QGridLayout(legend_text_widget, 7, 5);
	legend_text_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_text_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_max_char_length_label = new QLabel("Max char length: ", legend_text_widget, "legend_max_char_length_label");
	legend_text_layout->addWidget(legend_max_char_length_label, 1, 0, Qt::AlignRight);
	_legend_max_char_length_fld = new QSpinBox(legend_text_widget, "legend_max_char_length");
	_legend_max_char_length_fld->setMinValue(5);
	_legend_max_char_length_fld->setMaxValue(80);
	_legend_max_char_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_max_char_length_fld, 1, 1, Qt::AlignLeft);
	_legend_max_char_length_fld->setValue(15);
	connect(_legend_max_char_length_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendMaxCharLength(int)));

	QLabel* legend_line_length_label = new QLabel("Line length fraction: ", legend_text_widget, "legend_line_length_label");
	legend_text_layout->addWidget(legend_line_length_label, 1, 2, Qt::AlignRight);
	_legend_line_length_fld = new FloatSpinBox(0.1, 0.9, 2, .05, legend_text_widget, "legend_line_length");
	_legend_line_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_line_length_fld, 1, 3, Qt::AlignLeft);
	_legend_line_length_fld->setValue(0.2);
	connect(_legend_line_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendLineLength(double)));

	QLabel* legend_char_size_label = new QLabel("Character size: ", legend_text_widget, "legend_char_size_label");
	legend_text_layout->addWidget(legend_char_size_label, 3, 0, Qt::AlignRight);
	_legend_char_size_fld = new FloatSpinBox(0.5, 40., 1, .1, legend_text_widget, "legend_char_size");
	_legend_char_size_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_char_size_fld, 3, 1, Qt::AlignLeft);
	_legend_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LEGEND_CHAR_SIZE_ADD);
	connect(_legend_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendCharSize(double)));

	QLabel* legend_just_label = new QLabel("Justification: ", legend_text_widget, "legend_just_label");
	legend_text_layout->addWidget(legend_just_label, 5, 0, Qt::AlignRight);
	_legend_just_fld = new QComboBox(false, legend_text_widget, "legend_just");
	_legend_just_fld->insertItem("Centre", QPUI_VCENTRE);
	_legend_just_fld->insertItem("Top", QPUI_TOP);
	_legend_just_fld->insertItem("Bottom", QPUI_BOTTOM);
	QFontMetrics legend_just_fm(_legend_just_fld->font());
	int legend_just_w = legend_just_fm.width("centre");
	int legend_just_h = legend_just_fm.height();
	_legend_just_fld->setMinimumSize(legend_just_w, legend_just_h);
#if QT_VERSION < 0x040000
	_legend_just_fld->listBox()->setMinimumSize(legend_just_w, legend_just_h);
#else
        _legend_just_fld->setMinimumSize(legend_just_w, legend_just_h);
#endif
	QToolTip::add( _legend_just_fld, "Select legend text justification");
	legend_text_layout->addWidget(_legend_just_fld, 5, 1, Qt::AlignLeft);

	connect( _legend_just_fld, SIGNAL(activated(int)), this, SLOT(setLegendJust(int)) );

	QLabel* legend_font_label = new QLabel("Font: ", legend_text_widget, "legend_font_label");
	legend_text_layout->addWidget(legend_font_label, 5, 2, Qt::AlignRight);
	_legend_font_fld = new FontButton(legend_text_widget);
	QToolTip::add( _legend_font_fld, "Select legend text font");
	legend_text_layout->addWidget(_legend_font_fld, 5, 3, Qt::AlignLeft);

	connect( _legend_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setLegendFamily(int)) );
	connect( _legend_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setLegendStyle(bool)) );
	connect( _legend_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setLegendWeight(bool)) );

	legend_text_layout->addRowSpacing(0,0);
	legend_text_layout->addRowSpacing(1, _legend_max_char_length_fld->height());
	legend_text_layout->addRowSpacing(2,0);
	legend_text_layout->addRowSpacing(3, _legend_char_size_fld->height());
	legend_text_layout->addRowSpacing(4,0);
	legend_text_layout->addRowSpacing(5, _legend_font_fld->height());
	legend_text_layout->addRowSpacing(6,0);

	legend_text_layout->setRowStretch(0,0);
	legend_text_layout->setRowStretch(1,1);
	legend_text_layout->setRowStretch(2,0);
	legend_text_layout->setRowStretch(3,1);
	legend_text_layout->setRowStretch(4,0);
	legend_text_layout->setRowStretch(5,1);
	legend_text_layout->setRowStretch(6,0);

	legend_text_layout->setColStretch(0,0);
	legend_text_layout->setColStretch(1,0);
	legend_text_layout->setColStretch(2,0);
	legend_text_layout->setColStretch(3,0);
	legend_text_layout->setColStretch(4,1);

	legend_properties_layout->addWidget(legend_text_groupbox);
	legend_properties_layout->addSpacing(4);

	legend_top_layout->addWidget(legend_properties_groupbox);
	legend_text_groupbox->setEnabled(false);

	legend_position_groupbox=new QGroupBox(1, Qt::Vertical, this,"legend_position_groupbox");

	legend_position_groupbox->setTitle("Legend Position");

	QWidget* legend_position_widget = new QWidget(legend_position_groupbox, "legend_position_widget");

	QGridLayout *legend_position_layout = new QGridLayout(legend_position_widget, 5, 5);
	legend_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_align_label = new QLabel("Alignment: ", legend_position_widget, "legend_align_label");
	legend_position_layout->addWidget(legend_align_label, 1, 0, Qt::AlignRight);
	_legend_align_fld = new QComboBox(false, legend_position_widget, "legend_align");
	_legend_align_fld->insertItem("Inside top right", QPUI_INSIDE_TOP_RIGHT);
	_legend_align_fld->insertItem("Inside top left", QPUI_INSIDE_TOP_LEFT);
	_legend_align_fld->insertItem("Inside bottom right", QPUI_INSIDE_BOTTOM_RIGHT);
	_legend_align_fld->insertItem("Inside bottom left", QPUI_INSIDE_BOTTOM_LEFT);
	_legend_align_fld->insertItem("Above top centre", QPUI_ABOVE_TOP_CENTRE);
	_legend_align_fld->insertItem("Outside top right", QPUI_OUTSIDE_TOP_RIGHT);
	_legend_align_fld->insertItem("User defined", QPUI_USER_DEFINED);
	QFontMetrics legend_align_fm(_legend_align_fld->font());
	int legend_align_w = legend_align_fm.width("Inside bottom right");
	int legend_align_h = legend_align_fm.height();
	_legend_align_fld->setMinimumSize(legend_align_w, legend_align_h);
#if QT_VERSION < 0x040000
	_legend_align_fld->listBox()->setMinimumSize(legend_align_w, legend_align_h);
#else
        _legend_align_fld->setMinimumSize(legend_align_w, legend_align_h);
#endif
	QToolTip::add( _legend_align_fld, "Select legend box alignment");
	legend_position_layout->addWidget(_legend_align_fld,1,1,Qt::AlignLeft);
	_legend_align_fld->setCurrentItem(QPUI_INSIDE_TOP_RIGHT);
	connect( _legend_align_fld, SIGNAL(activated(int)), this, SLOT(setLegendAlign(int)) );

	QLabel* legend_loffset_label = new QLabel("Left offset: ", legend_position_widget, "legend_loffset_label");
	legend_position_layout->addWidget(legend_loffset_label, 3, 0, Qt::AlignRight);
	_legend_loffset_fld = new FloatSpinBox( -40., 40., 2, .05, legend_position_widget, "legend_loffset");
	_legend_loffset_fld->setFixedWidth(100);
	_legend_loffset_fld->setFocusPolicy(STRONG_FOCUS);
	legend_position_layout->addWidget(_legend_loffset_fld, 3, 1, Qt::AlignLeft);
	_legend_loffset_fld->setValue(0.75);
	connect( _legend_loffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendLoffset(double)) );

	QLabel* legend_toffset_label = new QLabel("Top offset: ", legend_position_widget, "legend_toffset_label");
	legend_position_layout->addWidget(legend_toffset_label, 3, 2, Qt::AlignRight);
	_legend_toffset_fld = new FloatSpinBox(-40., 40., 2, .05, legend_position_widget, "legend_toffset");
	_legend_toffset_fld->setFixedWidth(100);
	_legend_toffset_fld->setFocusPolicy(STRONG_FOCUS);
	legend_position_layout->addWidget(_legend_toffset_fld, 3, 3, Qt::AlignLeft);

	connect( _legend_toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendToffset(double)) );

	_legend_loffset_fld->setEnabled(false);
	_legend_toffset_fld->setEnabled(false);
	
	legend_position_layout->addRowSpacing(0,0);
	legend_position_layout->addRowSpacing(1, _legend_align_fld->height());
	legend_position_layout->addRowSpacing(2,0);
	legend_position_layout->addRowSpacing(3, _legend_loffset_fld->height());
	legend_position_layout->addRowSpacing(4,0);

	legend_position_layout->setRowStretch(0,0);
	legend_position_layout->setRowStretch(1,0);
	legend_position_layout->setRowStretch(2,0);
	legend_position_layout->setRowStretch(3,0);
	legend_position_layout->setRowStretch(4,1);

	legend_position_layout->setColStretch(0,0);
	legend_position_layout->setColStretch(1,0);
	legend_position_layout->setColStretch(2,0);
	legend_position_layout->setColStretch(3,0);
	legend_position_layout->setColStretch(4,1);

	legend_top_layout->addWidget(legend_position_groupbox);
	legend_top_layout->addSpacing(4);
	legend_top_layout->addStretch(10);
	legend_position_groupbox->setEnabled(false);

	legend_properties_widget->adjustSize();
}
#else
QpLegendTab::QpLegendTab(QTabWidget *tabWidget,  QString name, QpPanelItem * panel):
		QpTab(tabWidget, name, QP_LEGEND_TAB)
{
	// Comstructor

	// keep track of item which owns this tab
	I_Panel = panel; 

	QVBoxLayout *legend_top_layout = new QVBoxLayout(this);
	legend_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *legend_properties_groupbox = new QGroupBox(this);

	legend_properties_groupbox->setTitle("Properties");

// 	QWidget* legend_properties_widget = new QWidget(legend_properties_groupbox, "legend_properties_widget");

	QVBoxLayout *legend_properties_layout = new QVBoxLayout;
	legend_properties_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_properties_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_show_legend_fld = new QCheckBox("Show legend");
	_show_legend_fld->setChecked(false);
	legend_properties_layout->addWidget(_show_legend_fld, 0, Qt::AlignLeft);

	connect( _show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend(bool)) );

	/*QGroupBox **/legend_box_groupbox = new QGroupBox(legend_properties_groupbox);
	legend_box_groupbox->setTitle("Box");

// 	legend_box_widget = new QWidget(legend_box_groupbox, "legend_box_widget");

	QGridLayout *legend_box_layout = new QGridLayout;
	legend_box_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_box_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_border_label = new QLabel("Border: ");
	legend_box_layout->addWidget(legend_border_label, 1, 0, Qt::AlignRight);
	_legend_border_fld = new QComboBox(false);
	_legend_border_fld->insertItem("No border", QPUI_NO_BORDER);
	_legend_border_fld->insertItem("Line", QPUI_LINE);
	QFontMetrics legend_border_fm(_legend_border_fld->font());
	int legend_border_w = legend_border_fm.width("No border");
	int legend_border_h=legend_border_fm.height();
	_legend_border_fld->setMinimumSize(legend_border_w, legend_border_h);

	QToolTip::add( _legend_border_fld, "Select style of legend border");
	legend_box_layout->addWidget(_legend_border_fld, 1, 1, Qt::AlignLeft);
	_legend_border_fld->setCurrentItem(QPUI_LINE);
	connect( _legend_border_fld, SIGNAL(activated(int)), this, SLOT(setLegendBorder(int)) );

	QLabel* legend_background_label = new QLabel("Background: ");
	legend_box_layout->addWidget(legend_background_label, 1, 2, Qt::AlignRight);
	_legend_background_fld = new QComboBox(false);
	_legend_background_fld->insertItem("Transparent", QPUI_TRANSPARENT);
	_legend_background_fld->insertItem("Opaque", QPUI_OPAQUE);
	QFontMetrics legend_background_fm(_legend_background_fld->font());
	int legend_background_w = legend_background_fm.width("Transparent");
	int legend_background_h = legend_background_fm.height();
	_legend_background_fld->setMinimumSize(legend_background_w, legend_background_h);

	QToolTip::add( _legend_background_fld, "Select legend background");
	legend_box_layout->addWidget(_legend_background_fld, 1, 3, Qt::AlignLeft);

	connect( _legend_background_fld, SIGNAL(activated(int)), this, SLOT(setLegendBackground(int)) );

	QLabel* num_rows_label = new QLabel("Number of rows: ");
	legend_box_layout->addWidget(num_rows_label, 3, 0, Qt::AlignRight);
	_legend_num_rows_fld = new QSpinBox;
	_legend_num_rows_fld->setMinValue(1);
	_legend_num_rows_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_rows_fld, 3, 1, Qt::AlignLeft);

	connect(_legend_num_rows_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendNumRows(int)));

	QLabel* num_cols_label = new QLabel("Number of cols: ");
	legend_box_layout->addWidget(num_cols_label, 3, 2, Qt::AlignRight);
	_legend_num_cols_fld = new QSpinBox;
	_legend_num_cols_fld->setMinValue(1);
	_legend_num_cols_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_num_cols_fld, 3, 3, Qt::AlignLeft);

	connect(_legend_num_cols_fld,SIGNAL(valueChanged(int)), this, SLOT(setLegendNumCols(int)));

	QLabel* legend_width_label = new QLabel("Width fraction: ");
	legend_box_layout->addWidget(legend_width_label, 5, 0, Qt::AlignRight);
	_legend_width_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_groupbox, "legend_width");
	_legend_width_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_width_fld, 5, 1, Qt::AlignLeft);
	_legend_width_fld->setValue(QPUI_LEGEND_WIDTH);

	connect(_legend_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendWidth(double)));

	QLabel* legend_height_label = new QLabel("Height fraction: ");
	legend_box_layout->addWidget(legend_height_label, 5, 2, Qt::AlignRight);
	_legend_height_fld = new FloatSpinBox(0.00, 40.00, 2, .05, legend_box_groupbox, "legend_height");
	_legend_height_fld->setFixedWidth(100);
	legend_box_layout->addWidget(_legend_height_fld, 5, 3, Qt::AlignLeft);
	_legend_height_fld->setValue(QPUI_LEGEND_HEIGHT);

	connect(_legend_height_fld,SIGNAL(valueChanged(double)), this, SLOT(setLegendHeight(double)));

// 	legend_box_layout->addRowSpacing(0,0);
// 	legend_box_layout->addRowSpacing(1, _legend_border_fld->height());
// 	legend_box_layout->addRowSpacing(2,0);
// 	legend_box_layout->addRowSpacing(3,_legend_num_rows_fld->height());
// 	legend_box_layout->addRowSpacing(4,0);
// 	legend_box_layout->addRowSpacing(5,_legend_width_fld->height());
// 	legend_box_layout->addRowSpacing(6,0);

	legend_box_layout->setRowStretch(0,0);
	legend_box_layout->setRowStretch(1,1);
	legend_box_layout->setRowStretch(2,0);
	legend_box_layout->setRowStretch(3,1);
	legend_box_layout->setRowStretch(4,0);
	legend_box_layout->setRowStretch(5,1);
	legend_box_layout->setRowStretch(6,0);

	legend_box_layout->setColStretch(0,0);
	legend_box_layout->setColStretch(1,0);
	legend_box_layout->setColStretch(2,0);
	legend_box_layout->setColStretch(3,0);
	legend_box_layout->setColStretch(4,1);

	legend_properties_layout->addWidget(legend_box_groupbox);
	legend_properties_layout->addSpacing(4);
	legend_box_groupbox->setEnabled(false);
	legend_box_groupbox->setLayout(legend_box_layout);

	legend_text_groupbox = new QGroupBox(legend_properties_groupbox);
	legend_text_groupbox->setTitle("Text");

// 	legend_text_widget = new QWidget(legend_text_groupbox, "legend_text_widget");

	QGridLayout *legend_text_layout = new QGridLayout;
	legend_text_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_text_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_max_char_length_label = new QLabel("Max char length: ");
	legend_text_layout->addWidget(legend_max_char_length_label, 1, 0, Qt::AlignRight);
	_legend_max_char_length_fld = new QSpinBox;
	_legend_max_char_length_fld->setMinValue(5);
	_legend_max_char_length_fld->setMaxValue(80);
	_legend_max_char_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_max_char_length_fld, 1, 1, Qt::AlignLeft);
	_legend_max_char_length_fld->setValue(15);
	connect(_legend_max_char_length_fld, SIGNAL(valueChanged(int)), this, SLOT(setLegendMaxCharLength(int)));

	QLabel* legend_line_length_label = new QLabel("Line length fraction: ");
	legend_text_layout->addWidget(legend_line_length_label, 1, 2, Qt::AlignRight);
	_legend_line_length_fld = new FloatSpinBox(0.1, 0.9, 2, .05, legend_text_groupbox, "legend_line_length");
	_legend_line_length_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_line_length_fld, 1, 3, Qt::AlignLeft);
	_legend_line_length_fld->setValue(0.2);
	connect(_legend_line_length_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendLineLength(double)));

	QLabel* legend_char_size_label = new QLabel("Character size: ", legend_text_groupbox, "legend_char_size_label");
	legend_text_layout->addWidget(legend_char_size_label, 3, 0, Qt::AlignRight);
	_legend_char_size_fld = new FloatSpinBox(0.5, 40., 1, .1, legend_text_groupbox, "legend_char_size");
	_legend_char_size_fld->setFixedWidth(100);
	legend_text_layout->addWidget(_legend_char_size_fld, 3, 1, Qt::AlignLeft);
	_legend_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LEGEND_CHAR_SIZE_ADD);
	connect(_legend_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setLegendCharSize(double)));

	QLabel* legend_just_label = new QLabel("Justification: ");
	legend_text_layout->addWidget(legend_just_label, 5, 0, Qt::AlignRight);
	_legend_just_fld = new QComboBox(false);
	_legend_just_fld->insertItem("Centre", QPUI_VCENTRE);
	_legend_just_fld->insertItem("Top", QPUI_TOP);
	_legend_just_fld->insertItem("Bottom", QPUI_BOTTOM);
	QFontMetrics legend_just_fm(_legend_just_fld->font());
	int legend_just_w = legend_just_fm.width("centre");
	int legend_just_h = legend_just_fm.height();
	_legend_just_fld->setMinimumSize(legend_just_w, legend_just_h);

	QToolTip::add( _legend_just_fld, "Select legend text justification");
	legend_text_layout->addWidget(_legend_just_fld, 5, 1, Qt::AlignLeft);

	connect( _legend_just_fld, SIGNAL(activated(int)), this, SLOT(setLegendJust(int)) );

	QLabel* legend_font_label = new QLabel("Font: ");
	legend_text_layout->addWidget(legend_font_label, 5, 2, Qt::AlignRight);
	_legend_font_fld = new FontButton;

	QToolTip::add( _legend_font_fld, "Select legend text font");
	legend_text_layout->addWidget(_legend_font_fld, 5, 3, Qt::AlignLeft);

	connect( _legend_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setLegendFamily(int)) );
	connect( _legend_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setLegendStyle(bool)) );
	connect( _legend_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setLegendWeight(bool)) );

// 	legend_text_layout->addRowSpacing(0,0);
// 	legend_text_layout->addRowSpacing(1, _legend_max_char_length_fld->height());
// 	legend_text_layout->addRowSpacing(2,0);
// 	legend_text_layout->addRowSpacing(3, _legend_char_size_fld->height());
// 	legend_text_layout->addRowSpacing(4,0);
// 	legend_text_layout->addRowSpacing(5, _legend_font_fld->height());
// 	legend_text_layout->addRowSpacing(6,0);

	legend_text_layout->setRowStretch(0,0);
	legend_text_layout->setRowStretch(1,1);
	legend_text_layout->setRowStretch(2,0);
	legend_text_layout->setRowStretch(3,1);
	legend_text_layout->setRowStretch(4,0);
	legend_text_layout->setRowStretch(5,1);
	legend_text_layout->setRowStretch(6,0);

	legend_text_layout->setColStretch(0,0);
	legend_text_layout->setColStretch(1,0);
	legend_text_layout->setColStretch(2,0);
	legend_text_layout->setColStretch(3,0);
	legend_text_layout->setColStretch(4,1);

	legend_text_groupbox->setLayout(legend_text_layout);
	
	legend_properties_layout->addWidget(legend_text_groupbox);
	legend_properties_layout->addSpacing(4);
	
	legend_properties_groupbox->setLayout(legend_properties_layout);
	
	legend_top_layout->addWidget(legend_properties_groupbox);
	legend_text_groupbox->setEnabled(false);

	legend_position_groupbox=new QGroupBox(this);

	legend_position_groupbox->setTitle("Legend Position");

// 	legend_position_widget = new QWidget(legend_position_groupbox, "legend_position_widget");

	QGridLayout *legend_position_layout = new QGridLayout;
	legend_position_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	legend_position_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* legend_align_label = new QLabel("Alignment: ");
	legend_position_layout->addWidget(legend_align_label, 1, 0, Qt::AlignRight);
	_legend_align_fld = new QComboBox(false);
	_legend_align_fld->insertItem("Inside top right", QPUI_INSIDE_TOP_RIGHT);
	_legend_align_fld->insertItem("Inside top left", QPUI_INSIDE_TOP_LEFT);
	_legend_align_fld->insertItem("Inside bottom right", QPUI_INSIDE_BOTTOM_RIGHT);
	_legend_align_fld->insertItem("Inside bottom left", QPUI_INSIDE_BOTTOM_LEFT);
	_legend_align_fld->insertItem("Above top centre", QPUI_ABOVE_TOP_CENTRE);
	_legend_align_fld->insertItem("Outside top right", QPUI_OUTSIDE_TOP_RIGHT);
	_legend_align_fld->insertItem("User defined", QPUI_USER_DEFINED);
	QFontMetrics legend_align_fm(_legend_align_fld->font());
	int legend_align_w = legend_align_fm.width("Inside bottom right");
	int legend_align_h = legend_align_fm.height();
	_legend_align_fld->setMinimumSize(legend_align_w, legend_align_h);

	QToolTip::add( _legend_align_fld, "Select legend box alignment");
	legend_position_layout->addWidget(_legend_align_fld,1,1,Qt::AlignLeft);
	_legend_align_fld->setCurrentItem(QPUI_INSIDE_TOP_RIGHT);
	connect( _legend_align_fld, SIGNAL(activated(int)), this, SLOT(setLegendAlign(int)) );

	QLabel* legend_loffset_label = new QLabel("Left offset: ");
	legend_position_layout->addWidget(legend_loffset_label, 3, 0, Qt::AlignRight);
	_legend_loffset_fld = new FloatSpinBox( -40., 40., 2, .05, legend_position_groupbox, "legend_loffset");
	_legend_loffset_fld->setFixedWidth(100);
	_legend_loffset_fld->setFocusPolicy(STRONG_FOCUS);
	legend_position_layout->addWidget(_legend_loffset_fld, 3, 1, Qt::AlignLeft);
	_legend_loffset_fld->setValue(0.75);
	connect( _legend_loffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendLoffset(double)) );

	QLabel* legend_toffset_label = new QLabel("Top offset: ");
	legend_position_layout->addWidget(legend_toffset_label, 3, 2, Qt::AlignRight);
	_legend_toffset_fld = new FloatSpinBox(-40., 40., 2, .05, legend_position_groupbox, "legend_toffset");
	_legend_toffset_fld->setFixedWidth(100);
	_legend_toffset_fld->setFocusPolicy(STRONG_FOCUS);
	legend_position_layout->addWidget(_legend_toffset_fld, 3, 3, Qt::AlignLeft);

	connect( _legend_toffset_fld, SIGNAL(valueChanged(double)),this, SLOT(setLegendToffset(double)) );

	_legend_loffset_fld->setEnabled(false);
	_legend_toffset_fld->setEnabled(false);
	
// 	legend_position_layout->addRowSpacing(0,0);
// 	legend_position_layout->addRowSpacing(1, _legend_align_fld->height());
// 	legend_position_layout->addRowSpacing(2,0);
// 	legend_position_layout->addRowSpacing(3, _legend_loffset_fld->height());
// 	legend_position_layout->addRowSpacing(4,0);

	legend_position_layout->setRowStretch(0,0);
	legend_position_layout->setRowStretch(1,0);
	legend_position_layout->setRowStretch(2,0);
	legend_position_layout->setRowStretch(3,0);
	legend_position_layout->setRowStretch(4,1);

	legend_position_layout->setColStretch(0,0);
	legend_position_layout->setColStretch(1,0);
	legend_position_layout->setColStretch(2,0);
	legend_position_layout->setColStretch(3,0);
	legend_position_layout->setColStretch(4,1);
	
	legend_position_groupbox->setLayout(legend_position_layout);

	legend_top_layout->addWidget(legend_position_groupbox);
	legend_top_layout->addSpacing(4);
	legend_top_layout->addStretch(10);
	legend_position_groupbox->setEnabled(false);

}
#endif

// ---------------------------------------------

void QpLegendTab::setShowLegend(bool choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   legend_box_groupbox->setEnabled(choice);
   legend_text_groupbox->setEnabled(choice);
   legend_position_groupbox->setEnabled(choice);
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(SHOW_LEGEND, 0, 0.0, "", choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendBorder(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_BORDER, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendBackground(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_BACKGROUND, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendNumRows(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_NUM_ROWS, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendNumCols(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_NUM_COLS, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendWidth(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   setLegendAlign(_legend_align_fld->currentItem());
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_WIDTH, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendHeight(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   setLegendAlign(_legend_align_fld->currentItem());
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_HEIGHT, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendMaxCharLength(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_MAX_CHAR_LENGTH, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendLineLength(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_LINE_LENGTH, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendCharSize(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_CHAR_SIZE, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendJust(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_JUST, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendFamily(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_FONT_FAMILY, choice);
}

void QpLegendTab::setLegendStyle(bool choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_FONT_STYLE,0,0.,"", choice);
}

void QpLegendTab::setLegendWeight(bool choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_FONT_WEIGHT,0,0.,"", choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendAlign(int choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   	// calculate loffset and toffset based on alignment
	float legend_width = (float) _legend_width_fld->value();
	float legend_height = (float) _legend_height_fld->value();
	
	switch(choice){
	
		case QPUI_INSIDE_TOP_RIGHT:
			_legend_loffset_fld->setValue(1.0 - legend_width);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_TOP_LEFT:
			_legend_loffset_fld->setValue(0.0);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_BOTTOM_RIGHT:
			_legend_loffset_fld->setValue(1.0 - legend_width);
			_legend_toffset_fld->setValue(1.0 - legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_INSIDE_BOTTOM_LEFT:
			_legend_loffset_fld->setValue(0.0);
			_legend_toffset_fld->setValue(1.0 - legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_ABOVE_TOP_CENTRE:
			_legend_loffset_fld->setValue(0.5 - 0.5 * legend_width);
			_legend_toffset_fld->setValue(- legend_height);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_OUTSIDE_TOP_RIGHT:
			_legend_loffset_fld->setValue(1.0);
			_legend_toffset_fld->setValue(0.0);
			_legend_loffset_fld->setEnabled(false);
			_legend_toffset_fld->setEnabled(false);
			break;
		case QPUI_USER_DEFINED:
			_legend_loffset_fld->setEnabled(true);
			_legend_toffset_fld->setEnabled(true);
			break;
	}

   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_ALIGN, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendLoffset(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_L_OFFSET, 0, choice);
}

// ---------------------------------------------

void QpLegendTab::setLegendToffset(double choice)
{
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
   if(this->isVisible() ) I_Panel->I_ListView->setLegendOption(LEGEND_T_OFFSET, 0, choice);
}


// -------------------------------------

void QpLegendTab::updateNumTraces()
{
	if(QuSlotDebug()) cout << "QpLegendTab::updateNumTraces " << endl;

	// updates number of rows when trace number changed  
	int num_traces = I_Panel->numberTraces();
	if(num_traces > 0)
	{
		int num_rows = (int)ceil((float)num_traces / _legend_num_cols_fld->value());
		_legend_num_rows_fld->setValue(num_rows);
	}
	
}

