// qpui_window.cc
//
// Plot Interface Implementation - Plot Window and ListView components
//
// Janet Barnes
//
// July 2002

#include "qpui.h"
#include "qwui.h"
#include "qdutil.h"

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release; 

//     QpuiWindow CLASS
//     ----------------

QpWindow::QpWindow( QWidget *parent ):QguWindow(parent,"Plot")
{
	// Constructor
	if(QuSlotDebug())
	{
		printf("QpWindow::QpWindow %ld\n", this);
		fflush(stdout);
	}

	QguWindow::_saveset_filter = QString("Plot savesets (*.qs_plot)");
	
	// set up colour group palettes
	extern QPalette QPUI_normal_palette;
	extern QPalette QPUI_different_palette;
	QPUI_normal_palette = QPalette(qApp->palette());
	this->setPalette(QPUI_normal_palette);

	QColorGroup different_cg = QColorGroup(QPUI_normal_palette.disabled());
	QPUI_different_palette.setActive(different_cg);
	QPUI_different_palette.setInactive(different_cg);
	QPUI_different_palette.setDisabled(different_cg);

	setAcceptDrops(true);
	setFocusPolicy(WHEEL_FOCUS);

	_top_widget = new QWidget(this, "top_widget");
	setCentralWidget( _top_widget );

	_top_gl = new QGridLayout( _top_widget, 2, 1 );
	_top_gl->setSpacing(QSAS_profiles->current()->GUImargins);
	_top_gl->setMargin(QSAS_profiles->current()->GUImargins);
	
	_top_splitter=new QSplitter(_top_widget);
	_top_gl->addWidget( _top_splitter, 0,0 );


	
	//  list view of plot components
	_plot_listview = new QpListView( _top_splitter, this, "Plot List");
	_plot_listview->addColumn("Plot Components");  	
	_plot_listview->addColumn("Link", 30);

	_plot_listview->setRootIsDecorated(true);
	_plot_listview->setMinimumWidth(280);
	_plot_listview->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
	connect(_plot_listview, SIGNAL(viewWL_ObjRequest()), qsasWindow->WorkingList, SIGNAL(viewObjRequest()));
  
#if QT_VERSION <0x030000
	Q_SCROLL_VIEW *scroll=new Q_SCROLL_VIEW(_top_splitter, "scroll");
	scroll->setResizePolicy(Q_SCROLL_VIEW::AutoOne);
#else
	Q_SCROLL_VIEW *scroll=new Q_SCROLL_VIEW(_top_splitter);
#if QT_VERSION <0x040000
	scroll->setResizePolicy(Q_SCROLL_VIEW::AutoOneFit);
#else
	scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	scroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
#endif // if QT_VERSION <0x040000
#endif // end of #if
	
#if QT_VERSION < 0x040000
	QGroupBox* plot_groupbox = new QGroupBox( 1, Qt::Vertical, scroll->viewport(), "plot_groupbox");
#else
	QGroupBox* plot_groupbox = new QGroupBox(scroll->viewport());
	QVBoxLayout * plot_layout=new QVBoxLayout;
#endif
	plot_groupbox->setTitle("Plot Options");
	plot_groupbox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));

	// create tab widget for ALL tabs attached to a QpListItem
	_plot_listview->tabWidget = new QTabWidget(plot_groupbox, "tabWidget");
	
#if QT_VERSION >= 0x040000
	plot_layout->addWidget(_plot_listview->tabWidget);
	plot_groupbox->setLayout(plot_layout);
#endif
	
	_plot_listview->tabWidget->setMargin(QSAS_profiles->current()->GUImargins);

	// set up menu bar
	_menu_bar = menuBar();
	_menu_bar->clear();
	Q_POPUP_MENU* file_menu = new Q_POPUP_MENU(this);
	Q_POPUP_MENU* insert_menu = new Q_POPUP_MENU(this);
	Q_POPUP_MENU* edit_menu = new Q_POPUP_MENU(this);
	Q_POPUP_MENU* help_menu = new Q_POPUP_MENU( this);
	Q_POPUP_MENU* template_menu = new Q_POPUP_MENU( this);	


	file_menu->clear();
	file_menu->insertItem("&Save", this, SLOT(saveWithOut()));
	file_menu->insertItem("Save &As...", this, SLOT(saveWithOutAs()));
	file_menu->insertSeparator();
	file_menu->insertItem("&Include...", this, SLOT(include()));
	file_menu->insertItem("&Restore...", this, SLOT(restore()));
	file_menu->insertSeparator();
	file_menu->insertItem("P&lot Selected Pages", _plot_listview, SLOT(plotSelectedItems()));
	file_menu->insertSeparator();
	file_menu->insertItem("Use Defaults in Selected Items", _plot_listview, SLOT(setDefaultSelectedItems()));
	file_menu->insertSeparator();
	file_menu->insertItem("&Close", this, SLOT(close()));
	_menu_bar->insertItem("&File", file_menu );

	template_menu->clear();
	template_menu->insertItem("Vector RXYZ", _plot_listview, SLOT(makeVector_RXYZ_Item()));
	template_menu->insertItem("Vector XYZ", _plot_listview, SLOT(makeVector_XYZ_Item()));
	template_menu->insertSeparator();
	template_menu->insertItem("Vector R_XYZ", _plot_listview, SLOT(makeVector_R_XYZ_Item()));
	template_menu->insertItem("Vector R_LP", _plot_listview, SLOT(makeVector_R_LP_Item()));
	template_menu->insertItem("Vector R_TP", _plot_listview, SLOT(makeVector_R_TP_Item()));
	template_menu->insertSeparator();
	template_menu->insertItem("Vector R_X_Y_Z", _plot_listview, SLOT(makeVector_R_X_Y_Z_Item()));
	template_menu->insertItem("Vector X_Y_Z", _plot_listview, SLOT(makeVector_X_Y_Z_Item()));
	template_menu->insertItem("Vector R_L_P", _plot_listview, SLOT(makeVector_R_L_P_Item()));
	template_menu->insertItem("Vector R_T_P", _plot_listview, SLOT(makeVector_R_T_P_Item()));
	template_menu->insertSeparator();
	template_menu->insertItem("2 Panels", _plot_listview, SLOT(make2Panel_Item()));
	template_menu->insertItem("3 Panels", _plot_listview, SLOT(make3Panel_Item()));
	template_menu->insertItem("4 Panels", _plot_listview, SLOT(make4Panel_Item()));
	template_menu->insertSeparator();
	template_menu->insertItem("C1_2_3_4", _plot_listview, SLOT(makeC1_2_3_4_Item()));
	template_menu->insertItem("C1234", _plot_listview, SLOT(makeC1234_Item()));

	insert_menu->clear();
	insert_menu->insertItem("Trace", _plot_listview, SLOT(makeTraceItem()));
	insert_menu->insertItem("SpectroGram", _plot_listview, SLOT(makeSpectrumItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("Panel", _plot_listview, SLOT(makePanelItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("TS Frame", _plot_listview, SLOT(makeTSFrameItem()));
	insert_menu->insertItem("XY Frame", _plot_listview, SLOT(makeXYFrameItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("New Page", _plot_listview, SLOT(makePageItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("Constant X or T Trace", _plot_listview, SLOT(makeConstXTraceItem()));
	insert_menu->insertItem("Constant Y Trace", _plot_listview, SLOT(makeConstYTraceItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("Event Label", _plot_listview, SLOT(makeEventItem()));
	insert_menu->insertSeparator();
	insert_menu->insertItem("Template...", template_menu);

	_menu_bar->insertItem("Insert", insert_menu );

	edit_menu->clear();
	edit_menu->insertItem("Rename", _plot_listview, SLOT(renameSelectedItem()));
	edit_menu->insertItem("&Copy", _plot_listview, SLOT(copy()), Qt::CTRL+Qt::Key_C);
	edit_menu->insertItem("Paste", _plot_listview, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_menu->insertSeparator();
	edit_menu->insertItem("Select &All", _plot_listview, SLOT( selectAll() ),Qt::CTRL+Qt::Key_A );
	edit_menu->insertItem("Move Up", _plot_listview, SLOT(moveItemUp()), Qt::CTRL+Qt::Key_Up);
	edit_menu->insertItem("Move Down", _plot_listview, SLOT(moveItemDown()), Qt::CTRL+Qt::Key_Down);
	edit_menu->insertSeparator();
	edit_menu->insertItem("Delete", _plot_listview, SLOT(deleteSelectedItems()), Qt::CTRL+Qt::Key_X);
	edit_menu->insertSeparator();
	edit_menu->insertItem("&Link Selected Traces", _plot_listview, SLOT( linkPlots() ),Qt::CTRL+Qt::Key_L);
	edit_menu->insertItem("&Unlink Selected Traces", _plot_listview, SLOT( unlinkPlots() ),Qt::CTRL+Qt::Key_U);  
	_menu_bar->insertItem("Edit", edit_menu );

	_menu_bar->insertSeparator();
	help_menu->clear();
	help_menu->insertItem("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help_menu->insertSeparator();
	help_menu->insertItem("...Plotting", this, SLOT( launchManPlot() ));
	help_menu->insertSeparator();
	help_menu->insertItem("Module Version", this, SLOT( launchVersion() ));
	_menu_bar->insertItem("&Help", help_menu );

	_status_bar = statusBar();
	_status_bar->message(" ");

	QHBoxLayout *buttons_layout = new QHBoxLayout();

	plotBtn = new QPushButton("Plot on Screen",_top_widget,"plotBtn");
	plotBtn->setMinimumSize(plotBtn->sizeHint());
	plotBtn->setFocusPolicy(STRONG_FOCUS);
	connect(plotBtn, SIGNAL(clicked()), this, SLOT(doPlot()));
	buttons_layout->addWidget(plotBtn);
	plotBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss", _top_widget, "dismissBtn" );
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	dismissBtn->setFocusPolicy(STRONG_FOCUS);
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	//buttons_layout->insertSpacing(0, 50);
	buttons_layout->addWidget(dismissBtn);

	_top_gl->addLayout(buttons_layout,2,0);
	_top_gl->setRowStretch(0,1);
	_top_gl->setRowStretch(1,0);

	connect(_plot_listview, SIGNAL(status_msg(const QString&)), this, SLOT(status(const QString&)));

	connect(this, SIGNAL(WLmodified()), _plot_listview, SLOT(refreshWLitems()));

	QpPageItem *startPage = _plot_listview->makePage();
	startPage->showTabs();
	_plot_listview->tabWidget->adjustSize();
	
	connect(_plot_listview, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));
	
	this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));

#if QT_VERSION < 0x040000
	scroll->addChild(plot_groupbox);
#else
	_top_splitter->setStretchFactor(0, 1);
	_top_splitter->setStretchFactor(1, 10);
	scroll->setWidgetResizable(true);
	scroll->setWidget(plot_groupbox);
#endif
	
	adjustSize();
}

void QpWindow::keyPressEvent(QKeyEvent * e )
{  
	if(QuSlotDebug())
	{
		printf("QpWindow::keyPressEvent %ld\n", this);
		fflush(stdout);
	}

	// use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
		if(   plotBtn->isDefault()) {
			plotBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault())  {
			dismissBtn->setDefault(false);
			plotBtn->setDefault(true);
		}
		return;
	}
	// use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if(plotBtn->isDefault()) doPlot();
		else if (dismissBtn->isDefault()) close();
		return;
	}

	// else pass on to list view 
	e->ignore(); // let selection change

}

void QpWindow::deleteAllItems()
{
	if(QuSlotDebug())
	{
		printf("QpWindow::deleteAllItems \n");
		fflush(stdout);
	}
	_plot_listview->selectAll();
	_plot_listview->deleteSelectedItems();
}


// --------------------------------------

void QpWindow::resetDefaultBtn()
{  
	dismissBtn->setDefault(false);
	//printBtn->setDefault(false);
	plotBtn->setDefault(true);
}

// --------------------------------------

void QpWindow::doPlot()
{
	// slot invoked when Plot push button clicked
  	if(QuSlotDebug())
	{
		printf("QpWindow::doPlot %ld\n", this);
		fflush(stdout);
	}

   if(!_plot_listview)
	{
		(void)QMessageBox::warning(this,"Print","No active listview");
		return;
	}

	QpPageItem* page_item= dynamic_cast<QpPageItem*>(_plot_listview->firstChild());

	if(!page_item)
	{
		(void)QMessageBox::warning(this,"Print","No pages to plot");
		return;
	}

	_plot_listview->_plot_page_list.clear();
	while(page_item)
	{
		_plot_listview->_plot_page_list.push_back(page_item);
		page_item = dynamic_cast<QpPageItem*>(page_item->nextSibling());
	}

	_plot_listview->plot();

}

// --------------------------------------

// void QpWindow::doPrint()
// {
// 	// slot invoked when Print push button clicked
// 	if(QuSlotDebug()) printf("QpWindow::doPrint %ld\n", this);
// 
// 	doPlot(true);
// }

// --------------------------------------

void QpWindow::launchManPlot()
{
	// slot invoked from Plotting Help menu sub item
	// launches browser showing Qsas Plot help docuumentation

	if(QuSlotDebug())
	{
		printf("QpWindow::launchManPlot %ld\n", this);
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_plotting.html");
}

void QpWindow::launchManual()
{
	// slot invoked from Qsas Manual Help menu sub item
	// launches browser showing Qsas help index

	if(QuSlotDebug())
	{
		printf("QpWindow::launchManual %ld\n", this);
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_help_index.html");
}

void QpWindow::launchVersion()
{
	// slot invoked from Module Version Help menu sub item
	// launches browser showing Qsas version numbers

	if(QuSlotDebug())
	{
		printf("QpWindow::launchVersion %ld\n",this);
		fflush(stdout);
	}

	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n Plotting  ";
	text += QString(QPUI_VERSION);
	QMessageBox::information( this, "QSAS information", text.latin1(), "OK" );
}


void QpWindow::restore(Vistream& v_is,bool replace)
{
	// restore
	
	// May need to rewind if old saveset
	int pos=v_is.getpos();
	
	string guiID;
	v_is >> guiID;

	if(QuSlotDebug())
	{
		printf("QpWindow::restore set ID %s\n-------------------------------------\n",guiID.c_str());
		fflush(stdout);
	}
	// set QSAS global restore id number

	setRestoreVersion(guiID.c_str()); 
	
	if( qsas_restore_release < 20300)
	{
		// Note old plot sets either give correct id or zero, both get here.
		printf("QpWindow::restoring old plot V%d\n-------------------------------------\n",qsas_restore_release);
		v_is.setpos(pos);
		restoreV22(v_is, replace);
		return;
	}

	try{
		_restored_objects->unbind(Name("qpui"));
	}
	catch(NamingException&)
	{;}
	_restored_objects->bind_new_context(Name("qpui"));
  
	if(replace==true)  {
		_plot_listview->clear();
		_plot_listview->LinkNumber=0;
	}

	_plot_listview->blockSignals(true);	
	v_is >> *_plot_listview;
	_plot_listview->blockSignals(false);	

	_plot_listview->clearSelectedList();
	_plot_listview->setCurrentItem(_plot_listview->firstChild());
	_plot_listview->updateOnSelection();

}

void QpWindow::save(Vostream& v_os)
{
	// save
   
	string guiID;
	guiID=QSAS_VERSION_ID;

	v_os << guiID;
	if(with)
	{
		try{
			_saved_objects->unbind(Name("qpui"));
		}
		catch(NamingException&)
		{;}
		_saved_objects->bind_new_context(Name("qpui"));
	}
	_plot_listview->with = with;
	v_os << *_plot_listview;

}


