
#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <qglobal.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qobject.h>
#include <qpalette.h>
#include <qlineedit.h>
#include <qinputdialog.h>
#include <qstatusbar.h>
#include <qregexp.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qdatetime.h> 
#include <qstring.h> 
#include <qwidget.h>
#include <qcheckbox.h>

#include "Qdos.h"
#include "Xrefs.h"
#include "qdutil.h"

using namespace QSAS;

ObjectNamesTable_var QDOSlist = new ObjectNamesTable;
static double CLOSE_ENOUGH = 1.e-4;
int qsas_restore_release;

// ------------------------------------------

QString QuGetDefaultComp(const QString& WL, const QString& compDef)
{
	if(QuSlotDebug()) cout << "qdutil::QuGetDefaultComp" << endl;
	QString outText = QString(WL.latin1());
	
	QString compTxt = " ";
	QdObject_var dobj = QuGetObjByName(WL);
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(dobj);
	if(ms.is_nil()) return outText;
	
	if((ms[0].dims()).size()!=3) return outText;
	if(!ms->is_homogeneous_seq()) return false;
	
	int d1 = (ms[0].dims())[0];
	int d2 = (ms[0].dims())[1];
	int d3 = (ms[0].dims())[2];

	if(compDef=="iSS")
	{
		compTxt += QString("[(1,%1) Sum(1,%2) Sum(1,%3) ]").arg(d1).arg(d2).arg(d3);
	}
	else if(compDef=="SiS")
	{
		compTxt += QString("[Sum(1,%1) (1,%2) Sum(1,%3) ]").arg(d1).arg(d2).arg(d3);
	}
	else if(compDef=="SSi")
	{
		compTxt += QString("[Sum(1,%1) Sum(1,%2) (1,%3) ]").arg(d1).arg(d2).arg(d3);
	}
	else if(compDef=="SSS")
	{
		compTxt += QString("[Sum(1,%1) Sum(1,%2) Sum(1,%3) ]").arg(d1).arg(d2).arg(d3);
	}
	else if(compDef=="AAA")
	{
		compTxt += QString("[Ave(1,%1) Ave(1,%2) Ave(1,%3) ]").arg(d1).arg(d2).arg(d3);
	}
	else compTxt = QString(" ")+compDef;
	
	outText += QString(compTxt);
	
	return outText;
}

// ------------------------------------------

void QuReplaceTokens(QString &newText, QString &oldText)
{
	int ptr = oldText.find("?");
	while( ptr != -1)
	{
		oldText.remove(0, ptr+1);

		int ptr2 = oldText.find("=");
		QString before = QString(oldText.latin1()); 
		before.truncate(ptr2);
		oldText.remove(0, ptr2+1);
		
		int ptr3 = oldText.find("?");
		QString after = QString(oldText.latin1()); 
		after.truncate(ptr3);

		newText.replace(before, after, false);
		
		oldText.remove(0, ptr3+1);

		ptr = oldText.find("?");

	}
}

bool QuIsMatch(double value, double fill){
	// used by Filters in Calc and Filter plugins
	// Does not check strip toggles on main window

    if( fill == 0 && value == fill) return true;
    if( fill == -1 && value == fill) return true;
    else if ( fabs((value - fill)/fill) < CLOSE_ENOUGH ) return true;
	
  return false;
}

QCheckBox * checkBadData(QWidget *parent){
   static QCheckBox *StripBadData = NULL;
   
   if(StripBadData == NULL) {
      StripBadData = new QCheckBox("Strip NaN", parent);
   }
   return StripBadData;
}

QCheckBox * checkFills(QWidget *parent){
   static QCheckBox *StripFills = NULL;
   
   if(StripFills == NULL) {
      StripFills = new QCheckBox("Strip Fill Values", parent);
   }
   return StripFills;
}

bool QuSlotDebug(){

	return (QSAS_profiles->current()->Debug);
}


string QuCommonLabl(string str1, string str2){
  string commonStr="";
  // remove mod signs "|"
  size_t pos = str1.find("|");
  if(pos != string::npos) {
    str1 = str1.substr(pos+1, str1.size() - pos -1);
	pos = str1.find("|");
	if(pos != string::npos) {
       str1 = str1.substr(0,pos);
	}
  }
  
  pos = str2.find("|");
  if(pos != string::npos) {
    str2 = str2.substr(pos+1, str2.size() - pos -1);
	pos = str2.find("|");
	if(pos != string::npos) {
       str2 = str2.substr(0,pos);
	}
  }

  
  if (str1 == str2) return str1;
  
  // break into stem and component
  pos =  str2.find("_");
  if(pos != string::npos) {
    string s1 = str2.substr(0,pos);
    string s2 = str2.substr(pos+1, str2.size() - pos -1);
	pos = s2.find(" ");
	if(pos != string::npos) s2 = s2.substr(0,pos);
	
	pos = str1.find(s1);
	if(pos != string::npos) return s1;
	pos = str1.find(s2);
	if(pos != string::npos) return s2;
  }
  else {
    for(int i=0; i<str1.size(); i++){
	  if(i == str2.size() ) break;
	  if(str1[i] == '(') break;
	  if(str1[i] == str2[i]) commonStr += str1[i];
	  else break;
	}
  }
  
  return commonStr;
}


QdTimeSeq_var get_timetags(const QdObject_var& obj)
{
    QdTimeSeq_var nil_obj;
    if(obj.is_nil()) return  nil_obj;  // NULL var

    QdTimeSeq_var ts=QdTimeSeq_var::narrow(obj);
    if(ts.is_nil())
        if(obj->xref_exists(TIME_TAGS))
            ts=QdTimeSeq_var::narrow(obj->get_xref(TIME_TAGS));

    return ts;
}

QdTimeInterval_var get_timeinterval(const QdObject_var& obj)
{
    QdTimeInterval_var tint;
    if(obj.is_nil()) return tint; // must test return with is_nil()
    tint=QdTimeInterval_var::narrow(obj);
    if(tint.is_nil())
    {
        QdTimeSeq_var ts=get_timetags(obj);
        if(!ts.is_nil())
            tint=new QdTimeInterval(ts->get_interval_bounds());
    }
    return tint;
}


QdTimeSeq_var get_timetags_inrange(const QdTimeSeq_var& ts_in, const QdTimeInterval_var& tint)
{
    int recStart, recEnd;
	bool OK = QuGetIvlRecBound( ts_in, recStart, recEnd, tint->start(), tint->end());
    if( OK )
	{
	    QdTimeSeq_var ts_out =new QdTimeSeq(recEnd - recStart +1);
        for(int i=recStart, j=0; i<= recEnd; i++, j++) ts_out[j] = ts_in->elt(i);

	    return ts_out;
	}
   
	QdTimeSeq_var nilObj;
    return nilObj;
}


double get_time_spacing(const QdObject_var& obj)
{
    double spacing;
    if(obj.is_nil()) return  -1.0;

    QdTimeSeq_var ts=get_timetags(obj);
    if(ts.is_nil()) return -1.0;
    if(ts->sequence_size() < 2) spacing = -1;
    else{
      spacing = fabs(Seconds(ts[1] - ts[0]));
    }
    for(int i=2; i<ts->sequence_size(); i++){
      double next = Seconds(ts[i] - ts[i-1]);
      if( fabs(next) < spacing ) spacing = fabs(next);
    }
    // data glitches can give spacing too small

    return spacing;
}

void set_timetags(QdObject_var obj, QdTimeSeq_var ts)
{
    if(obj->xref_exists(TIME_TAGS))
        obj->delete_xref(TIME_TAGS);
    obj->set_xref(TIME_TAGS,ts);
}

int num_timetags_inrange(const QdTimeSeq_var& ts, const QdTimeInterval_var& tint)
{
    if(ts.is_nil() || tint.is_nil() ) return 0;
    int recStart, recEnd;
	
    if( QuGetIvlRecBound( ts, recStart, recEnd, tint->start(), tint->end())  ) return recEnd - recStart + 1;
    else return 0;

}

bool is_empty(const QdObject_var& obj)
{
    // if var pointer is nil then empty
    if(obj.is_nil())
        return true;

    // if a sequence then empty if sequence_size() is zero
    if(obj->is_sequence())
        return (obj->sequence_size()==0);

    // time ... empty if zero (for time being!!)
    QdTime_var time_var = QdTime_var::narrow( obj );
    if( !time_var.is_nil() )
        return (time_var->get_time_rep()==(QDOS::Time::TRep)0);

    // time interval ... empty if start=end time
    QdTimeInterval_var timeinterval_var = QdTimeInterval_var::narrow( obj );
    if( !timeinterval_var.is_nil() )
        return timeinterval_var->empty();

    // ..otherwise if a single object, always set up with a default value
    // so "not empty"
    return false;
}
 /***********************************************************************/

QDOS::ObjectNamesTable_var QuGet_object_names_table(){

   return QDOSlist;
}

 /***********************************************************************/

/// \fn  dimTripleStack getDimTripleStack(QString &compTxt)
/// \brief Method that parses text in QguLineEdit and finds the array reduction requested
/// \param     QString
/// \exception None
/// \return    dimTripleStack

dimTripleStack  getDimTripleStack(QString &compTxt){

  dimTripleStack arrayTriples;

  bool notEmpty = true;
   while (notEmpty){
      dimTriple nextDim = getDimTriple(compTxt);
	 if (nextDim.size() == 3) arrayTriples.push_back(nextDim);
	 else notEmpty = false;
   }

   return arrayTriples;
 }


/// \fn  dimTriple * getDimTriple(QString &compTxt)
/// \brief Method that parses text in QguLineEdit and finds the array reduction requested
/// \param     QString
/// \exception None
/// \return    dimTriple *

dimTriple  getDimTriple(QString &compTxt){

   dimTriple newTriple;
   int ptr;

   // get the next dimension only and cut from input str
   ptr = compTxt.find(")");
   if(ptr == -1) return newTriple; // badly formed, newTriple will have size() == 1

   QString thisDim =  QString(compTxt.unicode(), ptr); // deep copy of first n=ptr characters
   compTxt.remove(0, ptr+1); // remove ")" for next call

   ptr = thisDim.find("Sum");
   int option;
   if(ptr != -1){
     option = 1;   // sum
     thisDim.remove(0, ptr+3);
   }
   else {
	  ptr = thisDim.find("Ave");
	  if(ptr != -1){
        option = 3;   // ave
        thisDim.remove(0, ptr+3);
      }
      else option = 0;  // elements (or slice done below)
   }

   ptr = thisDim.find("(");
   if(ptr == -1) return newTriple; // no more dimensions, newTriple will have size() == 1

   thisDim.remove(0, ptr+1); // remove " ("

   ptr = thisDim.find(",");
   int start;
   int stop;
   bool ok;
   if (ptr == -1) {
     // one index, slice requested
     start = thisDim.toInt(&ok);
	 if(ok){
	 stop = start;
	 option = 2;
	   newTriple.push_back(option);
	   newTriple.push_back(start);
	   newTriple.push_back(stop);
     }
   }
   else{
     QString startStr = QString(thisDim.unicode(), ptr); // deep copy of first n=ptr characters
	 thisDim.remove(0, ptr+1); // remove ")" for next call

	 newTriple.push_back(option);
     start = startStr.toInt(&ok);
	 if(ok){
	   newTriple.push_back(start);
     }

     stop = thisDim.toInt(&ok);
	 if(ok){
	   newTriple.push_back(stop);
     }

   }

   return newTriple;

}

/// \fn vector <dimTriple *> getSliceTriple()
/// \brief Method that parses text in QguLineEdit and finds the time slice requested
/// \param     None
/// \exception None
/// \return    vector <dimTriple *>

dimTriple getSliceTriple(const QString &Txt)
{
   if ( QuSlotDebug() )
	printf( "getSliceTriple this = %s \n", Txt.latin1() );

   dimTriple sliceTriple;
   int option = 0; // default is subset
   int start;
   int stop;
   QString compTxt = QString( Txt.latin1() ); // deep copy
   int ptr = compTxt.find(" <");
   if(ptr == -1) return sliceTriple;  // no T slice section found, sliceTriple is empty vector

   compTxt.remove(0, ptr+2); // remove " <"
   ptr = compTxt.find(">");
   compTxt.truncate(ptr); // remove ">"


   ptr = compTxt.find("Ave");
   if(ptr != -1) {
     option = 1;   // average
     compTxt.remove(0, ptr+3);
   }

   ptr = compTxt.find(",");

   bool ok;
   if (ptr == -1) {
      // one index, slice requested
	  start = compTxt.toInt(&ok);
	  if(ok){
	    stop = start;
	    option = 2;
	    sliceTriple.push_back(option);
	    sliceTriple.push_back(start);
	    sliceTriple.push_back(stop);
	  }
   }
   else{
	 sliceTriple.push_back(option);

     QString startStr = QString(compTxt.unicode(), ptr); // deep copy of first n=ptr characters
	 compTxt.remove(0, ptr+1); // remove "," for next call

     start = startStr.toInt(&ok);
	 if(ok){
        sliceTriple.push_back(start);
	 }

     stop = compTxt.toInt(&ok);
	 if(ok){
	    sliceTriple.push_back(stop);
	 }
  }
  return sliceTriple;
 }

QString compText(QString objTxt)
{
   QString compTxt(objTxt.latin1()); // deep copy
   int compStartAt = compTxt.find(" [");
   int sliceStartAt = compTxt.find(" <");
   int startAt=-1;
   if(compStartAt>0 && sliceStartAt>0)
     startAt=std::min(compStartAt,sliceStartAt);
   else if(compStartAt>0)
      startAt=compStartAt;
   else if(sliceStartAt>0)
     startAt=sliceStartAt;
   if(startAt != -1)
     compTxt.remove(0,startAt);
   else
     compTxt.remove(0,compTxt.length());

   return compTxt;
}

QString compText(QString objTxt, const dimTriple& slice_triple)
{
  QString compTxt(objTxt.latin1());
  int sliceStartAt = compTxt.find(" <");
  if(sliceStartAt!=-1)
  {
    int sliceEndAt=compTxt.find(">");
    compTxt.remove(sliceStartAt,1+sliceEndAt-sliceStartAt);
  }
  compTxt+=" < ";
  if(slice_triple[0]==1)
    compTxt+="Ave ";
  compTxt+=QChar(slice_triple[1]);
  if(slice_triple[1]!=slice_triple[2])
  {
      compTxt+=",";
      compTxt+=QChar(slice_triple[2]);
  }
  compTxt+=" >";
  return compTxt;
}

/// \fn QString getXrefName(QString objTxt)
/// \brief Method that returns the xref name from the text in the input slot
/// \param     objTxt of type QString
/// \exception None
/// \return    text of type QString
///

QString getXrefName(QString objTxt)
{
   QString txt = QString(objTxt.latin1()); // deep copy
   int ptr = txt.find("[=>");
   if (ptr == -1) return QString("");
   txt.remove(0, ptr+4);
   ptr = txt.find(" ]");
   txt.truncate(ptr);
   return txt;
}

/// \fn QString baseObjectText()
/// \brief Method that returns the base Object text with component text removed
/// \param     None
/// \exception None
/// \return    text of type QString
///
QString baseObjectText(QString objTxt)
{
   QString baseTxt(objTxt.latin1());
   int endAt = baseTxt.find(" [");
   if(endAt != -1) baseTxt.truncate(endAt);
   else {
	  endAt = baseTxt.find(" <");
      if(endAt != -1) baseTxt.truncate(endAt);
   }
   return baseTxt;
}

/// \fn QguComp compEntry(const char* cmpTxt, const char* cmpType)
/// \brief Method that returns the component entry in the dropdown box from the type of component and the component text
/// \param     cmpTxt of type const char*
/// \param     cmpType of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt, const char* cmpType)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpType,"VECTOR") ==0)
  {
    if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,X_STR_CMP)==0)
      comp=X_COMP;
    else if(strcmp(cmpTxt,Y_STR_CMP)==0)
      comp=Y_COMP;
    else if(strcmp(cmpTxt,Z_STR_CMP)==0)
      comp=Z_COMP;
    else if(strcmp(cmpTxt,R_STR_CMP)==0)
      comp=R_COMP;
    else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
      comp=LAT_DEG_COMP;
    else if(strcmp(cmpTxt,Td_STR_CMP)==0)
      comp=THETA_DEG_COMP;
    else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
      comp=PHI_DEG_COMP;
    else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
      comp=LAT_RAD_COMP;
    else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
      comp=THETA_RAD_COMP;
    else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
      comp=PHI_RAD_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"TIME") ==0)
  {
    if (strcmp(cmpTxt,T_STR_START)==0)
      comp=T_START_COMP;
    else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
       comp=T_CENTRE_COMP;
    else if (strcmp(cmpTxt,T_STR_END)==0)
      comp=T_END_COMP;
    else
      comp=UNKNOWN_COMP;
  }
  else if(strcmp(cmpType,"ARRAY") ==0)
    comp=ARRAY_COMP;
  else if(strcmp(cmpType,"XREF")==0)
    comp=XREF_OBJ;
  else if(strcmp(cmpType,"SLICE") ==0)
    comp=SLICE_COMP;
  else
    comp=UNKNOWN_COMP;

  return comp;
}

/// \fn QguComp compEntry(const char* cmpTxt)
/// \brief Method that returns the component entry in the dropdown box from the component text
/// \param     cmpTxt of type const char*
/// \exception None
/// \return    comp of type QguComp
///

QguComp compEntry(const char* cmpTxt)
{
  QguComp comp=UNKNOWN_COMP;
  if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,X_STR_CMP)==0)
    comp=X_COMP;
  else if(strcmp(cmpTxt,Y_STR_CMP)==0)
    comp=Y_COMP;
  else if(strcmp(cmpTxt,Z_STR_CMP)==0)
    comp=Z_COMP;
  else if(strcmp(cmpTxt,R_STR_CMP)==0)
    comp=R_COMP;
  else if(strcmp(cmpTxt,Ld_STR_CMP)==0)
    comp=LAT_DEG_COMP;
  else if(strcmp(cmpTxt,Td_STR_CMP)==0)
    comp=THETA_DEG_COMP;
  else if(strcmp(cmpTxt,Pd_STR_CMP)==0)
    comp=PHI_DEG_COMP;
  else if(strcmp(cmpTxt,Lr_STR_CMP)==0)
    comp=LAT_RAD_COMP;
  else if(strcmp(cmpTxt,Tr_STR_CMP)==0)
    comp=THETA_RAD_COMP;
  else if(strcmp(cmpTxt,Pr_STR_CMP)==0)
    comp=PHI_RAD_COMP;
  else if (strcmp(cmpTxt,T_STR_START)==0)
    comp=T_START_COMP;
  else if (strcmp(cmpTxt,T_STR_CENTRE)==0)
   comp=T_CENTRE_COMP;
  else if (strcmp(cmpTxt,T_STR_END)==0)
    comp=T_END_COMP;
  else if (strncmp(cmpTxt, " [=>",4) == 0)
    comp=XREF_OBJ;
  else if (strncmp(cmpTxt, " [",2)==0)
    comp=ARRAY_COMP;
  else if (strncmp(cmpTxt, " <",2)==0)
    comp=SLICE_COMP;
  else
    comp=WL_OBJ;
  
  return comp;
}

QdObject_var QuSetCompFromObjText(const QString& from, QString& to)
{
  // Uses component and time slice of from object to get comp and slice of to obj
  // Care is taken since the slice is by record, so must be converted to an interval
   if ( QuSlotDebug() )
	printf( "QuSetCompFromObjText from = %s to = %s\n", from.latin1(), to.latin1() );

  // get from object
  QdObject_var nil_obj;
  QString to_obj_txt=baseObjectText(to);
  QString from_obj_txt=baseObjectText(from);

  // Check this isn't just the same object and selection
  if(to_obj_txt == from_obj_txt) 
  {
     to = from; // reset slot in plot
     return QuGetObjByNameAndComp(from);
  }
  // get 'to' object
  QdObject_var to_obj=QuGetObjByName(to_obj_txt);
  if(to_obj.is_nil())  return nil_obj;
  
  // get component string from from_obj
  QString from_comp_str=compText(from);

  // get slice info from from_obj
  dimTriple slice_trip=getSliceTriple(from);
  // do we have slice info
  bool newSlice = false;
  if(slice_trip.size()==3)
  {
     // slice 'to' object
     QdObject_var from_obj = QuGetObjByName(from_obj_txt);
	
     if( !from_obj.is_nil() ) {
     
       // find time interval from slice and from_object
	   QdTimeInterval_var tint=QuGetIvlFromSlice(from_obj,slice_trip);
       if( !tint.is_nil() )
	   {
         // update slice for to_object using time interval
	 	 if(QuGetSliceFromIvl(to_obj, tint, slice_trip))
	         newSlice = true; // safe to replace time slice (it exists)
         }
	  }
   }
   
   // update component text with new slice info
   if( newSlice )
   {
      to_obj_txt += compText(from_comp_str, slice_trip);
   }
   else{
      // remove any slice info - cannot apply it
      int cutAt = from_comp_str.find(" <");
	  if (cutAt != -1 ) from_comp_str.left(cutAt); // time, < >, is always to rt of comp, [ ]
      to_obj_txt += from_comp_str;
   }
   // try and fetch this object
   QdObject_var obj=QuGetObjByNameAndComp(to_obj_txt);
   // if nil then not a valid object so return to unchanged
   if(obj.is_nil())
     return QuGetObjByNameAndComp(to);
 
   // update 'to' and new object
   to=to_obj_txt;
   return obj;
  
}

// -----------------------------------------------------------------

QdTimeInterval_var QuGetIvlFromSlice(QdObject_var obj, const dimTriple& slice_trip)
{
   QdTimeInterval_var tint;
   if(obj.is_nil())
     return tint;
  // fetch ttags from from_obj
  QdTimeSeq_var tt = get_timetags(obj);
  if(tt.is_nil())
    return tint;
  // ensure time slice/subset
  if(slice_trip.size()!=3)
    return tint;
  // fetch from and to times
  if( tt->sequence_size() > slice_trip[2])
  {
    tint = new QdTimeInterval(tt[slice_trip[1]], tt[slice_trip[2]] );
    return tint;
  }
  else
    return tint;
}

bool QuGetSliceFromIvl(QdObject_var obj,QdTimeInterval_var tint, dimTriple& slice_trip)
{
  if(obj.is_nil())
    return false;
  // fetch ttags from obj
  QdTimeSeq_var tt = get_timetags(obj);
  if(tt.is_nil())
    return false;
  // ensure it is a time slice/subset
  if(slice_trip.size()!=3)
    return false;
  bool found_ivl=false;
  int rec_start,rec_end;
  // find bounding records in tt for these start and end times
  found_ivl=QuGetIvlRecBound(tt,rec_start,rec_end,tint->start(),tint->end());
  if(found_ivl)
  {
    slice_trip[1]=rec_start;
    slice_trip[2]=rec_end;
  }
  return found_ivl;
}

// ---------------------------------------------------

QdObject_var QuGetObjByNameAndComp(QString name)
{
  if ( QuSlotDebug() )
  {
	printf( "Utility: QuGetObjByNameAndComp \n" );
	fflush(stdout);
  }

  QdObject_var retObj;

  QdObject_var baseObj;
  if(name.isNull() || name.isEmpty())  return retObj;

  QString baseObjStr;
  int compAt = name.find(" [");
  int timeAt = name.find(" <"); 
  if(compAt == -1 && timeAt == -1) baseObjStr = name; 
  else if(compAt != -1) baseObjStr = name.left(compAt);
  else baseObjStr = name.left(timeAt);

  Name baseObjName(baseObjStr.latin1());
 
  try
  {
     // if name is not found then resolve throws a NameNotFound exception
     baseObj=QDOSlist->resolve(baseObjName);
  }
  catch(Exception &e)
  {
     cout << "Cannot find base object named " << baseObjStr.latin1() << endl;
	 return retObj;
  }
  
  retObj = baseObj; // default for no component processed

  if(compAt != -1)
  {  
    QString compStr=QString(name.latin1());
	compStr.remove(0,compAt);
	
    if (compStr.contains(R_STR_CMP) > 0) 
      retObj = QuGetVectorComponent(baseObj, "r(mag)" );
    else if (compStr.contains(X_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "x" );
    else if (compStr.contains(Y_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "y" );
    else if (compStr.contains(Z_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "z" );
    else if (compStr.contains(Ld_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "Lat(deg)" );
    else if (compStr.contains(Lr_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "Lat(rad)" );
    else if (compStr.contains(Td_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "theta(deg)" );
    else if (compStr.contains(Tr_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "theta(rad)" );
    else if (compStr.contains(Pd_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "phi(deg)" );
    else if (compStr.contains(Pr_STR_CMP) > 0)
      retObj = QuGetVectorComponent(baseObj, "phi(rad)" );
    else if (compStr.contains(T_STR_START) > 0)
      retObj = QuGetTimeComponent(baseObj, "Tstart" );
    else if (compStr.contains(T_STR_CENTRE) > 0)
      retObj = QuGetTimeComponent(baseObj, "Tcentre" );
    else if (compStr.contains(T_STR_END) > 0)
      retObj = QuGetTimeComponent(baseObj, "Tend" );
    else if (compStr.contains("[=>") > 0)
    {
       QString xref_name=getXrefName(compStr);
       try{
         QdObject_var xrefObj = baseObj->get_xref(xref_name.latin1());
         retObj = xrefObj;
       }
       catch(Exception &e){
         QuMsgBar("Xref not found: ", xref_name.latin1(), true);
       }
    }
    else if (compStr.contains("[") > 0)
    {
	   string arrComp_xref = string(compStr.latin1());
	   arrComp_xref[0] = '_';
       compStr.remove(0,2);
       dimTripleStack dimStack=getDimTripleStack(compStr);
       retObj = QuGetArrayComponent(baseObj, dimStack, arrComp_xref );
    }

 }
 if(timeAt != -1) 
 { 
    // see if this exact object already exists (comp and time slice)
	QString tCompStr = QString(name.latin1());
	tCompStr.remove(0, timeAt);
	string tComp = string(tCompStr.latin1());
	tComp[0] = '_';

	// Use cache if found
    if(retObj->xref_exists(tComp.c_str())) return retObj->get_xref(tComp.c_str());
	
    // now subset on time slice if relevant
    dimTriple sliceTriple = getSliceTriple(tCompStr);

    if(sliceTriple.size() == 3){
      vector <string> objNameList;
      objNameList.push_back(baseObjName.srep());
	  QdObject_var slicedObj = QuTakeSlice(objNameList, retObj, sliceTriple, false);
	  retObj->change_xref(tComp.c_str(), slicedObj);
	  retObj = slicedObj;
    }
 }

 return retObj;
}


// ------------------------------------------------

QdObject_var QuGetIntervalByNameAndComp(QString name)
{

  // initialise var pointer here (will be nil if resolve fails)
  QdObject_var baseObj;
  if(name.isNull() || name.isEmpty())  return baseObj;
 
  QdObject_var retObj;
 
  QString baseObjStr;
  int compAt = name.find(" [");
  int timeAt = name.find(" <"); 
  if(compAt == -1 && timeAt == -1) baseObjStr = name; 
  else if(compAt != -1) baseObjStr = name.left(compAt);
  else baseObjStr = name.left(timeAt);
 
  
  Name baseObjName(baseObjStr.latin1());

  try
  {
     // if name is not found then resolve throws a NameNotFound exception
     baseObj=QDOSlist->resolve(baseObjName);
  }
  catch(Exception &e)
  {
     cout << "Cannot find base interval object named " << baseObjStr.latin1() << endl;
	 return retObj;
  }

  string ivl_xref;
  if( timeAt != -1) 
  {
    QString ivlStr=QString(name.latin1());
    ivlStr.remove(0,timeAt); 
	ivl_xref = "_ivl" + string(ivlStr.latin1());
  }
  else ivl_xref = "_ivl";
  
  if(baseObj->xref_exists(ivl_xref)) return baseObj->get_xref(ivl_xref);
  
  // Apply time slice if relevant
  if(ivl_xref != "_ivl") 
  {
    dimTriple sliceTriple = getSliceTriple(name);
    if(sliceTriple.size() == 3){
      vector <string> objNameList;
      objNameList.push_back(baseObjName.srep());
      retObj = QuTakeSlice(objNameList, baseObj, sliceTriple, true); // true returns just the interval
	  
	  baseObj->change_xref(ivl_xref, retObj);
      return retObj;
    }
  }
  
  // else get ivl of base object
  retObj = get_timeinterval(baseObj);

  baseObj->change_xref(ivl_xref, retObj);
  return retObj;
}

 /***********************************************************************/

QdObject_var QuTakeSlice(vector <string> dataObjectNames, QdObject_var dobj, dimTriple sliceTriple, bool getInterval){

 // initialise var pointer here (will be nil if resolve fails)
 QdObject_var retObject;
 
 if(dobj.is_nil())    return retObject;
 
 QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(dobj);
 QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(dobj);
 QdTimeSeq_var    ts = QdTimeSeq_var::narrow(dobj);

    // subsample of sequence chosen
	if(QuSlotDebug())   cout << "QuTakeSlice: SUBSAMPLE " << sliceTriple[0] <<" "<< sliceTriple[1] <<" "<< sliceTriple[2] << endl;

   if(sliceTriple[0] == 0 && getInterval){
      // Return interval only
	  // ------------------------
      if(QuSlotDebug())	  cout << "QuTakeSlice: SUBSET INTERVAL" << endl;

	  int recStart = sliceTriple[1];
	  int recEnd = sliceTriple[2];
      QdTimeSeq_var tt = get_timetags(dobj);
	  if( !tt.is_nil() && tt->sequence_size() > recEnd){
	         QdTimeInterval_var ti = new QdTimeInterval(tt[recStart], tt[recEnd] );
			 return (QdObject_var) ti;
	  }
      else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }

   }
   else  if(sliceTriple[0] == 0){
      // Subset
	  // ------------------------
      if(QuSlotDebug()) cout << "QuTakeSlice: SUBSET" << endl;

	  int record = sliceTriple[1];
	  int recCount = sliceTriple[2] - record + 1;

	  if( !ss.is_nil() && ss->sequence_size() > sliceTriple[2]){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     ss->get_subsequence(*retScalar, record, recCount);
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > sliceTriple[2]) {

	         QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	   	     ms->get_subsequence(*retMatrix, record, recCount);
	         retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > sliceTriple[2]) {

	         QdTimeSeq_var ttNew = new QdTimeSeq();
	   	     ts->get_subsequence(*ttNew, record, recCount);
	         retObject = (QdObject_var) ttNew;
	  }
      else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }
      
   }
   else if(sliceTriple[0] == 1){
      // Average
	  //-------------------------
	  if(QuSlotDebug()) cout << "QuTakeSlice: AVE" << endl;

	  int recStart = sliceTriple[1];
	  int recEnd = sliceTriple[2] ;

	  if( !ss.is_nil() && ss->sequence_size() > recEnd){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     double aveScalar=0;
	     for(int i=recStart; i<=recEnd; i++) aveScalar += ss[i];
	     aveScalar= aveScalar/(recEnd - recStart + 1);
		 retScalar->push_back( QdRScalar(aveScalar) );
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > recEnd) {

		  QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	      QdRMatrix_var aveMatrix = new QdRMatrix(ms[recStart]);
		  for(int i=recStart+1; i<=recEnd; i++) *aveMatrix += ms[i];
		  *aveMatrix = *aveMatrix/(recEnd - recStart + 1);
		  retMatrix->push_back(*aveMatrix);
	      retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > recEnd) {
           QdTime newTime = (ts[recStart] +  ts[recEnd])/2.0;
		   QdTimeSeq_var ttNew = new QdTimeSeq();
		   ttNew->push_back(newTime);
		   retObject = (QdObject_var) ttNew;
	  }
	  else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
	        return dobj;
      }
   }
   else if(sliceTriple[0] == 2){
      // Slice
	  //--------------------------

      if(QuSlotDebug()) cout << "QuTakeSlice: SLICE" << endl;
	  int record = sliceTriple[1];
	  if( !ss.is_nil() && ss->sequence_size() > record){

	     QdRScalarSeq_var retScalar = new QdRScalarSeq();
	     ss->get_subsequence(*retScalar, record, 1);
	     retObject = (QdObject_var) retScalar;
	  }
	  else if( !ms.is_nil() && ms->sequence_size() > record) {

	         QdRMatrixSeq_var retMatrix = new QdRMatrixSeq();
	   	     ms->get_subsequence(*retMatrix, record, 1);
	         retObject = (QdObject_var) retMatrix;
	  }
	  else if( !ts.is_nil() && ts->sequence_size() > record) {

	         QdTimeSeq_var ttNew = new QdTimeSeq();
	   	     ts->get_subsequence(*ttNew, record, 1);
	         retObject = (QdObject_var) ttNew;
	  }
	  else{
	        // not scalar or matrix sequence
	        QuMsgBar("Time Slice failed", false);
			return dobj;
      }

   }
   else{
      QuMsgBar("Time Slice failed", false);
      return dobj;
   }

	// slice xrefs with same sequence size (if sequence_size > 1)
	// This also slices Time Tags
    vector<string> sl;
	dobj->list_xref_names(sl);
	for(int i=0; i<sl.size(); i++)
	{
	    for(int j=0; j<dataObjectNames.size(); j++){
	       if(sl[i] == dataObjectNames[j]) {
		      sl[i][0] = CACHE_XREF_CHAR; // break infinite loop if circularity returns to an ancestor
			  break;
		   }
		}
		if(sl[i][0]==CACHE_XREF_CHAR) continue;
		QdObject_var xref_obj = dobj->get_xref(sl[i]);
		if( xref_obj->sequence_size() > 1 &&  xref_obj->sequence_size() == dobj->sequence_size() ){
		    dataObjectNames.push_back(sl[i]);
			if(QuSlotDebug()) cout<< " slice Xref " << sl[i].c_str() << endl;
			QdObject_var sliced_xref_obj = QuTakeSlice(dataObjectNames, xref_obj, sliceTriple, false);
			retObject->change_xref(sl[i], sliced_xref_obj);
		}
		else retObject->change_xref(sl[i], xref_obj);
	}

    // Add new xref to identify slice etc
	QdTimeSeq_var tt_out = get_timetags(retObject);
    if (tt_out.is_nil() ) return retObject;
	
	 QdString_var timeSliceAttr;

	if(sliceTriple[0] == 0){
	  timeSliceAttr = new QdString(tt_out[0].iso() + " to " + tt_out[tt_out->size()-1].iso());
	}
	else  if(sliceTriple[0] == 1){
	  timeSliceAttr = new QdString("Ave " + tt_out[0].iso() + " to " + tt_out[tt_out->size()-1].iso());
	}
	else  if(sliceTriple[0] == 2){
	  timeSliceAttr = new QdString("At " + tt_out[0].iso());
	}
	 retObject->change_xref("Time_Slice", timeSliceAttr);
	return retObject;

}


 /***********************************************************************/
 QdObject_var QuGetSubset(QdObject_var ts_in, int recStart, int recEnd){
 
// Subset any ts object on records
       
	 if(QuSlotDebug()) cout << "QuGetSubset" << endl;
    
	 int recCount = recEnd - recStart +1;

      // note that sub_sequence() call uses size of sequence to determine length of subseq
      
	 QdTimeSeq_var ts = QdTimeSeq_var::narrow(ts_in);
	 if( !ts.is_nil() && ts->sequence_size() > recEnd) {

		 QdTimeSeq_var ttNew = new QdTimeSeq(recCount);
		 ts->get_subsequence(*ttNew, recStart, recCount);
		 return (QdObject_var) ttNew;
	 }

	 QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(ts_in);
	 if( !ss.is_nil() && ss->sequence_size() > recEnd){

		 QdRScalarSeq_var retScalar = new QdRScalarSeq(recCount);
		 ss->get_subsequence(*retScalar, recStart, recCount);
		 return (QdObject_var) retScalar;
	 }
      
	 QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(ts_in);
	 if( !ms.is_nil() && ms->sequence_size() > recEnd) {

		 QdRMatrixSeq_var retMatrix = new QdRMatrixSeq(recCount);
		 ms->get_subsequence(*retMatrix, recStart, recCount );
		 return (QdObject_var) retMatrix;
	 }
      
	 QdObject_var nil_obj;
	 return nil_obj;
   }

 /***********************************************************************/

QdObject_var QuGetObjByName(QString name){

 // initialise var pointer here (will be nil if resolve fails)
 QdObject_var obj;
 Name objName(name.latin1());


 try
 {
 // if name is not found then resolve throws a NameNotFound exception
     obj=QDOSlist->resolve(objName);
 }
 catch(Exception &e)
 {
    // do nothing
 }
 return obj;

}

/***********************************************************************/

void QuDeleteObject(QString name){

 QuMsgBar(name.latin1(), "deleted", false);

 try
 {
     QDOSlist->unbind(Name(name.latin1()));    

 }
 catch(Exception &e)
 {
     QuMsgBar("name not found", name.latin1(), true);
 }

 return;    

}

/***********************************************************************/

void QuReplaceObject(QString name, QdObject_var new_obj){

  QdObject_var old_obj = QuGetObjByName(name);
  
  // safe against old object not existing
  if ( !old_obj.is_nil() ){
    try
    {
        QDOSlist->unbind(Name(name.latin1()));    
    }
    catch(Exception &e)
    {
        QuMsgBar( QString(e.msg().c_str()), name.latin1(), true);
    }
  }
  
  try
  {
      QuSetObjByName(new_obj, name);
  }
  catch(Exception &e)
  {
      QuMsgBar( QString(e.msg().c_str()), name.latin1(), true);
  }

  
 return;    

}

/***********************************************************************/

QdObject_var QuDuplicateObject(QString name){

 QdObject_var duplicateObj;
 Name objName(name.latin1());
 
 try
 {
 // if name is not found then resolve throws a NameNotFound exception
     QdObject_var obj=QDOSlist->resolve(objName);
     duplicateObj = Factory::clone( (QdObject_var) obj);
 }
 catch(Exception &e)
 {
     QuMsgBar("duplicate failed:", name.latin1(), true);

 }
 return duplicateObj;    


}

/***********************************************************************/

void QuRenameObject(QString oldName, QString newName){
  QdObject_var obj;
  
  try{
     obj = QuGetObjByName(oldName);
  }
  catch(Exception &e)
  {
     QuMsgBar(oldName.latin1(), "not found", true);
     return;
  }
 
  // unbind old name
  try{
      QDOSlist->unbind(Name(oldName.latin1()));    
  }
  catch(Exception &e)
  {
     QuMsgBar("cannot unbind", oldName.latin1(), true); 
     return;
  }  
    // bind new name 
  
  try{
      QDOSlist->bindx(Name(newName.latin1()), obj);    // and safe to replace
  }
  catch(Exception &e)
  {
     QuMsgBar("cannot bind", oldName.latin1(), true);
     return;
   }  
   QuSetOK();
}

//*********************************************************

bool QuSetObjByName(QdObject_var ToSetObj, QString name){
  QdString_var name_ptr;
  QdObject_var obj = ToSetObj;
  
  Name objName(name.latin1());

  // Find if objects are to be cleaned before putting on WL

  if ( checkBadData()->isChecked() || checkFills()->isChecked() )
       obj = QuCleanObject(ToSetObj);

  if( is_empty(obj) ) {
    QuMsgBar("empty object rejected", true);
    return false;
  }
  
  QdObject_var tmp = QuGetObjByName( name );
  bool name_found = !tmp.is_nil();

   if( name_found )
        QDOSlist->unbind( objName );  // remove binding if name already bound     
  
   try{
       QDOSlist->bindx(objName, obj);    // and safe to replace
   }
   catch(Exception &e)
   {
      e.print_msg();
      QuMsgBar( "QuSetObjByName: error binding ", name.latin1(), false);
      return false;
   }

   try{      
      int n = objName.num_components();
      name_ptr = new QdString(objName.get_component(n-1).get_id());
      obj->change_xref("ObjectName", name_ptr);
   }
   catch(Exception &e){
      QuMsgBar("QuSetObjByName: cannot add xref ", name_ptr->c_str(), true);
      return false;
   }
   
   return true;
   

}
/***********************************************************************/
void QuSetTxtAttr(char *attrName, const char *value, QdObject_var obj)
{
   // create Xref DO
    try{
      if(obj->xref_exists(attrName))
         obj->delete_xref(attrName); 
	 
      QdString_var DOx = new QdString(value);
    
// Add Xref called "attrName" to parent Object
// In this case DOx (the attribute) does NOT have a name of its own

      obj->set_xref(attrName, DOx);
    }
    catch(Exception &e){
      QuMsgBar("cannot set xref ", attrName, true);
    }

  return;

 } // end QuSetTextAttr


/***********************************************************************/
void applyMaskToXrefs(QdObject_var ms_in, QdObject_var ms_out, SequenceMask msk, QdTimeSeq_var tt_out)
{
	bool hasTT = TRUE;
	if (tt_out.is_nil() || tt_out->sequence_size() == 0) hasTT = FALSE;
	  
	// Recursively fix sequence type xrefs (Note: TT already done on fly)
	vector<string> sl;
	ms_in->list_xref_names(sl);
	for(int i=0;i<sl.size();i++)
	{
		if(sl[i][0]==CACHE_XREF_CHAR) continue;
		
		QdObject_var xref_i = ms_out->get_xref(sl[i]);
		
		if(xref_i.is_nil() || xref_i->sequence_size() != ms_in->sequence_size()) continue; 

		QdRScalarSeq_var xref_ss = QdRScalarSeq_var::narrow(xref_i);
		if( !xref_ss.is_nil() ){
			QdRScalarSeq_var xref_i_out = new QdRScalarSeq;
			xref_ss->get_subsequence(*xref_i_out, msk);
			xref_i_out->copy_xrefs_from(xref_ss);
		    QdTimeSeq_var tt=get_timetags(xref_i_out);
			if( !tt.is_nil() && hasTT){
				set_timetags(xref_i_out,tt_out);
			}
			applyMaskToXrefs(xref_ss, xref_i_out, msk, tt_out);
			ms_out->change_xref(sl[i].c_str(), xref_i_out);
		}
		else{
			QdRMatrixSeq_var xref_ms = QdRMatrixSeq_var::narrow(xref_i);
			if( !xref_ms.is_nil() ){
				QdRMatrixSeq_var xref_i_out = new QdRMatrixSeq;
				xref_ms->get_subsequence(*xref_i_out, msk);
			    xref_i_out->copy_xrefs_from(xref_ms);
		        QdTimeSeq_var tt=get_timetags(xref_i_out);
			    if( !tt.is_nil() && hasTT){
				set_timetags(xref_i_out,tt_out);
			    }
			    applyMaskToXrefs(xref_ms, xref_i_out, msk, tt_out);
				ms_out->change_xref(sl[i].c_str(), xref_i_out);
			}
		}
		// else leave as is
	  }
      return;
}	   

/***********************************************************************/
QdObject_var QuCleanObject(QdObject_var dobj){
  QdObject_var fill_xref;
  double fill;
  bool hasTT;
  QdObject_var obj_out;


    if (dobj->xref_exists("FILLVAL"))
      fill_xref = dobj->get_xref("FILLVAL");

    fill = -10.e+30; // in case xref not set

    QdTimeSeq_var ts_in=get_timetags(dobj);
    hasTT = !(ts_in.is_nil());

    QdTimeSeq_var ts_out=new QdTimeSeq;

    // is sequence a matrix sequence
    QdRMatrixSeq_var ms_in=QdRMatrixSeq_var::narrow(dobj);
    if(!ms_in.is_nil())
    {
      if(ms_in->empty())
          return obj_out;

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

      QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions

      if(fill_m.is_nil())
      {
          QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
          if(fill_v.is_nil())
              fill_v = new QdRScalar(fill);

          fill_m=new QdRMatrix(ms_in[0].dimdata(),fill_v->scalar());

      }

      // initialise matrix output sequence and reserve max space
      QdRMatrixSeq_var ms_out=new QdRMatrixSeq;
      ms_out->reserve(ms_in->size());
    
	// initialise mask
      SequenceMask msk(ms_in->size(),true);


      // get underlying data for fill matrix
      valarray<double> v_fill(fill_m->data()[fill_m->getgslice()]);

	for(register int i=0;i<ms_in->size();i++)
	{
		// catch bad time tags
		if(hasTT && ts_in[i].get_time_rep() < -6.e10) {
			msk[i] = false;
			continue;
		}

		bool valid_data=true;

		// get underlying data for matrix element
		valarray<double> v_in(ms_in[i].data()[ms_in[i].getgslice()]);
		for(register int j=0;j<v_in.size();j++)
		{
			const double& dfill = v_fill[j%v_fill.size()];
			valid_data=!QuIsBadData(v_in[j], dfill);
			if(!valid_data){
				msk[i] = false;
				break;
			}
		}
		// if valid update output matrix sequence
		if(valid_data)
		{
			ms_out->push_back(ms_in[i]);
			if(hasTT) ts_out->push_back(ts_in[i]);
		}
	}

	ms_out->copy_xrefs_from(ms_in);
	  
	if( hasTT){
		ts_out->copy_xrefs_from(ts_in);
		set_timetags(ms_out,ts_out);
	}

	applyMaskToXrefs(ms_in, ms_out, msk, ts_out);
	  
	return ms_out;
    }

    // scalar sequence case

   QdRScalarSeq_var ss_in=QdRScalarSeq_var::narrow(dobj);
   if(!ss_in.is_nil())
   {
     if(ss_in->empty())
          return obj_out;

      QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
      if( !fill_v.is_nil())
          fill = (double) *fill_v;

      // initialise mask
      SequenceMask msk(ss_in->size(),true);

      for(int i=0;i<ss_in->size();i++){
          msk[i] = !QuIsBadData(ss_in[i], fill);
		// catch bad time tags
		if(hasTT && ts_in[i].get_time_rep() < -6.e10) msk[i] = false;
      }

      QdRScalarSeq_var ss_out=new QdRScalarSeq;
      ss_in->get_subsequence(*ss_out,msk);
      ss_out->copy_xrefs_from(ss_in);

      if( !ts_in.is_nil() ){
        ts_in->get_subsequence(*ts_out,msk);
        ts_out->copy_xrefs_from(ts_in);
        set_timetags(ss_out,ts_out);
      }
      
      applyMaskToXrefs(ss_in, ss_out, msk, ts_out);
      return ss_out;
   }


    QdRMatrix_var m_in=QdRMatrix_var::narrow(dobj);
    if(!m_in.is_nil())
    {

      // if so, do we have a single fill-value or a matrix of different fill
      // values for each dimension

      QdRMatrix_var fill_m=QdRMatrix_var::narrow(fill_xref);

      // ...either way create matrix of fill values
      // this assumes the MatrixSeq is homogeneous i.e. each matrix
      // element has the same no. and sizes of dimensions

      if(fill_m.is_nil())
      {
          QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
          if(fill_v.is_nil())
              fill_v = new QdRScalar(fill);

          fill_m=new QdRMatrix(m_in->dimdata(),fill_v->scalar());

      }


      // iterator for fill values
      RMatrix::iterator fitr=fill_m->begin();

      // iterator for matrix
      RMatrix::iterator itr=m_in->begin();
      int j=0;
      while(itr!=m_in->end() )
      {
          const double& d=*itr++;
          if( QuIsBadData(d, *(fitr+j++) ) )
             return obj_out; // empty ptr
      }

      // catch bad time tags
      if(hasTT && ts_in[0].get_time_rep() < -6.e10) return obj_out;

      return dobj;    // return original object

    }

   QdRScalar_var s_in=QdRScalar_var::narrow(dobj);
   if(!s_in.is_nil())
   {
      QdRScalar_var fill_v=QdRScalar_var::narrow(fill_xref);
      if( !fill_v.is_nil())
          fill = (double) *fill_v;

      if( QuIsBadData( *s_in, fill))
          return obj_out; // empty ptr

      // catch bad time tags
      if(hasTT && ts_in[0].get_time_rep() < -6.e10)  return obj_out;

      return dobj; // return original object as clean
   }

   // string and other input types not cleaned - return input object

   return dobj;

}
/***********************************************************************/

bool QuIsBadData(double value, double fill){
  if( checkFills()->isChecked()){
    if( fill == 0 && value == fill) return true;
    else if ( fabs((value - fill)/fill) < CLOSE_ENOUGH ) return true;
  }

  if(checkBadData()->isChecked() ){
    if ( !finite(value) ) return true; // finds NaN, Infinity, -Infinity
  }

  return false;
}

/***********************************************************************/

bool QuIsBad(double value, double fill){

	// base level test, does not ask if strip is set
    if( fill == 0 && value == fill) return true;
    else if ( fabs((value - fill)/fill) < CLOSE_ENOUGH ) return true;
  
    if ( !finite(value) ) return true; // finds NaN, Infinity, -Infinity

  return false;
}

/***********************************************************************/

QString QuNameOnly(QString nameContext){
 
  int i = nameContext.findRev('/');
  int len = nameContext.length() - i -1;
  
  if (i < 0 || len <= 0 ) return nameContext;
  
  return nameContext.right((uint) len);

}

/***********************************************************************/

void QuMsgBar(const char* text, bool error){
  static QString Stxt;
  QPalette *palette = new QPalette(qApp->palette());
  QColorGroup g = QColorGroup(palette->active());
  QColorGroup in_g = QColorGroup(palette->inactive());
  if(error){
    g.setColor(QColorGroup::Text, g.midlight());
    g.setColor(QColorGroup::Background,g.link());
    in_g.setColor(QColorGroup::Background, QColor("#ff8888"));
  }

  palette->setActive(g);
  palette->setInactive(in_g);
  ((QMainWindow *)qApp->mainWidget())->statusBar()->setPalette(*palette);

  Stxt = text;
  ((QMainWindow *)qApp->mainWidget())->statusBar()->message(Stxt);
   
}

/***********************************************************************/

void QuMsgBar(const char* text1, const char* text2, bool error){
  static QString SStxt;

  SStxt = text1;
  SStxt += " ";
  SStxt += text2;
  QuMsgBar(SStxt.latin1(), error);

}

/***********************************************************************/

void QuSetOK(){
  static QString OKtxt = "  ok  ";

  // restore colour palette as last write may have been in error colours
  QPalette palette = QPalette(qApp->palette());
  QColorGroup g = QColorGroup(palette.active());
  QColorGroup in_g = QColorGroup(palette.inactive());
  palette.setActive(g);
  palette.setInactive(in_g);
  
  ((QMainWindow *)qApp->mainWidget())->statusBar()->setPalette(palette);

  ((QMainWindow *)qApp->mainWidget())->statusBar()->message(OKtxt,10000);

}

/***********************************************************************/

int QuStrcpy(char * To, const char * From, int ToLen){

  /* ToLen is the size of the To string available
     (i.e. malloc as To[ToLen+1] ) */

 size_t FromLen;

   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncpy(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cpy", From, true);
   }
   else strcpy(To, From);

   return 0;
}

/***********************************************************************/

int QuStrcat(char * To, const char * From, int ToLen){

  /* ToLen is the remaining size of the To string available 
     (i.e. malloc as To[UsedLen + ToLen + 1] ) */
  
 size_t FromLen;
   
   FromLen = strlen(From);
   if (FromLen > ToLen){
     strncat(To, From, ToLen);
     To[ToLen] = '\0';
     QuMsgBar("String truncated on cat", From, true);
   }
   else strcat(To, From);
   
   return 0;
}
/***********************************************************************/

void QuSetFrame(QdObject_var obj, const char *txt, const char *frm, const char *rep){
// Accepts either CSDS syntax frame or components of frame "txt>frm_rep"
// txt = vector, rot, scalar, tensor
// frm = gse, gsm, sc, sc2 etc
// rep = xyz, rtp, rlp ( or rpz, xy, rp, which are not handled by QSAS yet)
// if txt is in "vector>gse_xyz" syntax dis-assemble
  int MaxLen = 30;
  int MaxLen3 = 90;
  char flag[MaxLen+1];
  char frame[MaxLen+1];
  char representation[MaxLen+1];
  char frameAttr[MaxLen3+1];
  double i_order;

  // initialise strings
  strcpy(flag,"");
  strcpy(frame,"");
  strcpy(representation,"");
  i_order = 0;
  
  if(strstr(QuToUpper(txt),"SCALAR") != NULL) {
    strcpy(flag,"scalar");
    strcpy(frame,"na");
    strcpy(frameAttr,"scalar>na");
    i_order = 0;
  }
  else{
     if(strstr(txt,">") != 0){
         // parse txt
       char * buffer = (char *)  malloc(strlen(txt)+1);
	  strcpy(buffer, txt);
	  char * ptr = strtok(buffer, ">");
	  if(ptr != NULL) {
	    strncpy(flag, ptr, MaxLen);

	    ptr = strtok(NULL, "_");
         if (ptr != NULL){
	       strncpy(frame, ptr, MaxLen);
	    }
	    ptr = strtok(NULL, "");
         if (ptr != NULL){
	       strncpy(representation, ptr, MaxLen);
	    }
	  }
	  free(buffer);
     }
     else{
	   strncpy(flag, txt, MaxLen);
	   if(frm != 0) strncpy(frame, frm, MaxLen);
	   if(rep != 0) strncpy(representation, rep, MaxLen);
     }
	// assemble CSDS syntax Frame
	strcpy(frameAttr, flag);
	strcat(frameAttr, ">");
	strcat(frameAttr, frame);
	if(strcmp(representation,"") != 0){
	  strcat(frameAttr, "_");
	  strcat(frameAttr, representation);
	}
	
	if(strcmp(QuToUpper(flag),"VECTOR") == 0) i_order = 1;
	else if(strcmp(QuToUpper(flag),"TENSOR") == 0) i_order = 2; // We only go to order 2 in QSAS so far
	else if(strcmp(QuToUpper(flag),"ROT") == 0) i_order = 2;
	else i_order = 0; // no idea so this will block s/w doing vector & tensor stuff
	
  }// end else on scalar
  
  // Inputs parsed, now set attributes
  
  QuSetTxtAttr("Frame", &(frameAttr[0]), obj);
  QuSetTxtAttr("REPRESENTATION",&(representation[0]) , obj);
   
  if(strcmp(QuToUpper(frame),"GSE") == 0) strcpy(frame, "GSE>Geocentric Solar Ecliptic");
  else if (strcmp(QuToUpper(frame),"GSM") == 0) strcpy(frame, "GSM>Geocentric Solar Magnetic");
  QuSetTxtAttr("COORDINATE_SYSTEM", &(frame[0]), obj);
  
  QdSMatrix_var rep1 = new QdSMatrix(Dim(3), "");
  representation[3] = '\0';   // terminate at 3 char just in case 

  for(int j=2;j>=0;j--){
	(*rep1)[j] = QdString(&(representation[j]));
	representation[j] = '\0';
  }
  obj->change_xref("REPRESENTATION_1", rep1);
  

  try{ 
	   QdRScalar_var Torder = new QdRScalar(i_order);
        obj->change_xref("TENSOR_ORDER", Torder);
  }
  catch(Exception &e){
      QuMsgBar("cannot set xref ","TENSOR_ORDER" , true);
  }
  return;
}

/***********************************************************************/

char * QuGetFrameAttr(QdObject_var obj){
 // returns CSDS syntax frame string "flag>frame_rep"
   int MaxLen = 80;
   static char frameAttr[81];
   const char *frm   = QuGetFrame(obj);
   const char *rep   = QuGetRep(obj);
   const char *order = QuGetOrder(obj);

   if(strcmp(order, "scalar") == 0){
      strcpy(frameAttr,"scalar>na");
   }
   else if(strcmp(order, "vector") == 0){
      strcpy(frameAttr,"vector>");
	 strcat(frameAttr, frm);
	 strcat(frameAttr, "_");
	 strcat(frameAttr, rep);
   }
   else if(strcmp(order, "rot") == 0){
      strcpy(frameAttr,"rot>");
	 strcat(frameAttr, frm);
	 strcat(frameAttr, "_");
	 strcat(frameAttr, rep);
   }
   else if(strcmp(order, "tensor") == 0){
      strcpy(frameAttr,"tensor>");
	 strcat(frameAttr, frm);
	 strcat(frameAttr, "_");
	 strcat(frameAttr, rep);
   }
   else
      strcpy(frameAttr,"unknown>na");

   return frameAttr;
}

/***********************************************************************/

char * QuGetFrameAttr(QdObject_var obj1, QdObject_var obj2){
   char *frameAttr;
   const char *fr = QuGetFrame(obj1);
   if(strcmp(fr,"") != 0) {
      frameAttr = QuGetFrameAttr(obj1);
   }
   else{
      frameAttr = QuGetFrameAttr(obj2); // will be scalar if none found
   }
   return frameAttr;  // QuGetFrameAttr always returns valid string, so this does too
}

/***********************************************************************/

const char * QuGetOrder(QdObject_var obj){
   int MaxLen = 30;
   static char order[31];
   strcpy(order, "");  // return empty string if not found
   
   char *attrPtr = QuGetAttrText( "Frame", obj );
   if( strcmp(attrPtr, "") == 0){

      // didn't find frame, look for TENSOR_ORDER
	   if(obj->xref_exists("TENSOR_ORDER")){
           QdObject_var xref=obj->get_xref("TENSOR_ORDER");
           QdRScalar_var xrefS= QdRScalar_var::narrow(xref);
           QdString_var xrefStr= QdString_var::narrow(xref);
           if( !xrefS.is_nil()) {
		     int i_order = (int) *xrefS;
		     if(i_order == 0) strcpy(order, "scalar");
		     else if(i_order == 1) strcpy(order, "vector");
		     else strcpy(order, "tensor");
		   }
		   else if ( !xrefStr.is_nil())
		   {
		     int i_order = atoi(xrefStr->c_str());
		     if(i_order == 0) strcpy(order, "scalar");
		     else if(i_order == 1) strcpy(order, "vector");
		     else strcpy(order, "tensor");
		   }
        }
   }
   else{
      // use Frame attribute if present, allows user to override CAA metadata
      char * buffer = (char *) malloc(strlen(attrPtr)+1);
	 strcpy(buffer, attrPtr);
	 char * ptr;
	 
	 ptr = strtok(buffer, ">");

	 if(ptr != NULL) {
	   strncpy(order, ptr, MaxLen);
	 }
	 free(buffer);
   }
   
   return &(order[0]);
}

/***********************************************************************/

const char * QuGetFrame(QdObject_var obj){
   int MaxLen = 30;
   static char frame[31];
   strcpy(frame, ""); // always return something, so empty this first
   
   char *attrPtr = QuGetAttrText( "Frame", obj );
   if( strcmp(attrPtr, "") == 0){
      // didn't find frame, look for COORDINATE_SYSTEM
	 attrPtr = QuGetAttrText( "COORDINATE_SYSTEM", obj );
	 char * buffer = (char*) malloc(strlen(attrPtr)+1);
	 strcpy(buffer, attrPtr);
	 char * ptr = strtok(buffer, ">");
	 if(ptr != NULL) {
	   strncpy(frame, ptr, MaxLen);
	   free(buffer);
	   return &(frame[0]);
	 }
	 else return attrPtr; // this is itself static
   }
   else{
      char * buffer = (char *) malloc(strlen(attrPtr)+1);
	 strcpy(buffer, attrPtr);
	 char * ptr;
	 
	 ptr = strtok(buffer, ">");
	 if(ptr != NULL) {
	    // if scalar return empty
	    if(strcmp(QuToUpper(ptr),"SCALAR") == 0) return frame; // will be empty string
	    
	    ptr = strtok(NULL, "_");
         if (ptr != NULL){
	       strncpy(frame, ptr, MaxLen);
	    }
	 }
 
	 free(buffer);
   }
   
   return &(frame[0]);
}

/***********************************************************************/


const char * QuGetRep(QdObject_var obj){
  int MaxLen = 30;
   static char rep[31];
   strcpy(rep, ""); // always return something, so empty this first
   
   const char *attrPtr = QuGetAttrText( "Frame", obj );
   if( strcmp(attrPtr, "") == 0){
      // didn't find frame, look for REPRESENTATION
	 attrPtr = QuGetAttrText( "REPRESENTATION", obj );
	 if( strcmp(attrPtr, "") == 0){
	     attrPtr = QuGetAttrCondensed("REPRESENTATION_1", obj);

		int j=0;
		while(j < strlen(attrPtr) && isdigit(attrPtr[j]) ){
		  j++;
		}
		strncpy(rep, &(attrPtr[j]), MaxLen);
		return  &(rep[0]);
	 }
	 return  attrPtr; 
   }
   else{
      char * buffer = (char *) malloc(strlen(attrPtr)+1);
	 strcpy(buffer, attrPtr);
	 char * ptr;
	 
	 ptr = strtok(buffer, ">");
	 if(ptr != NULL) {
	    // if scalar return empty
	    if(strcmp(QuToUpper(ptr),"SCALAR") == 0) return rep; // will be empty string
	    
	    ptr = strtok(NULL, "_"); // gets frame
	    ptr = strtok(NULL, ""); // gets rep (rest of string)
         if (ptr != NULL){
	      strncpy(rep, ptr, MaxLen);
	    }
	 }
 
	 free(buffer);
   }

   return &(rep[0]);
}

/***********************************************************************/

bool QuIsThreeVector(QdObject_var obj)
{
  bool frameFound;
  // first test for frame (CSDS or CAA syntax)
  const  char * framePtr = QuGetFrameAttr(obj);

  if(strcmp(framePtr, "" ) == 0) frameFound = false; // neither Frame nor COORDINATE_SYSTEM present
  else 
  {
     frameFound = true;
     if(strstr(framePtr, "vector") == NULL) return false;
  }
 
  QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
  if(rmatrix.is_nil())
  {
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(rmatrixseq.is_nil()) return false;
    if(!rmatrixseq->sequence_size())  return false;
    try
    {
      if((rmatrixseq[0].dims()).size()!=1) return false;
      if((rmatrixseq[0].dims())[0]!= 3) return false;
      if(!rmatrixseq->is_homogeneous_seq()) return false;
	 if(frameFound == true) return true;
	 else{
	   QuMsgBar("3D object with no frame", false);
	   return false;
	 }
    }
    catch(Exception &e)
    {
       e.print_msg();
       return false;
    }
  } 
  
  try
  {
    if((rmatrix->dims()).size()!=1) return false;
    if((rmatrix->dims())[0]!=3) return false;
    if(frameFound == true)  return true;
    else{
	 QuMsgBar("3D object with no frame", false);
	 return false;
    }
  }
  catch(Exception &e)
  {
    e.print_msg();
    return false;
  }
}

int QuArrayDims(QdObject_var obj)
{
  QdRMatrix_var rmatrix=QdRMatrix_var::narrow(obj);
  if(rmatrix.is_nil())
  {
    QdRMatrixSeq_var rmatrixseq=QdRMatrixSeq_var::narrow(obj);
    if(rmatrixseq.is_nil())
      return 0;
    if(!rmatrixseq->sequence_size())
      return 0;
    try
    {
      if((rmatrixseq[0].dims()).size()>0 && rmatrixseq->is_homogeneous_seq())
        return (rmatrixseq[0].dims()).size();
    }
    catch(Exception &e)
    {
       e.print_msg();
       return 0;
    }
  } 
  
  try
  {
    if((rmatrix->dims()).size() > 0) return (rmatrix->dims()).size();
  }
  catch(Exception &e)
  {
    e.print_msg();
    return 0;
  }
  return 0;
}

/***********************************************************************/

bool QuIsScalar(QdObject_var dobj){

  if ( dobj.is_nil() ) return false;
  
  QdRScalar_var sc=QdRScalar_var::narrow(dobj);
  if(!sc.is_nil()) return true;
  
  QdRScalarSeq_var scs=QdRScalarSeq_var::narrow(dobj);
  if(!scs.is_nil()) return true;
 
  return false;
}



/***********************************************************************/
char* QuGetAttrConcat(const char *attrName, QdObject_var obj)
{
  static StringRep string_rep;
  string_rep.set_token(StringRep::SR_TOKEN_SEPARATOR,"");
  string_rep.set_token(StringRep::SR_TOKEN_BEGIN_DIMS,"");
  string_rep.set_token(StringRep::SR_TOKEN_END_DIMS,"");
  string_rep.set_token(StringRep::SR_TOKEN_DIM_SEPARATOR,"");
  string_rep.set_token(StringRep::SR_TOKEN_BEGIN_MATRIX,"");
  string_rep.set_token(StringRep::SR_TOKEN_END_MATRIX,"");
  static string str;
  static char *empty="";
  
  if(obj.is_nil()) return (char *) empty;
  
  if(obj->xref_exists(attrName))
  {
    QdObject_var xref=obj->get_xref(attrName);
    str=string_rep(xref);
    return (char *)str.c_str(); 
  }
  else
    return (char *) empty;
}
/***********************************************************************/

const char* QuGetAttrCondensed(const char *attrName, QdObject_var obj)
{
  string str ="";
  const char *empty="";
  int lenNoWhite;
  
  if(obj.is_nil()) return (char *) empty;
  
  if(obj->xref_exists(attrName))
  {
    QdObject_var xref=obj->get_xref(attrName);
    QdSMatrix_var Sxref = QdSMatrix_var::narrow(xref);
	if ( Sxref.is_nil() ) return empty;
	
	for(QdSMatrix::iterator itr=Sxref->begin(); itr != Sxref->end(); itr++){
	     lenNoWhite = itr->size();
		 while (lenNoWhite > 0 && (*itr)[lenNoWhite-1] == ' ' ) lenNoWhite--; // remove Themis trailing space
         str += itr->substr(0, lenNoWhite);
	}
	return str.c_str(); 
  }
  else
    return empty;
}



/***********************************************************************/
char* QuGetAttrText(const char *attrName, QdObject_var obj)
{
  static StringRep string_rep;
  string_rep.set_token(StringRep::SR_TOKEN_STRING_DELIMITER,"");
  static string str;
  static char *empty="";
  
  if(obj.is_nil()) return (char *) empty;
  
  if(obj->xref_exists(attrName))
  {
    QdObject_var xref=obj->get_xref(attrName);
    str=string_rep(xref);
	if(str.size() > 80) {
		str = str.substr(0, 80);
		str += "...\0";
	}
    return (char *)str.c_str(); 
  }
  else
    return (char *) empty;
}
/***********************************************************************/
char * QuGetVectorRep(QdObject_var obj)
{
    static char rep[10];

    strcpy(rep,"unk");

    const char *attr=QuGetRep(obj);
    if(strcmp(attr, "") == 0) return rep;
 
    strncpy(rep,attr,4);
    
    // default is Rad
    if(strcmp(QuToUpper(rep), "RTP") == 0 || strcmp(QuToUpper(rep),"RLP") == 0)
    {
      if( QuIsAngleDegrees( obj) )
        strcat(rep,"(deg)");
	 
    }
    
    return &(rep[0]);
}
/***********************************************************************/

QString QuTextInputDialog(const char *caption, const char *label,
                          const char * defText, bool *OK, QWidget * parent){

QString input;

#ifdef QT_VERSION
#if QT_VERSION <0x030000
    input = QInputDialog::getText( QString(caption), QString(label), 
                               QString(defText), OK, parent, "QuInput_2");

#else
    input = QInputDialog::getText(QString(caption), QString(label),
                QLineEdit::Normal, QString(defText), OK, parent, "QuInput_3");

#endif // end of #if
#endif // end of #ifdef

return input;

}
/***********************************************************************/

QdObject_var QuGetVectorXYZ(QdObject_var dov){
  
 if ( !QuIsThreeVector(dov) ) return (QdObject_var) 0;


 const double RadToDeg = 45.0 / atan(1.0);  
 const double DegToRad = atan(1.0) / 45.0;  

 // if xyz already return input var
    
 string in_rep = QuGetRep(dov);
 if( in_rep == "xyz")  return dov;

 // if rtp convert to xyz
 
 if (in_rep =="rtp"){
   bool IsDeg = QuIsAngleDegrees(dov);
   try{
     // try sequence first
     QdRMatrixSeq_var in_seq = QdRMatrixSeq_var::narrow(dov);
     if(!in_seq.is_nil()){
       int sz = in_seq->sequence_size();
       
       QdRMatrixSeq_var out_seq = new QdRMatrixSeq(sz, Dim(3));
       out_seq->copy_xrefs_from(in_seq); 
       string attr = QuGetAttrText("Frame", out_seq);
       QuSetRep("xyz", out_seq);
  
       for(int i=0; i<sz; i++){
         RMatrix xyz(Dim(3),0.);

         if( IsDeg ){
           rtp_deg_to_xyz(in_seq[i], xyz);  //convert rtp to xyz
	      out_seq[i] = xyz;
         }
         else{
           rtp_to_xyz(in_seq[i],xyz);  //convert rtp to xyz
	      out_seq[i] = xyz;
         }
       } 
       return (QdObject_var) out_seq;
     }
     else {
       QdRMatrix_var in_var = QdRMatrix_var::narrow(dov);
       if(!in_var.is_nil()){

         QdRMatrix_var out_var = new QdRMatrix( Dim(3), 0);
         out_var->copy_xrefs_from(in_var);
	    QuSetRep("xyz", out_var);

         if( IsDeg ){
           rtp_deg_to_xyz(*in_var,*out_var);  //convert rtp to xyz
         }
         else{
           rtp_to_xyz(*in_var, *out_var);  //convert rtp to xyz
         }
         return (QdObject_var) out_var;
       }
     } // end if/else on sequence
   }
   catch(Exception &e){
      e.print_msg();
      return (QdObject_var) 0;   
   }
 }

 // if rlp convert to  xyz
 
 if (in_rep == "rlp" ) {
   bool IsDeg = QuIsAngleDegrees(dov);
   try{
     // try sequence first
     QdRMatrixSeq_var in_seq = QdRMatrixSeq_var::narrow(dov);
     if(!in_seq.is_nil()){
       int sz = in_seq->sequence_size();
       QdRMatrixSeq_var out_seq = new QdRMatrixSeq(sz, Dim(3));
       out_seq->copy_xrefs_from(in_seq); 
       QuSetRep("xyz", out_seq);

       for(int i=0; i<sz; i++){
         RMatrix xyz(Dim(3),0.);

         if( IsDeg ){
           rlp_deg_to_xyz(in_seq[i],xyz);  //convert rlp to xyz
	   
	   out_seq[i] = xyz;
         }
         else{
           rlp_to_xyz(in_seq[i],xyz);  //convert rlp to xyz
	   out_seq[i] = xyz;
         }
       } 
       return (QdObject_var) out_seq;
     }
     else {
       QdRMatrix_var in_var = QdRMatrix_var::narrow(dov);
       if(!in_var.is_nil()){
       
         QdRMatrix_var out_var = new QdRMatrix( Dim(3), 0);
         out_var->copy_xrefs_from(in_var); 
         QuSetRep("xyz", out_var);

         if( IsDeg ){
           rlp_deg_to_xyz(*in_var, *out_var);  //convert rlp to xyz
         }
         else{
           rlp_to_xyz(*in_var, *out_var);  //convert rlp to xyz
         }
         return (QdObject_var) out_var;
       }
     } // end if/else on sequence
   }
   catch(Exception &e){
      e.print_msg();
      return (QdObject_var) 0;   
   }
 }
 
 // can't handle other frame representations
 
 return (QdObject_var) 0;
 
}
/***********************************************************************/
 
void QuSetRep(const char* rep, QdObject_var do_out){
  QdString_var frame_xref = QdString_var::narrow(do_out->get_xref(FRAME));
  if( !frame_xref.is_nil() ){
    size_t pos = frame_xref->find("_");
    if(pos != string::npos){
       string stem = frame_xref->substr(0,pos+1);
       stem += rep;
       QdString_var new_frame_xref= new QdString(stem);
       do_out->change_xref(FRAME, new_frame_xref);
     } 
  }
  
  // and SI_conversion
  
  QdString_var SI_xref = QdString_var::narrow(do_out->get_xref(SI_CONVERSION));
  if( !SI_xref.is_nil() ){
    size_t pos = SI_xref->find(" ");
    if(pos != string::npos){
       string stemSI = SI_xref->substr(0,pos+1);
       QdString_var new_SI_xref= new QdString(stemSI);
       do_out->change_xref(SI_CONVERSION, new_SI_xref);
     } 
  }
  
    // and UNITS
  
  QdString_var units_xref = QdString_var::narrow(do_out->get_xref(UNITS));
  if( !units_xref.is_nil() ){
    size_t pos = units_xref->find(" ");
    if(pos != string::npos){
       string stemUnits = units_xref->substr(0,pos+1);
       QdString_var new_units_xref= new QdString(stemUnits);
       do_out->change_xref(UNITS, new_units_xref);
     } 
  }
  
  return;
}

/***********************************************************************/

bool QuIsAngleDegrees(QdObject_var dobj){
  StringRep string_rep;
  static string str;

  string_rep.set_token( StringRep::SR_TOKEN_STRING_DELIMITER, "" );

  QdObject_var xref;

  if(dobj->xref_exists(SI_CONVERSION))
    xref = dobj->get_xref(SI_CONVERSION);

  if( xref.is_nil() ) {
    printf("No SI_conversion: Units for polar angle assumed to be in degrees\n");
	fflush(stdout);
    return true;  // default to assuming degrees
  }

  str=string_rep(xref);

  if (strstr(str.c_str(), "deg") == NULL &&
      strstr(str.c_str(), "Deg") == NULL ) return false;

  return true;


}
/***********************************************************************/

QdObject_var QuGetArrayComponent(QdObject_var doa, dimTripleStack dimStack, string arrComp_xref_in)
{
   string arrComp_xref;
   size_t posn = arrComp_xref_in.find(" <");
   if(posn != string::npos) arrComp_xref = arrComp_xref_in.substr(0, posn);
   else  arrComp_xref = string(arrComp_xref_in.c_str());
   
   if (arrComp_xref != ""){
	 // look for cache
     if(doa->xref_exists(arrComp_xref)) return doa->get_xref(arrComp_xref);
   }
   QdObject_var reducedArr = doa;
   int i_offset = 1;
   // Reduce each dimension in turn according to operation choice
   for(int i=0; i<dimStack.size(); i++)
   {
      int choice = dimStack[i][0];
      if(choice == 0 )
	  {
	   if(dimStack[i][1] > dimStack[i][2]){
	      // elements cannot be sliced cyclic
		 QuMsgBar("start entry higher than end for element selection", true);
	      return doa;
	   }
	    // elements  (keeps DEPEND_i but modifies - can have m x 1 array, not exactly same as slice)
	    try{
	       reducedArr = QuSubsetArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2]);
	    }
	    catch(Exception &e){
	       cerr << e.msg() << endl;
		  QuMsgBar("element selection: slice out of range", true);
            return doa;
	    }
		// this option does not remove dimension i
	 }
	 else  if(choice == 1)
	 {
	    // sum (removes DEPEND_i)
	    reducedArr = QuSumArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2], false);
	    i_offset--;  // dimension has been removed, next call in loop must be adjusted
	 }
	 else  if(choice == 3)
	 {
	    // ave (removes DEPEND_i)
	    reducedArr = QuSumArray(reducedArr, i + i_offset, dimStack[i][1], dimStack[i][2], true);
	    i_offset--;  // dimension has been removed, next call in loop must be adjusted
     }
	 else
	 {
	    // slice (removes DEPEND_i)
	    reducedArr = QuSliceArray(reducedArr, i + i_offset, dimStack[i][1]);
	    i_offset--; // dimension has been removed, next call in loop must be adjusted
	 }
	 if (reducedArr.is_nil() ) return doa; // break from loop and give up
   }
      
   if (arrComp_xref != "") doa->change_xref(arrComp_xref.c_str(), reducedArr);
   
   return reducedArr;
}


/***********************************************************************/

QdObject_var QuSliceArray(QdObject_var doa, int indx, int entry )
{
   // Reduce dimension by slicing at entry along dimension indx
   QdRScalar_var s_out;
   QdRMatrix_var m_out;
   QdRScalarSeq_var ss_out;
   QdRMatrixSeq_var ms_out;

   QdObject_var reducedArr; // output object for return  
   if(doa.is_nil()) return reducedArr;
   
   Dim mDim;
   int mRank;

   // GUI works [1,n] qdos works [0,n-1]
   int ii = indx - 1;
   int ei = entry - 1;

   QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
   QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

     // Handle sequence and single matrix separately
     // Matrix first
   if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
      if(ii >= mRank || ii < 0){
        cout << "Array Slice: ERROR - index out of range" << endl;
        return reducedArr; 
      }
      if(ei >= (Mat_in->dims())[ii] || ei < 0){
        cout << "Array Slice: ERROR - entry out of range for this index" << endl;
        return reducedArr; 
      }
   
      // handle each case separately
      if(mRank == 1){
        s_out = new QdRScalar((*Mat_in)[ei]);
        reducedArr = (QdObject_var) s_out;
      }
      else if(mRank == 2){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[ei][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][ei]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
      }
      else if(mRank == 3){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[ei][all][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][ei][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 2){
          m_out = new QdRMatrix((*Mat_in)[all][all][ei]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
     }
     else if(mRank == 4){
       if(ii == 0){
         m_out = new QdRMatrix((*Mat_in)[ei][all][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
         m_out = new QdRMatrix((*Mat_in)[all][ei][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
         m_out = new QdRMatrix((*Mat_in)[all][all][ei][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 3){
         m_out = new QdRMatrix((*Mat_in)[all][all][all][ei]);
         reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
        cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
        return reducedArr; 
     }
   } 	

   // try sequence
   else if ( !MatSeq_in.is_nil() ) {
     // sequence size
     int ndata = MatSeq_in->sequence_size();

     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

     if(ii >= mRank || ii < 0){
       cout << "Array Slice: ERROR - index out of range" << endl;
       return reducedArr; 
     } 
     if(ei >= (MatSeq_in[0].dims())[ii] || ei < 0){
       cout << "Array Slice: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }

     // handle each case separately
     if(mRank == 1){
       ss_out = new QdRScalarSeq((*MatSeq_in)[all][ei]);
       reducedArr = (QdObject_var) ss_out;
     }
     else if(mRank == 2){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else if(mRank == 3){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else if(mRank == 4){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][ei][all][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][ei][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][ei][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 3){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][ei]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
       cout << "Array Slice: Sorry, only matrices up to Rank 4 implemented" << endl;
       return reducedArr; 
     }
  
   }
   else{	
     cout << "Array Slice: ERROR - input not an array or array series!" << endl;
     return reducedArr; 
   }        

   
   // copy basic Xrefs across...  
   // this will ensure new object points to same time tags.
       
   reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char * lab="";
   char * comp=""; 
   char * unit="";
   char num_str[50];
   string labstr;
   static StringRep string_rep;

   if (reducedArr->xref_exists(LABLAXIS) ){
  
     // handle case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if(!labi.is_nil() && labi->size() > ei){
       // works for 1-D ARRAY ONLY await cef final spec.
       labstr == string(labi[ei]);
     }
     else{
       // regular LABLAXIS string
       labstr = string(QuGetAttrText(LABLAXIS, reducedArr));
     }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     lab = QuGetAttrText("ObjectName", reducedArr);
     labstr = string(lab);
   }
   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr = QString("DEPEND_%1").arg(indx);

   if (reducedArr->xref_exists(labl_attr.latin1()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.latin1());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if (!labi.is_nil() && labi->size() > ei){
       labstr += (*labi)[ei];  
     }
     else {
       labstr += " entry ";
       sprintf(num_str, "%d", ei+1);
       labstr += string(num_str);
       labstr += " from: ";
       labstr += string(QuGetAttrText(labl_attr.latin1(), reducedArr));
     }
   }
   else if (reducedArr->xref_exists(dep_attr.latin1())){
      labstr += string(" Slice(");
      QdObject_var d_xref=reducedArr->get_xref(dep_attr.latin1());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(d_xref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(d_xref);
      if(!Dxref.is_nil()){
        if (Dxref->xref_exists(LABLAXIS)){
          comp = QuGetAttrText(LABLAXIS, Dxref);
          labstr += string(comp);  
        }
        else if (Dxref->xref_exists(FIELDNAM)){
          comp = QuGetAttrText(FIELDNAM, Dxref);
          labstr += string(comp);   
        }
        if (Dxref->size() > ei){
          labstr += " ";
		  QdRScalar_var elt = new QdRScalar((*Dxref)[ei]);
          labstr += string_rep(elt);
        }
        if (Dxref->xref_exists(UNITS)){
          labstr += " ";
          unit = QuGetAttrText(UNITS, Dxref);
          labstr += string(unit);
        }
      }
      else if(!DSxref.is_nil()){
        cout <<"CAUTION: Index dependency is RECORD VARYING!  Slice may not be meaningful!" << endl;;

        if (DSxref->xref_exists(LABLAXIS)){
          comp = QuGetAttrText(LABLAXIS, DSxref);
          labstr += string(comp);  
        }
        else if (DSxref->xref_exists(FIELDNAM)){
          comp = QuGetAttrText(FIELDNAM, DSxref);
          labstr += string(comp);   
        }
        if (DSxref->size() > ei){
          labstr += " ";
          QdRScalar_var elt = new QdRScalar((*DSxref)[0][ei]);
          labstr += string_rep(elt);
        }
        if (DSxref->xref_exists(UNITS)){
          labstr += " ";
          unit = QuGetAttrText(UNITS, DSxref);
          labstr += string(unit);
        }
      }
      else{
        char compstr[3];
	    sprintf(compstr, "%1d", ei+1); // Note this is %one-d not %ell-d
        labstr += string(compstr);
      }
      labstr += string(")");
   }
   QuSetTxtAttr(LABLAXIS, labstr.c_str(), reducedArr);

    string mods_str = "Array sliced at entry ";
    sprintf(num_str, "%d", entry);
    mods_str += string(num_str);
    mods_str += " of index "; 
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str);
    mods_str += " (see LABLAXIS)"; 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

    QString label_i_1 = "?";
    QString depend_i_1 ="?";
 
    for(int i=1; i <= mRank; i++)
    {
      if(i < indx) continue;  // indices below slice are unaffected
	 
      QString labl_i = QString("LABEL_%1").arg(i);
      QString dep_i  = QString("DEPEND_%1").arg(i);

      if(i == indx){
        // slice removes this dimension
        if (reducedArr->xref_exists(labl_i.latin1()) ) reducedArr->delete_xref(labl_i.latin1());
        if (reducedArr->xref_exists(dep_i.latin1()) )  reducedArr->delete_xref(dep_i.latin1());
	   label_i_1 = QString(labl_i.latin1()); // deep copy
	   depend_i_1 = QString(dep_i.latin1()); // deep copy
      }
     else{
        // shuffle higher dimensions down
        if(reducedArr->xref_exists(labl_i.latin1())) reducedArr->rename_xref(labl_i.latin1(), label_i_1.latin1());
        if(reducedArr->xref_exists(dep_i.latin1()))  reducedArr->rename_xref(dep_i.latin1(), depend_i_1.latin1());
	   label_i_1 = QString(labl_i.latin1()); // deep copy
	   depend_i_1 = QString(dep_i.latin1()); // deep copy
      }
    }
 
   
   return reducedArr;
   
   // END QuSliceArray
}  


/***********************************************************************/

QdObject_var QuSumArray(QdObject_var doa, int indx, int fromK, int toK, bool average )
{
   // Reduce dimension by summing along dimension indx
   QdRScalar_var s_out;
   QdRMatrix_var m_out;
   QdRScalarSeq_var ss_out;
   QdRMatrixSeq_var ms_out;

   QdObject_var reducedArr; // output object for return  
   if(doa.is_nil()) return reducedArr;
   
   Dim mDim;
   int mRank;

   // GUI works [1,n] qdos works [0,n-1]
   int ii = indx - 1;

   QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
   QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

     // Handle sequence and single matrix separately
     // Matrix first
   if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
      if(ii >= mRank || ii < 0){
        cout << "Array Sum: ERROR - index out of range" << endl;
        return reducedArr; 
      }
      if(fromK > mDim[ii] || fromK < 1){
        cout << "Array Sum: ERROR - 'from' entry out of range for this index" << endl;
        return reducedArr; 
      }
      if(toK > mDim[ii] || toK < 1){
        cout << "Array Sum: ERROR - 'to' entry out of range for this index" << endl;
        return reducedArr; 
      }
   
     // handle each case separately
     if(mRank == 1){
	  if( fromK <= toK ){
          s_out = new QdRScalar((*Mat_in)[fromK-1]);
          for (int ei=fromK; ei< toK; ei++){
            *s_out += (*Mat_in)[ei];
          }
		  if(average) {
		     double nave = (double)(toK - fromK + 1);
		     *s_out /= nave;
		  }
       }
	   else{
		// take exterior sum
          s_out = new QdRScalar((*Mat_in)[fromK-1]);
          for (int ei=fromK; ei< mDim[ii] ; ei++){
            *s_out += (*Mat_in)[ei];
          }
          for (int ei=0; ei < toK ; ei++){
            *s_out += (*Mat_in)[ei];
          }
		  if(average) {
		     double nave = (double)( mDim[ii] - toK + 1 + fromK);
		     *s_out /= nave;
          }
	  }
       reducedArr = (QdObject_var) s_out;
     }
     else if(mRank == 2){
       if(ii == 0){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all]);
             for (int ei=fromK; ei< toK; ei++){
               *m_out += (*Mat_in)[ei][all];
             }
		     if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			}
	     }
	     else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all]);
             for (int ei=fromK; ei<  mDim[ii]; ei++){
               *m_out += (*Mat_in)[ei][all];
             }
             for (int ei=0; ei< toK; ei++){
               *m_out += (*Mat_in)[ei][all];
             }
		     if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			}
	     }
          reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[all][ei];
             }
		     if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			}
	    }
	    else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[all][ei];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[all][ei];
             }
		     if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			}
	    }
         reducedArr = (QdObject_var) m_out;
       }
       else{
      cout << "Array Sum: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else if(mRank == 3){
       if(ii == 0){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all][all]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[ei][all][all];
             }
		     if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			}
	    }
	    else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all][all]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[ei][all][all];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[ei][all][all];
             }
		     if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			}
	    }
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
	    if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1][all]);
             for (int ei=fromK; ei< toK; ei++){
               *m_out += (*Mat_in)[all][ei][all];
             }
		     if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			}
	    }
	    else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1][all]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
               *m_out += (*Mat_in)[all][ei][all];
             }
             for (int ei=0; ei< toK; ei++){
               *m_out += (*Mat_in)[all][ei][all];
             }
		     if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
	    }
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][all][fromK-1]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][ei];
	        }
			if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			}
          }
		else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][all][fromK-1]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[all][all][ei];
	        }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][ei];
	        }
			if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
		}
          reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Sum: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else if(mRank == 4){
       if(ii == 0){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all][all][all]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[ei][all][all][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			 }
		}
		else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[fromK-1][all][all][all]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[ei][all][all][all];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[ei][all][all][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
		}
          reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1][all][all]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			 }
		}
		else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][fromK-1][all][all]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[all][ei][all][all];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
		}
          reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][all][fromK-1][all]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			 }
	     }
	     else{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][all][fromK-1][all]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[all][all][ei][all];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
	     }
          reducedArr = (QdObject_var) m_out;
       } 
       else if(ii == 3){
	     if( fromK <= toK ){
             m_out = new QdRMatrix((*Mat_in)[all][all][all][fromK-1]);
             for (int ei=fromK; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
		        *m_out /= nave;
			 }
		}
		else
		{
		   // take exterior sum
             m_out = new QdRMatrix((*Mat_in)[all][all][all][fromK-1]);
             for (int ei=fromK; ei< mDim[ii]; ei++){
                *m_out += (*Mat_in)[all][all][all][ei];
             }
             for (int ei=0; ei< toK; ei++){
                *m_out += (*Mat_in)[all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *m_out /= nave;
			 }
		}
          reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Sum: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else{
        cout << "Array Sum: Matrix Rank error - tell csc support" << endl;
        return reducedArr;
     } 	
   }
   // try sequence
   else if ( !MatSeq_in.is_nil() ) {

     // sequence size
     int ndata = MatSeq_in->sequence_size();

     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

     if(ii >= mRank || ii < 0){
       cout << "Array Sum: ERROR - index out of range" << endl;
       return reducedArr; 
     } 
     if(fromK > mDim[ii] || fromK < 1){
       cout << "Array Sum: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }
     if(toK > mDim[ii] || toK < 1){
       cout << "Array Sum: ERROR - entry out of range for this index" << endl;
       return reducedArr; 
     }

     if(mRank == 1){
	  if( fromK <= toK ){
          ss_out = new QdRScalarSeq((*MatSeq_in)[all][fromK-1]);
          for (int ei=fromK; ei < toK; ei++){
            *ss_out += (*MatSeq_in)[all][ei];
          }
		  if(average) { 
			double nave = (double)(toK - fromK + 1);
			*ss_out /= nave;
		  }
	  }
	  else{
		// take exterior sum
          ss_out = new QdRScalarSeq((*MatSeq_in)[all][fromK-1]);
          for (int ei=fromK; ei < mDim[ii]; ei++){
            *ss_out += (*MatSeq_in)[all][ei];
          }
          for (int ei=0; ei < toK; ei++){
            *ss_out += (*MatSeq_in)[all][ei];
          }
		   if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ss_out /= nave;
			}
	  }
       reducedArr = (QdObject_var) ss_out;
     }
     else if(mRank == 2){
       if(ii == 0){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all]);
             for (int ei=fromK; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
	        // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all];
             }
             for (int ei=0; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1]);
             for (int ei=fromK; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][all][ei];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
               *ms_out += (*MatSeq_in)[all][all][ei];
             }
             for (int ei=0; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][all][ei];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else{
          cout << "Array Sum: ERROR - matrix rank, tell CSC support" << endl;
          return reducedArr; 
       }
     }
     else if(mRank == 3){

       if(ii == 0){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all][all]);
             for (int ei=fromK; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all][all];
             }
             for (int ei=0; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1][all]);
             for (int ei=fromK; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all];
             }
             for (int ei=0; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][fromK-1]);
             for (int ei=fromK; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][fromK-1]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
               *ms_out += (*MatSeq_in)[all][all][all][ei];
             }
             for (int ei=0; ei < toK; ei++){
               *ms_out += (*MatSeq_in)[all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
     }
     else if(mRank == 4){
       if(ii == 0){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all][all][all]);
             for (int ei=fromK; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][ei][all][all][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][fromK-1][all][all][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
                *ms_out += (*MatSeq_in)[all][ei][all][all][all];
             }
             for (int ei=0; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][ei][all][all][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1][all][all]);
             for (int ei=fromK; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][fromK-1][all][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all][all];
             }
             for (int ei=0; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][ei][all][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			 }
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][fromK-1][all]);
             for (int ei=fromK; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][fromK-1][all]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][ei][all];
             }
             for (int ei=0; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][ei][all];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 3){
	     if( fromK <= toK ){
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][fromK-1]);
             for (int ei=fromK; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)(toK - fromK + 1);
			    *ms_out /= nave;
		     }
		}
		else{
		   // take exterior sum
             ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][fromK-1]);
             for (int ei=fromK; ei < mDim[ii]; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][all][ei];
             }
             for (int ei=0; ei < toK; ei++){
                *ms_out += (*MatSeq_in)[all][all][all][all][ei];
             }
			 if(average) { 
			    double nave = (double)( mDim[ii] - toK + 1 + fromK);
		        *ms_out /= nave;
			}
		}
          reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Sum/Ave: ERROR - matrix rank, tell CSC support" << endl;
         return reducedArr; 
       }
     }
     else{
       cout << "Array Sum/Ave: ERROR - matrix rank, tell CSC support" << endl;
       return reducedArr; 
     } 
   }
   else{	
     cout << "Array Sum/Ave: ERROR - matrix rank, tell CSC support" << endl;
     return reducedArr; 
   }        

   if (reducedArr.is_nil() ) {
     cout << "Array Sum/Ave: Unable to create summed array object" << endl;
     return reducedArr; 
   }
   
   // copy basic Xrefs across...  
   // this will ensure new object points to same time tags.
     reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char * lab="";
   char * comp=""; 
   char * unit="";
   char num_str[20];
   string labstr;
   static StringRep string_rep;

   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr  = QString("DEPEND_%1").arg(indx);

   if (reducedArr->xref_exists(labl_attr.latin1()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.latin1());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref); // try pointer
	
     if (!labi.is_nil()){
       // works only for 1-D array, await cef final spec...
       if (average) labstr = "Ave ";
	   else labstr = "Sum ";
	  if (fromK <= labi->size() && toK <= labi->size()){
	     if( toK >= fromK){
	       for (int ei=fromK - 1; ei<toK; ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	     }
	     else{
	       for (int ei=fromK - 1; ei<labi->size(); ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	       for (int ei=0; ei<toK; ei++){
              labstr += ", ";
              labstr += labi[ei];
            }
	     }
       }
       else {
         if (average) labstr += string(" ave down ");
		 else labstr += string(" sum down ");
         labstr += string(QuGetAttrText(labl_attr.latin1(), reducedArr));
       }
	}
   }
   else if ( reducedArr->xref_exists(LABLAXIS)) {
     // look for case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
	if (!labi.is_nil()){
       // works only for 1-D array, await cef final spec...
       if (average) labstr = "Ave ";
       else labstr = "Sum ";
	  if (fromK <= labi->size() && toK <= labi->size()){
	     if( toK >= fromK){
	       for (int ei=fromK - 1; ei<toK; ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	     }
	     else{
	       for (int ei=fromK - 1; ei<labi->size(); ei++){
              if(ei != fromK - 1) labstr += ", ";
              labstr += labi[ei];
            }
	       for (int ei=0; ei<toK; ei++){
              labstr += ", ";
              labstr += labi[ei];
            }
	     }
       }
       else {
         if (average) labstr += string(" ave down ");
		 else labstr += string(" sum down ");
         labstr += string(QuGetAttrText(labl_attr.latin1(), reducedArr));
       }
	}

   }
   else if (reducedArr->xref_exists(dep_attr.latin1())){
      if (average) labstr += string(" Ave(");
      else labstr += string(" Sum(");
      QdObject_var dxref=reducedArr->get_xref(dep_attr.latin1());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(dxref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(dxref);
      if(!Dxref.is_nil()){
        if (Dxref->xref_exists(LABLAXIS)){
          comp = QuGetAttrText(LABLAXIS, Dxref);
          labstr += string(comp);  
        }
        else if (Dxref->xref_exists(FIELDNAM)){
          comp = QuGetAttrText(FIELDNAM, Dxref);
          labstr += string(comp);   
        }
        if (Dxref->xref_exists(UNITS)){
          labstr += " ";
          unit = QuGetAttrText(UNITS, Dxref);
          labstr += string(unit);
        }
      }
      else if(!DSxref.is_nil()){
        cout <<"CAUTION: Index dependency is RECORD VARYING!  Sum may not be meaningful!" << endl;;

        if (DSxref->xref_exists(LABLAXIS)){
          comp = QuGetAttrText(LABLAXIS, DSxref);
          labstr += string(comp);  
        }
        else if (DSxref->xref_exists(FIELDNAM)){
          comp = QuGetAttrText(FIELDNAM, DSxref);
          labstr += string(comp);   
        }
        if (DSxref->xref_exists(UNITS)){
          labstr += " ";
          unit = QuGetAttrText(UNITS, DSxref);
          labstr += string(unit);
        }
      }
      labstr += string(")");
   }
      
   else if (reducedArr->xref_exists(LABLAXIS) ){
  
      // handle case when LABLAXIS is not an array 
      QdObject_var xref=reducedArr->get_xref(LABLAXIS);
      QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
      if(labi.is_nil() ){
          // regular LABLAXIS string
          labstr = string(QuGetAttrText(LABLAXIS, reducedArr));
	 }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     lab = QuGetAttrText("ObjectName", reducedArr);
     labstr = string(lab);
   }
   else {
      if (average) labstr = string("Ave");
	  else labstr = string("Sum");
   }
   QuSetTxtAttr(LABLAXIS, labstr.c_str(), reducedArr);

    string mods_str;
	if (average) mods_str = "Array averaged over index ";
	else mods_str = "Array summed over index ";
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str);
    mods_str += " (see LABLAXIS)"; 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

   if (reducedArr->xref_exists(UNITS)){
      string units_str = string(QuGetAttrText(UNITS, reducedArr));
      if (average) units_str += " averaged";  
	  else  units_str += " summed"; 
      QuSetTxtAttr("UNITS", units_str.c_str(), reducedArr);
    }

    QString label_i_1 = "?";
    QString depend_i_1 ="?";
 
    for(int i=1; i <= mRank; i++)
    {
      if(i < indx) continue;  // indices below slice are unaffected
	 
      QString labl_i = QString("LABEL_%1").arg(i);
      QString dep_i  = QString("DEPEND_%1").arg(i);

      if(i == indx){
        // slice removes this dimension
        if (reducedArr->xref_exists(labl_i.latin1()) ) reducedArr->delete_xref(labl_i.latin1());
        if (reducedArr->xref_exists(dep_i.latin1()) )  reducedArr->delete_xref(dep_i.latin1());
	   label_i_1 = QString(labl_i.latin1()); // deep copy
	   depend_i_1 = QString(dep_i.latin1()); // deep copy
      }
     else{
        // shuffle higher dimensions down
        if(reducedArr->xref_exists(labl_i.latin1())) reducedArr->rename_xref(labl_i.latin1(), label_i_1.latin1());
        if(reducedArr->xref_exists(dep_i.latin1()))  reducedArr->rename_xref(dep_i.latin1(), depend_i_1.latin1());
	   label_i_1 = QString(labl_i.latin1()); // deep copy
	   depend_i_1 = QString(dep_i.latin1()); // deep copy
      }
    }
 

   return   reducedArr;
   
   // END QuSumArray
}

/***********************************************************************/

QdObject_var QuSubsetArray(QdObject_var doa, int indx, int fromK, int toK )
{
   
     // Remove elements from an array

   QdRMatrix_var m_out;
   QdRMatrixSeq_var ms_out;

   QdObject_var reducedArr; // output object for return  
   if(doa.is_nil()) return reducedArr;
   
   Dim mDim;
   int mRank;

   // GUI works [1,n] qdos works [0,n-1]
   int ii = indx - 1;
   int j = fromK - 1;
   int k = toK - 1;

   QdRMatrix_var Mat_in = QdRMatrix_var::narrow(doa);
   QdRMatrixSeq_var MatSeq_in = QdRMatrixSeq_var::narrow(doa);

     // Handle sequence and single matrix separately
     // Matrix first
   if ( !Mat_in.is_nil()  ) {
   
      // dimension sizes
      mDim = Mat_in->dimdata();
      mRank = mDim.rank();
 
	 if( fromK == 1 && toK == mDim[ii] ){
	    // change nothing, return
	    return doa;
	 }
	 
      if(ii >= mRank || ii < 0){
        cout << "Array Elements: ERROR - index out of range " << ii << " for range 0," << mRank << endl;
        return reducedArr; 
      }
      if(j >= mDim[ii] || j < 0){
        cout << "Array Elements: ERROR - start entry out of range for this index" << endl;
        return reducedArr; 
      }
      if(k >= mDim[ii] || k < 0){
        cout << "Array Elements: ERROR - end entry out of range for this index" << endl;
        return reducedArr; 
      }
   
      // handle each case separately
      if(mRank == 1){
        m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)]);
        reducedArr = (QdObject_var) m_out;
      }
      else if(mRank == 2){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
      }
      else if(mRank == 3){
        if(ii == 0){
          m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 1){
          m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)][all]);
          reducedArr = (QdObject_var) m_out;
        }
        else if(ii == 2){
          m_out = new QdRMatrix((*Mat_in)[all][all][slice(j,k-j+1,1)]);
          reducedArr = (QdObject_var) m_out;
        }
        else{
          cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
          return reducedArr;
        }
     }
     else if(mRank == 4){
       if(ii == 0){
         m_out = new QdRMatrix((*Mat_in)[slice(j,k-j+1,1)][all][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 1){
         m_out = new QdRMatrix((*Mat_in)[all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 2){
         m_out = new QdRMatrix((*Mat_in)[all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) m_out;
       }
       else if(ii == 3){
         m_out = new QdRMatrix((*Mat_in)[all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) m_out;
       }
       else{
         cout << "Array Slice: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
        cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
        return reducedArr; 
     }
   } 	

   // try sequence
   else if ( !MatSeq_in.is_nil() ) {
     // sequence size
     int ndata = MatSeq_in->sequence_size();

     // dimension sizes
     mDim = MatSeq_in[0].dimdata();
     mRank = mDim.rank();

	if( fromK == 1 && toK == mDim[ii] ){
	    // change nothing, return
	    return doa;
     }
	
     if(ii >= mRank || ii < 0){
       cout << "Array Elements: ERROR - index out of range at " << indx << " for rank " << mRank << endl;
       return reducedArr; 
     } 
     if(j >= mDim[ii] || j < 0){
       cout << "Array Elements: ERROR - start entry out of range for this index" << endl;
       return reducedArr; 
     }
     if(k >= mDim[ii] || k < 0){
       cout << "Array Elements: ERROR - end entry out of range for this index" << endl;
       return reducedArr; 
     }

     // handle each case separately
     if(mRank == 1){
       ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)]);
       reducedArr = (QdObject_var) ms_out;
     }
     else if(mRank == 2){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else if(mRank == 3){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }  
     else if(mRank == 4){
       if(ii == 0){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][slice(j,k-j+1,1)][all][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 1){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][slice(j,k-j+1,1)][all][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 2){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][slice(j,k-j+1,1)][all]);
         reducedArr = (QdObject_var) ms_out;
       }
       else if(ii == 3){
         ms_out = new QdRMatrixSeq((*MatSeq_in)[all][all][all][all][slice(j,k-j+1,1)]);
         reducedArr = (QdObject_var) ms_out;
       }
       else{
         cout << "Array Elements: Matrix Rank error - tell csc support" << endl;
         return reducedArr;
       }
     }
     else{
       cout << "Array Elements: Sorry, only matrices up to Rank 4 implemented" << endl;
       return reducedArr; 
     }
  
   }
   else{	
     cout << "Array Elements: ERROR - input not an array or array series!" << endl;
     return reducedArr; 
   }        

   // copy basic Xrefs across...  
   // this will ensure new object points to same time tags.
       
   reducedArr->copy_xrefs_from(doa); 

   // create new LABLAXIS 

   char * lab="";
   char * comp=""; 
   char * unit="";
   char num_str[50];
   QdString_var labstr;
   static StringRep string_rep;
   QdStringSeq_var labjk;
   QdStringSeq_var lab_i_jk;
   QdStringSeq_var dlab_i_jk;
   
   
   QString labl_attr = QString("LABEL_%1").arg(indx);
   QString dep_attr = QString("DEPEND_%1").arg(indx);
   if (reducedArr->xref_exists(LABLAXIS) ){
  
     // handle case when LABLAXIS is an array 
     QdObject_var xref=reducedArr->get_xref(LABLAXIS);
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
     if(!labi.is_nil() && labi->size() > k){
        // works for 1-D ARRAY ONLY await cef final spec.
	   for (int kidx=j; kidx <= k; kidx++){
	      string lab_kidx = (labi[kidx]).c_str();
		 labjk->push_back(lab_kidx);
	   }
     }
     else{
       // regular LABLAXIS string
       labstr = new QdString(QuGetAttrText(LABLAXIS, reducedArr));
     }
   }
   else if (reducedArr->xref_exists("ObjectName") ){
     lab = QuGetAttrText("ObjectName", reducedArr);
     labstr = new QdString(lab);
   }
   if( !labjk.is_nil() )
   {
      reducedArr->change_xref(LABLAXIS, labjk);
   }
   else if ( !labstr.is_nil() )
   {
      QuSetTxtAttr(LABLAXIS, labstr->c_str(), reducedArr);
   }
   // handle LABEL_i, 
   if (reducedArr->xref_exists(labl_attr.latin1()) ){
     QdObject_var xref=reducedArr->get_xref(labl_attr.latin1());
     QdStringSeq_var labi = QdStringSeq_var::narrow(xref);
	 QdSMatrix_var mlabi = QdSMatrix_var::narrow(xref);
	 if (!mlabi.is_nil() ){
	    QdSMatrix_var mlab_jk = new QdSMatrix((*mlabi)[slice(j,k-j+1,1)]);
		reducedArr->change_xref(labl_attr.latin1(), mlab_jk);
	    mlab_jk->copy_xrefs_from(mlab_jk); // ensure labels have their own xrefs
	 }

     if (!labi.is_nil() && labi->size() > k){
	   for (int kidx=j; kidx <= k; kidx++){
	      string lab_kidx = (labi[kidx]).c_str();
		  lab_i_jk->push_back(lab_kidx);
	   }
       reducedArr->change_xref(labl_attr.latin1(), lab_i_jk);
     }
	 
   }
   // handle DEPEND_i, 
   if (reducedArr->xref_exists(dep_attr.latin1()) ){

      QdObject_var dxref=reducedArr->get_xref(dep_attr.latin1());
      QdRMatrix_var Dxref=QdRMatrix_var::narrow(dxref);
      QdRMatrixSeq_var DSxref=QdRMatrixSeq_var::narrow(dxref);
      QdSMatrix_var dlabi = QdSMatrix_var::narrow(dxref);
      if (!dlabi.is_nil() ){
	    QdSMatrix_var dxref_jk = new QdSMatrix((*dlabi)[slice(j,k-j+1,1)]);
		reducedArr->change_xref(dep_attr.latin1(), dxref_jk);
	    dxref_jk->copy_xrefs_from(dlabi); // ensure depends have their own xrefs
      }
      if(!Dxref.is_nil()){  
	    QdRMatrix_var dxref_jk = new QdRMatrix((*Dxref)[slice(j,k-j+1,1)]);
         reducedArr->change_xref(dep_attr.latin1(), dxref_jk);
	    dxref_jk->copy_xrefs_from(Dxref); // ensure depends have their own xrefs
	    
	    // handle case when delta plus/minus are also arrays
		if(dxref_jk->xref_exists("DELTA_PLUS")){
			QdObject_var dp_xref=dxref_jk->get_xref("DELTA_PLUS");
			QdRMatrix_var dpMat_xref=QdRMatrix_var::narrow(dp_xref);
			if( !dpMat_xref.is_nil() ) {
				QdRMatrix_var dpMat_xref_jk = new QdRMatrix((*dpMat_xref)[slice(j,k-j+1,1)]);
				dxref_jk->change_xref("DELTA_PLUS", dpMat_xref_jk);
			}
		}
		if(dxref_jk->xref_exists("DELTA_MINUS")){
			QdObject_var dm_xref=dxref_jk->get_xref("DELTA_MINUS");
			QdRMatrix_var dmMat_xref=QdRMatrix_var::narrow(dm_xref);
			if( !dmMat_xref.is_nil() ) {
				QdRMatrix_var dmMat_xref_jk = new QdRMatrix((*dmMat_xref)[slice(j,k-j+1,1)]);
				dxref_jk->change_xref("DELTA_MINUS", dmMat_xref_jk);
			}
		}
      }
      if(!DSxref.is_nil()){    
	    cout <<"CAUTION: Index dependency is RECORD VARYING!  Slice may not be meaningful!" << endl;;
	    QdRMatrixSeq_var dsxref_jk = new QdRMatrixSeq((*DSxref)[all][slice(j,k-j+1,1)]);
         reducedArr->change_xref(dep_attr.latin1(), dsxref_jk);
	    dsxref_jk->copy_xrefs_from(DSxref); // ensure depends still have xrefs & time tags as t varying

	    // handle case when delta plus/minus are also arrays
	 QdObject_var dps_xref;
	 if(dsxref_jk->xref_exists("DELTA_PLUS"))
           dps_xref=dsxref_jk->get_xref("DELTA_PLUS");
         QdRMatrix_var dpMat_xref=QdRMatrix_var::narrow(dps_xref);
         QdRMatrixSeq_var dpMatS_xref=QdRMatrixSeq_var::narrow(dps_xref);
	    if( !dpMat_xref.is_nil() ) {
	      QdRMatrix_var dpMat_xref_jk = new QdRMatrix((*dpMat_xref)[slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_PLUS", dpMat_xref_jk);
	    }
	    else if( !dpMatS_xref.is_nil() ) { // DeltaPlus itself a sequence of matrices
	      QdRMatrixSeq_var dpMatS_xref_jk = new QdRMatrixSeq((*dpMatS_xref)[all][slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_PLUS", dpMatS_xref_jk);
	    }

	 QdObject_var dms_xref;
	 if(dsxref_jk->xref_exists("DELTA_MINUS"))
           dms_xref=dsxref_jk->get_xref("DELTA_MINUS");
         QdRMatrix_var dmMat_xref=QdRMatrix_var::narrow(dms_xref);
         QdRMatrixSeq_var dmMatS_xref=QdRMatrixSeq_var::narrow(dms_xref);
	    if( !dmMat_xref.is_nil() ) {
	      QdRMatrix_var dmMat_xref_jk = new QdRMatrix((*dmMat_xref)[slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_MINUS", dmMat_xref_jk);
	    }
	    if( !dmMatS_xref.is_nil() ) {  // DeltaMinus itself a sequence of matrices
	      QdRMatrixSeq_var dmMatS_xref_jk = new QdRMatrixSeq((*dmMatS_xref)[all][slice(j,k-j+1,1)]);
           dsxref_jk->change_xref("DELTA_MINUS", dmMatS_xref_jk);
	    }
	   
      }
   }
    string mods_str = "Array subset between entry ";
    sprintf(num_str, "%d", fromK);
    mods_str += string(num_str);
    mods_str += " and entry "; 
    sprintf(num_str, "%d", toK);
    mods_str += string(num_str);
    mods_str += " of index "; 
    sprintf(num_str, "%d", indx);
    mods_str += string(num_str); 
    QuSetTxtAttr("MODS", mods_str.c_str(), reducedArr);

   // index is retained, no need to shuffle LABEL and DEPEND indices down
   return   reducedArr; 
   
   // END QuSubsetArray
}

/***********************************************************************/
QdObject_var QuGetVectorComponent(QdObject_var obj, const char* vcomp)
{
   if(obj.is_nil() ) return obj;

   // First check for cached component
   string vcomp_str(vcomp);
   string vcomp_xref("_"+vcomp_str);
   if(obj->xref_exists(vcomp_xref)) return obj->get_xref(vcomp_xref);
   
   // not cached, so construct component

   QdObject_var vcomp_obj=0;
   
   if(!QuIsThreeVector(obj))  return vcomp_obj;
   
   QdRMatrixSeq_var in_seq=QdRMatrixSeq_var::narrow(obj);
   QdRMatrix_var in_matx=QdRMatrix_var::narrow(obj);
   if(in_seq.is_nil() && in_matx.is_nil()) return vcomp_obj;
   
   string inrep=QuGetVectorRep(obj);
   static CoordRepConversion<RMatrix,RScalar> coord_rep_conv;
   try{
     if(!in_matx.is_nil())
     {
       QdRScalar_var out_scalar=new QdRScalar;
       vcomp_obj=out_scalar;
       coord_rep_conv.get_vcomp(inrep,vcomp,*in_matx,*out_scalar);
     }
     else if(!in_seq.is_nil())
     {
       int sz=obj->sequence_size();
       QdRScalarSeq_var out_seq=new QdRScalarSeq(sz);
       vcomp_obj=out_seq;
       coord_rep_conv.get_vcomp(inrep,vcomp,*in_seq,*out_seq);
     }
     vcomp_obj->copy_xrefs_from(obj);
     QdString_var label;
     QdString_var label_axis_str;
     QdSMatrix_var label_axis;
	 
	 if(obj->xref_exists(LABEL_1))
     {
	   // Try LABEL_1 (CAA/CSDS)
       QdObject_var label_one_obj=obj->get_xref(LABEL_1);	     
       label_axis=QdSMatrix_var::narrow(label_one_obj);
	   vcomp_obj->delete_xref(LABEL_1);
     }
	 else if(obj->xref_exists(LABL_PTR_1))
     {
	   // Try LABL_PTR_1 (ISTP)
       QdObject_var label_ptr_obj=obj->get_xref(LABL_PTR_1);	     
       label_axis=QdSMatrix_var::narrow(label_ptr_obj);
	   vcomp_obj->delete_xref(LABL_PTR_1);
     }

     if(label_axis.is_nil() &&  obj->xref_exists(LABLAXIS))
     {
       QdObject_var label_axis_obj=obj->get_xref(LABLAXIS);	     
       label_axis=QdSMatrix_var::narrow(label_axis_obj);
       label_axis_str=QdString_var::narrow(label_axis_obj);
	   if(!label_axis_str.is_nil()) label = new QdString(*label_axis_str);
     }
     if(!label_axis.is_nil())
     {

		if(label_axis->size()==3 && label_axis->rank()==1)
		{
			label= new QdString((*label_axis)[0]);
			// get stem only
			size_t pos=label->find("x");
			if(pos!=string::npos)  *label=label->substr(0,pos);
		}
     }
     
     if(obj->xref_exists(FIELDNAM)){
        QdString_var fieldnam=new QdString(QuGetAttrText(FIELDNAM,in_seq));
	 
        if(label.is_nil() && fieldnam->size() > 0 ) label = new QdString( (*fieldnam).c_str() );
	 
        fieldnam->operator+=("(");
        if(vcomp==string(MAG))
        {
          fieldnam->operator+=("magnitude");
        }
        else{
          fieldnam->operator+=(string("")+vcomp);
        }
	 
	    fieldnam->operator+=(")");
        vcomp_obj->change_xref(FIELDNAM, fieldnam);

	 }
	 
     if(label.is_nil() && obj->xref_exists(OBJECT_NAME)){
	   // last gasp, try ObjectName created by qie
        QdString_var objectName=new QdString(QuGetAttrText(OBJECT_NAME,in_seq));
	 
        if(label.is_nil()  && objectName->size() >0) label = new QdString( (*objectName).c_str() );
	 
	 }
 
     QdString_var frame=new QdString("scalar>na");
     vcomp_obj->change_xref(FRAME,frame);
	 
     QdString_var si_conversion;
	 QdString_var units;
     if(vcomp==string(LAT_DEG) || vcomp==string(THETA_DEG) || vcomp==string(PHI_DEG))
     {
       si_conversion=new QdString("1>degree");
       units=new QdString("deg");
	   
       if( !label.is_nil() )
       {
         size_t pos=vcomp_str.find("(deg");
         if(pos!=string::npos)
           label->insert(0,vcomp_str.substr(0,pos)+" ");
         else
           label->insert(0,vcomp_str+" ");
       }
     }
     else if(vcomp==string(LAT) || vcomp==string(THETA) || vcomp==string(PHI))
     {
       si_conversion=new QdString("1>rad");
       units=new QdString("rad");
       if(!label.is_nil())
       {
         size_t pos=vcomp_str.find("(rad");
         if(pos!=string::npos)
           label->insert(0,vcomp_str.substr(0,pos)+" ");
         else
           label->insert(0,vcomp_str+" ");
       }
     }
	 else if(vcomp==string(MAG))
     {
	    si_conversion=new QdString(QuGetAttrText(SI_CONVERSION,obj));
	    units=new QdString(QuGetAttrText(UNITS,obj));
		if(!label.is_nil())
        {
		    label->insert(0,"|");
		    label->append("|");
        }
     }
     else
     {
       si_conversion=new QdString(QuGetAttrText(SI_CONVERSION,obj));
       units=new QdString(QuGetAttrText(UNITS,obj));
       if(!label.is_nil())
       {
           label->append("_");
           label->append(vcomp);
       }
	 }

	 if(si_conversion->size()==0) si_conversion->operator+=("1>(unknown)");
	   
	 if( !si_conversion.is_nil() ) vcomp_obj->change_xref(SI_CONVERSION, si_conversion);
     if( !units.is_nil() ) vcomp_obj->change_xref(UNITS, units);
     if( !label.is_nil() ) vcomp_obj->change_xref(LABLAXIS, label);
     QdString_var scid=new QdString(QuGetAttrText(SC_ID, obj));
     if(scid->size()!=0) vcomp_obj->change_xref(SC_ID,scid);
     obj->change_xref(vcomp_xref,vcomp_obj);
   }
   catch(Exception& e)
   { 
      e.print_msg();
      cout << "Failed to get component from a vector. " << endl;
      cout << "Note: vectors require a Frame attribute with syntax, e.g. 'vector>gse_xyz' " << endl;
   }


   return vcomp_obj;
}
// ------------------------------------------------------

QdObject_var QuGetTimeComponent(QdObject_var obj, const char* tcomp)
{
   // First check for cached component
   string tcomp_str(tcomp);
   string tcomp_xref("_"+tcomp_str);
   if(obj->xref_exists(tcomp_xref)) return obj->get_xref(tcomp_xref);

   // otherwise create and cache component
   QdObject_var out_obj=0;

   QdTimeInterval_var obj_ivl=get_timeinterval(obj);
   if(obj_ivl.is_nil()) return obj_ivl;

   QdTime_var tobj = 0;

   if(strcmp(tcomp, "Tstart") == 0 )  tobj = new QdTime(obj_ivl->start_abs());
   if(strcmp(tcomp, "Tcentre") == 0 ) tobj = new QdTime(obj_ivl->start_abs() + (obj_ivl->duration_abs()/2.0));
   if(strcmp(tcomp, "Tend") == 0 )    tobj = new QdTime(obj_ivl->end_abs());

   out_obj = (QdObject_var) tobj;
   out_obj->copy_xrefs_from(obj_ivl);
   
   // cache component
   obj->change_xref(tcomp_xref, out_obj);
   
   return out_obj;

}
// ------------------------------------------------------


double ConvertThetaRad(double theta){
  const double PiByTwo = 2.0 * atan(1.0);

  // works for colatitude to latitude and vice versa

  return (double) (PiByTwo - theta);
}

// -------------------------------------------------------

double ConvertThetaDeg(double theta){

  // works for colatitude to latitude and vice versa

  return (double)(90.0 - theta);
}

// -------------------------------------------------------


bool QuIsTSRegular(QdTimeSeq_var tt, double *spacing){
  double err;
  double Gap, Gaptest;
   
  *spacing = 0;
    
  if ( tt.is_nil() ) return false;
  if( !tt->is_sequence() ) return false;

  int n_seq = tt->sequence_size();
  
  Gap = (double) mSeconds(tt[1]) - (double) mSeconds(tt[0]);
  
  *spacing = Gap/1000.;
  if (Gap < 1) return false;  // work to no better than msec accuracy

  
  for (int i=2; i<n_seq; i++){
    Gaptest = (double)mSeconds(tt[i]) - (double)mSeconds(tt[i-1]);
  
    err = fabs((Gap - Gaptest)/Gap);
    if (err > CLOSE_ENOUGH ) return false;
  }
  return true;
}

// -------------------------------------------------------

char * QuSanitizeName ( const char * oldName){

  char * newName;

  int n_sl = 0;
  for(int i=0; i<strlen(oldName); i++)
    if(oldName[i] == '/') n_sl++;
    
  newName = (char *) malloc(strlen(oldName)+1+4*n_sl);
  int new_i=0;
  for(int i=0; i<strlen(oldName); i++){
    if(oldName[i] == '/'){
      strcpy(&(newName[new_i]),"_sl_");
      new_i += 4;
    }
    else{
      newName[new_i]=oldName[i];
      new_i++;
    }
  }
  newName[new_i]='\0';

  return newName;
}
/***********************************************************************/

const char * QuToUpper(const char *InStr)
{
  int Max_Str_Len = 300;
  int i;
  static char upper[301];
  int s_length;

  s_length = strlen(InStr);

  if(s_length >= Max_Str_Len) {
    char strTxt[50];
    sprintf(strTxt, "String too long at %ld", s_length);
    QuMsgBar(strTxt, TRUE);
    cout << " String too long '" << InStr << "'" << endl;
    s_length = Max_Str_Len;
  }

  for (int i=0; i < s_length; i++){
    upper[i]=toupper(InStr[i]);
  }
  upper[s_length] = '\0';

  return &(upper[0]);
}
/***********************************************************************/

bool QuStrCmp(const char *A, const char *B){
  bool flag = false;
  
  // QuToUpper uses static memory, so copy  first string
  char * strA = (char *) malloc(sizeof(char)*(strlen(A)+1));
  strcpy(strA,QuToUpper(A));
  
  if( strcmp(strA, QuToUpper(B)) == 0 ) flag = true;
  free (strA);

  return flag;

}
/***********************************************************************/

bool QuIsJoined(QdObject_var obj1, QdObject_var obj2){
 
    if(obj1.is_nil() || obj2.is_nil() ) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - Null object\n");
		fflush(stdout);
	  }
      return false;
    }
    
    if(obj1->sequence_size() != obj2->sequence_size())  {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - sequences of different lengths %ld and %ld \n", obj1->sequence_size(), obj2->sequence_size());
		fflush(stdout);
	  }
      return false;
    }
    
    QdTimeSeq_var tt1 = get_timetags(obj1);
    QdTimeSeq_var tt2 = get_timetags(obj2);
    
    if(tt1.is_nil() || tt2.is_nil() ) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - is_nil() input\n");
		fflush(stdout);
	  }
      return false;
    }
    
    if(*tt1 == *tt2) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - joined: same timetag object_var \n");    
		fflush(stdout);
	  }
      return true;
    }
    
    if(tt1->size() != tt2->size()) {
      if(QuSlotDebug())
	  {
		printf("QuIsJoined - number of time tags differ \n");  
		fflush(stdout);
	  }
      return false;
    }
    
    for(int i=0; i<tt1->size(); i++){
      if( abs(tt1[i] - tt2[i]) > Seconds(0.002) ) return false;  // fuzzy comparison to 2 msec
    }
    
    if(QuSlotDebug())
	{
		printf("QuIsJoined - joined to within 2 msec \n");  
		fflush(stdout);
	}
    
    return true;
}

/***********************************************************************/

bool copyXref(const char * name, QdObject_var from_ptr, QdObject_var to_ptr){

  if( from_ptr.is_nil() || to_ptr.is_nil() ) return false;
  
  if( from_ptr->xref_exists(name) ){
     QdObject_var xrefCopy = from_ptr->get_xref(name);
     to_ptr->set_xref(name, xrefCopy);
     return true;
  }
  
  return false;
}

/***********************************************************************/

bool copyDimXrefs( QdObject_var from_ptr, QdObject_var to_ptr){

 int mtxRank;
 char xrefName[13];

  if( from_ptr.is_nil() ) return false;

  QdRMatrix_var mtx = QdRMatrix_var::narrow(from_ptr);
  if(mtx.is_nil()) {
    QdRMatrixSeq_var mtxseq = QdRMatrixSeq_var::narrow(from_ptr);
    if(mtxseq.is_nil()) return false;
    if(mtxseq->sequence_size() == 0) return false;
    mtxRank = (mtxseq[0]).rank();
  }
  else
     mtxRank = mtx->rank();
  
     // NOTE: copyXref() is safe against non-existent xrefs, so just try it in each case and see.
  bool OK = false;
  for (int k=1; k <= mtxRank; k++){
    sprintf(xrefName, "DEPEND_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
    sprintf(xrefName, "LABEL_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
    sprintf(xrefName, "REFERENCE_%1.1d", k);
    if( copyXref(xrefName, from_ptr, to_ptr) ) OK=true;     
  }
  
  return OK;

}

/***********************************************************************/

QColor QuGetColour(QColor inC, bool *newChoice, QWidget * parent){

  *newChoice = false;
  static bool first = true;
  
  if(first)
  {
	int Ncustom = QColorDialog::customCount();
	if (Ncustom >= 16) 
	{
		QColorDialog::setCustomColor(0, QColor(255, 255, 255).rgb());
		QColorDialog::setCustomColor(1, QColor(0, 0, 0).rgb());
		QColorDialog::setCustomColor(2, QColor(255, 0, 0).rgb());
		QColorDialog::setCustomColor(3, QColor(0, 255, 0).rgb());
		QColorDialog::setCustomColor(4, QColor(0, 0, 255).rgb());
		QColorDialog::setCustomColor(5, QColor(0, 255, 255).rgb());
		QColorDialog::setCustomColor(6, QColor(255, 0, 255).rgb());
		QColorDialog::setCustomColor(7, QColor(255, 255, 0).rgb());
		QColorDialog::setCustomColor(8, QColor(255, 128, 0).rgb());
		QColorDialog::setCustomColor(9, QColor(128, 255, 0).rgb());
		QColorDialog::setCustomColor(10, QColor(0, 255, 128).rgb());
		QColorDialog::setCustomColor(11, QColor(0, 128, 255).rgb());
		QColorDialog::setCustomColor(12, QColor(128, 0, 255).rgb());
		QColorDialog::setCustomColor(13, QColor(255, 0, 128).rgb());
		QColorDialog::setCustomColor(14, QColor(85, 85, 85).rgb());
		QColorDialog::setCustomColor(15, QColor(170, 170, 170).rgb());
	}
	
	first = false;
  }
/*  
#if defined(Q_OS_MACX)

  QRgb newRGB = QColorDialog::getRgba(inC.rgb(), newChoice, parent );
  QColor c = QColor(newRGB);
  if(newChoice) return c; // changed, return new colour
  return inC;  // Cancelled - return same as input
     
#else
*/
  QColor c=QColorDialog::getColor(inC, parent);
  if(c.isValid() && c != inC){
    // 
    if(newChoice) *newChoice = TRUE;  // ensure valid bool pointer newChoice was passed in before setting it
    return c;
  }
  else{
    if(newChoice) *newChoice = FALSE;
    return inC;
  }

// #endif

} // end QuGetColour

/***********************************************************************/
// Find records bounding specified time interval using binary search on two ends

bool QuGetIvlRecBound(QdTimeSeq_var tt, int &recStart, int &recEnd, const Time& tStart, const Time& tEnd){
   int recL=0;
   int recU=tt->size()-1;
   if(tStart.get_time_rep() < tt[recL].get_time_rep() ) return false; // outside tt range
   if(tEnd.get_time_rep() > tt[recU].get_time_rep()  ) return false; // outside tt range

   QuGetBounds(tt, recL, recU, tStart);
   if(tt[recL].get_time_rep() == tStart.get_time_rep() ) recStart = recL;
   else recStart = recU;

   recL=0;
   recU=tt->size()-1;
   
   QuGetBounds(tt, recL, recU, tEnd);
   if(tt[recU].get_time_rep() == tEnd.get_time_rep() ) recEnd = recU;
   else recEnd = recL;
 
   return true;
}

// -------------------------------------------------------------------
// Find the records tha bound a time value using recursive binary search

void QuGetBounds(QdTimeSeq_var tt, int &recL, int &recU, const Time& t){

	if( recU == recL+1) return; // found bounding records
	
	int recH =  (recU + recL) / 2;
	if( tt[recH].get_time_rep() < t.get_time_rep() ) recL = recH;
	else recU = recH;
	
	return QuGetBounds(tt, recL, recU, t);
}

// --------------------------------------------------------------------

void QuNow(const char * txt){
	QDateTime  qdatetime ; 
	qdatetime = QDateTime::currentDateTime();
	printf(" >>>>Timed ------------------- %s at  %s\n",txt, qdatetime.toString( "dd.MM.yyyy-hh:mm:ss.zzz" ).latin1());

}

// --------------------------------------------------------------------

int QuLength(QdObject_var obj){
   int seqLen=1; 
   QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
   QdRScalar_var    sv = QdRScalar_var::narrow(obj);
   QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
   QdRMatrix_var    mv = QdRMatrix_var::narrow(obj);
   QdTimeSeq_var    tt = QdTimeSeq_var::narrow(obj);

   if( !tt.is_nil() ) return tt->sequence_size();
   if( !ss.is_nil() ) return ss->sequence_size();
   if( !sv.is_nil() ) return sv->sequence_size();
   if( !ms.is_nil() && ms->sequence_size() == 1) {
      if((ms[0].dimdata()).rank() == 1) return (ms[0].dims())[0];
   }
   if( !mv.is_nil() ) {
      if((mv->dimdata()).rank() == 1) return (mv->dims())[0];
   }
   return seqLen;
}

// --------------------------------------------------------------------

bool QuSet_Array(int ndata, double *floatArray, QdObject_var obj){
	// Note that QuLength call ensures we cannot run off array
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
	QdRScalar_var    sv = QdRScalar_var::narrow(obj);
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
	QdRMatrix_var    mv = QdRMatrix_var::narrow(obj);
	QdTimeSeq_var    ts = QdTimeSeq_var::narrow(obj);

	if( !ss.is_nil() ) {
		for(int i=0; i<ndata; i++){
			floatArray[i] = (double)ss[i];
		}
		return true;
	}
	if( !sv.is_nil() ) {
		floatArray[0] = (double)*sv;
		return true;
	}
	if( !ms.is_nil() && ms->sequence_size() == 1) {
		for(int i=0; i<ndata; i++){
			if((ms[0].dimdata()).rank() == 1) floatArray[i] = (double)(ms[0])[i];
		}
		return true;
	}
	if( !mv.is_nil() ) { 
		for(int i=0; i<ndata; i++){
			if((mv->dimdata()).rank() == 1) floatArray[i] = (double)(*mv)[i];
		}
		return true;
	}
	if( !ts.is_nil() && ts->sequence_size() > 0) {
		// t_midnite is midnight at start of day...
		Time t_midnight(ts[0]);

		t_midnight -= Seconds(ts[0].second_of_day());
		double secFromMidnight;

		for(int i=0; i<ndata; i++){
			secFromMidnight = Seconds(ts[i] - t_midnight);
			floatArray[i] = (double)secFromMidnight;
		}
		return true;
	}

	return false;
}


double getSpacingTolerance(QdRScalarSeq_var rseq){
   // find minimum spacing between elements 
   int len = rseq->sequence_size();
   double diff_sign = 1.0;
   double diffi;
   
   if(len == 0) return 0;
   if(len == 1) return (rseq[0] * 0.1); // allow something to be visible on the plot

   double diff = fabs(rseq[len-1] - rseq[0]);
   for(int i=1; i<rseq->sequence_size(); i++){
      diffi =  rseq[i] - rseq[i-1] ;
      if(fabs(diffi) < diff) diff = fabs(diffi);
	  diff_sign = diffi / diff;
   }
   return (diff * diff_sign);
}

// --------------------------------------------------------------------

QdObject_var makeSequence(QdObject_var inObj, int seqSize )
{  
    if(QuSlotDebug()) cout << " qdutil - makeSequence length " << seqSize << endl;
    
   QdObject_var outObj;
   
   QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(inObj);
   QdRScalar_var s = QdRScalar_var::narrow(inObj);
   QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(inObj);
   QdRMatrix_var m = QdRMatrix_var::narrow(inObj);
   
   if( !ss.is_nil() ){
      QdRScalarSeq_var out_ss = new QdRScalarSeq(seqSize, ss[0]);
      out_ss->copy_xrefs_from(inObj);
      outObj = out_ss;
   }
   else if ( !s.is_nil() ){
      QdRScalarSeq_var out_ss = new QdRScalarSeq(seqSize, *s);
      out_ss->copy_xrefs_from(inObj);
      outObj = out_ss;
   }
   else if ( !ms.is_nil() ){
      QdRMatrixSeq_var out_ms = new QdRMatrixSeq(seqSize, ms[0]);
      out_ms->copy_xrefs_from(inObj);
      outObj = out_ms;
   }
   else if ( !m.is_nil() ){
      QdRMatrixSeq_var out_ms = new QdRMatrixSeq(seqSize, *m);
      out_ms->copy_xrefs_from(inObj);
      outObj = out_ms;
   }
   else return inObj;
   
   return outObj;
}

// -------------------------------------------------------

bool QuGetDataRange(QdObject_var obj, double &min_val, double &max_val)
{
	if(QuSlotDebug()) cout << " QuGetDataRange " << endl;
	
	if(obj.is_nil()) return false;
	
	QdRScalarSeq_var ss = QdRScalarSeq_var::narrow(obj);
	if(!ss.is_nil())
	{
		if(ss->size() < 1) return false;
		
		min_val = ss[0];
		max_val = ss[0];
		for(int i=1; i<ss->size(); i++)
		{
			if(ss[i] < min_val) min_val = ss[i];
			if(ss[i] > max_val) max_val = ss[i];
		}
		return true;
	}
	
	QdRMatrixSeq_var ms = QdRMatrixSeq_var::narrow(obj);
	if(!ms.is_nil())
	{
		if(ms->size() < 1) return false;
		
		double value = min(ms[0]);
		min_val = value;
		value = max(ms[0]);
		max_val = value;
		for(int i=1; i<ms->size(); i++)
		{
			value = min(ms[i]);
			if(value < min_val) min_val = value;
			
			value = max(ms[i]);
			if(value > max_val) max_val = value;
		}	

		return true;
	}
	
	QdRMatrix_var m = QdRMatrix_var::narrow(obj);
	if(!m.is_nil())
	{
		
		min_val = min(*m);
		max_val = max(*m);

		return true;
	}
	
	QdTimeSeq_var ts = QdTimeSeq_var::narrow(obj);
	if(!ts.is_nil())
	{
		if(ts->size() < 1) return false;
		
		min_val = Seconds(ts[0]);
		max_val = Seconds(ts[ts->size()-1]);

		return true;
	}

	return false;
}

void setRestoreVersion(const char*version)
{
	int ptr=0;
	int len = strlen(version);
	
	while( ptr < len && !isdigit(version[ptr]) ) ptr++;
	if(ptr == len) 
	{ 
		qsas_restore_release = 0;
		return;
	}
	
	int r1 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len &&  version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len) 
	{ 
		qsas_restore_release = r1*10000;
		return;
	}
	
	int r2 = strtol( &(version[ptr]), NULL, 10);
	while( ptr < len && version[ptr] != '.' ) ptr++;
	ptr++;
	if(ptr >= len ) 
	{ 
		qsas_restore_release = r1*10000 + r2*100;
		return;
	}

	int r3 = strtol( &(version[ptr]), NULL, 10);
	
	qsas_restore_release = r1*10000 + r2*100 + r3;
	return;
}


// --------------------------------------------------------------------

namespace QSAS
{

// sort out exception handling

void QuMultiJoin( QdObjectSeq_var        join_obj_seq,
                  QdTimeSeq_var          join_pts,
                  QdObjectSeq_var        joined_obj_seq,
                  vector<KVDataBase_var> options,
                  int                    triage_cnt )
{
    if ( join_obj_seq.is_nil() )
    {
        throw BadJoin("[QuMultiJoin]: No Input Series specified");
    }

    if (join_pts.is_nil() )
    {
       throw BadJoin("[QuMultiJoin]: No target time tags specified");
    }

    int n_series = join_obj_seq->size();

    if ( options.size() != n_series )
    {
        throw BadJoin("[QuMultiJoin]: Options Wrong Size");
    }

    if (joined_obj_seq.is_nil() )
    {
       joined_obj_seq = new QdObjectSeq( n_series );
    }

    if ( joined_obj_seq->size()!= n_series )
    {
        joined_obj_seq->resize( n_series );
    }

    QdObjectSeq_var x_in        = new QdObjectSeq( n_series  );
    QdObjectSeq_var join_method = new QdObjectSeq( n_series  );
    QdObjectSeq_var gap_value   = new QdObjectSeq( n_series  );
    QdObjectSeq_var gap_handler = new QdObjectSeq( n_series  );
    QdTimeSeq_var   tt_out      = new QdTimeSeq(   *join_pts );

    for( int i = 0; i < n_series; i++ )
    {    
        KVDataBase_var opt = options[i];
        if ( opt.is_nil() )
        {
            throw BadJoin( "[QuMultiJoin]:No Options specified" );
        }

        if (join_obj_seq[i]->sequence_size() <= 1)
        {
             // replace input sequence with one of right length and all values the same
             join_obj_seq[i] = makeSequence(join_obj_seq[i], tt_out->sequence_size() );
	     set_timetags( join_obj_seq[i], tt_out) ; // is automatically joined
        }
	
        QdObject_var obj = join_obj_seq[ i ];
        x_in[i] = get_timetags( obj );

        if ( x_in[i].is_nil() )
        {
            throw BadJoin( "[QuMultiJoin]: No Time Tags specified" );
        }
      
	
        if ( joined_obj_seq[i].is_nil() )
        {
            joined_obj_seq[i]= obj->create();
        }

        QdObject_var obj_out = joined_obj_seq[ i ];
        obj_out->copy_xrefs_from( obj );
        set_timetags( obj_out, tt_out );

        QdString_var interp_method = QdString_var::narrow( opt->get( INTERP_METHOD ) );
        if ( interp_method.is_nil() )
        {
            throw BadJoin( "[QuMultiJoin]: No Interp Method specified" );
        }

        join_method[ i ] = interp_method;
        QdString_var gap_method = QdString_var::narrow( opt->get( GAP_METHOD ) );
        if ( gap_method.is_nil() )
        {
            throw BadJoin( "[QuMultiJoin]: No Gap Method specified" );
        }

        QdObject_var gap       =  opt->get(GAP_TOLERANCE);
        QdObject_var ave_width =  opt->get(BOXCAR_WIDTH);
        bool         boxcar    = ( *interp_method)  == ( String( BOXCAR )) ;
        if ( boxcar )
        {
           if ( ave_width.is_nil() )
           {
               throw BadJoin( "[QuMultiJoin]: No Average Width specified" );
           }

           if ( is_empty( ave_width ) )
           {
               throw BadJoin( "[QuMultiJoin]: Average Width is zero" );
           }

           gap_value[ i ] = ave_width;
        }
        else
        {
            if ( gap.is_nil() )
            {
                throw BadJoin("[QuMultiJoin]: No Gap Tolerance specified");
            }
            gap_value[i]=gap;
        }

        QdObject_var fill_value = opt->get( FILL_VALUE );
        double fill=-10.e+30; // set default
        String       gap_action = *gap_method;
        if ( gap_action == String(REMOVE_GAP) )
        {
            triage_cnt = n_series;
            // do nothing...gap handler is empty
        }
        else if ( gap_action == String( ZERO_FILL_GAP ) )
        {
            QdObject_var zero_fill = obj->create_elt();
            gap_handler[ i ] = zero_fill;
       }
       else if ( gap_action == String( FILL_GAP ) )
       {
          if(fill_value.is_nil()) fill_value=new QdRScalar(fill);
	     QdObject_var fill_obj=obj->create_elt();
	     if(fill_obj->object_type()!=fill_value->object_type())
	     {
	       QdRScalar_var fill_scalar=QdRScalar_var::narrow(fill_value);
	       if(!fill_scalar.is_nil()) fill=*fill_scalar;
	       fill_obj->setValue(fill);
	     }
	     else fill_obj=fill_value;
		
          gap_handler[i]=fill_obj;
       }
       else if( gap_action == String( LINEAR_INTERP_GAP ) )
       {
           gap_handler[ i ] = new QdString( LINEAR );
       }
       else if ( gap_action == String( NEAREST_NEIGHBOUR_GAP ) )
       {
           gap_handler[ i ] = new QdString( NEAREST_NEIGHBOUR );
       }
       else
	   {
           throw BadJoin( "[QuMultiJoin]: Unrecognised Gap Handler" );
	   }

   }


   multijoin(  x_in,
               join_obj_seq,
               tt_out,
               joined_obj_seq,
               join_method,
               gap_value,
               gap_handler,
               triage_cnt   );

    
    // handle time dependent xrefs for output objects
    // Note that this is done after the data are joined, so gaps are already removed
    // from target time tags, so we remove no gaps (and triage = 0)
    
    for(int i=0;i<n_series;i++){
	
	   QdObject_var gap  = options[i]->get(GAP_TOLERANCE);
	   
	   QuMultiJoinXrefs(join_obj_seq[i], joined_obj_seq[i], join_method[i], gap);
	   

    } // end for over joined variables


}

void QuMultiJoinXrefs(QdObject_var preJoinObj, QdObject_var joinedObj, QdObject_var joinMethod, QdObject_var gap){

       // xrefs will come from the pre-joined object (we need their original time tags)
	   // and be attached to the joined object
	   
	   // get target time tags from joined object. Note tt_out may have been changed by multijoin of data
       QdTimeSeq_var xtt_out = get_timetags(joinedObj);  

	   QdObjectSeq_var xt_in = new QdObjectSeq();
	   QdObjectSeq_var xobj_in = new QdObjectSeq();
       QdObjectSeq_var xjoin_method = new QdObjectSeq();
       QdObjectSeq_var xgap_value = new QdObjectSeq();
       QdObjectSeq_var xgap_handler = new QdObjectSeq();
       QdObjectSeq_var xjoined_obj_seq = new QdObjectSeq();

	  
	   vector<string> sl;
       preJoinObj->list_xref_names(sl);
	   vector<string> xrefNames;
       for(int ix=0;ix<sl.size();ix++)
       {
          if(sl[ix][0]==CACHE_XREF_CHAR) continue;
		  if(upper_case(sl[ix]) == "DEPEND_0" ) continue;  // helps avoid infinite loop
          QdObject_var xref_obj = preJoinObj->get_xref(sl[ix]);
		  QdNObject_var nobj = QdNObject_var::narrow(xref_obj);
		  if( nobj.is_nil() || nobj->sequence_size() != preJoinObj->sequence_size()) continue; // not numeric xref, can't interpolate
		  
		  
		  QdTimeSeq_var tobj = get_timetags(xref_obj);
		  if(tobj.is_nil() ) {
		     // sequence length is same as object, so use tt from object
			 tobj = get_timetags(preJoinObj);
			 set_timetags(xref_obj, tobj); // keep them for recursive call for xrefs of xrefs

		  }

		  if(!tobj.is_nil()){
		     // has time tags and is a numeric type
		     xrefNames.push_back(sl[ix]);

		     xt_in->push_back((QdObject_var) tobj);
		     xobj_in->push_back((QdObject_var) nobj);
		   	
			 xjoin_method->push_back(joinMethod);     // inherit same method as data obj

             xgap_value->push_back(gap);
		   
		     QdString_var xgap = new QdString(NEAREST_NEIGHBOUR); // don't know how to cross gaps
             xgap_handler->push_back(xgap);	
 
		   
		     QdObject_var xref_out = xref_obj->create();
		     xjoined_obj_seq->push_back(xref_out);
		     xref_out->copy_xrefs_from(xref_obj);
             set_timetags(xref_out,xtt_out);
		}
		
       } // end for over variable's xrefs
	  
       int triage_cnt = 0;
	   try{
           multijoin(xt_in, 
		             xobj_in, 
				     (QdObject_var) xtt_out, 
				     xjoined_obj_seq, 
				     xjoin_method, 
				     xgap_value, 
				     xgap_handler, 
				     triage_cnt);
       }
       catch(Exception &e){
             cout << e.msg().c_str() << endl;
             return ;
       }   
	  
	   // attach new xrefs
	   for (int ix=0; ix < xjoined_obj_seq->size(); ix++){
	     joinedObj->change_xref(xrefNames[ix], xjoined_obj_seq[ix]);
		 // recursively join xrefs of t varying xrefs
		 QuMultiJoinXrefs(xobj_in[ix], xjoined_obj_seq[ix], joinMethod, gap);
	  }
}

} // end namespace QSAS
